// Copyright 1999-2014. Parallels IP Holdings GmbH. All Rights Reserved.
Smb.FormApsCatalog = Class.create(Jsw.FormAjax, {

    _isSearchResultsShown: false,
    _catalogType: 'apscatalog',
    _isCategoriesLoaded: false,

    _initConfiguration: function($super, config) {
        $super(config);
        Jsw.baseUrl = this._getConfigParam('baseUrl', '');
        this._catalogType = $('catalogType').value;

        this._resizeCallback = this._getConfigParam('resizeCallback', null);
    },

    _initComponentElement: function($super) {
        $super();

        element = new Element('div', {
            'class': 'objects-content'
        });
        element.appendChild(
            new Element('div', {
                'id': 'overview-catalog',
                'class': 'objects-content-area'
            })
        );
        element.appendChild(
            new Element('div', {
                'id': 'category-info',
                'class': 'objects-content-area'
            })
        );
        element.appendChild(
            new Element('div', {
                'id': 'details-info',
                'class': 'objects-content-area'
            })
        );

        this._componentElement.appendChild(this._getSearchBox());
        this._componentElement.appendChild(element);
    },

    _executeResizeCallback: function($super) {
        if (this._resizeCallback) {
            this._resizeCallback();
        }
    },

    render: function($super) {
        $super();
        this._addSearchHandlers();
        this._initBlocksResize();

        if ('apscatalog' == this._catalogType && document.location.hash != '#show-all') {
        	this.showCategories();
        } else {
            this.showSearchResults(false);
        }

        var category = this._getConfigParam('category');
        if (category) {
            this.showCategory(category);
        }

        var details = this._getConfigParam('details');
        if (details) {
            this.showDetails(details);
        }

        this._executeResizeCallback();
    },

    _showPanel: function(panel) {
        if (panel == 'overview-catalog') {
            $('overview-catalog').show();
        } else {
            $('overview-catalog').hide();
        }

        if (panel == 'category-info') {
            $('category-info').show();
        } else {
            $('category-info').hide();
        }

        if (panel == 'details-info') {
            $('details-info').show();
            $('current-action-navigation').hide();
            $('current-action-buttons').show();
            $('search-control').hide();
            $('search-box').hide();
        } else {
            $('details-info').hide();
            $('current-action-navigation').show();
            $('current-action-buttons').hide();
            $('search-control').show();
            $('search-box').show();
        }

        if (('marketplace' == this._catalogType) && ('category-info' == panel)) {
            $('catalog-actions-box').setStyle({ display: 'none' });
        } else {
            $('catalog-actions-box').setStyle({ display: '' });
        }

        this._executeResizeCallback();
    },

    returnToHome: function() {
        this.showCategories();

        return false;
    },

    returnToCategory: function() {
        if ($('category-info').empty()) {
            this.showCategories();
        } else {
            this._showPanel('category-info');
        }
        return false;
    },

    showCategories: function() {
    	this._showPanel('overview-catalog');
        
        Element.update($('current-action-navigation'), this._locale.showCategories + ' | <a class="view-switcher-applications" href="#">' + this._locale.showApplications + '</a>');
        this._addViewSwitcherHandlers();
        
        if (this._isCategoriesLoaded) {
            return false;
        }

        Element.update($('overview-catalog'), '<div class="ajax-loading">' + this._locale.loading + '</div>');

        new Ajax.Updater("overview-catalog", Jsw.baseUrl + "/app/categories-data/catalogId/" + $('catalogId').value, {
            method: 'get'
        });

        this._isCategoriesLoaded = true;

        return false;
    },

    showDetails: function(downloadItem) {
        Element.update($('details-info'), '<div class="ajax-loading">' + this._locale.loading + '</div>');

        this._showPanel('details-info');

        var applicationDetails = $H(downloadItem);
        new Ajax.Updater({success: "details-info"}, Jsw.baseUrl + "/app/details-data/catalogId/" + $('catalogId').value, {
            method: 'post',
            parameters: { applicationDetails: Object.toJSON(applicationDetails) },
            evalScripts: true,
            onFailure: this.onErrorShowDetails.bind(this, downloadItem)
        });
        return false;
    },

    onErrorShowDetails: function(downloadItem) {
        $('details-info').update(this._locale.connectionFailed.replace('%%retryLink%%', '<a href="#">' + this._locale.retry + '</a>'));
        $('details-info').select('a').each(function(element) {
            element.observe('click', this.showDetailsRetry.bind(this, downloadItem));
        }, this);
    },

    showDetailsRetry: function(downloadItem) {
        this.showDetails(downloadItem);
    },

    showCategory: function(catExternal) {
        this.resetSearchResults();

        this._showPanel('category-info');

        Element.update($('category-info'), '<div class="ajax-loading">' + this._locale.loading + '</div>');

        var context = this;

        new Ajax.Request(Jsw.baseUrl + "/app/applications-list-data/catalogId/" + $('catalogId').value, {
            method: 'get',
            parameters: { category: catExternal, cert: 'any' },
            onSuccess: function(transport) {
                try {
                    var result = transport.responseText.evalJSON();

                    if (result.list) {
                        Element.update($('category-info'), result.list);
                    }
                    if (result.navigation) {
                        Element.update($('current-action-navigation'), result.navigation);
                    }

                    context._executeResizeCallback();
                } catch(e) {

                }
            }
        });

        return false;
    },

    showCategoryRetry: function(catExternal) {
        this.showCategory(catExternal);
    },

    install: function(item) {
        $('name').value = item.name;
        var params = ['version', 'release', 'packager', 'vendor', 'isNonStorefrontCommercial', 'advancedInstall'];
        for (var i = 0, length = params.length; i < length; i++) {
            if ($(params[i]).value === '') {
                $(params[i]).value = item[params[i]];
            }
        }
        this._componentElement.action = Jsw.baseUrl + "/app/download/";
        this._componentElement.submit();
        return false;
    },

    changeAppVersion: function(item) {
        var params = ['version', 'release', 'packager', 'vendor', 'isNonStorefrontCommercial'];
        for (var i = 0, length = params.length; i < length; i++) {
            $(params[i]).value = item[params[i]];
        }
        $('package-version-to-install').update(item.version);
        $('package-version-list').hide();
        return false;
    },

    uninstall: function(packageId) {
       $('packageId').value = packageId;
       this._componentElement.action = Jsw.baseUrl + "/app/remove/";
       this._componentElement.submit();
       return false;
    },

    _getSearchBox: function() {
        var searchStateClass = ' actions-box-search-show';

        if ('false' == Jsw.Cookie.get(this._id + '-search-show')) {
            searchStateClass = '';
        }

        var div = new Element('div', {
            'id': 'catalog-actions-box',
            'class': 'actions-box' + searchStateClass
        }).update(
                '<div class="objects-toolbar clearfix">' +
                    '<span id="current-action" class="list-actions">' +
	                    '<span id="current-action-navigation"></span>' +
	                    '<span id="current-action-buttons">' +
	                        '<span class="link-back-to-category">' +
	                            '<a href="#tolist">' + this._locale.returnToCategories + '</a>' +
	                        '</span>' +
	                    '</span>' +
	                '</span>' +
                	'<div class="quick-search-box" id="search-control">' +
                		'<div class="search-field">' +
                			'<form>' +
                				'<input id="quick-search-name" type="text" value="" name="searchFilter">' +
                				'<em><span></span></em>' +
                			'</form>' +
                		'</div>' +
                		'<span class="search-control">' +
                			'<a class="s-btn sb-search-show" href="#"><i><i><i><span>' + this._locale.showSearch + '</span></i></i></i></a>' +
                			'<a class="s-btn sb-search-hide" href="#"><i><i><i><span>' + this._locale.hideSearch + '</span></i></i></i></a>' +
                		'</span>' +
                	'</div>' +
                '</div>' +
                '<div class="search-box" id="search-box">' +
                    '<ul>' +
                        '<li><div><span>' + this._locale.searchName + '</span><input id="search-name" type="text" value=""/></div></li>' +
                        '<input id="search-vendor" type="hidden" value=""/>' +
                        '<li><div><span>' + this._locale.searchPackager + '</span><input id="search-packager" type="text" value=""/></div></li>' +
                        '<li><div><span>' + this._locale.searchLicenseType + '</span>' +
                            '<select id="search-license-type">' +
                                '<option value="any" selected>' + this._locale.searchLicenseTypeAny + '</option>' +
                                '<option value="freeware">' + this._locale.searchLicenseTypeFreeware + '</option>' +
                                '<option value="commercial">' + this._locale.searchLicenseTypeCommercial + '</option>' +
                            '</select></div></li>' +
                    '</ul>' +

                    '<a class="s-btn sb-search" href="#">' +
                        '<i><i><i><span>' + this._locale.buttonSearch + '</span></i></i></i>' +
                    '</a>' +
                    '<a class="s-btn sb-show-all" href="#">' +
                        '<i><i><i><span>' + this._locale.buttonResetSearch + '</span></i></i></i>' +
                    '</a>' +
                '</div>'

        );

        return div;
    },

    _addSearchHandlers: function() {
        var context = this;

        this._componentElement.select('.search-control .s-btn').each(function(element) {
            element.observe('click', function() {
                this.up('.actions-box').toggleClassName('actions-box-search-show');
                Jsw.Cookie.set(context._id + '-search-show', this.up('.actions-box').hasClassName('actions-box-search-show'));
                context._executeResizeCallback();
                return false;
            });
        });

        var quickSearchLink = this._componentElement.select('.quick-search-box').first().select('em').first();
        quickSearchLink.observe('click', function() {
            context.showSearchResults(true);
        });

        var searchLink = this._componentElement.select('.sb-search').first();
        searchLink.observe('click', function() {
            context.showSearchResults(false);
        });

        var resetSearchLink = this._componentElement.select('.sb-show-all').first();
        resetSearchLink.observe('click', function() {
            context.resetSearchResults();
        });

        var searchFields = $w('quick-search-name search-name search-vendor search-packager search-license-type');
        searchFields.each(function(searchFieldName) {
            $(searchFieldName).observe('keydown', function(event) {
                // search by Enter keydown
                if (13 == event.keyCode) {
                	var isQuick = false;
                	if ('quick-search-name' == searchFieldName) {
                		isQuick = true;
                	}
                    context.showSearchResults(isQuick);
                    Event.stop(event);
                    return false;
                }
            });
        });

        this._componentElement.select('.link-back-to-category a').each(function(element) {
            element.observe('click', this.returnToCategory.bind(this));
        }, this);
    },
    
    _addViewSwitcherHandlers: function() {
    	var context = this;
    	
    	var showCategoriesLink = this._componentElement.select('.view-switcher-categories').first();
	    if (showCategoriesLink) {
	    	showCategoriesLink.observe('click', function() {
	    		context.resetSearchResults();
	    		context.showCategories();
	        });
	    }
    	var showApplicationsLink = this._componentElement.select('.view-switcher-applications').first();
    	if (showApplicationsLink) {
	    	showApplicationsLink.observe('click', function() {
	            context.showSearchResults(false);
	        });
    	}
    },

    showSearchResults: function(isQuick) {
    	if (isQuick) {
            $('search-packager').value = '';
            $('search-license-type').value = 'any';
    		$('search-name').value = $('quick-search-name').value;
    	} else {
    		$('quick-search-name').value = $('search-name').value;
    	}
        this._showPanel('category-info');
        
        Element.update($('current-action-navigation'), '<a class="view-switcher-categories" href="#">' + this._locale.showCategories + '</a> | ' + this._locale.showApplications);
        this._addViewSwitcherHandlers();

        Element.update($('category-info'), '<div class="ajax-loading">' + this._locale.loading + '</div>');

        new Ajax.Updater('category-info', Jsw.baseUrl + "/app/search-data/catalogId/" + $('catalogId').value, {
            method: 'get',
            evalScripts: true,
            parameters: {
                name: $('search-name').value,
                vendor: $('search-vendor').value,
                packager: $('search-packager').value,
                cert: 'any',
                licenseType: $('search-license-type').value
            },

            onComplete: function() {
                Smb.Views.Apps.fixContentWidth();
                Smb.Views.Apps.fixMarketplaceContentHeight();
            }
        });

        this._isSearchResultsShown = true;

        return false;
    },

    resetSearchResults: function() {
    	$('quick-search-name').value = '';
        $('search-name').value = '';
        $('search-vendor').value = '';
        $('search-packager').value = '';
        $('search-license-type').value = 'any';

        if (!this._isSearchResultsShown) {
            return ;
        }

        this._isSearchResultsShown = false;

        this.showSearchResults(false);
    },

    // resizable height for div elements
    _initBlocksResize: function() {
        try {
            // this needs to be executed twice for proper work on Firefox
            this._resizeBlocks();
            this._resizeBlocks();
            var context = this;
            Event.observe(window, 'resize', function(){
              context._resizeBlocks();
              context._resizeBlocks();
            }, false);
        } catch (e) {
//            alert(e.description);
        }
    },

    _resizeBlocks: function() {
        try {
            var bodyHeight = document.body.clientHeight - 12;
            if (Prototype.Browser.IE) {
                bodyHeight = document.documentElement.clientHeight - 12;
            }
            var blockTop = Element.cumulativeOffset($('application-info'))[1];
            var blockBottom = Element.getHeight("footer-wrapper");
            var newBlockHeight = bodyHeight - blockTop - blockBottom;

            if (newBlockHeight < 200) { newBlockHeight = 200; } // minimum height

            $('application-info').style.height = newBlockHeight + 'px';
            $('category-select').style.height = newBlockHeight + 'px';

        } catch (e) {
//            alert(e.description);
        }
    }
});

Smb.ApsCatalogDownloadBar = Class.create(Jsw.Component, {
    STATUS_STARTED: 0,
    STATUS_NOT_STARTED: 1,
    STATUS_DONE: 2,
    STATUS_ERROR: 3,

    _initConfiguration: function($super, config) {
        $super(config);
        Jsw.baseUrl = this._getConfigParam('baseUrl', '');
        var context = this;
        window.setTimeout(function(){context.updateProgress();}, 1000);
    },

    _initComponentElement: function() {
        this._progressTextElement = new Element('div', {
            'class': 'progress-counter'
        });
        //TODO: define progress bar element
        this._progressBarElement = new Element('div', {
            'class': 'progress-bar'
        }).update('<div class="progress-bar-wrap"><div class="progress-bar-fill"><span></span></div></div>');
        this._componentElement = new Element('div', {
            'class': this._cls
        });
        this._componentElement.appendChild(this._progressTextElement);
        this._componentElement.appendChild(this._progressBarElement);
        this._update(0);
    },

    _update: function(percents) {
        this._progressTextElement.update(this._locale.progress.sub('%%percents%%', percents));
        this._progressBarElement.select('div.progress-bar-fill').each(function(element) {
            $(element).setStyle({
                width: percents + '%'
            });
        });
    },

    updateProgress: function() {
        var context = this;
        var requestUrl = Jsw.baseUrl + "/app/download-progress-data/catalogId/" +
            this._getConfigParam('catalogId') + "/taskId/" + this._getConfigParam('taskId');
        new Ajax.Request(requestUrl, {
            method: 'get',
            onSuccess: function(transport) {
                try {
                    var result = transport.responseText.evalJSON();
                    if (result.status == context.STATUS_STARTED ||
                        result.status == context.STATUS_NOT_STARTED ||
                        result.status == context.STATUS_DONE) {

                        context._update(result.progress);

                        if (result.status == context.STATUS_DONE) {

                            var redirectUrl = context._getConfigParam('redirectUrl').replace("%packageId%", result.packageId);

                            if (context._getConfigParam('redirectMethod') == 'post') {
                                Jsw.redirectPost(redirectUrl);
                            } else {
                                Jsw.redirect(redirectUrl);
                            }
                        } else {
                            window.setTimeout(function(){context.updateProgress();}, 1000);
                        }
                    } else {
                        alert('Error downloading package: ' + result.message);
                        Jsw.redirect("/app");
                    }
                } catch (e) {
//                    alert('updateProgress exception: ' + e);
                }
            }
       });
    }
});