// Copyright 1999-2014. Parallels IP Holdings GmbH. All Rights Reserved.
Jsw.namespace('Smb.BackupManager');

Smb.BackupManager.WebContentDynamic = Class.create(Jsw.DynamicSubForm, {
    _initComponentElement: function($super) {
        $super();

        this._dataUrl = this._getConfigParam('dataUrl', '');
        this._dataUrlParams = this._getConfigParam('dataUrlParams', '');

        new Jsw.SmallButton({
            title: this._locale.addButtonTitle,
            addCls: 'sb-item-add',
            handler: this._addContentDialog.bind(this),
            disabled: false,
            renderTo: this._id + '-buttonAddItem'
        });
    },

    _bindDynamicContent: function(content) {
        new Jsw.SmallButton({
            title: this._locale.removeButtonTitle,
            addCls: 'sb-item-remove',
            handler: this.deleteContent.bind(this, content),
            renderTo: content.select('.buttonRemoveItem').last().identify()
        });

        return content;
    },

    _addEmptyContent: function() {
        var element = this._prepareDynamicContent(this._getDynamicContent(), this._locale.noItems);
        element.select('input').last().remove();
        element.removeClassName('dynamicSubFormContent').addClassName('emptySubFormContent');
        this._insertDynamicContent(element);
    },

    _prepareDynamicContent: function(content, value) {
        content.select('.webContentPath').first().update(value);
        content.select('input').last().setValue(value);
        return content;
    },

    _onUpdateDynamic: function() {
        this._componentElement.select('.emptySubFormContent').last().toggle(this.isEmpty());

        this._componentElement.select('.dynamicSubFormContent').each(function(child, i) {
                child.select('label').last().toggle(0 == i);
        });

        this.onChange();
    },

    onChange: function() {},

    isEmpty: function() {
        return 0 == this._componentElement.select('.dynamicSubFormContent').size();
    },

    hasContent: function(value) {
        var hasContent = false;
        this._componentElement.select('.dynamicSubFormContent').each(function(child, i) {
            if (value == child.select('input').last().getValue()) {
                hasContent = true;
                throw $break;
            }
        });

        return hasContent;
    },

    setDataUrlParams: function(params) {
        this._dataUrlParams = params;
        if (this._webContentPopup) {
            this._webContentPopup.setDataUrlParams(params);
        }
    },

    _onSubmit: function(items) {
        items.each(function(item) {
            if (!this.hasContent(item)) {
                this.addContent(item);
            }
        }.bind(this));
    },

    _addContentDialog: function() {
        if (this._webContentPopup) {
            this._webContentPopup.show();
            this._webContentPopup.reload();
            return;
        }

        this._webContentPopup = new Smb.BackupManager.WebContentPopup({
            locale: this._locale,
            dataUrl: this._dataUrl,
            dataUrlParams: this._dataUrlParams,
            onSubmit: this._onSubmit.bind(this)
        });
    }
});

Smb.BackupManager.WebContentExplorer = Class.create(Jsw.List, {

    _initComponentElement: function($super) {
        $super();

        this._dataUrlParams = this._getConfigParam('dataUrlParams', '');
        this._pageable = false;
        this._currentDir = '/';
        this.isDisabledItem = function(item) {
            return ('..' == item.name);
        };
    },

    setDataUrlParams: function(params) {
        this._dataUrlParams = params;
    },

    changeDirectory: function(event, newDirectory, absolutePath) {
        Event.stop(event);
        this._currentDir = absolutePath ? newDirectory : (this._currentDir + '/' + newDirectory);
        this._currentDir = this._normalizePath(this._currentDir);
        this.reload();
    },

    _normalizePath: function(path) {
        path = path.replace(new RegExp('/+', 'g'), '/');
        var upReg = new RegExp('/[^/]+/\\.\\.(/|$)', 'g');
        while (upReg.test(path)) {
            path = path.replace(upReg, '/');
        }
        path = path.replace(new RegExp('/\\.\\.(/|$)', 'g'), '/');
        path = path.replace(new RegExp('/+', 'g'), '/');
        path = path.replace(new RegExp('/$', 'g'), '');

        return ('' === path) ? '/' : path;
    },

    reload: function($super, params) {
        params = (params || '') + this._dataUrlParams + '?currentDir=' + encodeURIComponent(this.getCurrentDir());

        $super(params);
    },

    getCurrentDir: function() {
        return this._currentDir;
    },

    processReloadError: function(data) {
        if (data.status) {
            Jsw.clearStatusMessages();
            $A(data.statusMessages).each(function(message) {
                Jsw.addStatusMessage(message.status, message.content, {renderTo: this._id});
            }.bind(this));
        } else {
            Jsw.showInternalError('Unable to load list data.');
        }
    },

    _isColumnSelectionPresent: function() {
        return true;
    },

    _getListHtml: function($super) {
        return '' +
            '<div id="' + this._id + '-objects-content-area" class="' + this._listCls + '">' +
                '<table width="100%" cellspacing="0" id="' + this._tableId + '">' +
                    '<thead>' +
                        this._getListHeadersHtml() +
                    '</thead>' +
                    '<tbody>' +
                        this._getListDataHtml() +
                    '</tbody>' +
                '</table>' +
            '</div>';
    },

    disable: function() {
        [this._id + '-pathbar', this._id + '-objects-content-area'].each(function(element) {
            if (element = $(element)) {
                element.hide();
            }
        });

        var element = $(this._id + '-loading-container');
        if (element) {
            element.show();
        } else {
            this._componentElement.insert(
                '<div class="ajax-loading" id="' + this._id + '-loading-container">' +
                    this._locale.loadingTitle +
                '</div>'
            );
        }
    },

    enable: function($super) {
        $(this._id + '-loading-container').hide();

        [this._id + '-pathbar', this._id + '-objects-content-area'].each(function(element) {
            if (element = $(element)) {
                element.show();
            }
        });

        $super();
    },

    _addResponsiveHtml: function($super) {}
});

Smb.BackupManager.WebContentPopup = Class.create(Jsw.PopupForm, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._cls = this._getConfigParam('cls', 'popup-panel');
        this._dataUrl = this._getConfigParam('dataUrl', '');
        this._dataUrlParams = this._getConfigParam('dataUrlParams', '');
        this._onSubmitHandler = this._getConfigParam('onSubmit', function(items) {});
    },

    setDataUrlParams: function(params) {
        this._dataUrlParams = params;
        if (this._webContentExplorer) {
            this._webContentExplorer.setDataUrlParams(params);
        }
    },

    reload: function(params) {
        if (this._webContentExplorer) {
            this._webContentExplorer.reload(params);
        }
    },

    render: function($super) {
        $super();

        this.setBoxType('fm-box');
        this.setTitle(this._locale.title);
        this.setHint1(this._locale.hint);

        var contentAreaId = this._contentAreaId;

        this._webContentExplorer = new Smb.BackupManager.WebContentExplorer({
            applyTo: contentAreaId,
            dataUrl: this._dataUrl,
            dataUrlParams: this._dataUrlParams,
            locale: this._locale,
            columns: [
                Object.extend(Object.clone(Jsw.list.COLUMN_SELECTION), { cls: 'select first', headerCls: 'select first', hideDisabled: true }),
                {
                    header: this._locale.columnName,
                    sortable: true,
                    dataIndex: 'name',
                    cls: 'js-file-name fm-file-name',
                    renderer: function(item) {
                        var itemTitle = '' +
                            '<img src="' + item.icon + '"> ' +
                            '<span>' + item.name.escapeHTML() + '</span>';

                        if (item.isDirectory) {
                            itemTitle = '' +
                                '<a class="fm-file-name-link" href="#" onclick="Jsw.getComponent(\'' + contentAreaId + '\').changeDirectory(event, \'' + item.name.replace(/"/g, '\\&quot;').replace(/'/g, '\\&apos;') + '\')">' +
                                    itemTitle +
                                '</a>';
                        }

                        return '' +
                            '<table class="fm-file-name-wrap">' +
                                '<tbody>' +
                                    '<tr class="" id="">' +
                                        '<td class="fm-file-name-container">' +
                                            itemTitle +
                                        '</td>' +
                                    '</tr>' +
                                '</tbody>' +
                            '</table>';
                    }
                }, {
                    header: this._locale.columnChangeDate,
                    sortable: true,
                    dataIndex: 'modificationDate',
                    cls: 'minor number'
                }, {
                    header: this._locale.columnSize,
                    sortable: true,
                    dataIndex: 'formatedSize',
                    cls: 't-r number js-file-size last',
                    headerCls: 'last'
                }
            ]
        });

        this.addRightButton(this._locale.buttonOk, this._onSubmit, true, true);
        this.addRightButton(this._locale.buttonCancel, this.hide, false, false);
    },

    _getButtonsArea: function() {
        return '' +
            '<div class="btns-box" id="' + this._actionButtonsId + '">' +
                '<div class="box-area">' +
                    '<div class="form-row">' +
                        '<div class="single-row">' +
                            '<div class="field-value" id="' + this._rightActionButtonsAreaId + '"> </div>' +
                        '</div>' +
                    '</div>' +
                '</div>' +
            '</div>';
    },

    _onSubmit: function() {
        this.hide();

        var webContentExplorer = Jsw.getComponent(this._contentAreaId);
        var currentDir = webContentExplorer.getCurrentDir().replace(/[\/]+$/g, '');
        var selectedItems = [];
        webContentExplorer.getSelectedItemsIds().each(function(item) {
            selectedItems.push(currentDir + '/' + item);
        });
        this._onSubmitHandler(selectedItems);
    },

    disable: function() {
        var context = this;
        $(this._rightActionButtonsAreaId).select('.btn.action').each(
            function(actionButtonWrapper) {
                var sendButton = actionButtonWrapper.down('button');
                context._okButtonTitle = sendButton.innerHTML;
                sendButton.update(
                    '<span class="wait">' + context._locale.waitButtonTitle + '</span>'
                );
            }
        );

        $(this._rightActionButtonsAreaId).select('.btn').each(
            function(actionButtonWrapper) {
                actionButtonWrapper.down('button').disabled = true;
                actionButtonWrapper.addClassName('disabled');
            }
        );
    },

    enable: function() {
        var context = this;
        $(this._rightActionButtonsAreaId).select('.btn.action').each(
            function(actionButtonWrapper) {
                var sendButton = actionButtonWrapper.down('button');
                sendButton.update(context._okButtonTitle);
            }
        );
        $(this._rightActionButtonsAreaId).select('.btn').each(
            function(actionButtonWrapper) {
                actionButtonWrapper.down('button').disabled = false;
                actionButtonWrapper.removeClassName('disabled');
            }
        );
    },

    _onOkHandlerException: function(transport, exception) {
        this._addErrorMessage('Internal error: ' + exception);
    },

    _onOkHandlerCreate: function(transport) {
        this.disable();
    },

    _onOkHandlerComplete: function(transport) {
        this.enable();
    }

});

Smb.BackupManager.Pathbar = Class.create(Jsw.Pathbar, {

    _renderItem: function($super, item) {
        $super(item);

        var context = this;

        if (item instanceof Jsw.Pathbar.Item) {
            item.addEventObserver('click', function(event) {
                var contentAreaId = context._renderTargetId.replace(/-pathbar$/, '');
                Jsw.getComponent(contentAreaId).changeDirectory(event, item.getHref(), true);
            });
        }
    },

    _renderItemSuffix: function($super, item) {
        if (item instanceof Jsw.Pathbar.Item) {
            $super(item);
        }
    }
});
