// Copyright 1999-2014. Parallels IP Holdings GmbH. All Rights Reserved.
Jsw.namespace('Smb.Views');

Smb.Views.Email = {
    isAttachmentsSupported: true,

    updateRedirectControlsState: function() {
        if ($('redirect-redirectSection-enabled').checked) {
            $('redirect-redirectSection-subscribers').enable();
        } else {
            $('redirect-redirectSection-subscribers').disable();
        }
    },

    updateAutoresponderControlsState: function() {
    	var elements = $A([
        	'autoResponder-autoResponderSection-contentType-textplain',
        	'autoResponder-autoResponderSection-contentType-texthtml',
        	'autoResponder-autoResponderSection-codepage',
        	'autoResponder-autoResponderSection-subject',
        	'autoResponder-autoResponderSection-message',
        	'autoResponder-autoResponderSection-replyTo',
            'autoResponder-autoResponderSection-forwardAddress',
            'autoResponder-autoResponderSection-replyLimit'
        ]);
    	if ($('autoResponder-autoResponderSection-enabled').checked) {
    		elements.each(function(item) {
    			if ($(item)) {
    				$(item).enable();
    			}
			});
    		if (Smb.Views.Email.isAttachmentsSupported) {
                $('attachments').show();
            }
        } else {
    		elements.each(function(item) {
    			if ($(item)) {
    				$(item).disable();
    			}
			});
            if (Smb.Views.Email.isAttachmentsSupported) {
                $('attachments').hide();
            }
        }
    },

    updateAntivirusControlsState: function() {
        if ($('antivirus-antivirusSection-enabled').checked) {
            if ($('antivirus-antivirusSection-antivirusSettings-incoming')) {
                $('antivirus-antivirusSection-antivirusSettings-incoming').enable();
            }
            if ($('antivirus-antivirusSection-antivirusSettings-outgoing')) {
                $('antivirus-antivirusSection-antivirusSettings-outgoing').enable();
            }
            if ($('antivirus-antivirusSection-antivirusSettings-both')) {
                $('antivirus-antivirusSection-antivirusSettings-both').enable();
            }
        } else {
            if ($('antivirus-antivirusSection-antivirusSettings-incoming')) {
                $('antivirus-antivirusSection-antivirusSettings-incoming').disable();
            }
            if ($('antivirus-antivirusSection-antivirusSettings-outgoing')) {
                $('antivirus-antivirusSection-antivirusSettings-outgoing').disable();
            }
            if ($('antivirus-antivirusSection-antivirusSettings-both')) {
                $('antivirus-antivirusSection-antivirusSettings-both').disable();
            }
        }
    },

    updateAntispamControlsState: function() {
    	antispamEnabled = $('antispam-mainSection-enabled').checked;


    	Smb.Views.Email.enableControl($('antispam-mainSection-antispamSettings-mark'), antispamEnabled);
    	markEnabled = $('antispam-mainSection-antispamSettings-mark').checked;
    	Smb.Views.Email.enableControl($('antispam-mainSection-subjText'), antispamEnabled && markEnabled);
    	Smb.Views.Email.enableControl($('antispam-mainSection-reportText'), antispamEnabled && markEnabled);
    	Smb.Views.Email.enableControl($('antispam-mainSection-antispamSettings-delete'), antispamEnabled);
    	Smb.Views.Email.enableControl($('antispam-mainSection-antispamSettings-move'), antispamEnabled);
    	Smb.Views.Email.enableControl($('antispam-advancedSection-filterScore-score'), antispamEnabled);
    	Smb.Views.Email.enableControl($('antispam-advancedSection-whiteList-list'), antispamEnabled);
    	Smb.Views.Email.enableControl($('antispam-advancedSection-blackList-list'), antispamEnabled);
    	Smb.Views.Email.enableControl($('antispam-advancedSection-trustedNetworks-networks'), antispamEnabled);
    },

    enableControl: function (ctrl, value) {
    	if (!ctrl) {
    		return;
    	}
    	if (value) {
    		ctrl.enable();
    	} else {
    		ctrl.disable();
    	}
    }
};

Smb.Views.EmailFormAjax = Class.create(Jsw.FormAjax, {
	disable: function($super) {
		this.attachments = [];
        if (Smb.Views.Email.isAttachmentsSupported) {
            $('attachments').select('div.dynamicSubFormContent input').each(function(input) {
                if ('' == input.value) {
                    return;
                }
                this.attachments.push(input);
                input.disable();
            }, this);
        }
		$super();
	},

	enable: function($super) {
		this.attachments.each(function(input) {
			input.enable();
		});
		$super();
	},

	_processForm: function($super, response) {
		if (response.emailId) {
			this._processAttachments($super, response);
		} else {
			$super(response);
		}
	},

	_processAttachments: function(continueCallback, response) {
		var form = new Element('form', {
			action: Jsw.baseUrl + '/email-address/responder-attachments/id/' + response.emailId,
			method: 'post',
			enctype: 'multipart/form-data',
			encoding: 'multipart/form-data',
			style: 'display:none'
		});
        if ($('forgery_protection_token')) {
        	form.insert(new Element('input', {
                type: 'hidden',
                name: 'forgery_protection_token',
                value: $('forgery_protection_token').content
            }));
        }
		this._componentElement.insert({after: form});
		this.attachments.each(function(input) {
			input.enable();
			input.name = 'file[]';
			var inputCopy = input.clone();
			input.hide();
			input.insert({after: inputCopy});
			form.insert(input);
		});
		form.submit();
	}
});

Jsw.onReady(function() {
    Smb.Views.Email.isAttachmentsSupported = !!$('attachments');

    Smb.Views.Email.updateRedirectControlsState();
    $('redirect-redirectSection-enabled').observe('click', Smb.Views.Email.updateRedirectControlsState);

    Smb.Views.Email.updateAutoresponderControlsState();
    $('autoResponder-autoResponderSection-enabled').observe('click', Smb.Views.Email.updateAutoresponderControlsState);

    if ($('antivirus-antivirusSection-enabled')) {
        Smb.Views.Email.updateAntivirusControlsState();
        $('antivirus-antivirusSection-enabled').observe('click', Smb.Views.Email.updateAntivirusControlsState);
    }

    if ($('antispam-mainSection-enabled')) {
    	Smb.Views.Email.updateAntispamControlsState();
    	$('antispam-mainSection-enabled').observe('click', Smb.Views.Email.updateAntispamControlsState);
    	$('antispam-mainSection-antispamSettings-mark').observe('click', Smb.Views.Email.updateAntispamControlsState);
    	$('antispam-mainSection-antispamSettings-delete').observe('click', Smb.Views.Email.updateAntispamControlsState);
    	if ($('antispam-mainSection-antispamSettings-move')) {
    		$('antispam-mainSection-antispamSettings-move').observe('click', Smb.Views.Email.updateAntispamControlsState);
    	}
    }
});
