// Copyright 1999-2014. Parallels IP Holdings GmbH. All Rights Reserved.
Jsw.namespace('Smb.FileManager');

Smb.FileManager.resizeBlocks = function() {
    var minBlockHeight = 200;
    var bodyHeight = $(document.body).getHeight();
    var blockTop = Position.cumulativeOffset($('objects-list'))[1];

    var blockBottom = $$('.fm-footer')[0].getHeight() + 2;
    if (!$(document.body).hasClassName('classic-mode')) {
        blockBottom += $('footer-wrapper').getHeight();
    }

    var blockHeight = bodyHeight - blockTop - blockBottom;
    if (blockHeight < minBlockHeight) {
        blockHeight = minBlockHeight;
    }

    $('objects-details').setStyle({ 'height': blockHeight + 'px' });
    $('objects-list-content-area').setStyle({ 'height': blockHeight + 'px' });

    var topBlocksHeight = $('fm-content-operations').getHeight() + $('fm-content-pathbar').getHeight();
    $('objects-details').down('.objects-content').setStyle({ 'height': (blockHeight - topBlocksHeight) + 'px' });
};

Smb.FileManager.initSidebarVisibility = function() {
    $('fm-frame-switcher').observe('click', function(){
        if ($('fm').hasClassName('objects-management-box-collapsed')) {
            $('fm').removeClassName('objects-management-box-collapsed');
            Jsw.Cookie.remove('fm-hidden-tree');
        } else if ($('fm').hasClassName('fm-hidden-tree')) {
            $('fm').removeClassName('fm-hidden-tree');
            Jsw.Cookie.remove('fm-hidden-tree');
        } else {
            $('fm').addClassName('objects-management-box-collapsed');
            Jsw.Cookie.set('fm-hidden-tree', true);
        }
    });
};

Smb.FileManager.Explorer = Class.create(Jsw.List, {

    _initComponentElement: function($super) {
        $super();

        var dropArea = '<div id="drop-area" class="fm-drop-area">' +
            '<div class="fm-drop-area-wrap">' +
                '<span class="fm-drop-area-text"></span>' +
            '</div>' +
        '</div>';

        this._componentElement.insert({bottom: dropArea});
        $('drop-area').hide();
    },

    changeDirectory: function(newDirectory, absolutePath) {
        newDirectory = absolutePath ? newDirectory : (this._state.currentDir + '/' + newDirectory);
        this.reload('?currentDir=' + encodeURIComponent(newDirectory));
    },

    getCurrentDir: function() {
        return this._state.currentDir;
    },

    getReadonlyHint: function() {
        return this._state.currentDirReadonly
            ? '<span class="error-hint">' + this._pathbar.locale.readonlyHint + '</span>'
            : '';
    },

    showPopup: function(popupClass, options) {
        var popupId = options['id'];
        var component = Jsw.getComponent(popupId);

        if (component) {
            component.reset(options);
        } else {
            new popupClass(options);
        }
    },

    calculateSize: function() {
        var context = this;
        var params = {
            url: '/file-manager/calculate-size',
            skipConfirmation: true
        };
        params.submitHandler = function(url, ids) {
            new Ajax.Request(Jsw.prepareUrl(url), {
                method: 'get',
                parameters: ids,
                onSuccess: function (transport) {
                    var status = transport.responseText.evalJSON();
                    var table = $('fm-content-table');
                    var selectedRows = table.select('tr.selected');
                    selectedRows.each(function(selectedRow) {
                        var fileNameElement = selectedRow.select('td.js-file-name .fm-file-name-container a span').first();
                        if (!fileNameElement) {
                            return;
                        }
                        var fileName = fileNameElement.firstChild.nodeValue;
                        if (fileName in status.fileSizes) {
                            var fileSize = status.fileSizes[fileName];
                            selectedRow.select('td.js-file-size').first().update(fileSize);
                        }
                    });

                    Jsw.clearStatusMessages();
                    $A(status.statusMessages).each(function(message) {
                        Jsw.addStatusMessage(message.status, message.content);
                    });
                    context.enable();
                },

                onCreate: function() {
                    context.disable();
                },

                onFailure: function() {
                    context.enable();
                }
            });
        };
        this.execGroupOperation(params);
    },

    processReloadError: function(data) {
        if (data.status) {
            Jsw.clearStatusMessages();
            Jsw.addStatusMessage(data.status, data.message);
        } else {
            Jsw.showInternalError('Unable to load list data.');
        }
    },

    _initConfiguration: function($super, config) {
        $super(config);

        var hideTree = Jsw.Cookie.get('fm-hidden-tree');
        if (hideTree) {
            this.hideTree();
        } else {
            this.showTree();
        }

        this._treeConfig = this._getConfigParam('tree', '');
        this._tree = new Smb.FileManager.Tree(this._treeConfig);
        this._itemActions = this._getConfigParam('itemActions', {});
    },

    redraw: function($super) {
        $super();

        this._tree.setRenderTarget('fm-tree');
        this._tree.setDirectory(this.getCurrentDir());
        this._tree.render();

        Smb.FileManager.resizeBlocks();

        this._addItemActionHandlers();

        var dragAndDropZone = Jsw.getComponent('drop-area');
        if (dragAndDropZone && dragAndDropZone.reset) {
            dragAndDropZone.reset();
        }
    },

    _isColumnSelectionPresent: function() {
        return true;
    },

    _getListHtml: function($super) {
        if (0 == this._pager.totalItemCount) {
            var html = '<span class="empty-list">' + this._locale.noEntriesFound + '</span>';
        } else {
            var html =
                this._getPagerHtml() +
                '<div class="' + this._listCls + '">' +
                    '<table width="100%" cellspacing="0" id="' + this._tableId + '">' +
                        '<thead>' +
                            this._getListHeadersHtml() +
                        '</thead>' +
                        '<tbody>' +
                            this._getListDataHtml() +
                        '</tbody>' +
                    '</table>' +
                '</div>' +
                this._getPagerHtml();
        }


        html = '' +
            '<div class="objects-content">' +
                '<div id="' + this._id + '-objects-content-area" class="objects-content-area">' +
                    '<div class="list-box">' +
                        '<div class="box-area">' +
                            '<div class="content">' +
                                '<div class="content-area">' +
                                    html +
                                '</div>' +
                            '</div>' +
                        '</div>' +
                    '</div>' +
                '</div>' +
            '</div>';

        return html;
    },

    _addItemActionHandlers: function() {

        var context = this;

        var rows = this._componentElement.select('#fm-content-table > tbody > tr');
        rows.each(function(row) {
            if (row.select('td.js-file-name .js-operations-button').length == 0) {
                return;
            }

            row.observe('mouseover', function(e) {
                rows.each(function(innerRow) {
                    innerRow.removeClassName('row-over');
                });
                this.addClassName('row-over');
            });

            row.observe('mouseleave', function(e) {
                var popup = $$('.fm-popup-file-operations').first();
                if (popup && popup.visible()) {
                    return;
                }
                this.removeClassName('row-over');
            });
        });
        $(document).observe('click', function() {
            var popup = $$('.fm-popup-file-operations').first();
            if (popup) {
                popup.remove();
            }
        });
        this._componentElement.select('#fm-content-table > tbody > tr td.js-file-name').each(
            function(cell) {
                cell.observe('click', function(e) {

                    if (e.findElement('a.fm-file-name-link')) {
                        return;
                    }
                    Event.stop(e);
                    var popup = $$('.fm-popup-file-operations').first();
                    if (popup) {
                        popup.remove();
                        return;
                    }
                    var item = context.getItemById(this.up('tr').select('.select input')[0].value);
                    popup = context.getActionsForItem(item);
                    $$('body')[0].insert({ top: popup });
                    popup = $$('.fm-popup-file-operations').first();

                    popup.removeClassName('collapsed');

                    var popupTopPosition = Event.pointerY(e);
                    var popupLeftPosition = Event.pointerX(e);
                    var windowHeight = $(document.body).getHeight();
                    var windowWidth = $(document.body).getWidth();
                    var popupHeight = $(popup).getHeight();
                    var popupWidth = $(popup).getWidth();

                    if ((popupTopPosition + popupHeight) > windowHeight) {
                        popupTopPosition = Event.pointerY(e) - popupHeight;
                    }
                    if ((popupLeftPosition + popupWidth) > windowWidth) {
                        popupLeftPosition = Event.pointerX(e) - popupWidth;
                    }

                    popup.setStyle({
                        left: popupLeftPosition + 'px',
                        top: popupTopPosition + 'px'
                    });

                    popup.observe('mouseleave', function(e) {
                        this.remove();
                    });

                    popup.select('li a').each(function(action) {
                        action.observe('click', function() {
                            popup.remove();

                            var actionName = this.up('li').select('input[name=item-action-name]')[0].value;
                            var actionInfo = context.getActionByName(item, actionName);
                            if (actionName in context._itemActions) {
                                context._itemActions[actionName](item, actionInfo);
                            } else if (actionInfo.href) {
                                document.location = Jsw.prepareUrl(actionInfo.href);
                            }
                        });
                    });
                });
            });
    },

    callDefaultItemAction: function(itemId) {
        var item = this.getItemById(itemId);
        var defaultAction;
        item.actions.each(function(action) {
            if (action['default']) {
                defaultAction = action;
            }
        });
        if (defaultAction.name in this._itemActions) {
            this._itemActions[defaultAction.name](item, defaultAction);
        }
    },

    getActionByName: function(item, actionName) {
        var foundAction;

        item.actions.each(function(actionInfo) {
            if (actionInfo.name == actionName) {
                foundAction = actionInfo;
            }
        });

        return foundAction;
    },

    getActionsForItem: function(item) {
        if (!item.actions) {
            return '';
        }
        return ' ' +
            '<div class="popup-box fm-popup-file-operations collapsed">' +
                '<table class="popup-wrapper" cellspacing="0"><tbody><tr><td class="popup-container">' +
                    '<div class="c1"><div class="c2"><div class="c3"><div class="c4"><div class="c5">' +
                        '<div class="popup-content">' +
                            '<div class="popup-content-area">' +
                                '<ul>' + this._getActionsForItem(item) + '</ul>' +
                            '</div>' +
                        '</div>' +
                    '</div></div></div></div></div>' +
                '</td></tr></tbody></table>' +
            '</div>';
    },

    _getActionsForItem: function(item) {
        var actions = [];
        item.actions.each(function(actionInfo) {
            var action = '';
            if (actionInfo.separator) {
                action = '<li class="separator"/>';
            } else {
                action =
                    '<li><a class="sb-btn sb-no-icon" href="#">' +
                        '<i><i><i><span>' + actionInfo.title + '</span></i></i></i>' +
                        '<input type="hidden" name="item-action-name" value="' + actionInfo.name + '" />' +
                    '</a></li>';
            }
            actions.push(action);
        });
        return actions.join('');
    },

    showTree: function() {
        $('fm').removeClassName('fm-hidden-tree');
    },

    hideTree: function() {
        $('fm').addClassName('fm-hidden-tree');
    },

    disable: function() {
        [this._id + '-pathbar', this._id + '-objects-content-area'].each(function(element) {
            $(element).hide();
        });

        var element = $(this._id + '-loading-container');

        if (element) {
            element.show();
        } else {
            this._componentElement.insert(
                '<div class="ajax-loading" id="' + this._id + '-loading-container">' +
                    this._locale.loadingTitle +
                '</div>'
            );
        }

        this._disablerOverlay.show();
        this._disablerOverlay.clonePosition(this._componentElement.down('.actions-box'));
    },

    enable: function($super) {
        $(this._id + '-loading-container').hide();

        [this._id + '-pathbar', this._id + '-objects-content-area'].each(function(element) {
            $(element).show();
        });

        $super();
    },

    _addResponsiveHtml: function($super) {}

});

Smb.FileManager.Pathbar = Class.create(Jsw.Pathbar, {

    _renderItem: function($super, item) {
        $super(item);

        if (item instanceof Jsw.Pathbar.Item) {

            var context = this;

            item.addEventObserver('click', function(event) {
                Event.stop(event);
                Jsw.getComponent('fm-content').changeDirectory(item.getHref(), true);
            });
        }
    },

    _renderItemSuffix: function($super, item) {
        if (!(item instanceof Jsw.Pathbar.Item)) {
            return;
        }
        $super(item);
    }

});

Smb.FileManager.Tree = Class.create(Jsw.Component, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._data = this._getConfigParam('data', []);
        this._dataUrl = this._getConfigParam('dataUrl', '');
        this._onNodeClick = this._getConfigParam('onNodeClick');
        this._rootNodeTitle = this._getConfigParam('rootNodeTitle', '');
        this._onRootNodeClick = this._getConfigParam('onRootNodeClick');
        if (this._getConfigParam('showMessage')) {
            this._showMessage = this._getConfigParam('showMessage');
        }
    },

    _initComponentElement: function($super) {
        $super();
        this._initTreeView();
    },

    _initTreeView: function() {
        var container = this._componentElement.update(new Element('div', { 'class': 'tree-wrap' })).down();

        container.update(this._getRootNode());
        container.insert(this._getNodes(this._data));
    },

    _getNodes: function(items) {
        var context = this;
        var ulElement = new Element('ul', { 'class': 'tree-container' });

        items.each(function(item) {
            var liElement = new Element('li', { 'class': 'tree-item' });
            var wrapper = liElement.update('<div class="tree-item-wrap"></div>').down();
            liElement.dataPath = item.path;

            var itemSelect = new Element('div', { 'class': 'tree-item-select' });
            wrapper.insert(itemSelect);
            itemSelect.observe('mouseover', context._onNodeMouseover.bind(context, liElement));
            itemSelect.observe('mouseleave', context._onNodeMouseleave.bind(context, liElement));
            itemSelect.observe('click', context._onNodeSelect.bind(context, liElement));
            itemSelect.observe('click', context._onNodeClick.bind(context, item.path, liElement));
            wrapper.insert('<span class="tree-item-state"><img src="' + Jsw.skinUrl + '/images/tree-open.gif' + '" height="16" width="16"></span>');
            wrapper.down('.tree-item-state').observe('click', context._onNodeToggle.bind(context, item.path, liElement));
            wrapper.down('.tree-item-state').observe('mouseover', context._onNodeMouseover.bind(context, liElement));
            wrapper.down('.tree-item-state').observe('mouseleave', context._onNodeMouseleave.bind(context, liElement));
            var linkElement = new Element('a', { 'class': 'tree-item-content', 'href': '#' });
            linkElement.update('<span><img src="' + Jsw.skinUrl + '/icons/16/plesk/file-folder.png" alt=""><b>' + item.name.escapeHTML() + '</b></span>');
            linkElement.observe('mouseover', context._onNodeMouseover.bind(context, liElement));
            linkElement.observe('mouseleave', context._onNodeMouseleave.bind(context, liElement));
            linkElement.observe('click', context._onNodeSelect.bind(context, liElement));
            linkElement.observe('click', context._onNodeClick.bind(context, item.path, liElement));
            wrapper.insert(linkElement);

            ulElement.insert(liElement);
        });

        return ulElement;
    },

    _getRootNode: function() {
        var element = new Element('div', { 'class': 'tree-item tree-item-root' });
        var wrapper = element.update('<div class="tree-item-wrap"></div>').down();
        element.dataPath = '/';

        var itemSelect = new Element('div', { 'class': 'tree-item-select' });
        wrapper.insert(itemSelect);
        itemSelect.observe('mouseover', this._onNodeMouseover.bind(this, element));
        itemSelect.observe('mouseleave', this._onNodeMouseleave.bind(this, element));
        itemSelect.observe('click', this._onNodeSelect.bind(this, element));
        itemSelect.observe('click', this._onRootNodeClick.bind(this));

        var linkElement = new Element('a', { 'class': 'tree-item-content', 'href': '#' });
        linkElement.update('<span><img src="' + Jsw.skinUrl + '/icons/16/plesk/file-folder.png" alt=""><b>' + this._rootNodeTitle.escapeHTML() + '</b></span>');
        linkElement.observe('mouseover', this._onNodeMouseover.bind(this, element));
        linkElement.observe('mouseleave', this._onNodeMouseleave.bind(this, element));
        linkElement.observe('click', this._onNodeSelect.bind(this, element));
        linkElement.observe('click', this._onRootNodeClick.bind(this));

        wrapper.insert(linkElement);

        return element;
    },

    _onNodeToggle: function(directoryPath, containerElement) {
        var subTreeElement = containerElement.down('ul');
        var expandElement = containerElement.down('.tree-item-state');

        if (subTreeElement) {
            expandElement.update('<img src="' + Jsw.skinUrl + '/images/tree-open.gif' + '" height="16" width="16">');
            subTreeElement.remove();
            return;
        }

        this._loadNode(directoryPath, containerElement);
    },

    _loadNode: function(directoryPath, containerElement) {
        if (containerElement.down('.tree-item-state').down('.js-loader')) {
            return;
        }
        var request = new Ajax.Request(
            Jsw.baseUrl + this._dataUrl,
            {
                method:'post',
                parameters: { 'rootDir': directoryPath },
                onCreate: this._onNodeLoadStart.bind(this, containerElement),
                onSuccess: this._onNodeLoadSuccess.bind(this, containerElement),
                onFailure: this._onNodeLoadFailure.bind(this)
            }
        );
    },

    _isNodeExpanded: function(nodeElement) {
        if (nodeElement.down('ul')) {
            return true;
        } else {
            return false;
        }

    },

    _onNodeLoadStart: function(containerElement, request) {
        var expandElement = containerElement.down('.tree-item-state');
        expandElement.update('<img src="' + Jsw.skinUrl + '/images/indicator.gif' + '" height="16" width="16" class="js-loader">');
    },

    _onNodeLoadSuccess: function(containerElement, request) {
        var expandElement = containerElement.down('.tree-item-state');
        var items = request.responseText.evalJSON();
        if (items.status) {
            expandElement.update('<img src="' + Jsw.skinUrl + '/images/tree-open.gif' + '" height="16" width="16">');
            this._showMessage(items.status, items.message);
            return;
        }

        Jsw.clearStatusMessages();
        expandElement.update('<img src="' + Jsw.skinUrl + '/images/tree-close.gif' + '" height="16" width="16">');
        containerElement.insert(this._getNodes(items));
    },

    _showMessage: function(status, message) {
        Jsw.clearStatusMessages();
        Jsw.addStatusMessage(status, message);
    },

    _onNodeSelect: function(containerElement) {
        this.resetSelection();
        containerElement.down('.tree-item-wrap').addClassName('tree-item-row-active');
    },
    resetSelection: function() {
        this._componentElement.select('.tree-item-wrap').each(function(element) {
            element.removeClassName('tree-item-row-active');
        });
    },
    _onNodeMouseover: function(containerElement) {
        this.resetHover();
        containerElement.down('.tree-item-select').addClassName('tree-item-hover');
    },
    _onNodeMouseleave: function(containerElement) {
        this.resetHover();
        containerElement.down('.tree-item-select').removeClassName('tree-item-hover');
    },
    resetHover: function() {
        this._componentElement.select('.tree-item-select').each(function(element) {
            element.removeClassName('tree-item-hover');
        });
    },

    setDirectory: function(directory) {
        var node = this._getNodeElement(directory);
        if (node) {
            this._onNodeSelect(node);
        }
    },

    reload: function() {
        this._componentElement.update('<div class="ajax-loading">Please wait...</div>');

        var request = new Ajax.Request(
            Jsw.baseUrl + this._dataUrl,
            {
                method: 'post',
                parameters: { 'rootDir': '/' },
                onSuccess: this._onFullReloadSuccess.bind(this),
                onFailure: this._onNodeLoadFailure.bind(this)
            }
        );
    },

    _onFullReloadSuccess: function(request) {
        this._data = request.responseText.evalJSON();
        this._initTreeView();
    },

    _onNodeLoadFailure: function() {
        Jsw.showInternalError('Failed to load tree data.');
    },

    reloadPath: function(path) {
        if (path == '/') {
            this.reload();
            return;
        }

        var node = this._getNodeElement(path);
        if (!node) {
            return;
        }

        if (this._isNodeExpanded(node)) {
            node.down('ul').remove();
            this._loadNode(path, node);
        }
    },

    _getNodeElement: function(path) {
        var nodes = this._componentElement.select('.tree-item');
        for (var i = 0; i < nodes.length; i++) {
            if (nodes[i].dataPath == path) {
                return nodes[i];
            }
        }
    }

});

Smb.FileManager.PopupForm = Class.create(Jsw.PopupForm, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._handlerUrl = this._getConfigParam('handlerUrl', '');
    },

    render: function($super) {
        $super();

        this.setBoxType('form-box');
        this.setTitle(this._locale.title);
    },

    reset: function() {
        this.show();
    },

    _onSubmitEvent: function(event) {
        this._onSubmit();
        Event.stop(event);
        return false;
    },

    _initOnSubmitEvent: function() {
        var form = $(this._id + '-form');
        form._formSubmit = form.submit;
        form.submit = this._onSubmit.bind(this);
        form.observe('submit', this._onSubmitEvent.bind(this));
    },

    disable: function() {
        var context = this;
        $(this._rightActionButtonsAreaId).select('.btn.action').each(
            function(actionButtonWrapper) {
                var sendButton = actionButtonWrapper.down('button');
                context._okButtonTitle = sendButton.innerHTML;
                sendButton.update(
                    '<span class="wait">' + context._locale.waitButtonTitle + '</span>'
                );
            }
        );

        $(this._rightActionButtonsAreaId).select('.btn').each(
            function(actionButtonWrapper) {
                actionButtonWrapper.down('button').disabled = true;
                actionButtonWrapper.addClassName('disabled');
            }
        );
    },

    enable: function() {
        var context = this;
        $(this._rightActionButtonsAreaId).select('.btn.action').each(
            function(actionButtonWrapper) {
                var sendButton = actionButtonWrapper.down('button');
                sendButton.update(context._okButtonTitle);
            }
        );
        $(this._rightActionButtonsAreaId).select('.btn').each(
            function(actionButtonWrapper) {
                actionButtonWrapper.down('button').disabled = false;
                actionButtonWrapper.removeClassName('disabled');
            }
        );
    },

    _initSelectionHint: function() {
        var filesNumber = this._selectedItems.length;

        if (1 == filesNumber) {
            this.setHint1(this._locale.hintFile.replace('%%file%%', '<b>' + this._selectedItems[0].name.escapeHTML() + '</b>'));
        } else {
            this.setHint1(this._locale.hintFiles.replace('%%number%%', filesNumber));
        }
    },

    _onOkHandlerException: function(transport, exception) {
        this._addErrorMessage('Internal error: ' + exception);
    },

    _onOkHandlerCreate: function(transport) {
        this.disable();
    },

    _onOkHandlerComplete: function(transport) {
        this.enable();
    }

});

Smb.FileManager.ListPopup = Class.create(Jsw.PopupForm, {

    _initConfiguration: function($super, config) {
        $super(config);
        this._itemsAreaId = this._id + '-items';
    },

    _addEvents: function($super) {
        $super();
        Event.observe(
            window,
            'resize',
            this.resizeList.bind(this),
            false
        );
    },

    resizeList: function() {
        if (this._isRemoved) {
            return;
        }

        var minHeigh = 150;
        var list = $(this._contentAreaId).down('table');
        var listContainer = $(this._contentAreaId).down('.list');

        var bodyHeight = (Prototype.Browser.IE)
            ? $(document.documentElement).getHeight()
            : $(document.body).getHeight();

        var buttonsHeight = $(this._actionButtonsId).getHeight();
        var top = Position.cumulativeOffset(list)[1];
        var height = list.getHeight();
        var newHeight = bodyHeight - top - buttonsHeight - 100; // 100px below dialog

        if (newHeight > height) {
            listContainer.style.height = '';
            listContainer.removeClassName('with-scroll');
            return;
        }

        if (newHeight < minHeigh) {
            newHeight = minHeigh;
        }

        listContainer.style.height = newHeight + 'px';
        listContainer.style.overflowY = 'auto';
        listContainer.style.overflowX = 'hidden';
        listContainer.addClassName('with-scroll');
    },

    _renderContentArea: function() {
        $(this._contentAreaId).insert (
            '<div class="list">' +
                '<table width="100%" cellspacing="0">' +
                    '<tbody id="' + this._itemsAreaId + '">' +
                    '</tbody>' +
                '</table>' +
            '</div>'
        );
    },

    render: function($super) {
        $super();
        this._renderContentArea();
        $(this._actionButtonsId).addClassName('no-separator');
    }
});

Smb.FileManager.CreateFile = Class.create(Smb.FileManager.PopupForm, {

    render: function($super) {
        $super();

        $(this._contentAreaId).insert(
            '<form method="post" action="" enctype="application/x-www-form-urlencoded" id="' + this._id + '-form">' +
                '<div>' +
                    '<div class="form-row">&#160;</div>' +
                    '<div class="form-row">' +
                        '<div class="field-name">' +
                            '<label for="' + this._id + '-file-name">' +
                                this._locale.fileName +
                            '</label>' +
                        '</div>' +
                        '<div class="field-value">' +
                            '<input name="fileName" id="' + this._id + '-file-name" ' +
                                'value="" class="f-middle-size input-text" type="text">' +
                            '<span class="field-errors" style="display: none;"></span>' +
                        '</div>' +
                    '</div>' +
                    '<div class="form-row">' +
                        '<div class="single-row">' +
                            '<div class="indent-box">' +
                                '<input type="checkbox" class="checkbox" value="1" ' +
                                    'id="' + this._id + '-html-template"/>' +
                                '<div class="indent-box-content">' +
                                    '<label for="' + this._id + '-html-template">' +
                                        this._locale.htmlTemplate +
                                    '</label>' +
                                    '<span class="hint">' +
                                        this._locale.htmlTemplateHint +
                                    '</span>' +
                                '</div>' +
                            '</div>' +
                        '</div>' +
                    '</div>' +
                '</div>' +
            '</form>'
        );

        this._initOnSubmitEvent();

        this.addRightButton(this._locale.buttonOk, this._onSubmit, true, true);
        this.addRightButton(this._locale.buttonCancel, this.hide, false, false);

        this.reset();
    },

    reset: function($super) {
        this.setHint(
            this._locale.hint.replace(
                '%%folder%%', '<b>' + Jsw.getComponent('fm-content').getCurrentDir().escapeHTML() + '</b>'
            ) + Jsw.getComponent('fm-content').getReadonlyHint()
        );

        this._clearMessages();

        $(this._id + '-file-name').value = '';
        $(this._id + '-html-template').checked = false;

        $super();

        try {
            $(this._id + '-file-name').focus();
        } catch (e) {}
    },

    _onSubmit: function() {
        var fileName = $(this._id + '-file-name').value;
        var htmlTemplate = $(this._id + '-html-template').checked;

        new Ajax.Request(
            this._handlerUrl,
            {
                method: 'post',
                parameters: {
                    'newFileName': fileName,
                    'htmlTemplate': htmlTemplate
                },
                onCreate: this._onOkHandlerCreate.bind(this),
                onSuccess: (function(transport) {
                    this._onOkHandlerSuccess(transport, fileName);
                }).bind(this),
                onException: this._onOkHandlerException.bind(this),
                onComplete: this._onOkHandlerComplete.bind(this)
            }
        );
    },

    _onOkHandlerSuccess: function(transport, fileName) {
        this._clearMessages();
        try {
            var response = transport.responseText.evalJSON();
            if ('success' == response.status) {
                Jsw.getComponent('fm-content').reload();
                var message = response.message.replace(
                    '%%file%%',
                    fileName.escapeHTML()
                );

                Jsw.addStatusMessage('info', message);

                this.hide();
            } else {
                this._addErrorMessage(response.message);
            }
        } catch (e) {
            this._addErrorMessage(e.message);
            this._addErrorMessage(
                'Internal error: ' + transport.responseText
            );
        }
    }

});

Smb.FileManager.CreateFolder = Class.create(Smb.FileManager.PopupForm, {

    render: function($super) {
        $super();

        $(this._contentAreaId).insert(
            '<form method="post" action="" enctype="application/x-www-form-urlencoded" id="' + this._id + '-form">' +
                '<div>' +
                    '<div class="form-row">&#160;</div>' +
                    '<div class="form-row">' +
                        '<div class="field-name">' +
                            '<label for="' + this._id + '-folder-name">' +
                                this._locale.folderName +
                            '</label>' +
                        '</div>' +
                        '<div class="field-value">' +
                            '<input name="folderName" id="' + this._id + '-folder-name" ' +
                                'value="" class="f-middle-size input-text" type="text">' +
                            '<span class="field-errors" style="display: none;"></span>' +
                        '</div>' +
                    '</div>' +
                '</div>' +
            '</form>'
        );

        this._initOnSubmitEvent();

        this.addRightButton(this._locale.buttonOk, this._onSubmit, true, true);
        this.addRightButton(this._locale.buttonCancel, this.hide, false, false);

        this.reset();
    },

    reset: function($super) {
        this.setHint(
            this._locale.hint.replace(
                '%%folder%%', '<b>' + Jsw.getComponent('fm-content').getCurrentDir().escapeHTML() + '</b>'
            ) + Jsw.getComponent('fm-content').getReadonlyHint()
        );

        this._clearMessages();

        $(this._id + '-folder-name').value = '';

        $super();

        try {
            $(this._id + '-folder-name').focus();
        } catch (e) {}
    },

    _onSubmit: function() {
        var folderName = $(this._id + '-folder-name').value;
        new Ajax.Request(
            this._handlerUrl,
            {
                method: 'post',
                parameters: {
                    'newDirectoryName': folderName
                },
                onCreate: this._onOkHandlerCreate.bind(this),
                onSuccess: (function(transport) {
                    this._onOkHandlerSuccess(transport, folderName);
                }).bind(this),
                onException: this._onOkHandlerException.bind(this),
                onComplete: this._onOkHandlerComplete.bind(this)
            }
        );
    },

    _onOkHandlerSuccess: function(transport, folderName) {
        this._clearMessages();
        try {
            var response = transport.responseText.evalJSON();
            if ('success' == response.status) {
                Jsw.getComponent('fm-content').reload();
                Jsw.getComponent('fm-tree').reloadPath(Jsw.getComponent('fm-content').getCurrentDir());
                var message = response.message.replace(
                    '%%folder%%',
                    '<a href="#" id="' + this._id + '-success-message-link">' +
                        folderName.escapeHTML() +
                    '</a>'
                );

                Jsw.addStatusMessage('info', message);

                $(this._id + '-success-message-link').observe('click', function() {
                    Jsw.getComponent('fm-content').changeDirectory(folderName);
                });

                this.hide();
            } else {
                this._addErrorMessage(response.message);
            }
        } catch (e) {
            this._addErrorMessage(e.message);
            this._addErrorMessage(
                'Internal error: ' + transport.responseText
            );
        }
    }

});

Smb.FileManager.Rename = Class.create(Smb.FileManager.PopupForm, {

    render: function($super) {
        $super();

        $(this._contentAreaId).insert(
            '<form method="post" action="" enctype="application/x-www-form-urlencoded" id="' + this._id + '-form">' +
                '<div>' +
                    '<div class="form-row">&#160;</div>' +
                    '<div class="form-row">' +
                        '<div class="field-name">' +
                            '<label for="' + this._id + '-file-name">' +
                                this._locale.fileName +
                            '</label>' +
                        '</div>' +
                        '<div class="field-value">' +
                            '<input name="newFileName" id="' + this._id + '-file-name" ' +
                                'value="" class="f-middle-size input-text" type="text">' +
                            '<span class="field-errors" style="display: none;"></span>' +
                        '</div>' +
                    '</div>' +
                '</div>' +
            '</form>'
        );

        this._initOnSubmitEvent();

        this.addRightButton(this._locale.buttonOk, this._onSubmit, true, true);
        this.addRightButton(this._locale.buttonCancel, this.hide, false, false);

        this.reset(this._config);
    },

    reset: function($super, options) {
        this._itemId = options.itemId;
        this._fileName = options.fileName;

        this.setHint(
            this._locale.hint.replace(
                '%%file%%', '<b>' + this._fileName.escapeHTML() + '</b>'
            )
        );

        this._clearMessages();

        $(this._id + '-file-name').value = this._fileName;

        $super();

        try {
            $(this._id + '-file-name').focus();
        } catch (e) {}
    },

    _onSubmit: function() {
        var fileName = $(this._id + '-file-name').value;

        new Ajax.Request(
            this._handlerUrl,
            {
                method: 'post',
                parameters: {
                    'ids[0]': this._itemId,
                    'newFileName': fileName
                },
                onCreate: this._onOkHandlerCreate.bind(this),
                onSuccess: (function(transport) {
                    this._onOkHandlerSuccess(transport);
                }).bind(this),
                onException: this._onOkHandlerException.bind(this),
                onComplete: this._onOkHandlerComplete.bind(this)
            }
        );
    },

    _onOkHandlerSuccess: function(transport) {
        this._clearMessages();
        try {
            var response = transport.responseText.evalJSON();
            var message = $A(response.statusMessages).first();
            if ('info' == message.status) {
                Jsw.getComponent('fm-content').reload();
                Jsw.getComponent('fm-tree').reloadPath(Jsw.getComponent('fm-content').getCurrentDir());
                Jsw.addStatusMessage('info', message.content);

                this.hide();
            } else {
                this._addErrorMessage(message.content);
            }
        } catch (e) {
            this._addErrorMessage(e.message);
            this._addErrorMessage(
                'Internal error: ' + transport.responseText
            );
        }
    }

});

Smb.FileManager.CopyMoveFiles = Class.create(Smb.FileManager.PopupForm, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._treeConfig = this._getConfigParam('tree');
    },

    render: function($super) {
        $super();

        $(this._contentAreaId).insert(
            '<form method="post" action="" enctype="application/x-www-form-urlencoded" id="' + this._id + '-form">' +
                '<div class="form-box">' +
                    '<div class="box-area">' +
                        '<div class="content">' +
                            '<div class="content-area">' +
                                '<div class="form-row">' +
                                    '<div class="field-name">' +
                                        this._locale.destinationFolderHint +
                                    '</div>' +
                                    '<div class="field-value">' +
                                        '<div class="scrollable fm-scrollable">' +
                                            '<div class="scrollable-wrap">' +
                                                '<div id="fm-modal-dialog-box-files-tree" class="tree"></div>' +
                                            '</div>' +
                                        '</div>' +
                                    '</div>' +
                                '</div>' +
                                '<div class="form-row">' +
                                    '<div class="single-row">' +
                                        '<div class="indent-box">' +
                                            '<input type="checkbox" class="checkbox" value="1" ' +
                                                'id="' + this._id + '-overwrite"/>' +
                                            '<div class="indent-box-content">' +
                                                '<label for="' + this._id + '-overwrite">' +
                                                    this._locale.overwriteExisting +
                                                '</label>' +
                                            '</div>' +
                                        '</div>' +
                                    '</div>' +
                                '</div>' +
                            '</div>' +
                        '</div>' +
                    '</div>' +
                '</div>' +
            '</form>'
        );

        var treeConfig = Object.extend(this._treeConfig, {
            onNodeClick: this._onTreeNodeClick.bind(this),
            onRootNodeClick: this._onTreeRootNodeClick.bind(this),
            showMessage: this._showMessage.bind(this)
        });

        this._tree = new Smb.FileManager.Tree(treeConfig);

        this._initOnSubmitEvent();

        this.addRightButton(this._locale.buttonOk, this._onSubmit, true, true);
        this.addRightButton(this._locale.buttonCancel, this.hide, false, false);

        this.reset();
    },

    _onTreeNodeClick: function(directoryPath) {
        this.setDestinationDir(directoryPath);
    },

    _onTreeRootNodeClick: function() {
        this.setDestinationDir('/');
    },

    reset: function($super) {
        this._selectedItems = Jsw.getComponent('fm-content').getSelectedItems();
        this._destinationDir = '';
        this._tree.reload();
        $(this._id + '-overwrite').checked = false;

        this._initSelectionHint();

        this._clearMessages();

        $super();
    },

    _onSubmit: function() {
        if (this._destinationDir == '') {
            this._showMessage('error', this._locale.destinationRequired);
            return;
        }
        this.hide();
        var overwriteExisting = $(this._id + '-overwrite').checked;

        Jsw.getComponent('fm-content').execGroupOperation({
            url: Jsw.addUrlParams(this._handlerUrl, 'destinationDir=' + this._destinationDir + '&overwrite=' + overwriteExisting),
            skipConfirmation: true,
            onSuccess: this._onSuccess.bind(this)
        });
    },

    _showMessage: function(status, message) {
        // Currently, various statuses don't supported, value of status is ignored and all messages have error status.
        this._clearMessages();
        this._addErrorMessage(message);
    },

    setDestinationDir: function(destinationDir) {
        this._destinationDir = destinationDir;
    },

    _onSuccess: function() {
        Jsw.getComponent('fm-content').enable();
        Jsw.getComponent('fm-tree').reloadPath(Jsw.getComponent('fm-content').getCurrentDir());
        Jsw.getComponent('fm-tree').reloadPath(this._destinationDir);
    }

});

Smb.FileManager.CopyFiles = Class.create(Smb.FileManager.CopyMoveFiles, {

});

Smb.FileManager.MoveFiles = Class.create(Smb.FileManager.CopyMoveFiles, {

});

Smb.FileManager.UploadCapabilities = {
    _isMultipleFileSupported: null,
    _isUploadProgressSupported: null,

    isAdvancedUploadSupported: function() {
        return Smb.FileManager.UploadCapabilities.isMultipleFileSupported() &&
            Smb.FileManager.UploadCapabilities.isFormDataSupported() &&
            Smb.FileManager.UploadCapabilities.isFileApiSupported() &&
            !Smb.FileManager.UploadCapabilities.isReadonlyDirectory();
    },

    isMultipleFileSupported: function() {
        if (null === Smb.FileManager.UploadCapabilities._isMultipleFileSupported) {
            var element = new Element('input', {'type' : 'file'});
            Smb.FileManager.UploadCapabilities._isMultipleFileSupported =  'multiple' in element;
        }
        return Smb.FileManager.UploadCapabilities._isMultipleFileSupported;
    },

    isReadonlyDirectory: function() {
        return Jsw.getComponent('fm-content').getReadonlyHint();
    },

    isFormDataSupported: function() {
        return typeof FormData != 'undefined';
    },

    isFileApiSupported: function() {
        return typeof File != "undefined";
    },

    isUploadProgressSupported: function() {
        if (null === Smb.FileManager.UploadCapabilities._isUploadProgressSupported) {
            var transport = Ajax.getTransport();
            Smb.FileManager.UploadCapabilities._isUploadProgressSupported = transport && typeof transport.upload != 'undefined';
        }
        return Smb.FileManager.UploadCapabilities._isUploadProgressSupported;
    }
};

Smb.FileManager.UploadButton = Class.create(Jsw.SmallButton, {
    _initConfiguration: function($super, config) {
        $super(config);
        this._uploadPopupParams = this._getConfigParam('uploadPopupParams', {});
        this._fileInputId = this._getConfigParam('fileUploadInputId', 'file-upload-input');

        if (!Smb.FileManager.UploadCapabilities.isAdvancedUploadSupported()) {
            this._handler = this.uploadHandler.bind(this);
        }
    },

    _initComponentElement: function($super) {
        $super();

        if (!Smb.FileManager.UploadCapabilities.isAdvancedUploadSupported()) {
            return;
        }

        this._componentElement.setStyle({
            position: 'relative',
            overflow: 'hidden'
        });

        this.resetFileInputElement();
    },

    resetFileInputElement: function() {
        var fileInputElement = this._componentElement.select('#' + this._fileInputId).first();
        if (fileInputElement) {
            fileInputElement.remove();
        }

        fileInputElement = new Element('input', {
            type: 'file',
            name: 'file[]',
            id: this._fileInputId,
            multiple: true
        });

        fileInputElement.setStyle({
            position: 'absolute',
            right: 0,
            top: 0,
            cursor: 'pointer',
            opacity: 0
        });
        this._componentElement.insert({bottom: fileInputElement});

        fileInputElement.observe('change', this.multipleUploadHandler.bind(this));
    },

    uploadHandler: function() {
        Jsw.getComponent('fm-content').showPopup(Smb.FileManager.Upload, this._uploadPopupParams);
    },

    multipleUploadHandler: function() {
        this._uploadPopupParams.fileInputId = this._fileInputId;
        this._uploadPopupParams.resetFileInputElement = this.resetFileInputElement.bind(this);
        Jsw.getComponent('fm-content').showPopup(Smb.FileManager.MultipleUpload, this._uploadPopupParams);
    }
});

Smb.FileManager.Upload = Class.create(Smb.FileManager.PopupForm, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._selectFilesDialogId = this._id + '-select-files-dialog';
        this._selectFilesDialogItemsAreaId = this._selectFilesDialogId + '-items';
    },

    render: function($super) {
        $super();

        this.setHint(this._locale.hint + Jsw.getComponent('fm-content').getReadonlyHint());

        $(this._contentAreaId).insert('<div id="' + this._selectFilesDialogId + '">' +
                '<div id="' + this._selectFilesDialogItemsAreaId + '"></div>' +
                '<div class="form-row">' +
                    '<div class="single-row">' +
                        '<a href="#" class="s-btn sb-item-add"><i><i><i><span>' + this._locale.addFileItem + '</span></i></i></i></a>' +
                    '</div>' +
                '</div>' +
            '</div>'
        );

        this.addSelectFilesItem();

        $(this._selectFilesDialogId).select('.sb-item-add').each(function(element) {
            element.observe('click', this._onAddSelectFilesItemClick.bindAsEventListener(this));
        }, this);

        this.addRightButton(this._locale.buttonOk, this._onSubmit, false, false);
        this.addRightButton(this._locale.buttonCancel, this._onCancel, false, false);
    },

    reset: function($super) {
        this._clearMessages();
        $(this._selectFilesDialogItemsAreaId).update('');
        this._uploadCanceled = false;

        $super();
    },

    _onSubmit: function() {
        if (!this._checkUploadQueue()) {
            return;
        }
        if (this._hasExistingFilesUploadQueue()) {
            this.showReplaceExisitingFilesDialog();
            return;
        }
        this.showUploadProgressDialog();
        this.startNextQueueUpload();
    },

    _onCancel: function() {
        this.hide();
    },

    _onAddSelectFilesItemClick: function(event) {
        this.addSelectFilesItem();
        Event.stop(event);
    },

    addSelectFilesItem: function() {
        var itemsArea = $(this._selectFilesDialogItemsAreaId);
        var itemsCount = itemsArea.select('form').length;

        var itemId = this._selectFilesDialogId + '-item-' + itemsCount;
        var item = '<form id="' + itemId + '" action="' + this._handlerUrl + '" method="post" enctype="multipart/form-data" target="_uploadIFrame">' +
                '<input name="forgery_protection_token" type="hidden" value="' + $('forgery_protection_token').content + '"/>' +
                '<div class="form-row">' +
                    '<input name="file[]" type="file"/>' +
                    (itemsCount ? ' <a href="#" class="s-btn sb-item-remove"><i><i><i><span> ' + this._locale.removeFileItem + '</span></i></i></i></a>' : '') +
                '</div>' +
            '</form>';

        itemsArea.insert({ bottom : item });

        if (itemsCount) {
            $(itemId).select('.sb-item-remove').each(function(element) {
                element.observe('click', this._onRemoveSelectFilesItemClick.bindAsEventListener(this));
            }, this);
        }
    },

    _onRemoveSelectFilesItemClick: function(event) {
        var item = Event.element(event).up('form');
        if (item) {
            item.remove();
        }
        Event.stop(event);
    },

    _checkUploadQueue: function() {
        this._clearMessages();
        this._prepareUploadQueue();
        return this._hasFilesInUploadQueue();
    },

    _hasFilesInUploadQueue: function() {
        if (0 == this._uploadQueue.length) {
            this._addErrorMessage(this._locale.noFilesSelectedForUpload);
            return false;
        }
        return true;
    },

    _hasExistingFilesUploadQueue: function() {
        this._uploadExistingFilesQueue = $A();
        var currentDirectoryFiles = this._getCurrentDirectoryFiles();
        this._uploadQueue.each(function(queueItem) {
            if ( 0 <= currentDirectoryFiles.indexOf(queueItem.savedFileName)) {
                this._uploadExistingFilesQueue.push(queueItem);
            }
        }, this);
        return (0 < this._uploadExistingFilesQueue.length) ? true : false;
    },

    _getCurrentDirectoryFiles: function() {
        var filenames = [];
        $('fm-content-table').select('tr').each(function(selectedRow) {
            var fileNameElement = selectedRow.select('td.js-file-name .fm-file-name-container a span').first();
            if (!fileNameElement) {
                return;
            }
            var fileName = fileNameElement.firstChild.nodeValue;
            filenames.push(fileName);
        });
        return filenames;
    },

    _prepareUploadQueue: function() {
        this._uploadQueue = $A();
        var items = $(this._selectFilesDialogItemsAreaId).select('form');
        items.each(function(formElement) {
            var fileName = formElement.down('input[type="file"]').value;
            if (!fileName) {
                return;
            }

            // IE saves full path to file, but we needed in file name only
            fileName = fileName.replace(/\\/g,'/').replace( /.*\//, '');

            var queueItem = {
                inputId: formElement.id,
                progressId: formElement.id + '-progress',
                formElement: formElement,
                savedFileName: fileName,
                status: 'PENDING',
                type: 'unknown'
            };
            this._uploadQueue.push(queueItem);
        }, this);
    },

    showUploadProgressDialog: function() {
        this.hide();
        this._uploadProgressDialog = new Smb.FileManager.UploadProgressDialog({
            id: this._id + '-progress',
            renderTo: this._id + '-progress-dialog-box',
            cls: 'popup-panel',
            items: this._uploadQueue,
            locale: this._locale,
            onCancelClick: this._onCancelUploadClick.bind(this),
            uploadProgressSupported: Smb.FileManager.UploadCapabilities.isAdvancedUploadSupported() &&
                Smb.FileManager.UploadCapabilities.isUploadProgressSupported()
        });
    },

    _onCancelUploadClick: function(event) {
        this._uploadCanceled = true;
        this._uploadAdapter.cancel();
    },

    showReplaceExisitingFilesDialog: function() {
        this.hide();
        this._uploadProgressDialog = new Smb.FileManager.ReplaceExistingFilesDialog({
            id: this._id + '-progress',
            renderTo: this._id + '-progress-dialog-box',
            cls: 'popup-panel',
            items: this._uploadExistingFilesQueue,
            locale: this._locale,
            onSkipClick: this._onSkipExistingFilesClick.bind(this),
            onReplaceClick: this._onReplaceExistingFilesClick.bind(this)
        });
    },

    _onSkipExistingFilesClick: function(event) {
        this._uploadExistingFilesQueue.each(function(queueItem) {
            for(var i = 0; i < this._uploadQueue.length; i++) {
                if (queueItem.savedFileName == this._uploadQueue[i].savedFileName) {
                    this._uploadQueue.splice(i, 1);
                    break;
                }

            }
        }, this);
        if (!this._hasFilesInUploadQueue()) {
            this._uploadProgressDialog.remove();
            this.hide();
            return;
        }
        this.showUploadProgressDialog();
        this.startNextQueueUpload();
    },

    _onReplaceExistingFilesClick: function(event) {
        this.showUploadProgressDialog();
        this.startNextQueueUpload();
    },

    _prepareUploadAdapter: function() {
        if (!this._uploadAdapter) {
            var config = {
                contentAreaId: this._contentAreaId,
                handlerUrl: this._handlerUrl,
                startUploadHandler: function() {
                    if (this._scheduleStartUpload) {
                        this._scheduleStartUpload = null;
                        this.startNextQueueUpload();
                    }
                }.bind(this),
                finishUploadHandler: this._onNextQueueUploadFinished.bind(this)
            };
            this._uploadAdapter = new Smb.FileManager.UploadAdapterIframe(config);
        }
    },

    startNextQueueUpload: function() {
        this._prepareUploadAdapter();

        if (!this._uploadAdapter.isReady()) {
            this._scheduleStartUpload = true;
            return;
        }

        if (this._uploadCurrent) {
            return;
        }

        if (this._uploadCanceled) {
            this._onAllQueueUploadFinished();
            return;
        }

        this._started = false;

        this._processUploadQueue();

        if (!this._started) {
            this._onAllQueueUploadFinished();
            return;
        }

        var status = this.getQueueStatus();

        this._uploadProgressDialog.updateListDescription(
            this._locale.descriptionUploadProgress
                .replace('%%total%%', status.total)
                .replace('%%pending%%', status.pending)
                .replace('%%uploading%%', status.uploading)
                .replace('%%completed%%', status.completed)
                .replace('%%failed%%', status.failed)
        );
    },

    _processUploadQueue: function() {
        this._uploadQueue.each(this._processUploadItem, this);
    },

    _processUploadItem: function(queueItem) {
        if ('PENDING' != queueItem.status) {
            return;
        }
        this._started = true;
        this.updateUploadProgressItemStatus(queueItem, 'UPLOADING');
        this._uploadCurrent = queueItem;
        queueItem.formElement.select('input[name="fileName"]').each(function(element) {
            element.value = queueItem.savedFileName;
        }, this);
        queueItem.formElement.submit();
        throw $break;
    },

    _onNextQueueUploadFinished: function() {
        if (!this._uploadCurrent) {
            return;
        }

        var queueItem = this._uploadCurrent;
        this._uploadCurrent = null;

        var status = this._getNextQueueUploadResult();
        switch(status.status) {
            case 'SUCCESS':
            case 'CANCELED':
                this.updateUploadProgressItemStatus(queueItem, status.status);
                this.startNextQueueUpload();
                break;
            case 'ERROR':
            default:
                this.updateUploadProgressItemStatus(queueItem, 'ERROR');
                this._onQueueUploadFailed(status);
                break;
        }
    },

    _getNextQueueUploadResult: function() {
        if (this._uploadCanceled) {
            return {
                status: 'CANCELED',
                message: ''
            };
        }
        return this._uploadAdapter.getResult();
    },

    _onAllQueueUploadFinished: function() {
        Jsw.getComponent('fm-content').reload();
        this.reportUploadFinished.bind(this).delay(0.5);
    },

    _onQueueUploadFailed: function(status) {
        var errorItems = $A();
        this._uploadQueue.each(function(item) {
            errorItems.push({
                'name': item.savedFileName,
                'type': item.type,
                'status': item.status
            });
        });

        this.close();

        new Smb.FileManager.ErrorDialog({
            errorMessage: status.message,
            message: this._locale.failed,
            locale: this._locale,
            items: errorItems
        });
    },

    reportUploadFinished: function() {
        var messageType = null;
        var message = null;

        Jsw.clearStatusMessages();

        this.close();

        if (0 == this._uploadQueue.length) {
            return;
        }

        var status = this.getQueueStatus();

        if (this._uploadCanceled) {
            message = this._locale.uploadCanceled;
            messageType = 'warning';
        } else if (status.completed > 0) {
            message = this._locale.uploadFinished;
            messageType = 'info';
        } else {
            message = this._locale.uploadFinishedNoFiles;
            messageType = 'warning';
        }

        Jsw.addStatusMessage(messageType, message
            .replace('%%total%%', status.total)
            .replace('%%pending%%', status.pending)
            .replace('%%uploading%%', status.uploading)
            .replace('%%completed%%', status.completed)
            .replace('%%failed%%', status.failed)
            .replace('%%folder%%', Jsw.getComponent('fm-content').getCurrentDir().escapeHTML())
        );
    },

    updateUploadProgressItemStatus: function(queueItem, status) {
        queueItem.status = status;
        this._uploadProgressDialog.updateItemStatus(queueItem);
    },

    getQueueStatus: function() {
        var status = {
            completed: 0,
            pending: 0,
            failed: 0,
            uploading: 0,
            canceled: 0,
            total: this._uploadQueue.length
        };

        this._uploadQueue.each(function(queueItem) {
            switch (queueItem.status) {
                case 'PENDING':
                    status.pending++;
                    break;
                case 'UPLOADING':
                    status.uploading++;
                    break;
                case 'ERROR':
                    status.failed++;
                    break;
                case 'SUCCESS':
                    status.completed++;
                    break;
                case 'CANCELED':
                    status.canceled++;
                    break;
            }
        }, this);

        return status;
    },

    close: function() {
        if (this._uploadProgressDialog) {
            this._uploadProgressDialog.remove();
            this._uploadProgressDialog = null;
        }
        this.hide();
    }
});

Smb.FileManager.MultipleUpload = Class.create(Smb.FileManager.Upload, {
    _initConfiguration: function($super, config) {
        $super(config);
        this._files = this._getConfigParam('files', null);

        this.resetFileInputElement = this._getConfigParam('resetFileInputElement', function(){});
        this._fileInputId = this._getConfigParam('fileInputId');
    },

    render: function() {
        this._onSubmit();
    },

    reset: function($super, config) {
        if (config) {
            this._initConfiguration(config)
        }
        this._uploadCanceled = false;

        // Do not call $super() here
        this.show();

        this._onSubmit();
    },

    _checkUploadQueue: function($super) {
        var result = $super();
        if (!result) {
            this.hide();
        }
        return result;
    },

    _clearMessages: function() {
        Jsw.clearStatusMessages();
    },

    _addErrorMessage: function(message) {
        Jsw.addStatusMessage('error', message);
    },

    _prepareUploadQueue: function() {
        this._uploadQueue = $A();
        this._totalUploadedSize = 0;
        this._totalSize = 0;

        if (this._files) {
            var files = this._files;
        } else {
            var files = $(this._fileInputId).files;
        }
        for (var i = 0, file; file = files[i]; i++) {
            var fileSize = this._getFileSize(file);
            var queueItem = {
                progressId: 'upload-progress-' + i,
                savedFileName: file.name,
                status: 'PENDING',
                type: 'unknown',
                file: file,
                loaded: null,
                total: fileSize
            };
            this._totalSize += fileSize;
            this._uploadQueue.push(queueItem);
        }
    },

    _getFileSize: function(file) {
        return typeof file.fileSize != 'undefined' ? file.fileSize : file.size;
    },

    _processUploadItem: function(queueItem) {
        if ('PENDING' != queueItem.status) {
            return;
        }
        this._started = true;
        this.updateUploadProgressItemStatus(queueItem, 'UPLOADING');
        this._uploadCurrent = queueItem;
        this._uploadAdapter.upload(queueItem.file);
        throw $break;
    },

    _prepareUploadAdapter: function() {
        if (!this._uploadAdapter) {
            var config = {
                handlerUrl: this._handlerUrl,
                startUploadHandler: function() {
                    if (this._scheduleStartUpload) {
                        this._scheduleStartUpload = null;
                        this.startNextQueueUpload();
                    }
                }.bind(this),
                finishUploadHandler: this._onNextQueueUploadFinished.bind(this),
                progressUploadHandler: this._onProgress.bindAsEventListener(this)
            };
            this._uploadAdapter = new Smb.FileManager.UploadAdapterAjax(config);
        }
    },

    close: function($super) {
        $super();
        if (!this._files) {
            this.resetFileInputElement();
        }
        this._files = null;
    },

    _onProgress: function(e) {
        if (e.lengthComputable && this._uploadCurrent){
            //e.loaded in Chrome and Safari contains POST request upload progress, not file upload progress, so try to normalize
            this._uploadCurrent.loaded = e.total > 0 ? Math.round(e.loaded / e.total * this._uploadCurrent.total) : 0;
            this._uploadProgressDialog.updateTotalProgress(this._getTotalUploadPercent(this._totalUploadedSize + this._uploadCurrent.loaded));
            this._uploadProgressDialog.updateItemUploadProgress(this._uploadCurrent);
        }
    },

    _onNextQueueUploadFinished: function($super) {
        if (this._uploadCurrent) {
            this._uploadCurrent.loaded = this._uploadCurrent.total;
            this._totalUploadedSize += this._uploadCurrent.total;
            this._uploadProgressDialog.updateTotalProgress(this._getTotalUploadPercent(this._totalUploadedSize));
            this._uploadProgressDialog.updateItemUploadProgress(this._uploadCurrent);
        }
        $super();
    },

    _getTotalUploadPercent: function(totalUploaded) {
        return this._totalSize > 0 ? Math.round(totalUploaded / this._totalSize * 100) : 100;
    }
});

Smb.FileManager.UploadAdapterIframe = Class.create({
    initialize: function(config) {
        this._contentAreaId = config.contentAreaId;
        this._handlerUrl = config.handlerUrl;
        this._startUploadHandler = config.startUploadHandler;
        this._finishUploadHandler = config.finishUploadHandler;
        this._uploadFrameInitialized = false;
        this._getUploadIFrameElement();
    },

    _getUploadIFrameElement: function() {
        if (!$(this._contentAreaId).down('iframe')) {
            this._uploadFrameElement = new Element('iframe', {
                id: this._contentAreaId + '-upload-iframe',
                name: '_uploadIFrame',
                style: "display: none;",
                src: this._handlerUrl
            });
            $(this._contentAreaId).insert(this._uploadFrameElement);
            this._uploadFrameElement.observe('load', this._onUploadFrameInitialized.bindAsEventListener(this));
        }
        return this._uploadFrameElement;
    },

    _onUploadFrameInitialized: function()
    {
        if (this._uploadFrameInitialized) {
            return;
        }
        this._uploadFrameInitialized = true;
        this._uploadFrameElement.observe('load', this._finishUploadHandler);
        this._startUploadHandler();
    },

    isReady: function() {
        return this._uploadFrameInitialized;
    },

    getResult: function() {
        var statusFrame = this._getUploadIFrameElement();
        var statusDocument = statusFrame.contentDocument;
        if (statusDocument == undefined || statusDocument == null)
            statusDocument = statusFrame.contentWindow.document;
        var status = null;
        try {
            status = statusDocument.body.innerHTML.evalJSON();
        } catch(e) {
            status = {
                status: 'ERROR',
                message: 'No upload response'
            }
        }
        return status;
    },

    cancel: function() {
        if (this._uploadFrameElement){
            this._uploadFrameElement.setAttribute('src', 'javascript:false;');
        }
        this._finishUploadHandler();
    }
});

Smb.FileManager.AjaxUploadRequest = Class.create(Ajax.Request, {
    initialize: function($super, url, options) {
        if (options.onProgress) {
            this._onProgress = options.onProgress;
        }
        $super(url, options);
    },

    request: function($super, url) {
        if (this._onProgress) {
            this.transport.upload.onprogress = this._onProgress;
        }
        $super(url);
    },

    setRequestHeaders: function() {},

    abort: function() {
        this.transport.abort();
    }
});

Smb.FileManager.UploadAdapterAjax = Class.create({
    initialize: function(config) {
        this._handlerUrl = config.handlerUrl;
        this._startUploadHandler = config.startUploadHandler;
        this._finishUploadHandler = config.finishUploadHandler;
        this._progressUploadHandler = config.progressUploadHandler;
    },

    upload: function(file) {
        var formData = new FormData();
        formData.append(file.name, file);
        formData.append('forgery_protection_token', $('forgery_protection_token').content);

        this._request = new Smb.FileManager.AjaxUploadRequest(
            this._handlerUrl,
            {
                method: 'post',
                postBody: formData,
                onCreate: this._onCreate.bind(this),
                onSuccess: this._onSuccess.bind(this),
                onException: this._onException.bind(this),
                onProgress: this._progressUploadHandler
            }
        );
    },

    _onCreate: function() {
        this._startUploadHandler();
    },

    _onSuccess: function(transport) {
        try {
            this._result = transport.responseText.evalJSON();
        } catch (e) {
            this._result = {
                status: 'ERROR',
                message: 'No upload response'
            }
        }
        this._finishUploadHandler();
    },

    _onException: function(transport, exception) {
        this._result = {
            status: 'ERROR',
            message: 'Internal error: ' + exception
        }
    },

    isReady: function() {
        return true;
    },

    getResult: function() {
        return this._result;
    },

    cancel: function() {
        this._request.abort();
        this._finishUploadHandler();
    }
});

Smb.FileManager.UploadProgressDialog = Class.create(Smb.FileManager.ListPopup, {
    _initConfiguration: function($super, config) {
        $super(config);

        this._cls = this._getConfigParam('cls', 'popup-panel');
        this._titleCls = this._getConfigParam('titleCls', 'pp-upload');

        this._items = this._getConfigParam('items', $A());
        this._uploadProgressDialogDescriptionId = this._id + '-description';
        this._uploadProgressDialogPercentageId =  this._id + '-percentage';
        this._onCancelClickExternal = this._getConfigParam('onCancelClick', null);
        this._uploadProgressSupported = this._getConfigParam('uploadProgressSupported', false);
        this._units = [this._locale.bytes, this._locale.kbytes, this._locale.mbytes, this._locale.gbytes];
    },

    render: function($super) {
        $super();

        this.setHint1(this._locale.uploadDialogHint);
        this.setTitle(this._locale.uploadDialogTitle);

        $(this._contentAreaId).insert({
            top: '<div class="b-progress fm-upload-progress">' +
                '<ul class="b-progress-counter">' +
                    '<li class="b-progress-counter-item b-progress-counter-item-1" id="' + this._uploadProgressDialogDescriptionId + '"></li>' +
                    '<li class="b-progress-counter-item b-progress-counter-item-2" id="' + this._uploadProgressDialogPercentageId + '"></li>' +
                '</ul>' +
                '<div class="progress-bar">' +
                    '<div class="progress-bar-wrap">' +
                        '<div style="width: 0;" class="progress-bar-fill"><span></span></div>' +
                    '</div>' +
                '</div>' +
            '</div>'
        });

        this._items.each(function(item) {
            this._addItem(item);
        }, this);

        this._cancelButton = this.addRightButton(
            this._locale.buttonCancel,
            this._onCancelClick,
            true,
            false
        );
        this.resizeList();
    },

    _onCancelClick: function() {
        this._updateButton(this._cancelButton, {
            title: this._locale.buttonCancelUploadClicked,
            disabled: true
        });
        if (this._onCancelClickExternal) {
            this._onCancelClickExternal();
        }
    },

    _addItem: function(queueItem) {
        this._itemClass = ('odd' == this._itemClass) ? 'even' : 'odd';

        var itemHtml = '<tr class="' + this._itemClass + '" id="' + queueItem.progressId + '">' +
            '<td class="first">' +
                '<span class="type-' + queueItem.type.escapeHTML() + '">' +
                    queueItem.savedFileName.escapeHTML() +
                '</span>' +
            '</td>' + (this._uploadProgressSupported ?
                '<td class="upload-progress nowrap t-r">' +
                    '<span>' +
                        this._getUploadProgressDescription(queueItem) +
                    '</span>' +
                '</td>' : '') +
            '<td class="last" style="width: 100px;">' +
                '<span class="hint-not-started">' +
                    this._locale.statusNotStarted +
                '</span>' +
            '</td>' +
        '</tr>';

        $(this._itemsAreaId).insert({ bottom : itemHtml });
    },

    _getUploadProgressDescription: function(queueItem) {
        var progressText = '';
        if (queueItem.loaded === null || queueItem.loaded == queueItem.total) {
            progressText = this._getSize(queueItem.total);
        } else {
            progressText = this._locale.progressText
                .replace('%%loaded%%', this._getSize(queueItem.loaded))
                .replace('%%total%%', this._getSize(queueItem.total));
        }
        return progressText;
    },

    _getSize: function(size){
        var unitIndex = 0;
        while (size >= 1024) {
            size /= 1024;
            unitIndex++;
        }

        return (unitIndex > 0 ? size.toFixed(1) : size) + ' ' + this._units[unitIndex];
    },

    updateListDescription: function(html) {
        $(this._uploadProgressDialogDescriptionId).update(html);
    },

    updateTotalProgress: function(totalUploaded) {
        $(this._contentAreaId).down('.progress-bar-fill').setStyle({width: totalUploaded + '%'});
        $(this._uploadProgressDialogPercentageId).update(this._locale.totalProgressText.replace('%%percent%%', totalUploaded + '%'));
    },

    updateItemStatus: function(queueItem) {
        var statusIndicator = '';

        switch (queueItem.status) {
            case 'UPLOADING':
                statusIndicator = '<span class="hint-wait">' + this._locale.statusUploading +'</span>';
                break;
            case 'ERROR':
                statusIndicator = '<span class="hint-failed">' + this._locale.statusFailed +'</span>';
                break;
            case 'SUCCESS':
                statusIndicator = '<span class="hint-ok">' + this._locale.statusCompleted +'</span>';
                break;
            case 'CANCELED':
                statusIndicator = '<span class="hint-failed">' + this._locale.statusCanceled +'</span>';
                break;
        }
        $(queueItem.progressId).down('.last').update(statusIndicator);
    },

    updateItemUploadProgress : function(queueItem) {
        if (this._uploadProgressSupported) {
            var progressDescription = this._getUploadProgressDescription(queueItem);
            $(queueItem.progressId).down('.upload-progress').update(progressDescription);

            var progressPercent = queueItem.total > 0 ? Math.round(queueItem.loaded / queueItem.total * 100) : 100;
            var gauge = $(queueItem.progressId).down('.gauge-filled');
            if (gauge) {
                gauge.setStyle({width: progressPercent + '%'});
            } else {
                var progressBar = '<span title="" class="gauge-wrapper not_empty">' +
                    '<span class="gauge-container">' +
                        '<span class="gauge-filled" style="width: ' + progressPercent + '%;"/>' +
                    '</span>' +
                '</span>';
                $(queueItem.progressId).down('.last').update(progressBar);
            }
        }
    }
});


Smb.FileManager.ReplaceExistingFilesDialog = Class.create(Smb.FileManager.ListPopup, {
    _initConfiguration: function($super, config) {
        $super(config);

        this._cls = this._getConfigParam('cls', 'popup-panel');
        this._titleCls = this._getConfigParam('titleCls', 'pp-upload');

        this._items = this._getConfigParam('items', $A());
        this._onSkipClickExternal = this._getConfigParam('onSkipClick', null);
        this._onReplaceClickExternal = this._getConfigParam('onReplaceClick', null);
    },

    render: function($super) {
        $super();

        this.setHint1(this._locale.replaceExistingFilesDialogHint);
        this.setTitle(this._locale.uploadDialogTitle);

        this._items.each(function(item) {
            this._addItem(item);
        }, this);

        this._replaceButton = this.addRightButton(
            this._locale.buttonReplace,
            this.onReplaceClick,
            true,
            false
        );

        this._skipButton = this.addRightButton(
            this._locale.buttonSkip,
            this.onSkipClick,
            true,
            false
        );

        this.resizeList();
    },

    onSkipClick: function() {
        this._updateButton(this._skipButton, {
            disabled: true
        });
        if (this._onSkipClickExternal) {
            this._onSkipClickExternal();
        }
    },

    onReplaceClick: function() {
        this._updateButton(this._replaceButton, {
            disabled: true
        });
        if (this._onReplaceClickExternal) {
            this._onReplaceClickExternal();
        }
    },

    _addItem: function(queueItem) {
        this._itemClass = ('odd' == this._itemClass) ? 'even' : 'odd';

        var itemHtml = '<tr class="' + this._itemClass + '" id="' + queueItem.progressId + '">' +
            '<td class="first">' +
            '<span class="type-' + queueItem.type.escapeHTML() + '">' +
            queueItem.savedFileName.escapeHTML() +
            '</span>' +
            '</td>' +
            '</tr>';

        $(this._itemsAreaId).insert({ bottom : itemHtml });
    }
});

Smb.FileManager.DragAndDrop = Class.create(Jsw.Component, {
    _initConfiguration: function($super, config) {
        $super(config);
        this._uploadPopupParams = this._getConfigParam('uploadPopupParams', {});
        this._dropAreaId = this._getConfigParam('dropAreaId', 'drop-area');
    },

    _initComponentElement: function($super) {
        $super();
        if (!Smb.FileManager.UploadCapabilities.isAdvancedUploadSupported()) {
            return;
        }
        $(this._dropAreaId).down('.fm-drop-area-text').update(this._locale.dragAndDropArea);
    },

    _addEvents: function() {
        if (!Smb.FileManager.UploadCapabilities.isAdvancedUploadSupported()) {
            return;
        }
        this._addDocumentHandlers();
        this._addDropAreaHandlers();
    },

    _addDocumentHandlers: function() {
        $(document).observe('dragover', function(e){
            if (e.dataTransfer){
                e.preventDefault();
            }
        });

        $(document).observe('dragenter', function(e) {
            if (!this._isFileDrag(e)) {
                return;
            }
            $(this._dropAreaId).show();
        }.bindAsEventListener(this));

        $(document).observe('drop', function(e) {
            if (!this._isFileDrag(e)) {
                return;
            }
            $(this._dropAreaId).hide();
            e.preventDefault();
        }.bindAsEventListener(this));
    },

    _addDropAreaHandlers: function() {
        $(this._dropAreaId).observe('dragover', function(e) {
            if (!this._isFileDrag(e)) {
                return;
            }
            e.stopPropagation();
            e.preventDefault();
        }.bindAsEventListener(this));

        $(this._dropAreaId).observe('drop', function(e) {
            if (!this._isFileDrag(e)) {
                return;
            }
            e.preventDefault();

            $(this._dropAreaId).hide();
            this._uploadPopupParams.files = e.dataTransfer.files;
            Jsw.getComponent('fm-content').showPopup(Smb.FileManager.MultipleUpload, this._uploadPopupParams);
        }.bindAsEventListener(this));

        $(this._dropAreaId).observe('mouseover', function(e) {
            $(this._dropAreaId).hide();
        }.bindAsEventListener(this));
    },

    _isFileDrag: function(e) {
        var dataTransfer = e.dataTransfer;
        return dataTransfer && dataTransfer.effectAllowed != 'none' &&
            (dataTransfer.files || (!Prototype.Browser.WebKit && dataTransfer.types.contains && dataTransfer.types.contains('Files')));
    }
});

Smb.FileManager.ErrorDialog = Class.create(Smb.FileManager.ListPopup, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._cls = this._getConfigParam('cls', 'popup-panel');
        this._titleCls = this._getConfigParam('titleCls', 'pp-error');

        this._errorMessage = this._getConfigParam('errorMessage', '');
        this._message = this._getConfigParam('message', null);
        this._items = this._getConfigParam('items', $A());
    },

    render: function($super) {
        $super();

        this.setTitle(this._locale.errorDialogTitle);

        this._addErrorMessage(this._errorMessage);
        if (this._message) {
            this.setHint1(this._message);
        }

        this._items.each(function(item) {
            this._addItem(item);
        }, this);

        this.addRightButton(
            this._locale.buttonClose,
            this._onClose.bind(this),
            true,
            true
        );
        this.resizeList();
    },

    _onClose: function() {
        this.hide();
    },

    _addItem: function(item) {
        this._itemClass = ('odd' == this._itemClass) ? 'even' : 'odd';

        var statusClass = '';
        var statusTitle = '';

        switch (item.status) {
            case 'SUCCESS':
                statusClass = 'ok';
                statusTitle = this._locale.statusCompleted;
                break;
            case 'ERROR':
                statusClass = 'failed';
                statusTitle = this._locale.statusFailed;
                break;
            default:
                statusClass = 'not-started';
                statusTitle = this._locale.statusNotStarted;
        }

        var statusHtml = '<span class="hint-' + statusClass + '">' + statusTitle + '</span>';

        var itemHtml = '' +
            '<tr class="' + this._itemClass + '">' +
                '<td class="first">' +
                    '<span class="type-' + item.type.escapeHTML() + '">' +
                        item.name.escapeHTML() +
                    '</span>' +
                '</td>' +
                '<td class="last">' +
                    statusHtml +
                '</td>' +
            '</tr>';

        $(this._itemsAreaId).insert({ bottom : itemHtml });
    }
});

Smb.FileManager.ExtractArchive = Class.create(Smb.FileManager.PopupForm, {

    render: function($super) {
        $super();

        $(this._contentAreaId).insert(
            '<form method="post" action="" enctype="application/x-www-form-urlencoded" id="' + this._id + '-form">' +
                '<div>' +
                    '<div class="form-row">&#160;</div>' +
                    '<div class="form-row">' +
                        '<div class="field-name">' +
                            '<label for="' + this._id + '-destination-folder">' +
                                this._locale.destinationFolderHint +
                            '</label>' +
                        '</div>' +
                        '<div class="field-value">' +
                            '<div class="text-value">' +
                                '<span id="' + this._id + '-destination-folder">' +
                                '</span>' +
                            '</div>' +
                        '</div>' +
                    '</div>' +
                    '<div class="form-row">' +
                        '<div class="single-row">' +
                            '<div class="indent-box">' +
                                '<input type="checkbox" class="checkbox" value="1" ' +
                                    'id="' + this._id + '-overwrite"/>' +
                                '<div class="indent-box-content">' +
                                    '<label for="' + this._id + '-overwrite">' +
                                        this._locale.overwriteExisting +
                                    '</label>' +
                                '</div>' +
                            '</div>' +
                        '</div>' +
                    '</div>' +
                '</div>' +
            '</form>'
        );

        this._initOnSubmitEvent();

        this.addRightButton(this._locale.buttonOk, this._onSubmit, true, true);
        this.addRightButton(this._locale.buttonCancel, this.hide, false, false);

        this.reset(this._config);
    },

    reset: function($super, options) {
        if (options.fileName) {
            this._fileName = options.fileName;
            this._selectedItems = [Jsw.getComponent('fm-content').getItemById(this._fileName)];
        } else {
            this._fileName = '';
            this._selectedItems = Jsw.getComponent('fm-content').getSelectedItems();
        }

        $(this._id + '-overwrite').checked = false;

        this._initSelectionHint();

        $(this._id + '-destination-folder').update(Jsw.getComponent('fm-content').getCurrentDir().escapeHTML());

        var readonlyHint = Jsw.getComponent('fm-content').getReadonlyHint();
        if (readonlyHint) {
            readonlyHint = '<span class="field-errors">' + readonlyHint + '</span>';
            $(this._id + '-destination-folder').insert({after: readonlyHint});
        } else {
            readonlyHint = $(this._id + '-destination-folder').next('.field-errors');
            if (readonlyHint) {
                readonlyHint.remove();
            }
        }

        this._clearMessages();

        $super();
    },

    _onSubmit: function() {
        this.hide();
        var overwriteExisting = $(this._id + '-overwrite').checked;
        var params = {
            url: Jsw.addUrlParams(this._handlerUrl, 'overwrite=' + overwriteExisting),
            skipConfirmation: true
        };

        if (this._fileName) {
            params['ids[0]'] = this._fileName;
            new Ajax.Request(
                this._handlerUrl,
                {
                    method: 'post',
                    parameters: params,
                    onCreate: this._onOkHandlerCreate.bind(this),
                    onSuccess: (function(transport) {
                        this._onOkHandlerSuccess(transport);
                    }).bind(this),
                    onException: this._onOkHandlerException.bind(this),
                    onComplete: this._onOkHandlerComplete.bind(this)
                }
            );

        } else {
            params.onSuccess = this._onGroupExtractSuccess;
            Jsw.getComponent('fm-content').execGroupOperation(params);
        }
    },

    _onGroupExtractSuccess: function() {
        Jsw.getComponent('fm-content').enable();
        Jsw.getComponent('fm-tree').reloadPath(Jsw.getComponent('fm-content').getCurrentDir());
    },

    _onOkHandlerSuccess: function(transport) {
        this._clearMessages();
        try {
            var response = transport.responseText.evalJSON();
            var message = $A(response.statusMessages).first();
            if ('info' == message.status) {
                Jsw.getComponent('fm-content').reload();
                Jsw.getComponent('fm-tree').reloadPath(Jsw.getComponent('fm-content').getCurrentDir());
                Jsw.addStatusMessage('info', message.content);
                this.hide();
            } else {
                this._addErrorMessage(message.content);
            }
        } catch (e) {
            this._addErrorMessage(e.message);
            this._addErrorMessage(
                'Internal error: ' + transport.responseText
            );
        }
    }
});

Smb.FileManager.CreateArchive = Class.create(Smb.FileManager.PopupForm, {

    render: function($super) {
        $super();

        $(this._contentAreaId).insert(
            '<form method="post" action="" enctype="application/x-www-form-urlencoded" id="' + this._id + '-form">' +
                '<div>' +
                    '<div class="form-row">&#160;</div>' +
                    '<div class="form-row">' +
                        '<div class="field-name">' +
                            '<label for="' + this._id + '-archive-name">' +
                                this._locale.archiveName +
                            '</label>' +
                        '</div>' +
                        '<div class="field-value">' +
                            '<input name="archiveName" id="' + this._id + '-archive-name" ' +
                                'value="" class="f-middle-size input-text" type="text">.zip' +
                            '<span class="field-errors" style="display: none;"></span>' +
                        '</div>' +
                    '</div>' +
                '</div>' +
            '</form>'
        );

        this._initOnSubmitEvent();

        this.addRightButton(this._locale.buttonOk, this._onSubmit, true, true);
        this.addRightButton(this._locale.buttonCancel, this.hide, false, false);

        this.reset();
    },

    reset: function($super) {
        this.setHint(
            this._locale.hint.replace(
                '%%folder%%', '<b>' + Jsw.getComponent('fm-content').getCurrentDir().escapeHTML() + '</b>'
            ) + Jsw.getComponent('fm-content').getReadonlyHint()
        );

        this._clearMessages();

        $(this._id + '-archive-name').value = '';

        $super();

        try {
            $(this._id + '-archive-name').focus();
        } catch (e) {}
    },

    _onSubmit: function() {
        var context = this;
        var archiveName = $(this._id + '-archive-name').value;
        var params = {
            url: Jsw.addUrlParams(this._handlerUrl, 'destinationDir=' + this._destinationDir),
            skipConfirmation: true
        };
        params.submitHandler = function(url, parameters) {
            parameters.set('archiveName', archiveName);
            new Ajax.Request(
                context._handlerUrl,
                {
                    method: 'post',
                    parameters: parameters,
                    onCreate: context._onOkHandlerCreate.bind(context),
                    onSuccess: (function(transport) {
                        context._onOkHandlerSuccess(transport, archiveName);
                    }).bind(context),
                    onException: context._onOkHandlerException.bind(context),
                    onComplete: context._onOkHandlerComplete.bind(context)
                }
            );
        };
        Jsw.getComponent('fm-content').execGroupOperation(params);
    },

    _onOkHandlerSuccess: function(transport, archiveName) {
        this._clearMessages();
        try {
            var response = transport.responseText.evalJSON();
            if ('success' == response.status) {
                Jsw.getComponent('fm-content').reload();
                var message = response.message.replace('%%archive%%', archiveName.escapeHTML() + '.zip');

                Jsw.addStatusMessage('info', message);

                this.hide();
            } else {
                this._addErrorMessage(response.message);
            }
        } catch (e) {
            this._addErrorMessage(e.message);
            this._addErrorMessage(
                'Internal error: ' + transport.responseText
            );
        }
    }

});

Smb.FileManager.ViewSettings = Class.create(Smb.FileManager.PopupForm, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._settings = this._getConfigParam('settings', {});
    },

    render: function($super) {
        $super();

        var content = '' +
            '<form method="post" action="" enctype="application/x-www-form-urlencoded" id="' + this._id + '-form">' +
                '<div>' +
                    '<div class="form-row">&#160;</div>';

        if (this._settings.folderTree) {
            content += '' +
                    '<div class="form-row">' +
                        '<div class="single-row">' +
                            '<div class="indent-box">' +
                                '<input type="checkbox" class="checkbox" value="1" ' +
                                    'id="' + this._id + '-folder-tree"/>' +
                                '<div class="indent-box-content">' +
                                    '<label for="' + this._id + '-folder-tree">' +
                                        this._locale.folderTree +
                                    '</label>' +
                                '</div>' +
                            '</div>' +
                        '</div>' +
                    '</div>';
        }

        if (this._settings.previewPanel) {
            content += '' +
                    '<div class="form-row">' +
                        '<div class="single-row">' +
                            '<div class="indent-box">' +
                                '<input type="checkbox" class="checkbox" value="1" ' +
                                    'id="' + this._id + '-preview-panel"/>' +
                                '<div class="indent-box-content">' +
                                    '<label for="' + this._id + '-preview-panel">' +
                                        this._locale.previewPanel +
                                    '</label>' +
                                '</div>' +
                            '</div>' +
                        '</div>' +
                    '</div>';
        }

        content += '' +
                '</div>';

        if (this._settings.advanced) {
            content += '' +
                '<div id="' + this._id + '-advanced" class="form-box b-collapsible">' +
                    '<div id="' + this._id + '-advanced-content-area">';
        }

        if (this._settings.systemFiles) {
            content += '' +
                        '<div class="form-row">' +
                            '<div class="single-row">' +
                                '<div class="indent-box">' +
                                    '<input type="checkbox" class="checkbox" value="1" ' +
                                        'id="' + this._id + '-system-files"/>' +
                                    '<div class="indent-box-content">' +
                                        '<label for="' + this._id + '-system-files">' +
                                            this._locale.systemFiles +
                                        '</label>' +
                                    '</div>' +
                                '</div>' +
                            '</div>' +
                        '</div>';
        }

        if (this._settings.permissions) {
            content += '' +
                        '<div class="form-row">' +
                            '<div class="single-row">' +
                                '<div class="indent-box">' +
                                    '<input type="checkbox" class="checkbox" value="1" ' +
                                        'id="' + this._id + '-permissions"/>' +
                                    '<div class="indent-box-content">' +
                                        '<label for="' + this._id + '-permissions">' +
                                            this._locale.permissions +
                                        '</label>' +
                                    '</div>' +
                                '</div>' +
                            '</div>' +
                        '</div>';
        }

        if (this._settings.userAndGroup) {
            content += '' +
                        '<div class="form-row">' +
                            '<div class="single-row">' +
                                '<div class="indent-box">' +
                                    '<input type="checkbox" class="checkbox" value="1" ' +
                                        'id="' + this._id + '-user-group"/>' +
                                    '<div class="indent-box-content">' +
                                        '<label for="' + this._id + '-user-group">' +
                                            this._locale.userAndGroup +
                                        '</label>' +
                                    '</div>' +
                                '</div>' +
                            '</div>' +
                        '</div>';
        }

        if (this._settings.advanced) {
        content += '' +
                    '</div>' +
                '</div>';
        }

        content += '' +
            '</form>';

        $(this._contentAreaId).insert(content);

        if (this._settings.advanced) {
            new Jsw.Panel({
                title: this._locale.showAdvanced,
                hideContentTitle: this._locale.hideAdvanced,
                applyTo: this._id + '-advanced',
                hidden: true
            });
        }

        this._initOnSubmitEvent();

        this.addRightButton(this._locale.buttonOk, this._onSubmit, true, true);
        this.addRightButton(this._locale.buttonCancel, this.hide, false, false);

        this.reset();
    },

    reset: function($super) {
        this.setHint(this._locale.hint);

        if (this._settings.folderTree) {
            $(this._id + '-folder-tree').checked = !Jsw.Cookie.get('fm-hidden-tree');
        }

        if (this._settings.systemFiles) {
            $(this._id + '-system-files').checked = !Jsw.Cookie.get('fm-hidden-system-files');
        }

        if (this._settings.permissions) {
            $(this._id + '-permissions').checked = !Jsw.Cookie.get('fm-hidden-permissions-column');
        }

        if (this._settings.userAndGroup) {
            $(this._id + '-user-group').checked = !Jsw.Cookie.get('fm-hidden-user-column');
            $(this._id + '-user-group').checked = !Jsw.Cookie.get('fm-hidden-group-column');
        }

        this._clearMessages();

        $super();
    },

    _onSubmit: function() {

        if (this._settings.folderTree) {
            if ($(this._id + '-folder-tree').checked) {
                Jsw.getComponent('fm-content').showTree();
                Jsw.Cookie.remove('fm-hidden-tree');
            } else {
                Jsw.getComponent('fm-content').hideTree();
                Jsw.Cookie.set('fm-hidden-tree', true);
            }
        }

        if (this._settings.systemFiles) {
            this._updateColumnState($(this._id + '-system-files'), 'fm-hidden-system-files');
            Jsw.getComponent('fm-tree').reload();
        }

        if (this._settings.permissions) {
            this._updateColumnState($(this._id + '-permissions'), 'fm-hidden-permissions-column');
        }

        if (this._settings.userAndGroup) {
            this._updateColumnState($(this._id + '-user-group'), 'fm-hidden-user-column');
            this._updateColumnState($(this._id + '-user-group'), 'fm-hidden-group-column');
        }

        Jsw.getComponent('fm-content').reload();
        this.hide();
    },

    _updateColumnState: function(checkbox, stateCookie) {
        var state = checkbox.checked;
        if (state) {
            Jsw.Cookie.remove(stateCookie);
        } else {
            Jsw.Cookie.set(stateCookie, true);
        }
    }

});

Smb.FileManager.FileNamePopupForm = Class.create(Jsw.PopupForm, {
    _initConfiguration: function($super, config) {
        $super(config);
        this._handler = this._getConfigParam('handler', null);
    },

    render: function($super) {
        $super();

        this.setBoxType('form-box');
        this.setTitle(this._locale.popupTitle);

        $(this._contentAreaId).update(
            this._locale.popupFilename + ' <input type="text" id="saveAsFileNamePopup"/>'
        );

        this.addOkButton();
        this.addCancelButton();

    },

    addOkButton: function() {
        this.addRightButton(this._locale.buttonOk, this._onOkClick, true, true);
    },

    addCancelButton: function() {
        this.addRightButton(this._locale.buttonCancel, this._onCancelClick, false, false);
    },

    _onOkClick: function() {
        this._handler($('saveAsFileNamePopup').value);
        this.hide();
    },

    _onCancelClick: function() {
        this.hide();
    }
});