// Copyright 1999-2014. Parallels IP Holdings GmbH. All Rights Reserved.
/* SMB specific JavaScript classes */

Jsw.namespace('Smb');

Smb.Webmail = {

    urlWrapper: function(webmailUrl) {
        var wrapper = 'window.open("' + webmailUrl + '", "webmail", ' +
            '"width=1150,height=600,innerWidth=1150,innerHeight=600,toolbar=yes,personalbar=no,' +
            'locationbar=no,statusbar=no,scrollbars=yes,resizable=yes");';

        return wrapper;
    }

}

Smb.ApsLicense = {

    buy: function(action, body, inFrame) {
        if ('' == body) {
            window.open(action);
            return false;
        }
        var form = new Element('form');
        var input = new Element('input');

        form.hide();
        form.action = action;
        form.target = '_blank';
        form.method = 'post';
        form.enctype = 'application/x-www-form-urlencoded';

        input.type = 'hidden';
        input.name = 'data';
        input.value = body;

        form.insert(input);
        if (inFrame) {
            var frame = new Element('iframe', { id: 'storeFrame', name: 'storeFrame', scrolling: 'no', width: '100%', frameborder: '0', style: 'visibility: hidden;' });
            var img = new Element('img', { id: 'storeLoading', src: Jsw.skinUrl + '/icons/32/plesk/loading.gif'});
            form.target = 'storeFrame';
            frame.insert(form);
            $('main').update(img);
            $('main').insert(frame);
        } else {
            $(document.body).insert(form);
        }
        form.submit();
    },

    initForm: function() {
        $('apsLicense-uploadType-text').observe('click', this.onChangeUploadType.bindAsEventListener(this));
        $('apsLicense-uploadType-file').observe('click', this.onChangeUploadType.bindAsEventListener(this));
    },

    onChangeUploadType: function() {
        if ($('apsLicense-uploadType-text').checked) {
            $('apsLicense-uploadText').enable();
            $('uploadFile').disable();
        } else {
            $('apsLicense-uploadText').disable();
            $('uploadFile').enable();
        }
    }

};

Smb.Store = {
    init: function() {
        $('storeLoading').setStyle({ display: 'none' });
        $('storeFrame').setStyle({ visibility: 'visible' });
    },
    resize: function(height) {
        $('storeFrame').writeAttribute({height: height + 'px'});
    }
};

Smb.GettingStartedPopup = Class.create({

    initialize: function(containerId) {
        this._container = $(containerId);

        if (!this._container) {
            return;
        }

        this._container.select('.getting-started-link').each((
            function(element) {
                element.observe('click', this._onOkClick.bindAsEventListener(this));
            }
        ).bind(this));
        this._container.select('.s-btn.action').each((
            function(element) {
                element.observe('click', this._onOkClick.bindAsEventListener(this));
            }
        ).bind(this));
        this._container.select('.s-btn').each((
            function(element) {
                if (element.hasClassName('action')) {
                    return;
                }
                element.observe('click', this._onCancelClick.bindAsEventListener(this));
            }
        ).bind(this));

        if ('true' != Jsw.Cookie.get('show-getting-started-popup')) {
            return;
        }
        if ('true' == Jsw.Cookie.get('do-not-show-getting-started-popup')) {
            return;
        }

        this._container.addClassName('active');
        $(document.body).insert({
            top: '<div class="main-disabled-block"></div>'
        });
    },

    _onOkClick: function(event) {
        Event.stop(event);

        this._close();
    },

    _onCancelClick: function(event) {
        Event.stop(event);

        this._close();
    },

    _close: function() {
        this._container.removeClassName('active');

        $(document.body).select('.main-disabled-block').each(
            function(element) {
                element.remove();
            }
        );

        // use two cookies for backward compatibility
        Jsw.Cookie.set('show-getting-started-popup', 'false', null, Jsw.baseUrl + '/');
        Jsw.Cookie.setPermanent('do-not-show-getting-started-popup', 'true', Jsw.baseUrl + '/');
    }

});

Smb.NavigationTabs = Class.create(Jsw.bar.Tabs, {
    _initConfiguration: function($super, config) {
        $super(config);

        this._menuName = this._getConfigParam('menuName', '');
    },

    _renderItems: function($super) {
        $super();
        var contentAreaWidth = $(this._contentAreaId).getWidth();
        var items = $(this._contentAreaId).childElements();
        var itemsWidth = 0;

        items.each(function(item) {
            itemsWidth += item.getWidth() + 1;
        });

        if (itemsWidth > contentAreaWidth) {
            var delta = Math.ceil((itemsWidth - contentAreaWidth) / items.size());
            var paddingDelta = Math.ceil(delta / 2);

            var itemsPadding = 14 - paddingDelta;
            if (itemsPadding < 5) {
                itemsPadding = 5;
            }

            items.each(function(item) {
                item.down('span').setStyle({
                    paddingLeft: itemsPadding + 'px',
                    paddingRight: itemsPadding + 'px'
                })
            });
        }
    },

    _addResponsiveTabs: function() {
        var element = $(this._id);
        var div = new Element('div', { 'class': 'navbar-responsive-menu'});
        div.update('<a href="#navbar" class="nav-menu-link"><span>' + this._menuName + '</span></a>');
        element.insert({
            top: div
        });
        element.down('.navbar-responsive-menu').observe('click', function(event) {
            Event.stop(event);
            element.toggleClassName('responsive-nav-visible');
        });
        element.select('li a').each(function(el) {
            el.observe('click', function() {
                element.toggleClassName('responsive-nav-visible');
            });
        });
    }

});

//prevent AJAX request canceling on Escape button press in Firefox
$(document).observe('keydown', function(event) {
    if (Event.KEY_ESC == event.keyCode) {
        Event.stop(event);
    }
});

Jsw.namespace('Smb.UserPack');

Smb.UserPack.Promotion = Class.create(Jsw.Component, {

    _initConfiguration: function($super, config) {
        $super(config);
        this._buyUserPackLinkId = this._id + '-buy-userpack-link';
    },

    _initComponentElement: function($super) {
        $super();
        var gaugeClass = '';
        var value = this._getConfigParam('value', 0);
        if (100 <= value) {
            gaugeClass = 'over';
        } else if (0 < value) {
            gaugeClass = 'not_empty';
        }
        this._updateComponentElement(
            '<div class="info-box">' +
                '<table cellspacing="0"><tbody><tr><td>' +
                    '<div class="info-box-data">' +
                        this._locale.header +
                    '</div>' +
                    '<span title="" class="gauge-wrapper f5 ' + gaugeClass + '">' +
                        '<span class="gauge-container">' +
                            '<span style="width: ' +
                                (100 >= value ? value : 100) +
                                '%;" class="gauge-filled">' +
                            '</span>' +
                        '</span>' +
                    '</span>' +
                    this._getUserPackPromotionLink() +
                '</td></tr></tbody></table>' +
            '</div>'
        );
    },

    _getUserPackPromotionLink: function() {
        if (this._getConfigParam('buyUserPackUrl')) {
            return '' +
                '<div class="info-box-link">' +
                    '<a href="#" id="' + this._buyUserPackLinkId + '">' +
                        this._locale.link +
                    '</a>' +
                '</div>';
        } else {
            return '' + this._locale.contactToProvider;
        }
    },

    _addEvents: function($super) {
        $super();
        var userPackLink = $(this._buyUserPackLinkId);
        if (userPackLink) {
            userPackLink.observe('click', this._onBuyUserPackClick.bindAsEventListener(this));
        }
    },

    _onBuyUserPackClick: function(event) {
            Event.stop(event);
            Jsw.redirectPost(this._getConfigParam('buyUserPackUrl'), {
                    data: this._getConfigParam('buyUserPackData')
            }, '_blank');
    }

});

Smb.ToolPanel = Class.create(Jsw.Container, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._contentAreaId = this._id + '-tools-list';
        this._image = this._getConfigParam('image', '');
        this._title = this._getConfigParam('title', '');
    },

    _initComponentElement: function($super) {
        this._componentElement = new Element('div', {
            'class': 'tool-block'
        }).insert(
            '<span class="tool-icon">'+
                '<img src="' + this._image + '"/>'+
            '</span>'+
            '<span class="tool-name">' + this._title + '</span>'
        ).insert(new Element('ul', {
            'id': this._id + '-tools-list',
            'class': 'tool-info' })
        );
    }
});

Smb.ToolButton = Class.create(Jsw.BigButton, {

    _initConfiguration: function($super, config) {
        $super(config);
        this._cls = this._getConfigParam('cls', 'tool-block');
        this._disabledClass = this._getConfigParam('disabledClass', 'tool-block-disabled');
    },

    _initComponentElement: function($super) {
        $super();

        var comments = '';
        if (Object.isArray(this._config['additionalComments'])) {
            this._config['additionalComments'].each(function(comment) {
                comments += comment.escapeHTML() + '<br/>';
            });
            comments = '<span>' + comments + '</span>';
        }
        comments += this._config['comment'].escapeHTML();

	    var tool = '<span class="tool-name">' + this._config['title'].escapeHTML() + '</span>' +
            '<span class="tool-info">' + comments + '</span>';
	    if (null != this._icon) {
		    tool = '<span class="tool-icon">' + '<img src="' + this._icon + '"></span>' + tool;
	    }
        this._componentElement.update(tool);

        this._initVisibility();
    }

});

Smb.Dropdown = Class.create(Jsw.Container, {

    _initConfiguration: function($super, config) {
        $super(config);
        this._contentAreaId = this._id + '-content-area';
        this._cls = this._getConfigParam('cls', 'dropdown-box');
        this._onChange = this._getConfigParam('onChange', false);
        this._forceDropdown = this._getConfigParam('forceDropdown', false);
        this._responsiveClass = this._getConfigParam('responsiveClass', false);
        this._responsiveHeaderId = this._getConfigParam('responsiveHeaderId', false);
        this._responsiveHeaderClass = this._getConfigParam('responsiveHeaderClass', false);
    },

    _initComponentElement: function($super) {
        $super();

        this._updateComponentElement(
            '<div class="dropdown-heading">' +
                '<div class="dropdown-heading-area">' +
                    '<em></em>' +
                    '<span>' + this._getConfigParam('title') + '</span>' +
                '</div>' +
            '</div>' +
            '<div class="dropdown-content">' +
                '<div class="dropdown-content-area">' +
                    '<table class="dropdown-wrapper">' +
                        '<tr>' +
                            '<td class="dropdown-container"><ul id="' + this._contentAreaId + '" class="dropdown-list"></ul></td>' +
                        '</tr>' +
                    '</table>' +
                '</div>' +
            '</div>'
        );

        if (this._forceDropdown || this._items && this._items.size() > 1) {
            this._componentElement.addClassName(this._cls);
        }
    },

    _addEvents: function($super) {
        $super();

        var boxElement = this._componentElement;

        Event.observe(boxElement, 'mouseover', function() {
            boxElement.addClassName('expanded');
        });

        Event.observe(boxElement, 'mouseout', function() {
            boxElement.removeClassName('expanded');
        });

        this._addResponsiveEvents();
    },

    _addResponsiveEvents: function() {
        if (this._responsiveClass) {
            var dropdown = this;
            var responsiveElement = $$('.' + this._responsiveClass)[0];
            $(document).observe('click', function() {
                dropdown._hide();
            });

            Event.observe(responsiveElement, 'click', function(event) {
                dropdown._show(responsiveElement);
                event.stopPropagation();
            });

            $(dropdown._applyTargetId).observe('click', function(event) {
                event.stopPropagation();
            });
        }
    },

    _hide: function() {
        $(this._responsiveHeaderId).className = '';
        var element = $$('.r-shortcuts-item-active');
        if (element.length == 0) {
            return;
        }
        element[0].removeClassName('r-shortcuts-item-active');
    },

    _show: function(el) {
        var needShow = true;
        if (el.hasClassName('r-shortcuts-item-active')) {
            needShow = false;
        }

        this._hide();
        if (needShow) {
            $(this._responsiveHeaderId).addClassName(this._responsiveHeaderClass);
            el.addClassName('r-shortcuts-item-active');
        }
    },

    _renderItem: function(item) {
        var dropdownItem = new Jsw.DropdownItem(item);

        var attributes = {
            'href': item.href ? item.href : '#',
            'class': 's-btn ' + (item.styleClass ? item.styleClass : 'sb-no-icon')
        };
        if (item.target) {
            attributes.target = item.target;
        }

        var valueItem = new Element('a', attributes);
        valueItem.innerHTML = '<i><i><i><span>' + dropdownItem.title.escapeHTML() + '</span></i></i></i>';

        var dropdownItemClass = 'dropdown-item';
        if (this._getConfigParam('title') == dropdownItem.title.escapeHTML()) {
            dropdownItemClass += ' dropdown-item-current';
        }

        var liElement = new Element('li', { 'class': dropdownItemClass });
        liElement.appendChild(valueItem);
        $(this._contentAreaId).insert(liElement);

        if (this._onChange) {
            valueItem.observe('click', this._onChange.bindAsEventListener(this, dropdownItem));
        }
    }

});

Smb.SearchDropdown = Class.create(Smb.Dropdown, {

    _initComponentElement: function($super) {
    },

    _addEvents: function($super) {
        this._addResponsiveEvents();
    }

});

Smb.SearchDropdown = Class.create(Smb.Dropdown, {

    _initComponentElement: function($super) {
    },

    _addEvents: function($super) {
        this._addResponsiveEvents();
    }

});

Smb.RssFeedCustomSpot = Class.create(Jsw.Container, {
    _tag: 'div',

    _initConfiguration: function($super, config) {
        $super(config);
        this._dataUrl = this._getConfigParam('dataUrl', '');
        this._customSpotsBlock = this._getConfigParam('customSpotsBlock', '');
        this._itemCount = parseInt(this._getConfigParam('itemCount', ''));
        this._total = this._offset = 0;
    },

    _initComponentElement: function($super) {
        $super();
        this._initLayout();
    },

    _initLayout: function() {
        this._componentElement.update(
            '<div class="p-box-tl"><div class="p-box-tr"><div class="p-box-tc"><!----></div></div></div>' +
                '<div class="p-box-ml"><div class="p-box-mr"><div class="p-box-mc">' +
                '<div class="p-box-header">' +
                '<div class="p-box-header-wrap">' +
                '<div class="p-box-header-wrap-inner">' +
                '<div class="p-box-header-nav">' +
                '<a class="link-01" id="' + this._id + '-prev" href="#"><span>' + this._locale.prev + '</span></a> ' +
                '<a class="link-01" id="' + this._id + '-next" href="#"><span>' + this._locale.next + '</span></a>' +
                '</div>' +
                '</div>' +
                '</div>' +
                '</div>' +
                '</div></div></div>' +
                '<div class="p-box-bl"><div class="p-box-br"><div class="p-box-bc"><!----></div></div></div>'
        );
    },

    _renderItems: function() {
        this._showCurrentItems();
        return false;
    },

    _onNext: function(event) {
        this._offset += this._itemCount;
        this._showCurrentItems();
        Event.stop(event);
        return false;
    },

    _onPrev: function(event) {
        this._offset -= this._itemCount;
        this._showCurrentItems();
        Event.stop(event);
        return false;
    },

    _addEvents: function() {
        $(this._id + '-next').observe('click', this._onNext.bind(this));
        $(this._id + '-prev').observe('click', this._onPrev.bind(this));
    },

    _onCreateExecute: function() {
        $(this._renderTargetId).update('<div class="ajax-loading">' + this._locale.loading + '</div>');
    },

    _onSuccessExecute: function(transport) {
        try {
            var data = transport.responseText.evalJSON();
        } catch (e) {
            Jsw.showInternalError(transport.responseText);
            return false;
        }

        this._total = parseInt(data.total);
        this._togglePrev();
        this._toggleNext();
        this._renderCurrentItems(data.items);
        if (0 >= this._total && $(this._customSpotsBlock)) {
            $(this._customSpotsBlock).hide();
        }
        return false;
    },

    _onFailureExecute: function() {
        if ($(this._customSpotsBlock)) {
            $(this._customSpotsBlock).hide();
        }
    },

    _togglePrev: function() {
        if (0 >= this._offset) {
            $(this._id + '-prev').hide();
        } else {
            $(this._id + '-prev').show();
        }
    },

    _toggleNext: function() {
        if (this._total <= (this._offset + this._itemCount)) {
            $(this._id + '-next').hide();
        } else {
            $(this._id + '-next').show();
        }
    },

    _renderCurrentItems: function(items) {
        if (0 == items.length) {
            $(this._renderTargetId).update('');
            return;
        }

        var content = '<ul id="rssFeedItems" class="rss-feed-list">'
        for(var key in items) {
            if (!items.hasOwnProperty(key)) {
                continue;
            }
            content += '<li class="rss-feed-item">' +
                '<a class="i-link" href="' + items[key]['link'].escapeHTML() + '" target="_blank">' + items[key]['title'].escapeHTML() + '</a>' +
                '<p>' + items[key]['description'] + '</p>' +
                '</li>';
        }
        content += '</ul>';
        $(this._renderTargetId).update(content);
    },

    _showCurrentItems: function() {
        new Ajax.Request(Jsw.prepareUrl(this._dataUrl), {
            method: 'post',
            parameters: {
                offset: this._offset
            },
            onCreate: this._onCreateExecute.bind(this),
            onSuccess: this._onSuccessExecute.bind(this),
            onFailure: this._onFailureExecute.bind(this)
        });
    }
});

Smb.StickyNav = Class.create({
    containerPadding: null,
    navbarOffsetStore: null,

    update: function() {

        if ($('navbar')) { var nav = $('navbar'); }
        else if ($('classic-mode-navigation')) { var nav = $$('#classic-mode-navigation .tabs-area').first(); }

        var navContainer = nav.up();
        var navHeight = nav.getHeight();
        if (this.containerPadding == null) {
            this.containerPadding = parseInt(navContainer.getStyle('padding-bottom'));
        }
        if (this.navbarOffsetStore == null) {
            this.navbarOffsetStore = nav.cumulativeOffset().top;
        }
        if (document.viewport.getScrollOffsets().top > ( this.navbarOffsetStore - 3 )) {
            navContainer.setStyle({ paddingBottom: this.containerPadding + navHeight - 3 + 'px' });
            navContainer.addClassName('sticky-nav');
        } else {
            navContainer.setStyle({ paddingBottom: null });
            navContainer.removeClassName('sticky-nav');
        }
    }
});

Jsw.onReady(Jsw.priority.low, function() {
    if ($('navbar') || $('classic-mode-navigation')) {
        var stickyNav = new Smb.StickyNav;
        stickyNav.update();
        Event.observe(window, 'resize', stickyNav.update.bind(stickyNav));
        Event.observe(window, 'scroll', stickyNav.update.bind(stickyNav));
        Event.observe(document, 'mousewheel', stickyNav.update.bind(stickyNav));
    }
});

Jsw.onReady(function() {

    $$('.active-list-item .caption-control').each(function(element) {
        element.observe('click', function() {
            this.up('.active-list-item').toggleClassName('active-list-item-collapsed');
            this.up('.active-list-item').toggleClassName('active-list-item-expanded');

            var list = this.up('.active-list');
            var stateCookieName = list.id + '-state';
            var activeListState = Jsw.Cookie.get(stateCookieName);

            try {
                activeListState = activeListState.evalJSON();
            } catch (e) {
                activeListState = {}
            }

            var listItem = element.up('.active-list-item');
            var stateItemId = listItem.id.split('-').last();

            if (listItem.hasClassName('active-list-item-expanded')) {
                activeListState[stateItemId] = 1;
            } else {
                delete activeListState[stateItemId];
            }

            Jsw.Cookie.setPermanent(stateCookieName, Object.toJSON(activeListState), '/');
        });
    });

    $$('.panel-heading').each(function (element) {
        element.observe('click', function () {
            this.up('.panel').toggleClassName('panel-collapsed');
            this.up('.panel').toggleClassName('panel-expanded');

            var panel = this.up('.panel');
            var stateCookieName = panel.id + '-hidden';

            if (panel.hasClassName('panel-collapsed')) {
                Jsw.Cookie.setPermanent(stateCookieName, 1, '/');
            } else {
                Jsw.Cookie.remove(stateCookieName, '/');
            }
        });
    });

    $$('.active-list .dropdown-toggle').each(function(element) {
        element.observe('click', function(e){
            Event.stop(e);
            var hadClassName = this.up('.btn-group').hasClassName('btn-group-open');
            $$('.active-list .dropdown-toggle').each(function(elem){
                elem.up('.btn-group').removeClassName('btn-group-open');
            });
            if (!hadClassName) {
                this.up('.btn-group').addClassName('btn-group-open');
            }
        });
        $(document.body).observe('click', function() {
            element.up('.btn-group').removeClassName('btn-group-open');
        });
    });

    $$('.a-toolbar-buttons-misc .btn-search').each(function(element) {
        element.observe('click', function(){
            this.up('.a-toolbar').toggleClassName('a-toolbar-collapsed');
            this.up('.a-toolbar').toggleClassName('a-toolbar-expanded');
        });
    });

    $$('.a-toolbar-buttons-misc .btn-settings').each(function(element) {
        element.observe('click', function(e){
            Event.stop(e);
            this.up('.a-toolbar-buttons-settings').toggleClassName('a-toolbar-buttons-settings-inactive');
            this.up('.a-toolbar-buttons-settings').toggleClassName('a-toolbar-buttons-settings-active');
        });
        $(document.body).observe('click', function() {
            element.up('.a-toolbar-buttons-settings').addClassName('a-toolbar-buttons-settings-inactive');
            element.up('.a-toolbar-buttons-settings').removeClassName('a-toolbar-buttons-settings-active');
        });
    });

    $$('.caption-services .close').each(function(element) {
        element.observe('click', function(e){
            Event.stop(e);
            var closeUrl = this.readAttribute('data-close-url');
            this.up('.caption').select('.caption-services').map(Element.show);
            this.up('.caption-services').remove();
            new Ajax.Request(Jsw.baseUrl + closeUrl, { method: 'post' });
        });
    });
});
