// Copyright 1999-2014. Parallels IP Holdings GmbH. All Rights Reserved.
Jsw.namespace('Smb.WebSitesAndDomains');

Smb.WebSitesAndDomains.Panel = Class.create(Jsw.Panel, {

    render: function($super) {
        $super();
        this._renderHeader();
        this._renderLinks();
        this._renderAjaxPart();
        this._renderMoreLink();
    },

    _renderHeader: function() {
        var description = this._getConfigParam('description');
        if (!description) {
            return;
        }
        $(this._contentAreaId).insert({top: '<p>' + description + '</p>'});
    },

    _renderLinks: function() {
        var links = this._getConfigParam('links');
        if (!links) {
            return;
        }

        if (!$(this._contentAreaId + '-links')) {
            $(this._contentAreaId).insert('<div id="' + this._contentAreaId + '-links"></div>');
        }

        $(this._contentAreaId + '-links').update(
            '<table class="t-links" width="100%" cellspacing="0"><tbody>' +
                '<tr><td width="50%"></td><td width="50%"></td></tr>' +
                '</tbody></table>'
        );

        var firstList = new Element('ul');
        var secondList = new Element('ul');

        $(this._contentAreaId + '-links').down('td').update(firstList);
        $(this._contentAreaId + '-links').down('td', 1).update(secondList);

        // remove null items
        links = links.compact();

        var halfLinks = Math.round(links.size() / 2);
        var evenLink = false;

        links.each(function(unprocessedLink) {
            var linkElement = new Element('li');
            if (evenLink = !evenLink) {
                firstList.insert(linkElement);
            } else {
                secondList.insert(linkElement);
            }
            this._addLinkElement(linkElement, unprocessedLink);
        }, this);
    },

    _processLink: function(unprocessedLink) {
        return Object.clone(unprocessedLink);
    },

    _renderMoreLink: function() {
        var moreLink = this._getConfigParam('moreLink');
        if (!moreLink) {
            return;
        }

        var unprocessedLink = Object.clone(moreLink);
        unprocessedLink.title = moreLink.title;

        if (!$(this._contentAreaId + '-more')) {
            $(this._contentAreaId).insert('<div id="' + this._contentAreaId + '-more" class="panel-bottom-actions"></div>');
        }
        var parent = $(this._contentAreaId + '-more');
        parent.update('');
        this._addLinkElement(parent, unprocessedLink);
    },

    _addLinkElement: function(parent, unprocessedLink) {
        var link = this._processLink(unprocessedLink);
        if (link.disabled) {
            var linkElement = new Element('span')
                .update(link.title);
        } else {
            var linkElement = new Element('a', {
                href: Jsw.prepareUrl(link.href),
                target: link.target ? link.target : false
            }).update(link.title);
        }
        parent.insert(linkElement);
        if (link.description) {
            Jsw.Tooltip.init(linkElement, {text: link.description.escapeHTML()});
        }
        if (link.handler) {
            var handler = link.handler;
            if (Object.isString(handler)) {
                eval('handler = ' + handler);
            }
            linkElement.observe('click', handler);
        }
    },

    _renderAjaxPart: function() {
        var url = this._getConfigParam('ajaxPartSource');
        if (!url) {
            return;
        }

        if (!$(this._contentAreaId + '-ajax-content')) {
            $(this._contentAreaId).insert('<div id="' + this._contentAreaId + '-ajax-content"></div>');
        }
        $(this._contentAreaId + '-ajax-content').update(
            '<div class="ajax-loading">' + this._getConfigParam('ajaxLoadingCaption') + '</div>'
        );

        new Ajax.Updater($(this._contentAreaId + '-ajax-content'), Jsw.prepareUrl(url), {
            method: 'get',
            evalScripts: true
        });
    },

    _addTooltips: function() {
    }
});

Smb.WebSitesAndDomains.CatalogApplicationItem = Class.create(Jsw.Component, {
    _tag: 'div',

    _initConfiguration: function($super, config) {
        $super(config);
        this._cls = 'b-grid-item-wrap';
        this._catalogId = this._getConfigParam('catalogId');
        this._data = this._getConfigParam('data');
    },

    _initComponentElement: function($super) {
        $super();

        this._componentElement.update(
            '<a class="app-block" href="#">' +
                '<span class="app-icon">' +
                '<img alt="" src="' + this._data.icon.escapeHTML() + '"/>' +
                '</span>' +
                '<span class="app-name">' + this._data.title.escapeHTML() + '</span>' +
            '</a>'
        );
    },

    _addEvents: function() {
        this._componentElement.down('a').observe('click', this._onClickEvent.bind(this));
    },

    _onClickEvent: function(event)
    {
        Event.stop(event);
        Jsw.redirectPost('/app/market/id/' + this._catalogId, { details: this._data.details });
    }
});

Smb.WebSitesAndDomains.DeleteConfirmation = function(params) {
    var ids = {
        0: params.id
    };
    Jsw.messageBox.show({
        type: Jsw.messageBox.TYPE_YESNO,
        buttonTitles: {
            yes: params.buttonYes,
            no: params.buttonNo
        },
        text: params.confirmDeleteText,
        subtype: 'delete',
        onYesClick: function() {
            Jsw.redirectPost("/web/delete", {ids: ids, redirect: true});
        }
    });
};

Smb.WebSitesAndDomains.Promo = Class.create(Jsw.Promo, {

    _initLayout: function($super) {
        this._componentElement.update(
            '<ul class="tools-list" id="' + this._contentAreaId + '"></ul>' +
            '<div class="panel-bottom-actions">' +
                '<a class="prev-link" id="' + this._id + '-prev" href="#"><span>' + this._locale.prev + '</span></a> ' +
                '<a class="next-link" id="' + this._id + '-next" href="#"><span>' + this._locale.next + '</span></a>' +
            '</div>'
        );
    }

});

Smb.WebSitesAndDomains.UpdateProgressNotification = Class.create(Jsw.Component, {

    STATUS_WORKING: 'working',
    STATUS_DONE: 'done',
    STATUS_NOTWORKING: 'notworking',

    updateInterval: 5,

    _initConfiguration: function($super, config) {

        $super(config);
        this._message = this._getConfigParam('message');
    },

    _addEvents: function() {
        this._processStatus({
            status: this.STATUS_WORKING,
            message: this._message
        });
    },

    _scheduleUpdateStatus: function() {
        this._statusUpdate.bind(this).delay(this.updateInterval);
    },

    _statusUpdate: function() {
        new Ajax.Request('/admin/update/get-update-data', {
            method: 'get',
            onSuccess: this._onSuccess.bind(this),
            onFailure: this._onFailure.bind(this)
        });
    },


    _onSuccess: function(transport) {
        try {
            this._processStatus(transport.responseText.evalJSON());
        } catch (e) {
            // Schedule new update - control panel is probably down
            this._scheduleUpdateStatus();
        }
    },


    _onFailure: function(transport) {

        // Schedule new update - control panel is probably down
        // or othewise unavailable due to updates installation.
        // Have no information to display.

        this._scheduleUpdateStatus();
    },


    _processStatus: function(response) {

        if (!response) {

            // Schedule new update - control panel is probably down
            this._scheduleUpdateStatus();

        } else if (this.STATUS_WORKING == response.status) {

            this._componentElement.update(
                '<div class="msg-box msg-warning msg-update-running">' +
                    '<div><div><div><div><div>' +
                    '<div class="msg-content">' +
                    this._message +
                    '</div>' +
                    '</div></div></div></div></div>' +
                    '</div>'
            );

            this._scheduleUpdateStatus();

        } else if (this.STATUS_DONE == response.status || this.STATUS_NOTWORKING == response.status) {

            // Stop asynchronous notification and reload the whole page
            Jsw.redirect('/');
        }
    }

});

Smb.WebSitesAndDomains.UpdateResultNotification = Class.create(Jsw.Component, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._status = this._getConfigParam('status', 'info');
        this._message = this._getConfigParam('message', '');
    },

    _initComponentElement: function($super) {
        $super();

        this._updateComponentElement(
            '<div class="msg-box msg-' + this._status + '"><div><div><div><div><div>' +
                '<div class="msg-content">' + this._message + '</div></div></div></div></div></div></div>'
        );
    }

});

Smb.WebSitesAndDomains.StickySidebar = Class.create ({

    initialize: function (element, options) {

        this.element = $(element);
        this.options = options || {};
        this.offsetTop = this.options.offsetTop || 0;
        this.offsetBottom = this.options.offsetBottom || 0;

        Event.observe(window, 'resize', this.update.bindAsEventListener(this));
        Event.observe(window, 'scroll', this.update.bindAsEventListener(this));
        Event.observe(document, 'mousewheel', this.update.bindAsEventListener(this));
    },

    update: function () {
        var viewportTop = document.viewport.getScrollOffsets().top + this.offsetTop;
        var viewportBottom = viewportTop + document.viewport.getHeight() - this.offsetTop - this.offsetBottom;

        this.element.childElements().each(function(element) {

            var margin = parseInt(element.getStyle('marginTop'));
            var elementTop = element.cumulativeOffset().top;
            var elementBottom = elementTop + element.getHeight();

            if (elementBottom < viewportBottom) {
                var deltaBottom = viewportBottom - elementBottom;
                var delta = Math.min(deltaBottom , this.element.getHeight() - element.getHeight() - margin);
                if (elementTop < viewportTop) {
                    var deltaTop = viewportTop - elementTop;
                    delta = Math.min(deltaTop, delta);
                } else {
                    delta = 0;
                }
                if (delta > 0) {
                    element.setStyle({
                        marginTop: (margin + delta) + 'px',
                        height: element.getHeight() + 'px'
                    });
                    if (delta == deltaBottom || delta == deltaTop) {
                        element.down().setStyle({
                            bottom: delta != deltaTop && delta == deltaBottom  ? this.offsetBottom + 'px' : null,
                            top: delta == deltaTop ? this.offsetTop + 'px' : null,
                            width: parseInt(element.down().getStyle('width')) + 'px'
                        }).addClassName('fixed');
                    } else {
                        delta = 0;
                    }
                }
            }

            if (elementTop > viewportTop) {
                var deltaTop = elementTop - viewportTop;
                var delta = Math.min(deltaTop, margin);
                if (delta > 0) {
                    element.setStyle({
                        marginTop: (margin - delta) + 'px',
                        height: element.getHeight() + 'px'
                    });
                    if (delta == deltaTop) {
                        element.down().setStyle({
                            top: this.offsetTop + 'px',
                            width: parseInt(element.down().getStyle('width')) + 'px'
                        }).addClassName('fixed');
                    } else {
                        delta = 0;
                    }
                }
            }

            if (!delta) {
                element.setStyle({
                    height: null
                });
                element.down().removeClassName('fixed').setStyle({
                    width: null,
                    top: null,
                    bottom: null
                });
            }

        }, this);
    }
});

Jsw.onReady(Jsw.priority.low, function() {
    if ($('navbar') || $('classic-mode-navigation')) {
        var config = {};
        if ($('navbar')) {
            config = {
                offsetTop: $('navbar').getHeight() + parseInt($$('.heading').first().getStyle('marginBottom')),
                offsetBottom: parseInt($('footer-wrapper').getStyle('paddingTop'))
            };
        } else if ($('classic-mode-navigation')) {
            config = {
                offsetTop: $('classic-mode-navigation').getHeight() + parseInt($('classic-mode-navigation').getStyle('marginBottom')),
                offsetBottom: parseInt($('wrapper').getStyle('paddingBottom'))
            };
        }
        var stickySidebar = new Smb.WebSitesAndDomains.StickySidebar($$('.b-content-wrap').first(), config);
        $$('.active-list-details, .panel-content').each(function(element) {
            $w('webkitTransitionEnd transitionend msTransitionEnd oTransitionEnd').each(function(ev) {
                element.observe(ev, stickySidebar.update.bindAsEventListener(stickySidebar));
            });
        });
    }
});

Jsw.onReady(function() {
    $$('.js-webadmin').each(function(element) {
        element.observe('click', function() {
            var select = element.up('div').down('select');
            var selected = select.options[select.selectedIndex];
            var userLogin = selected.readAttribute('data-user-login');
            var passwordPrompt = selected.readAttribute('data-password-prompt');
            var domainId = element.up('.active-list-item').down('.js-domain-id').value;
            var params = {'domainId': domainId};

            if (userLogin) {
                var dbPassword = prompt(passwordPrompt);
                if (!dbPassword) {
                    return;
                }
                params['dbPassword'] = dbPassword;
            }

            Jsw.redirectPost(selected.value, params, 'dbedit');
        });
    });

    $$('.caption-service-toolbar .toggler').each(function(element) {
        element.observe('click', function(e) {
            var editBlock = this.up('.caption-service-toolbar').down('.item-invisible');
            var createBlock = this.up('.item-visible');
            editBlock.removeClassName('item-invisible').addClassName('item-visible');
            createBlock.removeClassName('item-visible').addClassName('item-invisible');

            Event.stop(e);
        });
    });

    $$('.so-panel-version').each(function(element) {
        element.observe('click', function(event) {
            Event.stop(event);
            var position = element.positionedOffset();
            var popUp = $$('.pb-panel-version').first();
            var popUpWidth = popUp.down('.popup-wrapper').getWidth();
            var leftOffset = position[0] - popUpWidth + 'px';
            var topOffset = position[1] + 'px';
            popUp.setStyle({'left': leftOffset, 'top': topOffset});
            popUp.toggleClassName('collapsed');
        });

        $$('.pb-panel-version').each(function(popUp) {
            popUp.observe('click', function(event) {
                event = Event.extend(event);
                event.stopPropagation();
            });

            popUp.down('.close').observe('click', function() {
                $$('.pb-panel-version').first().addClassName('collapsed');
            });
        });

        $(document.body).observe('click', function() {
            this.down('.pb-panel-version').addClassName('collapsed');
        });
    });
});

