<?php
/**
 * Lock manager client extension stub
 */
class Lock
{
    const AVAILABLE = 0;
    const LOCKED_ON_READ = 1;
    const LOCKED_ON_WRITE = 2;
    const LOCKED = 3;
    const ALREADY_LOCKED = 4;
    const CANNOT_LOCK = 5;
    const UNLOCKED = 6;
//    const DEADLOCK = 7;
    const NOT_LOCKED = 7;
//    const INVALID_QUERY = 9;
//    const RESULTS_COUNT = 10;

    /**
     * Creates an object-lock for a particular resource with name resourceName
     *
     * @param string $resourceName
     */
    public function __construct($resourceName)
    {}

    /**
     * Blocks resource for reading. Blocking call. If the resource can not be obtained, then execution is blocked.
     * Comment - optional argument is used in the detection of deadlocks.
     * In case of failure, throws an exception (DeadlockException, ConnectionException)
     * Returns the status of the transaction ("Locked" | "Already locked" - in an attempt to re-capture)
     *
     * @param string $comment
     * @return string
     */
    public function readLock($comment)
    {}

    /**
     * Blocks resource for reading. Non-blocking call.
     * Comment - optional argument is used in the detection of deadlocks.
     * In case of failure, throws an exception (ConnectionException)
     * Returns the status of the transaction ("Locked" | "Already locked" - in an attempt to re-capture | "Cannot lock" - if you can not get the resource)
     *
     * @param string $comment
     * @return string
     */
    public function tryReadLock($comment)
    {}

    /**
     * Blocks resource for writing. Blocking call. If the resource can not be obtained, then execution is blocked.
     * Comment - optional argument is used in the detection of deadlocks.
     * In case of failure, throws an exception (DeadlockException, ConnectionException)
     * Returns the status of the transaction ("Locked" | "Already locked" - in an attempt to re-capture)
     *
     * @param $comment
     * @return string
     */
    public function writeLock($comment)
    {}

    /**
     * Blocks resource for writing. Non-blocking call.
     * Comment - optional argument is used in the detection of deadlocks.
     * In case of failure, throws an exception (ConnectionException)
     * Returns the status of the transaction ("Locked" | "Already locked" - in an attempt to re-capture | "Cannot lock" - if you can not get the resource)
     *
     * @param $comment
     * @return string
     */
    public function tryWriteLock($comment)
    {}

    /**
     * Returns the lock status. During the time while on the service, or even locally comes the answer, the lock can change.
     * Possible states: 0 - the resource is available. 1 - the resource is read. 2 - the resource is on record.
     * In case of failure, throws an exception (ConnectionException)
     *
     * @return int
     */
    public function getState()
    {}

    /**
     * Unlocks a resource.
     * In case of failure, throws an exception (ConnectionException).
     * Returns the status of the operation ("Unlocked" | "Not locked")
     *
     * @return string
     */
    public function unlock()
    {}
}

class LockQuery
{
    const WRITE = 'EX';
    const READ = 'SH';

    /**
     * Blocks resource group. Blocking call.
     * Terminates all locks after the capture of the group.
     * Comment - optional argument is used in the detection of deadlocks.
     * In case of failure, throws an exception (DeadlockException, ConnectionException)
     * Returns the status of the operation
     *      Lock :: LOCKED | Lock :: ALREADY_LOCKED - if at least one of the resource group is already captured by this client
     *
     * @param string $comment
     * @return int
     */
    public function  lock($comment)
    {}

    /**
     * Blocks resource group. Non-blocking call.
     * Comment - optional argument is used in the detection of deadlocks.
     * In case of failure, throws an exception (ConnectionException)
     *
     * Returns the status of the operation
     *  Lock :: LOCKED | Lock :: ALREADY_LOCKED - if at least one of the resource group is already captured by this client
     *  Lock :: CANNOT_LOCK - if you can not get all the resources
     *
     * @param string $comment
     * @return string
     */
    public function tryLock($comment)
    {}

    /**
     * Unlocks the resource group.
     * In case of failure, throws an exception (ConnectionException).
     * Returns the status of the operation (Lock :: UNLOCKED | Lock :: NOT_LOCKED)
     *
     * @return int
     */
    public function unlock()
    {}
}

class LockManagerClient
{
    /**
     * Creates a new instance of the object to control the locks. This method is created or obtained from somewhere in the client ID.
     */
    public function __construct()
    {}

    /**
     * Creates a request to perform an atomic capture of resources.
     * For each resource specifies the type of lock: (READ | WRITE)
     *
     * @param $query
     * array(
     *  String resourceName1 => LockQuery::LockType type1,
     * )
     *
     * @return LockQuery
     */
    public  function getLockQuery($query)
    {}

    /**
     * Unlocks all resources associated with the identifier.
     * In case of failure, throws an exception (ConnectionException).
     *
     * @return void
     */
    public function unlockAll()
    {}

    /**
     * Changes the customer ID to the parameter token. If the client has locked and it is the last client with that ID, the lock "released."
     * In case of failure, throws an exception (ConnectionException, NoSuchTokenException)
     *
     * @param string $token
     * @return void
     */
    public function setToken($token)
    {}

    /**
     * Returns the current ID.
     * In case of failure, throws an exception (ConnectionException).
     *
     * @return string
     */
    public function getToken()
    {}

    /**
     * Creates a new identifier. If the client has locked and it is the last client with that ID, the lock "released."
     * In case of failure, throws an exception (ConnectionException)
     *
     * @return string
     */
    public function createToken()
    {}
}
