// Copyright 1999-2014. Parallels IP Holdings GmbH. All Rights Reserved.
Jsw.Chart = Class.create(Proto.Chart, {

    // override original method to hide labels near pies
    drawPieGraph: function(graphData) {
        var sumData = 0;
        var radius = 0;
        var centerX = this.chartWidth/2;
        var centerY = this.chartHeight/2;
        var startAngle = 0;
        var endAngle = 0;
        var fontSize = this.options.pies.fontSize;
        var labelWidth = this.options.pies.labelWidth;

        //determine Pie Radius
        if(!this.options.pies.autoScale)
            radius = this.options.pies.radius;
        else
            radius = (this.chartHeight * 0.85)/2;

        var labelRadius = radius * 1.05;

        for(i = 0; i < graphData.length; i++)
            sumData += graphData[i].data;

        // used to adjust labels so that everything adds up to 100%
        totalPct = 0;

        //lets draw the shadow first.. we don't need an individual shadow to every pie rather we just
        //draw a circle underneath to simulate the shadow...
        this.drawPieShadow(centerX, centerY, radius, 0, 0);

        //lets draw the actual pie chart now.
        graphData.each(function(gd, j){
            var pct = gd.data / sumData;
            startAngle = endAngle;
            endAngle += pct * (2 * Math.PI);
            var sliceMiddle = (endAngle - startAngle) / 2 + startAngle;
            var labelX = centerX + Math.cos(sliceMiddle) * labelRadius;
            var labelY = centerY + Math.sin(sliceMiddle) * labelRadius;
            var anchorX = centerX;
            var anchorY = centerY;
            var textAlign = null;
            var verticalAlign = null;
            var left = 0;
            var top = 0;

            //draw pie:
            //drawing pie
            this.context.beginPath();
            this.context.moveTo(anchorX, anchorY);
            this.context.arc(anchorX, anchorY, radius, startAngle, endAngle, false);
            this.context.closePath();
            this.context.fillStyle = this.parseColor(gd.color).scale(null, null, null, this.options.pies.fillOpacity).toString();

            if(this.options.pies.fill)  { this.context.fill(); }

            // drawing labels
            if (sliceMiddle <= 0.25 * (2 * Math.PI))
            {
                // text on top and align left
                textAlign = "left";
                verticalAlign = "top";
                left = labelX;
                top = labelY + fontSize;
            }
            else if (sliceMiddle > 0.25 * (2 * Math.PI) && sliceMiddle <= 0.5 * (2 * Math.PI))
            {
                // text on bottom and align left
                textAlign = "left";
                verticalAlign = "bottom";
                left = labelX - labelWidth;
                top = labelY;
            }
            else if (sliceMiddle > 0.5 * (2 * Math.PI) && sliceMiddle <= 0.75 * (2 * Math.PI))
            {
                // text on bottom and align right
                textAlign = "right";
                verticalAlign = "bottom";
                left = labelX - labelWidth;
                top = labelY - fontSize;
            }
            else
            {
                // text on top and align right
                textAlign = "right";
                verticalAlign = "bottom";
                left = labelX;
                top = labelY - fontSize;
            }

            left = left + "px";
            top = top + "px";
            var textVal = Math.round(pct * 100);

            if (j == graphData.length - 1) {
                if (textVal + totalPct < 100) {
                    textVal = textVal + 1;
                } else if (textVal + totalPct > 100) {
                    textVal = textVal - 1;
                };
            }

            totalPct = totalPct + textVal;
        }.bind(this));
    }

});