// Copyright 1999-2014. Parallels IP Holdings GmbH. All Rights Reserved.
Jsw.onReady(function() {

    var searchActionDelay = 200;
    var timer;
    var enterPressed = false;

    function lmsg(localeKey) {
        return searchBarParams.locale[localeKey].escapeHTML();
    }

    function fatalError(message) {
        alert(message);
    }

    function onPasteHandler() {
        scheduleSearch();
    }

    function onKeyUpHandler(event) {
        if ([Jsw.keyCode.UP_ARROW,  Jsw.keyCode.DOWN_ARROW, Jsw.keyCode.LEFT_ARROW, Jsw.keyCode.RIGHT_ARROW, Jsw.keyCode.ENTER].indexOf(event.keyCode) != -1) {
            return;
        }

        scheduleSearch();
    }

    function onKeyDownHandler(event) {
        if ([Jsw.keyCode.UP_ARROW,  Jsw.keyCode.DOWN_ARROW].indexOf(event.keyCode) != -1) {
            onArrowKeyPressed(event.keyCode);
        }

        enterPressed = (Jsw.keyCode.ENTER == event.keyCode);
        if (enterPressed) {
            scheduleSearch();
        }
    }

    function onFocusHandler(event) {
        var field = event.target;
        field.value = '';
        field.removeClassName('search-empty');
    }

    function onBlurHandler(event) {
        // workaround for Google Chrome
        setTimeout(function() {
            showSearchFieldStub();
            setTimeout(resetSearch, 300);
        }, 0);
    }

    function showSearchFieldStub() {
        var field = $('searchTerm');
        field.value = lmsg('fieldStub');
        field.addClassName('search-empty');
        field.blur();
    }

    function scheduleSearch() {
        $('searchButton').src = searchBarParams.skinUrl + 'images/indicator.gif';

        if (timer) {
            clearTimeout(timer);
        }
        timer = setTimeout(findTerm, searchActionDelay);
    }

    function onSearchSuccess(request) {
        if ($('searchTerm').value != request.request.options.parameters.term) {
            return;
        }

        try {
            var response = request.responseText.evalJSON();
        } catch (e) {
            fatalError('Failed to parse JSON response: ' + e.message);
            return;
        }

        if ('error' == response.status) {
            var result = '';
            response.statusMessages.each(function(message) {
                result += message.title + ': ' + message.content + "\n";
            });
            fatalError(result);
            return;
        }

        if (enterPressed) {
            goToSelectedItem(response);
        } else {
            showResults(response);
        }
    }

    function onSearchFailure(request) {
        fatalError('Search request failed due to following error: ' + request.responseText);
    }

    function onSearchComplete(request) {
        $('searchButton').src = searchBarParams.skinUrl + 'icons/16/plesk/search.png';
    }

    function goToSelectedItem(results) {
        var resultsBlock = getResultsBlock();
        var activeLink = resultsBlock.select('a.active')[0];

        if (!activeLink) {
            activeLink = resultsBlock.select('a')[0];
        }

        var linkHref = activeLink ? activeLink.href : results.records.first().link;
        var linkTarget = activeLink ? activeLink.target : results.records.first().target;

        if (!linkTarget && 'admin' == searchBarParams.panelType) {
            top.window.frames['workFrame'].document.location = linkHref;
        } else {
            Jsw.redirect(linkHref, null, linkTarget);
        }

        showSearchFieldStub();
        resetSearch();
    }

    function showResults(results) {
        var resultsBlock = getResultsBlock();

        var resultsHtml = '';

        if (0 == results.records.size()) {
            resultsHtml += '<div class="search-results-note"><span>' + lmsg('nothingFound') + '</span></div>';
        } else {
            resultsHtml += '<ul class="search-results-list">';

            results.records.each(function(item) {
                var itemDetails = ('undefined' != typeof item.details) ? item.details.escapeHTML() : '';
                var linkTarget = '';
                if (item.target) {
                    linkTarget = 'target="' + item.target + '"';
                }
                var icon = '';

                if ('undefined' != typeof item.icon) {
                    var iconUrl = (0 == item.icon.indexOf(searchBarParams.skinUrl))
                        ? item.icon
                        : searchBarParams.skinUrl + item.icon;
                    icon = '<img src="' + iconUrl + '"> ';
                }

                resultsHtml +=
                    '<li class="search-results-item">'
                        + '<a href="' + item.link.escapeHTML() + '" title="' + itemDetails + '" ' + linkTarget + '>'
                            + '<span>'
                                + icon
                                + item.title.escapeHTML()
                            + '</span>'
                        + '</a>'
                    + '</li>';
            });

            resultsHtml += '</ul>';

            if (results.meta.moreResultsFound) {
                resultsHtml +=
                    '<div class="search-results-note"><span>'
                    + lmsg('moreResultsFound')
                    + '</span></div>';
            }
        }

        resultsBlock.down('.top-search-results-content').update(resultsHtml);

        var links = resultsBlock.select('a');
        links.each(function(link) {
            link.observe('mouseover', onItemMouseOver.bind(link, resultsBlock));
            link.observe('mouseout', onItemMouseOut);
        });

        resultsBlock.show();

        if ('admin' == searchBarParams.panelType) {
            $('searchResultsBlockHead').show();
        }
    }

    function onItemMouseOver(resultsBlock) {
        var oldActiveLink = resultsBlock.select('a.active')[0];

        if (oldActiveLink) {
            oldActiveLink.removeClassName('active');
        }

        this.addClassName('active');
    }

    function onItemMouseOut() {
        this.removeClassName('active');
    }

    function getResultsBlock() {
        if ('admin' == searchBarParams.panelType) {
            var workFrameDoc = top.window.frames['workFrame'];
            var resultsBlock = workFrameDoc.$('searchResultsBlock');
            var searchBlock = top.window.frames['topFrame'].$('search');
            var leftFrameWidth = top.window.document.getElementById('MainFrameSet').getAttribute('cols').split(',')[0];
            var searchBlockOffsetLeft = searchBlock.viewportOffset()[0];
            var resultBlockOffset = searchBlockOffsetLeft - leftFrameWidth;
        } else {
            var resultsBlock = $('searchResultsBlock');
            var resultBlockOffset = 0;
        }

        if (!resultsBlock) {
            resultsBlock = new workFrameDoc.Element('div', {
                'id': 'searchResultsBlock',
                'class': 'top-search-results'
            });
            resultsBlock.hide();
            resultsBlock.update(
                '<table class="top-search-results-wrapper"><tbody><tr><td class="top-search-results-container">'
                    + '<div class="top-search-results-content">'
                    + '</div>'
                + '</td></tr></tbody></table>'
            );

            $(workFrameDoc.document.body).insert({ top: resultsBlock });

            resultsBlock.observe('click', resetSearch);
        }


        resultsBlock.setStyle({
            left: resultBlockOffset + 'px'
        });

        return resultsBlock;
    }

    function resetSearch() {
        if (timer) {
            clearTimeout(timer);
        }

        var resultsBlock = null;

        if ('admin' == searchBarParams.panelType) {
            $('searchResultsBlockHead').hide();

            try {
                resultsBlock = top.window.frames['workFrame'].$('searchResultsBlock')
            } catch (e) {
                // frame not loaded yet, no need to hide block
            }
        } else {
            resultsBlock = $('searchResultsBlock');
        }

        if (resultsBlock) {
            resultsBlock.hide();
        }

        onSearchComplete();
    }

    function onArrowKeyPressed(keyCode) {
        var resultsBlock = getResultsBlock();
        var links = resultsBlock.select('a');

        if (0 == links.length) {
            return;
        }

        var activeLink = resultsBlock.select('a.active')[0];

        if (!activeLink) {
            resultsBlock.select('a').first().addClassName('active');
            return;
        }

        if (Jsw.keyCode.DOWN_ARROW == keyCode) {
            var nextRow = activeLink.up('li').next();

            if (nextRow) {
                activeLink.removeClassName('active');
                nextRow.down('a').addClassName('active');
            }
        }

        if (Jsw.keyCode.UP_ARROW == keyCode) {
            var prevRow = activeLink.up('li').previous();

            if (prevRow) {
                activeLink.removeClassName('active');
                prevRow.down('a').addClassName('active');
            }
        }
    }

    function findTerm() {
        var term = $('searchTerm').value;

        if ('' == term) {
            resetSearch();
            return;
        }

        new Ajax.Request(
            searchBarParams.actionUrl,
            {
                method: 'get',
                parameters: { 'term': term },
                onSuccess: onSearchSuccess.bind(this),
                onFailure: onSearchFailure.bind(this),
                onComplete: onSearchComplete.bind(this)
            }
        );
    }

    showSearchFieldStub();

    $('searchTerm').observe('paste', onPasteHandler);
    $('searchTerm').observe('keyup', onKeyUpHandler);
    $('searchTerm').observe('keydown', onKeyDownHandler);
    $('searchTerm').observe('focus', onFocusHandler);
    $('searchTerm').observe('blur', onBlurHandler);

    Event.observe(window, 'resize', function() {
        setTimeout(getResultsBlock, 0);
    });
});
