// Copyright 1999-2014. Parallels IP Holdings GmbH. All Rights Reserved.
Jsw.namespace('Jsw.Task.ProgressBar');

Jsw.Task.ProgressBar = Class.create(Jsw.Container, {

    _initConfiguration: function($super, config) {
        $super(config);

        this.pe = null;
        this._contentAreaId = this._id + '-content-area';
    },

    _initComponentElement: function($super) {
        $super();

        var isAsyncProgressBarCollapsed = Jsw.Cookie.get('isAsyncProgressBarCollapsed');

        this._updateComponentElement(
            '<div id="asyncProgressBar" class="async-progress-bar ' + ("true" === isAsyncProgressBarCollapsed ? 'async-progress-bar-collapsed': '') + '"> ' +
                '<div class="async-progress-bar-wrap">' +
                    '<div class="async-progress-bar-top">' +
                        '<div class="async-progress-bar-control">' +
                            '<a id="asyncProgressBarControlHide" href="#" class="async-progress-bar-control-hide">' + this._locale.progressBarHide +'</a>' +
                            '<a id="asyncProgressBarControlShow" href="#" class="async-progress-bar-control-show">' + this._locale.progressBarShow +'</a>' +
                        '</div>' +
                        '<div class="async-progress-bar-title">' +
                            '<span id="asyncProgressBarTitleTasks" class="async-progress-bar-title-tasks hidden"></span>' +
                            '<span id="asyncProgressBarTitleTasksError" class="async-progress-bar-title-tasks-error hidden"><span>0</span></span>' +
                            '<span id="asyncProgressBarTitleTasksComplete" class="async-progress-bar-title-tasks-complete hidden"><span>0</span></span>' +
                            '<span id="asyncProgressBarTitleTasksWarning" class="async-progress-bar-title-tasks-warning hidden"><span>0</span></span>' +
                            '<a id="asyncProgressBarHideCompletedTasks" class="async-progress-bar-title-tasks-hide hidden" href="#">' + this._locale.hideCompletedTasks + '</a>' +
                        '</div>' +
                    '</div>' +
                    '<div class="async-progress-bar-body">' +
                        '<ul class="async-progress-bar-list" id="' + this._contentAreaId + '">' +
                        '</ul>' +
                    '</div>' +
                '</div>' +
            '</div>'
        );
    },

    toggle: function() {
        var element = $('asyncProgressBar');
        element.toggleClassName('async-progress-bar-collapsed');

        Jsw.Cookie.setPermanent(
            'isAsyncProgressBarCollapsed',
            element.hasClassName('async-progress-bar-collapsed'),
            '/'
        );
    },

    update: function() {
        var context = this;
        new Ajax.Request(Jsw.prepareUrl('/task/task-progress/'), {
            method: 'post',
            onSuccess: function(response) {
                var data = response.responseText.evalJSON();
                context.checkPreviousStatus(data.items);
                context.setItems(data.items);
                context.renderItems();
            }
        });
    },

    getLocale: function() {
        return this._locale;
    },

    renderItems: function() {
        this._initItems();
        this._renderItems();
    },

    removeItemsByIds: function(ids){
        var items = this.getItems();
        for (var i=0; i<ids.length; i++) {
            for (var j=0; j<items.length; j++) {
                if (items[j].getId() === ids[i]) {
                    items[j].getTargetElement().remove();
                    items.splice(j, 1);
                    break;
                }
            }
        }
        this.setItems(items);
        this.renderItems();
    },

    checkPreviousStatus: function(items) {
        items.each(function(item) {
            this._config['items'].each(function(item) {
                if (this['id'] == item.getId()) {
                    this['isRefreshLinkEnabled'] = item['_isRefreshLinkEnabled'];
                    if (('started' != this['status']) && item.isRunning()) {
                        this['isRefreshLinkEnabled'] = true;
                    }
                }
            }, item);
        }, this);
    },

    setItems: function(items) {
        this._config['items'] = items;
    },

    removeCompletedTask:function()
    {
        var context = this;
        var items = this.getItems();
        var ids = [];
        for (var i=0; i<items.length; i++) {
            if (!items[i].isRunning()) {
                ids.push(items[i].getId());
            }
        }
        if (ids.length > 0) {
            new Ajax.Request(Jsw.prepareUrl('/task/task-remove/'), {
                method: 'post',
                parameters: {'ids[]': ids},
                onSuccess: function(response) {
                    context.removeItemsByIds(ids);
                }
            });
        }
    },

    _renderItems: function($super) {
        $super();

        this._updateTitle();
        if (this.getItems().length > 0) {
            this._componentElement.removeClassName('hidden');
            this._setPeriodicalExecutor();
        } else {
            this._componentElement.addClassName('hidden');
        }
    },

    _setPeriodicalExecutor: function()
    {
        var context = this;
        if (null == this.pe) {
            this.pe = new PeriodicalExecuter(function() {
                context.update();
                var items = context.getItems();
                if (items && 0 == items.length) {
                    context.pe.stop();
                    context.pe = null;
                }
            }, 5);
        }
    },

    _updateTitle: function() {
        var countRunning = 0;
        var countCompleteError = 0;
        var countComplete = 0;
        var countCompleteWarning = 0;
        var countAll = 0;

        var items = this.getItems();
        for (var i=0; i<items.length; i++) {
            countAll += 1;
            if (items[i].isComplete()) {
                countComplete += 1;
            } else if (items[i].isCompleteWithWarning()) {
                countCompleteWarning += 1;
            } else if (items[i].isCompleteWithError()) {
                countCompleteError += 1;
            } else {
                countRunning += 1;
            }
        }
        var taskRunningElement = $('asyncProgressBarTitleTasks');
        if (countRunning > 0) {
            if (countRunning == countAll) {
                taskRunningElement.update(countRunning + ' ' + this._locale.taskInProgress);
            } else {
                taskRunningElement.update(countRunning);
            }
            $('asyncProgressBar').removeClassName('async-progress-bar-complete');
            taskRunningElement.removeClassName('hidden');
        } else {
            $('asyncProgressBar').addClassName('async-progress-bar-complete');
            taskRunningElement.addClassName('hidden');
        }
        if (countComplete > 0 || countCompleteWarning > 0 || countCompleteError > 0) {
            $('asyncProgressBarHideCompletedTasks').removeClassName('hidden');
        } else {
            $('asyncProgressBarHideCompletedTasks').addClassName('hidden');
        }

        this._updateTaskTitleElement('asyncProgressBarTitleTasksError', countCompleteError);
        this._updateTaskTitleElement('asyncProgressBarTitleTasksComplete', countComplete);
        this._updateTaskTitleElement('asyncProgressBarTitleTasksWarning', countCompleteWarning);
    },

    _updateTaskTitleElement: function(id, count) {
        var element = $(id);
        var renderedCount = element.down(0).textContent || element.down(0).innerText;
        if (count != Number(renderedCount)) {
            element.update('<span>' + count + '</span>');
            if (count > 0) {
                element.removeClassName('hidden');
            } else {
                element.addClassName('hidden');
            }
        }
    },

    _renderItem: function(item) {
        var renderTargetId = this._id + '-item-' + item.getId();
        if (!$(renderTargetId)) {
            $(this._contentAreaId).insert({
                top: '<li id="' + renderTargetId +'" class="async-progress-bar-item"></li>'
            });
        }
        item.setRenderTarget(renderTargetId);
        item.setLocale(this.getLocale());
        item.setProgressBarElement(this);
        item.render();
    },

    _addEvents: function($super) {
        $super();

        var context = this;
        $('asyncProgressBarControlHide').observe('click', function() {
            context.toggle();
        });
        $('asyncProgressBarControlShow').observe('click', function() {
            context.toggle();
        });
        $('asyncProgressBarHideCompletedTasks').observe('click', function() {
            context.removeCompletedTask();
        });
    }
});

Jsw.Task.ProgressBar.Item = Class.create(Jsw.Component, {

    STATUS_COMPLETE: 'done',
    STATUS_ERROR: 'error',
    STATUS_RUNNING: 'started',

    _initConfiguration: function($super, config) {
        $super(config);

        this._id = this._getConfigParam('id', '');
        this._type = this._getConfigParam('type', '');
        this._status = this._getConfigParam('status', '');
        this._errors = this._getConfigParam('errors', []);
        this._isRefreshLinkEnabled = this._getConfigParam('isRefreshLinkEnabled', false);
        this._progressTitle = this._getConfigParam('progressTitle', '');
        this._progressValue = this._getConfigParam('progressValue', 0);
        this._progressBarElement = {};
    },

    getTargetElement: function() {
        return $(this.getRenderTarget());
    },

    render: function() {
        var element = this.getTargetElement();

        if (this.isComplete()) {
            element.addClassName('async-progress-bar-item-complete');
            element.update(
                '<div class="async-progress-bar-item-wrap">' +
                    '<div class="async-progress-bar-item-msg">' +
                        '<a id="asyncProgressBarItemClose' + this.getId() + '" href="#"><span class="close"></span></a>' +
                        '<div class="async-progress-bar-item-msg-body">' + this.getProgressTitle() + this.getRefreshLink() + '</div>' +
                    '</div>' +
                '</div>'
            );
        } else if (this.isCompleteWithWarning()) {
            element.addClassName('async-progress-bar-item-warning');
            this._updateElementWithErrors(element);
        } else if (this.isCompleteWithError()) {
            element.addClassName('async-progress-bar-item-error');
            this._updateElementWithErrors(element);
        } else {
            element.update(
                '<div class="async-progress-bar-item-wrap">' +
                    '<div class="async-progress-bar-item-heading">' +
                        '<span class="async-progress-bar-item-control"><a id="asyncProgressBarItemCancel' + this.getId() + '" href="#">' + this._locale.cancel + '</a></span>' +
                        '<span class="async-progress-bar-item-title">' + this.getProgressTitle() + '</span>' +
                    '</div>' +
                    '<span class="gauge-wrapper" title="">' +
                        '<span class="gauge-container"><span id="" class="gauge-filled" style="width: ' + this.getProgressValue() + '%;"></span></span>' +
                    '</span>' +
                    '<div class="async-progress-bar-item-footer">' + this.getProgressValue() + '%</div>' +
                '</div>'
            );
        }

        this._addEvents();
        this._addTooltips();
    },

    setProgressBarElement: function(element) {
        this._progressBarElement = element;
    },

    getProgressBarElement: function() {
        return this._progressBarElement;
    },

    setLocale: function(locale) {
        this._locale = locale;
    },

    getProgressTitle: function() {
        return this._progressTitle;
    },

    getProgressValue: function() {
        return this._progressValue;
    },

    getStatus: function() {
        return this._status;
    },

    getRefreshLink: function() {
        return (this._isRefreshLinkEnabled) ? '<div><a href="#" id="refreshLink' + this.getId() + '">' + this._locale.refresh + '</a></div>' : '';
    },

    hasErrors: function() {
        return this._errors.length > 0;
    },

    getErrors: function() {
        return this._errors;
    },

    getErrorsString: function() {
        var messages = '';
        var errors = this.getErrors();
        for (var i=0; i<errors.length; i++) {
            messages += '<li>' + errors[i].escapeHTML() + '</li>';
        }

        return messages;
    },

    isComplete: function() {
        return this.STATUS_COMPLETE == this._status && !this.hasErrors();
    },

    isCompleteWithWarning: function() {
        return this.STATUS_COMPLETE == this._status && this.hasErrors();
    },

    isRunning: function() {
        return this.STATUS_RUNNING == this._status;
    },

    isCompleteWithError: function() {
        return this.STATUS_ERROR == this._status;
    },

    remove: function() {
        var context = this;
        new Ajax.Request(Jsw.prepareUrl('/task/task-remove/'), {
            method: 'post',
            parameters: {'ids[]': [this.getId()]},
            onSuccess: function(response) {
                context.getProgressBarElement().removeItemsByIds([context.getId()]);
            }
        });
    },

    _updateElementWithErrors: function(element) {
        element.update(
            '<div class="async-progress-bar-item-wrap">' +
                '<div class="async-progress-bar-item-msg">' +
                '<a id="asyncProgressBarItemClose' + this.getId() + '" href="#"><span class="close"></span></a>' +
                    '<div class="async-progress-bar-item-msg-body">' +
                        this.getProgressTitle() +
                        '<div class="error-details">' +
                            '<ul>' +
                                this.getErrorsString() +
                            '</ul>' +
                        '</div>' +
                        this.getRefreshLink() +
                    '</div>' +
                '</div>' +
            '</div>'
        );
    },

    _addEvents: function() {

        var context = this;
        var buttonClose = $('asyncProgressBarItemClose' + this.getId());
        var buttonCancel = $('asyncProgressBarItemCancel' + this.getId());
        var buttonRefresh = $('refreshLink' + this.getId());

        if (buttonClose) {
            buttonClose.observe('click', function() {
                context.remove();
            });
        }
        if (buttonCancel) {
            buttonCancel.observe('click', function() {
                context.remove();
            });
        }
        if (buttonRefresh) {
            buttonRefresh.observe('click', function(event) {
                Jsw.redirect(Jsw.prepareUrl(window.location.pathname));
                event.preventDefault();
            });
        }
    }
});