// Copyright 1999-2014. Parallels IP Holdings GmbH. All Rights Reserved.
Jsw.namespace('Jsw.Task.StatusMessage');

Jsw.Task.StatusMessage = Class.create(Jsw.Component, {

    //see common/php/plib/Db/Table/Row/LongTask.php
    STATUS_NOT_STARTED: 'not_started',
    STATUS_STARTED: 'started',
    STATUS_DONE: 'done',
    STATUS_CANCELED: 'canceled',
    STATUS_ERROR: 'error',

    _failedStatusUpdates: 0,
    _maxFailedStatusUpdates: 10,

    _initConfiguration: function($super, config) {
        $super(config);

        this._id = this._getConfigParam('id');
        this._onStatusUpdate = this._getConfigParam('onStatusUpdate', null);
        this._onTaskDone = this._getConfigParam('onDone', null);
        this._onTaskCancel = this._getConfigParam('onCancel', null);
        this._onTaskError = this._getConfigParam('onError', null);
        this._renderMessage = this._getConfigParam('renderMessage', function(status) {
            if (!status.message) {
                return 'Internal error.';
            }
            return status.message;
        });

        this._statusUrl = this._getConfigParam('statusUrl', Jsw.prepareUrl('/task/status/taskId/'));
        this._updateInterval = this._getConfigParam('updateInterval', 60);
        this._autoRender = true;
    },

    _showProgress: function(status) {
        var statusClass = 'msg-progress';
        if (this.STATUS_DONE == status.status) {
            statusClass = 'msg-info';
        } else
        if (this.STATUS_CANCELED == status.status) {
            statusClass = 'msg-warning';
        } else
        if (this.STATUS_ERROR == status.status) {
            statusClass = 'msg-error';
        }

        var message = this._renderMessage(status);

        if (this.STATUS_DONE != status.status && status.progress) {
            if (this._getConfigParam('percent', true)) {
                message = message + ': <span id="backupProgress">' + status.progress + '</span>%';
            }

            if (this._getConfigParam('gauge', true)) {
                message = '<table class="progress-container" cellspacing="0" width="100%"><tr>' +
                    '<td class="progress-indicator">' +
                    '<table cellspacing="0" width="100%"><tr>' +
                    '<td class="progress-status">' + message + '</td>' +
                    '<td>' +
                    '<span title="" class="gauge-wrapper f5 not_empty">' +
                    '<span class="gauge-container"><span id="backupProgressGauge" style="width: ' + status.progress + '%;" class="gauge-filled"></span></span>' +
                    '</span>' +
                    '</td>' +
                    '</tr></table>' +
                    '</td>' +
                    '</tr></table>';
            }
        }

        var content = '<div class="msg-box ' + statusClass + '">' +
            '<div><div><div><div><div>' +
            '<div class="msg-content">' + message +
            '</div>' +
            '</div></div></div></div></div>' +
            '</div>';

        this._componentElement.update(content);
    },

    _addEvents: function($super) {
        $super();
        this._processStatus(this._getConfigParam('status', null));
    },

    _scheduleUpdateStatus: function () {
        this._updateStatus.bind(this).delay(this._updateInterval);
    },

    _updateStatus: function () {
        new Ajax.Request(this._statusUrl + this._id, {
            method: 'get',
            onSuccess: this._onStatusUpdateSuccess.bind(this),
            onFailure: this._onStatusUpdateFailure.bind(this)
        });
    },

    _onStatusUpdateSuccess: function(transport) {
        var status = transport.responseText.evalJSON();
        this._failedStatusUpdates = 0,
        this._processStatus(status);
        if (this._onStatusUpdate) {
            this._onStatusUpdate(status);
        }
    },

    _onStatusUpdateFailure: function(transport) {
        this._failedStatusUpdates++;
        if (this._failedStatusUpdates < this._maxFailedStatusUpdates) {
            this._scheduleUpdateStatus();
        }
    },

    _processStatus: function(status) {
        if (!status) {
            this._showLoading();
            this._updateStatus();
        } else if (this.STATUS_DONE == status.status) {
            if (this._onTaskDone) {
                this._onTaskDone(status);
            }
            this._showFinished(status);
        } else if (this.STATUS_CANCELED == status.status) {
            if (this._onTaskCancel) {
                this._onTaskCancel(status);
            }
            this._showFinished(status);
        } else if (this.STATUS_ERROR == status.status) {
            if (this._onTaskError) {
                this._onTaskError(status);
            }
            this._showError(status);
        } else if (this.STATUS_STARTED == status.status || this.STATUS_NOT_STARTED == status.status) {
            this._showWorking(status);
            this._scheduleUpdateStatus();
        } else {
            this._showNone(status);
        }
    },

    _showFinished: function(status) {
        this._showProgress(status);
    },

    _showWorking: function(status) {
        this._showProgress(status);
    },

    _showError: function(status) {
        this._showProgress(status);
    },

    _showLoading: function() {
    },

    _showNone: function(status) {
    }
});
