// Copyright 1999-2014. Parallels IP Holdings GmbH. All Rights Reserved.
Jsw.namespace('Jsw.WordpressPopupManager');

Jsw.WordpressPopupManager.manageThemes = function(ids, name, locale, dashboardUrl, returnUrl) {
    new Jsw.WordpressPopupManager.ManageThemes({
        id: 'manageThemes',
        cls: 'popup-panel popup-panel-wordpress',
        prepareUrl: Jsw.prepareUrl('/wordpress/installed-themes/'),
        handlerUrl: Jsw.prepareUrl('/wordpress/update-installed-themes/'),
        returnUrl: returnUrl,
        locale: locale,
        ids: ids,
        name: name,
        dashboardUrl: dashboardUrl
    });
}

Jsw.WordpressPopupManager.managePlugins = function(ids, name, locale, dashboardUrl, returnUrl) {
    new Jsw.WordpressPopupManager.ManagePlugins({
        id: 'managePlugins',
        cls: 'popup-panel popup-panel-wordpress',
        prepareUrl: Jsw.prepareUrl('/wordpress/installed-plugins/'),
        handlerUrl: Jsw.prepareUrl('/wordpress/update-installed-plugins/'),
        returnUrl: returnUrl,
        locale: locale,
        ids: ids,
        name: name,
        dashboardUrl: dashboardUrl
    });
}

Jsw.WordpressPopupManager.securityScan = function(ids, name, locale) {
    new Jsw.WordpressPopupManager.CheckWordpressInstances({
        id: 'securityScan',
        cls: 'popup-panel popup-panel-wordpress popup-panel-security',
        prepareUrl: Jsw.prepareUrl('/wordpress/check/'),
        handlerUrl: Jsw.prepareUrl('/wordpress/check/resolve/1/'),
        rollbackUrl: Jsw.prepareUrl('/wordpress/check/rollback/1/'),
        locale: locale,
        ids: ids,
        name: name,
        sendButtonId: 'resolveButton',
        cancelButtonId: 'cancelButton'
    });
}

Jsw.WordpressPopupManager.CommonPopupForm = Class.create(Jsw.ConfirmationPopupManager.PopupForm, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._name = this._getConfigParam('name', '');
        this._dashboardUrl = this._getConfigParam('dashboardUrl', null);
        this._returnUrl = this._getConfigParam('returnUrl', null);
    },

    getName: function() {
        return this._name;
    },

    _getUrl: function() {
        return this._dashboardUrl;
    },

    _setHintToItems: function() {
        $$('span.hint.hint-info').each(function(hintElement) {
            var textContent  = hintElement.up('td').down('.jsItemTitle').textContent || hintElement.up('td').down('.jsItemTitle').innerText;
            var tooltip = new Jsw.AjaxTooltip.Instance({
                title: textContent.escapeHTML(),
                target: hintElement.id.escapeHTML()
            });
            tooltip.setText(hintElement.down('.tooltipData').innerHTML.replace(/\n/, '<br>'));
        });
    },

    _setTitle: function() {
        if (1 === this._ids.length) {
            this.setTitle(this._locale.titleSingle.replace('%%url%%', this.getName().escapeHTML()));
        } else {
            this.setTitle(this._locale.title.replace('%%number%%', this._ids.length));
        }
    }

});

Jsw.WordpressPopupManager.CheckWordpressInstances = Class.create(Jsw.WordpressPopupManager.CommonPopupForm, {

    _initConfiguration: function($super, config) {
        $super(config);
        
        this._formListAdditionalCheckersAreaId = this._id + '-form-list-additional-checkers';

        this._rollbackUrl = this._getConfigParam('rollbackUrl', null);
    },

    _onSuccessPreparePopup: function(transport) {

        this._clearMessages();

        try {
            this._response = transport.responseText.evalJSON();
            if ('success' != this._response.status) {
                if (this._response.redirect) {
                    Jsw.redirect(this._response.redirect);
                    return;
                }
                this._addErrorMessage(this._response.message.replace(/\n/, '<br>'));
            }

            if (0 >= this._response.checkers.length) {
                $(this._formBoxAreaId).update('');
                $(this._formListAreaId).update( '<p>' + this._locale.checkersNotFound + '</p>');
                return
            }

            $(this._formBoxAreaId).update('');
            $(this._formListAreaId).update(
                '<table cellspacing="0" width="100%">' +
                    '<tbody id="' + this._formListItemsAreaId + '">' +
                    '</tbody>' +
                '</table>' +
                (this._response.hasAdditionalCheckers ? '<br/>' +
                '<p>' + this._locale.additionalCheckersDescription + '</p>' +
                    '<table cellspacing="0" width="100%">' +
                    '<tbody id="' + this._formListAdditionalCheckersAreaId + '">' +
                    '</tbody>' +
                '</table>' : '')
            );

            this._response.checkers.each(function(checker) {
                this._addChecker(checker);
            }, this);

            this._onAllSuccess();

        } catch (e) {
            this._addErrorMessage(e.message);
            this._addErrorMessage(
                'Internal error: ' + transport.responseText
            );
        }
    },

    _getLoadingIndicatorItems: function(checkers) {
        var items = [];
        $$('.checkers').each(function(row) {
            if (row.checked) {
                items[row.title] = row.title;
            }
        });

        return items;
    },

    _addLoadingIndicator: function(checkers) {
        for (var checker in checkers) {
            if (!checkers.hasOwnProperty(checker)) continue;
            $('checker-image-' + checkers[checker]).src = Jsw.skinUrl + '/icons/16/plesk/indicator.gif';
        }
    },

    _addChecker: function(checker) {
        this._itemClass = ('odd' == this._itemClass) ? 'even' : 'odd';

        var image = (checker.resolved)
            ? '<img id="checker-image-' + checker.id + '" src="' + Jsw.skinUrl + '/icons/16/plesk/ok.png" title="">'
            : '<img id="checker-image-' + checker.id + '" src="' + Jsw.skinUrl + '/icons/16/plesk/' + (checker.isAdditional ? 'att.png' : 'warning.png') + '" title="' + checker.mainMessage.escapeHTML() + '">';
        var warning = ('' != checker.warningMessage)
            ? '<div class="error-hint hint-sub">' + checker.warningMessage.escapeHTML() + '</div>'
            : '';
        var hint = ('' != checker.hintMessage)
            ? '<span class="hint hint-info" id="hint_' + checker.id + '"> (?)<span class="tooltipData">' + checker.hintMessage.escapeHTML() + '</span></span>'
            : '';
        var itemHtml = '' +
            '<tr class="' + this._itemClass + '">' +
                '<td>' +
                    '<div class="b-indent">' +
                        '<span class="b-indent-icon">' +
                            image +
                        '</span>' +
                        '<span class="jsItemTitle">' +
                            checker.title.escapeHTML() +
                        '</span>' +
                        hint +
                        warning +
                    '</div>' +
                '</td>' +
                '<td class="last t-r">' + this._getCheckerHtml(checker) + '</td>' +
            '</tr>';
        
        if (checker.isAdditional) {
        	$(this._formListAdditionalCheckersAreaId).insert({bottom : itemHtml});
        } else {
        	$(this._formListItemsAreaId).insert({bottom : itemHtml});
        }
        
        this._addRollbackButton(checker);
    },

    _getCheckerHtml: function(checker) {
        var checked = (checker.chosenByDefault) ? 'checked' : '';
        var checkerHtml = (checker.resolved)
            ? '<span id="checker-text-' + checker.id + '">' + this._locale.ok + '</span>'
            : '<input class="checkers" type="checkbox" name="checkers[' + checker.id + ']" id="checker-checkbox-' + checker.id + '" ' + checked + ' title="' + checker.id + '">';

        return checkerHtml;
    },

    _onSuccessResponse: function(response) {
        this.enable();

        if (0 >= response.checkers.length) {
            return;
        }

        response.checkers.each(function(checker) {
            if (checker.resolved) {
                if ($('checker-checkbox-' + checker.id)) {
                	$('checker-checkbox-' + checker.id).up().update(this._getCheckerHtml(checker));
                }
                $('checker-image-' + checker.id).src = Jsw.skinUrl + '/icons/16/plesk/ok.png';
                this._addRollbackButton(checker);
            } else {
                if ($('checker-text-' + checker.id)) {
                    $('checker-text-' + checker.id).up().update(this._getCheckerHtml(checker));
                }
                $('checker-image-' + checker.id).src = Jsw.skinUrl + '/icons/16/plesk/' + (checker.isAdditional ? 'att.png' : 'warning.png');
            }
            this._onAllSuccess();
        }, this);
    },

    _getHeadDescription: function() {
        return '<p id="' + this._formDescriptionId + '-headDescription">' + this._locale.headDescription + '</p>';
    },

    _getBottomDescription: function() {
        return '<p style="display: none" align="center" id="' + this._formDescriptionId + '-bottomDescription">' + this._locale.bottomDescription + '</p>';
    },

    _getAdditionalParams: function(params) {
        $$('.checkers').each(function(row){
            params.set('checkers[' + row.title + ']', row.checked);
        });
        return params;
    },

    _onAllSuccess: function() {
        if (0 >= $$('.checkers').length) {
            $(this._sendButtonId).hide();
            $(this._formDescriptionId + '-bottomDescription').show();
        } else {
            $(this._sendButtonId).show();
            $(this._formDescriptionId + '-bottomDescription').hide();
        }
        this._setHintToItems();
    },

    _onRollbackSubmit: function(event) {
        var rollbackChecker = $(event.target).up().readAttribute('data');

        var params = $H();
        var count = 0;
        this._ids.each(function(id) {
            params.set('ids[' + count + ']', id);
            count++;
        });

        params.set('checkers[' + rollbackChecker + ']', true);

        this.disable();
        this._addLoadingIndicator([rollbackChecker]);
        new Ajax.Request(
            this._rollbackUrl,
            {
                method: 'post',
                parameters: params,
                onSuccess: this._onSuccess.bind(this),
                onException: this._onException.bind(this)
            }
        );
    },

    _addRollbackButton: function(checker) {
        if (checker.hasRollback && checker.resolved) {
            $('checker-text-' + checker.id).insert({before: this._createButton(this._locale.rollback, this._onRollbackSubmit, false, false, {'data': checker.id})});
        } else if (!checker.hasRollback && !checker.resolved) {
        	$('checker-checkbox-' + checker.id).insert({before: '<span class="minor">' + this._locale.unableRollback + '</span> '});
        }
    }
});

Jsw.WordpressPopupManager.ManageItems = Class.create(Jsw.WordpressPopupManager.CommonPopupForm, {

    _onSuccessPreparePopup: function(transport) {
        this._clearMessages();
        this._response = transport.responseText.evalJSON();
        if ('success' != this._response.status) {
            if (this._response.redirect) {
                Jsw.redirect(this._response.redirect);
                return;
            }
            this._addErrorMessage(this._response.message.replace(/\n/, '<br>'));
        }
        this._renderTable(this._response.items);
    },

    _getHeadDescription: function() {
        return '<p id="' + this._formDescriptionId + '-headDescription">' + this._locale.headDescription + '</p>';
    },

    _getRowHtml: function(items, totalInstances) {
        return ''
    },

    _addHandlers: function(twoStateControl) {
        return false;
    },

    _getUninstallUrl: function() {
        return '';
    },

    _addUninstallHandlers: function() {
        var context = this;

        $$('a.jsRemoveItem').each(function(link) {
            link.observe('click', function() {
                Jsw.messageBox.show({
                    type: Jsw.messageBox.TYPE_YESNO,
                    buttonTitles: {
                        yes: context._locale.removeDialogButtonYes,
                        no: context._locale.removeDialogButtonNo
                    },
                    text: context._locale.confirmUninstall,
                    subtype: 'delete',
                    onYesClick: function() {
                        Jsw.redirectPost(Jsw.prepareUrl(context._getUninstallUrl()), {
                            'ids': context._ids,
                            'item-id': link.readAttribute('data-item-id'),
                            'returnUrl': context._returnUrl
                        });
                    }
                });
            });
        })
    },

    _getUpdateUrl: function() {
        return '';
    },

    _addUpdateHandlers: function() {
        var context = this;

        $$('a.jsUpdateItem').each(function(link) {
            link.observe('click', function() {
                Jsw.redirectPost(Jsw.prepareUrl(context._getUpdateUrl()), {
                    'ids': context._ids,
                    'item-id': link.readAttribute('data-item-id'),
                    'returnUrl': context._returnUrl
                });
            })
        })
    },


    _preRenderTable: function(items) {
        $(this._formBoxAreaId).update('');
        if (0 == items.length) {
            $(this._formListAreaId).setStyle({ overflow: 'visible' });
            $(this._formListAreaId).update(this._locale.noItems);
            return;
        }
        $(this._formListAreaId).setStyle({ overflow: 'auto', height: '350px' });
    },

    _renderTableContent: function(items) {

        var totalInstances = this._ids.length;
        $(this._formListAreaId).update(
            '<table cellspacing="0" width="100%">' +
                '<thead>' +
                    '<tr>' +
                        '<th class="first">' + this._locale.fieldName + '</th>' +
                        '<th>' + this._locale.fieldActivate + '</th>' +
                        (totalInstances > 1 ? ('<th>' + this._locale.fieldTotal + '</th>') : '') +
                        '<th class="action-icon-set t-r last"></th>' +
                    '</tr>' +
                '</thead>' +
                '<tbody id="' + this._formListItemsAreaId + '">' +
                '</tbody>' +
            '</table>'
        );
        var rowsHtml = this._getRowHtml(items, totalInstances);
        $(this._formListItemsAreaId).update(rowsHtml);

        this._addHandlers(1 == totalInstances);
        this._addUpdateHandlers();
        this._addUninstallHandlers();
    },

    _postRenderTable: function(items) {
        this._setHintToItems();
    },

    _renderTable: function(items) {
        this._preRenderTable(items);
        this._renderTableContent(items);
        this._postRenderTable(items);
    },

    _getActionsBoxHtml: function() {
        var content = '<div class="actions-box">' +
                '<div class="objects-toolbar clearfix">' +
                    '<a class="s-btn sb-install" href="#"><i><i><i><span>' +
                        this._locale.installItem +
                    '</span></i></i></i></a>' +
                '</div>' +
            '</div>';

        return content;
    },

    _getUpdatesInfo: function(totalInstances, item) {
        if (0 == item._updatesTotal) {
            return '';
        }

        var updatesLabel = (1 == totalInstances) ?
            this._locale.updateAvailableSingle.replace('%%version%%', item._updateVersion.escapeHTML()) :
            this._locale.updateAvailable;
        var updatesLink = (1 == totalInstances) ?
            this._locale.linkUpdateSingle :
            this._locale.linkUpdate.replace('%%number%%', item._updatesTotal);

        var updatesInfo =
            '<div class="hint-sub hint-attention update-available">' + updatesLabel + '&nbsp;' +
                '<a href="#" class="jsUpdateItem" data-item-id="' + item._id + '">' + updatesLink + '</a>' +
            '</div>';

        return updatesInfo;
    },

    _getEscapedItem: function(item) {
        var escapedItem = {};
        for(var prop in item) {
            if (!item.hasOwnProperty(prop)) {
                continue;
            }
            escapedItem[prop] = String(item[prop]).escapeHTML();
        }

        return escapedItem;
    }
});

Jsw.WordpressPopupManager.ManagePlugins = Class.create(Jsw.WordpressPopupManager.ManageItems, {

    _getBottomDescription: function() {
        if (1 === this._ids.length) {
            return this._getUrl() ? ('<a href="' + this._getUrl() + '" target="_blank">' + this._locale.managePluginsInsideWordpress + '</a>') : '';
        } else {
            return '';
        }
    },

    _getRowHtml: function(items, totalInstances) {
        var rowsHtml = '';
        var oddRow = true;

        items.each(function(plugin) {
            var pluginEscaped = this._getEscapedItem(plugin);
            var stateLabel = '<label for="' + pluginEscaped._id + '_active">' + this._locale.activeState + '</label>';
            rowsHtml += '<tr class="' + (oddRow ? 'odd' : 'even') + '">' +
                    '<td class="first">' +
                        '<span class="jsItemTitle ' + ('active' != pluginEscaped._overallStatus ? 'minor' : '') +  '">' +
                            pluginEscaped.title + ' ' + pluginEscaped.version +
                        '</span>' +
                        '<span class="hint hint-info" id="hint_' + pluginEscaped._id + '"> (?)' +
                            '<span class="tooltipData">' + pluginEscaped.description + '</span>' +
                        '</span>' +
                        this._getUpdatesInfo(totalInstances, pluginEscaped) +
                    '</td>' +
                    '<td class="nowrap">' +
                        '<label for="' + pluginEscaped._id + '" class="e-checkbox ' + this._getCheckboxCssClass(totalInstances, pluginEscaped) + '">' +
                        '<input type="checkbox" class="checkbox" id="' + pluginEscaped._id + '" name="' + pluginEscaped._id + '" ' + this._getCheckboxStateAttribute(totalInstances, pluginEscaped) + '> ' +
                        this._getStateLabels() +
                        '</label>' +
                    '</td>' +
                    (totalInstances > 1 ? ('<td>' + pluginEscaped._total + '</td>') : '') +
                    '<td class="action-icon-set t-r last min">' +
                        '<a href="#" class="jsRemoveItem" data-item-id="' + pluginEscaped._id + '">' +
                            '<img src="' + Jsw.skinUrl + '/icons/16/plesk/uninstall-app.png">' +
                        '</a>' +
                    '</td>' +
                '</tr>';

            oddRow = !oddRow;
        }, this);

        return rowsHtml;
    },

    _getCheckboxCssClass: function(totalInstances, plugin) {
        if (1 == totalInstances) {
            return ('active' == plugin.status) ? 'e-checkbox-on' : 'e-checkbox-off';
        } else {
            return 'e-checkbox-pre';
        }
    },

    _getCheckboxStateAttribute: function(totalInstances, plugin) {
        if (1 == totalInstances && ('active' == plugin.status)) {
            return 'checked="checked"';
        }

        return '';
    },

    _getAdditionalParams: function(params) {
        $(this._formListAreaId).select('.e-checkbox').each(function(label) {
            if (label.hasClassName('e-checkbox-on')) {
                params.set('items[' + label.readAttribute('for') + ']', 1);
            } else if (label.hasClassName('e-checkbox-off')) {
                params.set('items[' + label.readAttribute('for') + ']', 0);
            }
        });

        params.set('returnUrl', this._returnUrl);

        return params;
    },

    _getStateLabels: function() {
        var labels = '<span class="e-checkbox-text e-checkbox-text-pre" style="width: 83px;">' + this._locale.stateDoNotChange + '</span>' +
            '<span class="e-checkbox-text e-checkbox-text-on" style="width: 83px;">' + this._locale.stateOn + '</span>' +
            '<span class="e-checkbox-text e-checkbox-text-off" style="width: 83px;">' + this._locale.stateOff + '</span>';
        return labels;
    },

    _addHandlers: function(twoStateControl) {
        $$('.e-checkbox').each(function(el) {
            var text = [el.down('.e-checkbox-text-off').getWidth(), el.down('.e-checkbox-text-on').getWidth(), el.down('.e-checkbox-text-pre').getWidth()];
            var textMax = text.max();
            var checkboxElement = el.down('.checkbox');

            checkboxElement.observe('click', function(e) {
                if (twoStateControl) {
                    if (checkboxElement.checked) {
                        el.addClassName('e-checkbox-on');
                        el.removeClassName('e-checkbox-off');
                        el.down('.e-checkbox-text-on').setStyle({width: textMax + 'px'});
                    } else {
                        el.addClassName('e-checkbox-off');
                        el.removeClassName('e-checkbox-on');
                        el.down('.e-checkbox-text-off').setStyle({width: textMax + 'px'});
                    }
                } else {
                    if (el.hasClassName('e-checkbox-pre')) {
                        el.addClassName('e-checkbox-on');
                        el.removeClassName('e-checkbox-pre');
                        el.down('.checkbox').checked = true;
                        el.down('.e-checkbox-text-on').setStyle({width: textMax + 'px'});
                    } else if (el.hasClassName('e-checkbox-on')) {
                        el.addClassName('e-checkbox-off');
                        el.removeClassName('e-checkbox-on');
                        el.down('.checkbox').checked = false;
                        el.down('.e-checkbox-text-off').setStyle({width: textMax + 'px'});
                    } else if (el.hasClassName('e-checkbox-off')) {
                        el.addClassName('e-checkbox-pre');
                        el.removeClassName('e-checkbox-off');
                        el.down('.checkbox').checked = false;
                        el.down('.e-checkbox-text-pre').setStyle({width: textMax + 'px'});
                    }
                }
            });
        });

        var context = this;
        $$('#' + this._contentAreaId + ' .sb-install').first().observe('click', function() {
            var locale = context._locale;
            locale.buttonOk = locale.buttonInstall;
            new Jsw.WordpressPopupManager.InstallPlugins({
                id: 'installPlugins',
                cls: 'popup-panel popup-panel-wordpress',
                prepareUrl: Jsw.prepareUrl('/wordpress/available-plugins/'),
                handlerUrl: Jsw.prepareUrl('/wordpress/install-plugins/'),
                locale: locale,
                ids: context._ids,
                name: context.getName(),
                returnUrl: (context._returnUrl ? context._returnUrl : '/wordpress/list')
            });
        });
    },

    _getUninstallUrl: function() {
        return '/wordpress/uninstall-plugin';
    },

    _getUpdateUrl: function() {
        return '/wordpress/update-plugin';
    }
});

Jsw.WordpressPopupManager.ManageThemes = Class.create(Jsw.WordpressPopupManager.ManageItems, {

    _getHeadDescription: function() {
        return '<p id="' + this._formDescriptionId + '-headDescription">' + this._locale.headDescription + '</p>';
    },

    _getBottomDescription: function() {
        if (1 === this._ids.length) {
            return this._getUrl() ? ('<a href="' + this._getUrl() + '" target="_blank">' + this._locale.manageThemesInsideWordpress + '</a>') : '';
        } else {
            return '';
        }
    },

    _getRadioStateAttribute: function(totalInstances, theme) {
        if (1 == totalInstances && ('active' == theme.status)) {
            return 'checked="checked"';
        }

        return '';
    },

    _getRowHtml: function(items, totalInstances) {
        var rowsHtml = '';
        var oddRow = true;
        var activeText = totalInstances > 1 ? this._locale.setActive : this._locale.active;

        items.each(function(theme) {
            var themeEscaped = this._getEscapedItem(theme);
            rowsHtml += '<tr class="' + (oddRow ? 'odd' : 'even') + '">' +
                    '<td class="first">' +
                        '<span class="jsItemTitle">' +
                            themeEscaped.title + ' ' + themeEscaped.version +
                        '</span>' +
                        '<span class="hint hint-info" id="hint_' + themeEscaped._id + '"> (?)' +
                            '<span class="tooltipData">' + themeEscaped.description + '</span>' +
                        '</span>' +
                        this._getUpdatesInfo(totalInstances, themeEscaped) +
                    '</td>' +
                    '<td class="nowrap">' +
                        '<label for="' + themeEscaped._id + '"><input class="radio theme-active" type="radio" ' + this._getRadioStateAttribute(totalInstances, themeEscaped) + ' name="themes_active" id="' + themeEscaped._id + '"> ' + activeText + '</label>' +
                    '</td>' +
                    (totalInstances > 1 ? ('<td>' + themeEscaped._total + '</td>') : '') +
                    '<td class="action-icon-set t-r last min">' +
                        '<a href="#" class="jsRemoveItem" data-item-id="' + themeEscaped._id + '">' +
                            '<img src="' + Jsw.skinUrl + '/icons/16/plesk/uninstall-app.png">' +
                        '</a>' +
                    '</td>' +
                '</tr>';

            oddRow = !oddRow;
        }, this);
        if (totalInstances > 1) {
            rowsHtml += '<tfoot>' +
                    '<tr>' +
                        '<td class="first"></td>' +
                        '<td>' +
                            '<label for="themes_active"><input class="radio" type="radio" checked="checked" name="themes_active" id="themes_active"> ' + this._locale.setDoNotChange + '</label>' +
                        '</td>' +
                        '<td class="last min nowrap"></td>' +
                    '</tr>' +
                '</tfoot>';
        }


        return rowsHtml;
    },

    _getAdditionalParams: function(params) {
        $(this._formListAreaId).select('.theme-active').each(function(element) {
            if (element.checked) {
                params.set('items[' + element.id + ']', 1);
            }
        });

        params.set('returnUrl', this._returnUrl);

        return params;
    },

    _addHandlers: function(twoStateControl) {
        var context = this;
        $$('#' + this._contentAreaId + ' .sb-install').first().observe('click', function() {
            var locale = context._locale;
            locale.buttonOk = locale.buttonInstall;
            new Jsw.WordpressPopupManager.InstallThemes({
                id: 'installThemes',
                cls: 'popup-panel popup-panel-wordpress',
                prepareUrl: Jsw.prepareUrl('/wordpress/available-themes/'),
                handlerUrl: Jsw.prepareUrl('/wordpress/install-themes/'),
                locale: locale,
                ids: context._ids,
                name: context.getName(),
                returnUrl: (context._returnUrl ? context._returnUrl : '/wordpress/list')
            });
        });

    },

    _getUninstallUrl: function() {
        return '/wordpress/uninstall-theme';
    },

    _getUpdateUrl: function() {
        return '/wordpress/update-theme';
    }
});


Jsw.WordpressPopupManager.InstallItems = Class.create(Jsw.WordpressPopupManager.ManageItems, {

    _setTitle: function() {
        if (1 === this._ids.length) {
            this.setTitle(this._locale.installTitleSingle.replace('%%url%%', this.getName()));
        } else {
            this.setTitle(this._locale.installTitle.replace('%%number%%', this._ids.length));
        }
    },

    _getHeadDescription: function() {
        return '<p id="' + this._formDescriptionId + '-headDescription">' + this._locale.installHeadDescription + '</p>';

    },

    _renderTableContent: function(items) {

        $(this._formListAreaId).update(
            '<table cellspacing="0" width="100%">' +
                '<thead>' +
                    '<tr>' +
                        '<th class="select first"></th>' +
                        '<th class="first">' + this._locale.fieldName + '</th>' +
                        '<th>' + this._locale.fieldVersion + '</th>' +
                        '<th class="last">' + this._locale.fieldRating + '</th>' +
                    '</tr>' +
                '</thead>' +
                '<tbody id="' + this._formListItemsAreaId + '">' +
                '</tbody>' +
            '</table>'
        );

        var rowsHtml = '';
        var oddRow = true;

        items.each(function(item) {
            var itemEscaped = this._getEscapedItem(item);
            var safeDescription = new Element('span');
            safeDescription.innerHTML = itemEscaped.description;
            rowsHtml += '<tr class="' + (oddRow ? 'odd' : 'even') + '">' +
                '<td class="select first"><input type="checkbox" class="checkbox" name="' + itemEscaped.name + '"></td>' +
                    '<td class="first">' +
                        '<span class="jsItemTitle">' +
                            itemEscaped.title +
                        '</span>' +
                        '<span class="hint hint-info" id="hint_' + itemEscaped.name + '"> (?)' +
                            '<span class="tooltipData">' + (safeDescription.textContent || safeDescription.innerText) + '</span>' +
                        '</span>' +
                    '</td>' +
                    '<td>' + itemEscaped.version + '</td>' +
                    '<td class="last min nowrap">' +
                        '<div class="star-holder">' +
                            '<div class="star-rating" style="width: ' + itemEscaped.rating + '%;"></div>' +
                        '</div>' +
                    '</td>' +
                '</tr>';

            oddRow = !oddRow;
        }, this);

        $(this._formListItemsAreaId).update(rowsHtml);
    },

    _getActionsBoxHtml: function() {
        var content = '<div class="actions-box">' +
                '<div class="objects-toolbar clearfix">' +
                    '<div class="quick-search-box">' +
                        '<span class="search-field">' +
                            '<input type="text" value="" id="popupListSearchTerm">' +
                            '<em><span></span></em>' +
                        '</span>' +
                    '</div>' +
                '</div>' +
            '</div>';

        return content;
    },

    _addActionBoxHandlers: function() {
        var context = this;
        var originalPrepareUrl = this._prepareUrl;

        var searchHandler = function() {
            if ($('popupListSearchTerm').value.length < 2) {
                context.renderTryToSearch();
            } else {
                context._prepareUrl = originalPrepareUrl + '?term=' + encodeURIComponent($('popupListSearchTerm').value);
                context.reload();
            }
        }

        $('popupListSearchTerm').observe('keydown', function(event) {
            if (Jsw.keyCode.ENTER == event.keyCode) {
                event.preventDefault();
                searchHandler();
            }
        });

        $$('#' + this._contentAreaId + ' .search-field em').first().observe('click', searchHandler);
    },

    _getAdditionalParams: function(params) {
        $(this._formListAreaId).select('.checkbox').each(function(checkbox) {
            if (checkbox.checked) {
                params.set('items[' + checkbox.name + ']', 1);
            }
        });

        params.set('returnUrl', this._returnUrl);

        if ($(this._contentAreaId + '-activate').checked) {
            params.set('activate', 1);
        }

        return params;
    },

    _renderPreparePopup: function() {
        this.renderTryToSearch();
    },

    renderTryToSearch: function() {
        $(this._formBoxAreaId).update('');
        $(this._formListAreaId).setStyle({ overflow: 'visible' });
        $(this._formListAreaId).update(this._locale.tryToSearch);
    }
});

Jsw.WordpressPopupManager.InstallPlugins = Class.create(Jsw.WordpressPopupManager.InstallItems, {

    _getBottomDescription: function() {
        var checkboxId = this._contentAreaId + '-activate';
        return '<div class="text-center">' +
            '<label for="' + checkboxId + '"><input type="checkbox" id="' + checkboxId + '" checked="checked"> ' + this._locale.activateAfterInstall + '</label>' +
        '</div>';
    },

    _getAdditionalParams: function(params) {
        $(this._formListAreaId).select('.checkbox').each(function(checkbox) {
            if (checkbox.checked) {
                params.set('items[' + checkbox.name + ']', 1);
            }
        });

        params.set('returnUrl', this._returnUrl);

        if ($(this._contentAreaId + '-activate').checked) {
            params.set('activate', 1);
        }

        return params;
    }
});

Jsw.WordpressPopupManager.InstallThemes = Class.create(Jsw.WordpressPopupManager.InstallItems, {

    _getBottomDescription: function() {
        return '';
    },

    _getAdditionalParams: function(params) {
        $(this._formListAreaId).select('.checkbox').each(function(checkbox) {
            if (checkbox.checked) {
                params.set('items[' + checkbox.name + ']', 1);
            }
        });

        params.set('returnUrl', this._returnUrl);

        params.set('activate', 0);

        return params;
    }
});