# Copyright 1999-2014. Parallels IP Holdings GmbH. All Rights Reserved.
"""
SplitFile provides minimal file-like interface which presents
concatenation of passed files.

Opener are passed to SplitFile to avoid excessive open files floating
around.

Opener must return consequent parts of backup until no more parts
found, then it must return None
"""

class SplitFile(object):
    def __init__(self, BackupPartsFinder):
        self.finder = BackupPartsFinder
        self.curFileObj = None
        self.__openNext()

    def read(self, size):
        if not self.curFileObj:
            return ""

        read_ = self.curFileObj.read(size)
        if len(read_) < size:
            self.__openNext()
            return read_ + self.read(size - len(read_))
        return read_

    def close(self):
        if self.curFileObj:
            self.curFileObj.close()
            self.curFileObj = None

    def __openNext(self):
        if self.curFileObj:
            self.curFileObj.close()

        self.backup = self.finder()
        try:
            self.curFileObj = self.backup.fh()
        except:
            self.curFileObj = None

# Local Variables:
# mode: python
# indent-tabs-mode: nil
# tab-width: 4
# End:
