# Copyright 1999-2014. Parallels IP Holdings GmbH. All Rights Reserved.
import libxml2, logging


_logger = logging.getLogger("api.util.validator")


class ValidatorErrorHandler:
    def __init__(self):
        self.errors = []
    def handler(self, msg, data):
        self.errors.append(msg)
    def __unicode__(self):
        return u"\n".join(self.errors)


class Validator:
    def __init__(self, schemafile):
        self.ctxt_parser = libxml2.schemaNewParserCtxt(schemafile)
        self.ctxt_schema = self.ctxt_parser.schemaParse()

    def __del__(self):
        libxml2.schemaCleanupTypes()
        libxml2.cleanupParser()

    def do(self, xmlstr):
        ctxt_valid = self.ctxt_schema.schemaNewValidCtxt()
        erh = ValidatorErrorHandler()

        ctxt_valid.setValidityErrorHandler(erh.handler, erh.handler)

        doc = libxml2.parseDoc(xmlstr)
        ret = doc.schemaValidateDoc(ctxt_valid)
        doc.freeDoc()
        del ctxt_valid
        del doc

        if ret != 0:
            _logger.error("Validation failed:\n" + unicode(erh))
        else:
            _logger.info("Packet succesfully validated.")

        return (ret, unicode(erh))

# Local Variables:
# mode: python
# indent-tabs-mode: nil
# tab-width: 4
# End:
