# Copyright 1999-2014. Parallels IP Holdings GmbH. All Rights Reserved.
import sys, shutil

import log, logging, dirutil, plesk_config, os

_logger = logging.getLogger("api.agent_copy")

def agents_dir():
	return os.path.join(plesk_config.get("PRODUCT_ROOT_D"), "PMM", "agents")

def usage():
	print "Usage: agent-copy <destination-directory>"

def copy_dir_files(destBaseDir, srcDir, files):
	destDir = os.path.normpath(os.path.join(destBaseDir, srcDir))

	_logger.debug("=> " + srcDir)
	if not os.path.exists(destDir):
		_logger.debug("Creating " + destDir)
		os.mkdir(destDir, 0750)
	for f in files:
		srcFile = os.path.normpath(os.path.join(srcDir, f))
		if os.path.isfile(srcFile):
			_logger.debug(srcFile + " -> " + destDir)
			shutil.copy2(srcFile, destDir)

def main():
	if len(sys.argv) != 2:
		usage()
		sys.exit(1)

	sessionDir = unicode(sys.argv[1], "utf8")

	try:
		dirutil.createNewDirByAbsoluteAsciiPath(sessionDir)
	except OSError, e:
		# skip chown error for nfs shares (errno 1), bug 94296.
		# it is unknown how nfs share can be detected simply and we skip the error 1 always
		if e.errno != 1:
			sys.stdout.write(unicode(e))
			sys.exit(1)

	log.initLog("agent-copy", sessionDir)

	agentDir = os.path.join(agents_dir(), "PleskX")
	sharedDir = os.path.join(agents_dir(), "shared")

	try:
		os.chdir(agentDir)
		_logger.debug("** " + agentDir)
		os.path.walk(".", copy_dir_files, sessionDir)

		os.chdir(sharedDir)
		_logger.debug("** " + sharedDir)
		os.path.walk(".", copy_dir_files, sessionDir)
	except OSError, e:
		print "Unable to copy agent into working directory: "  + str(e)
		sys.exit(1)
