<!-- Copyright 1999-2014. Parallels IP Holdings GmbH. All Rights Reserved. -->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
  <xsl:output indent="yes" encoding="UTF-8"/>

	<xsl:template match='psa'>
	<migration-dump agent-name='PleskX' dump-version="7.5" dump-original-version="7.5" >
	<xsl:if test="description">
		<xsl:copy-of select="description"/>
	</xsl:if>
	<xsl:choose>
		<xsl:when test="user[@type='client']">
			<xsl:for-each select="user[@type='client']">
				<xsl:call-template name="create-client"/>
			</xsl:for-each>
		</xsl:when>
		<xsl:otherwise>
			<xsl:if test="sites">
				<xsl:for-each select="sites/domain">
					<xsl:call-template name="create-domain"/>
				</xsl:for-each>
			</xsl:if>
		</xsl:otherwise>
	</xsl:choose>

	<xsl:if test="server">
		<xsl:call-template name="create-server"/>
	</xsl:if>
	</migration-dump>
	</xsl:template>

  <xsl:template name="create-client">
    <client id="{@id}" name="{login/@name}" cr-date="{@create_date}">
      <xsl:if test="vhost_skeleton">
        <xsl:attribute name="cid-skeleton">
          <xsl:value-of select="substring-after(vhost_skeleton/@src,':')"/>
        </xsl:attribute>
      </xsl:if>
      <xsl:call-template name="create-password">
        <xsl:with-param name='passwd' select='login/@password'/>
        <xsl:with-param name='type' select='login/@PW_TYPE'/>
      </xsl:call-template>

      <xsl:call-template name="create-status">
        <xsl:with-param name='status' select='@status'/>
      </xsl:call-template>

      <xsl:if test="card">
        <xsl:call-template name="create-card"/>
      </xsl:if>

      <xsl:if test="limits">
        <xsl:call-template name="create-limits"/>
      </xsl:if>

      <xsl:if test="permissions">
        <xsl:call-template name="create-permissions"/>
      </xsl:if>

      <xsl:if test="ip_pool">
        <ip_pool>
          <xsl:for-each select="ip_pool/*">
            <xsl:call-template name="create-ip">
              <xsl:with-param name="id" select="@id"/>
            </xsl:call-template>
          </xsl:for-each>
        </ip_pool>
      </xsl:if>

      <xsl:if test="sites">
        <xsl:for-each select="sites/domain">
          <xsl:call-template name="create-domain"/>
        </xsl:for-each>
      </xsl:if>

    </client>
  </xsl:template>  

  <xsl:template name="create-domain">
    <domain id="{@id}" name="{@name}" cr-date="{@cr_date}">
      <xsl:if test="@htype='vrt_hst'">
        <xsl:call-template name="create-ip">
          <xsl:with-param name="id" select="user/ip_address/@id"/>
        </xsl:call-template>
      </xsl:if>

      <xsl:if test="forwarding/@ip_address_id">
        <xsl:call-template name="create-ip">
          <xsl:with-param name="id" select="forwarding/@ip_address_id"/>
        </xsl:call-template>
      </xsl:if>

      <xsl:call-template name="create-status">
        <xsl:with-param name="status" select="@status"/>
      </xsl:call-template>

      <xsl:if test="limits">
        <xsl:call-template name="create-limits"/>
      </xsl:if>

      <xsl:if test="user/@type='mail'">
        <mailsystem>
          <xsl:call-template name="create-status">
            <xsl:with-param name="status" select="@status"/>
          </xsl:call-template>
          <xsl:for-each select="user[@type='mail']">
            <xsl:call-template name="create-mailuser"/>
          </xsl:for-each>
        </mailsystem>
      </xsl:if>

      <xsl:if test="dns">
        <dns-zone email="root@localhost.localdomain">
          <xsl:attribute name="type"><xsl:value-of select="@dns_type"/></xsl:attribute>
          <xsl:if test="@dns_zone='true'">
            <xsl:call-template name="create-status">
              <xsl:with-param name="status">0</xsl:with-param>
            </xsl:call-template>
          </xsl:if>
          <xsl:if test="@dns_zone!='true'">
            <xsl:call-template name="create-status">
              <xsl:with-param name="status">16</xsl:with-param>
            </xsl:call-template>
          </xsl:if>
          <xsl:for-each select="dns/host">
            <xsl:variable name="type" select="@type"/>
            <!--              <xsl:if test="@ip_address_id">
              <xsl:variable name="ip_id" select="@ip_address_id"/>
              <xsl:for-each select="/psa/server/ip_addresses_list">
                <xsl:if test="ip_address/@id = $ip_id">
                  <xsl:variable name="ip" select="ip_address/@ip_address"/>
                </xsl:if>
              </xsl:for-each>
            </xsl:if>
            -->
            <dnsrec>
              <xsl:attribute name="type"><xsl:value-of select="$type"/></xsl:attribute>
              <xsl:choose>
                <xsl:when test="$type = 'NS'">
                  <xsl:attribute name="src"><xsl:value-of select="@name"/></xsl:attribute>
                  <xsl:attribute name="dst"><xsl:value-of select="@ns"/></xsl:attribute>
                </xsl:when>
                <xsl:when test="$type = 'A'">
                  <xsl:attribute name="src"><xsl:value-of select="@name"/></xsl:attribute>
                  <xsl:choose>
                    <xsl:when test="@ip_address_id">
                      <xsl:variable name="ip" select="@ip_address_id"/>
                      <xsl:attribute name="dst"><xsl:value-of select="/psa/server/ip_addresses_list/ip_address[@id = $ip]/@ip_address"/></xsl:attribute>
                    </xsl:when>
                    <xsl:otherwise>
                      <xsl:attribute name="dst"><xsl:value-of select="@ip"/></xsl:attribute>
                    </xsl:otherwise>
                  </xsl:choose>
                </xsl:when>
                <xsl:when test="$type = 'CNAME'">
                  <xsl:attribute name="src"><xsl:value-of select="@name"/></xsl:attribute>
                  <xsl:attribute name="dst"><xsl:value-of select="@cname"/></xsl:attribute>
                </xsl:when>
                <xsl:when test="$type = 'MX'">
                  <xsl:attribute name="src"><xsl:value-of select="@name"/></xsl:attribute>
                  <xsl:attribute name="dst"><xsl:value-of select="@mx"/></xsl:attribute>
                  <xsl:attribute name="opt"><xsl:value-of select="@priority"/></xsl:attribute>
                </xsl:when>
                <xsl:when test="$type = 'PTR'">
                  <xsl:choose>
                    <xsl:when test="@ip_address_id">
                      <xsl:variable name="ip" select="@ip_address_id"/>
                      <xsl:attribute name="src"><xsl:value-of select="/psa/server/ip_addresses_list/ip_address[@id = $ip]/@ip_address"/></xsl:attribute>
                    </xsl:when>
                    <xsl:otherwise>
                      <xsl:attribute name="src"><xsl:value-of select="@ip"/></xsl:attribute>
                    </xsl:otherwise>
                  </xsl:choose>
                  <xsl:attribute name="dst"><xsl:value-of select="@name"/></xsl:attribute>
                  <xsl:attribute name="opt"><xsl:value-of select="@netmask"/></xsl:attribute>
                </xsl:when>
                <xsl:when test="$type = 'TXT'">
                  <xsl:attribute name="src"><xsl:value-of select="@name"/></xsl:attribute>
                  <xsl:attribute name="dst"><xsl:value-of select="@text"/></xsl:attribute>
                </xsl:when>
              </xsl:choose>
            </dnsrec>
          </xsl:for-each>
        </dns-zone>
      </xsl:if>

      <xsl:for-each select='data_bases'>
        <xsl:call-template name='create-db'/>
      </xsl:for-each>

      <!-- maillist -->
      <!-- traffic -->
      <!-- certificate -->
      <xsl:for-each select="certificates_list/certificate">
        <xsl:call-template name="create-cert">
          <xsl:with-param name='id' select="@id"/>
        </xsl:call-template>
      </xsl:for-each>

      <xsl:apply-templates select = "domain_services/domain_service[@type='tomcat']"/>

      <xsl:apply-templates  select="user[@type='domain_level']"/>

      <xsl:choose>
        <xsl:when test="@htype='vrt_hst'">
          <xsl:apply-templates select="user[@type='hosting']"/>
        </xsl:when>
        <xsl:when test="@htype='std_fwd'">
          <shosting><xsl:value-of select="forwarding/@redirect"/></shosting>
        </xsl:when>
        <xsl:when test="@htype='frm_fwd'">
          <fhosting><xsl:value-of select="forwarding/@redirect"/></fhosting>
        </xsl:when>
      </xsl:choose>

    </domain>
  </xsl:template>

  <xsl:template  match="user[@type='domain_level']">
    <domainuser status="{@status}">
      <xsl:if test="card/pname">
        <xsl:attribute name="contact">
          <xsl:value-of select="card/pname/text()"/>
        </xsl:attribute>
      </xsl:if>

      <xsl:call-template name='create-password'>
        <xsl:with-param name="passwd" select="login/@password"/>
        <xsl:with-param name="type" select="login/@PW_TYPE"/>
      </xsl:call-template>

      <xsl:if test='card'>
        <xsl:call-template name='create-card'>
          <xsl:with-param name="no_name" select="1"/>
        </xsl:call-template>
      </xsl:if>

      <xsl:if test='permissions'>
        <xsl:call-template name='create-permissions'/>
      </xsl:if>

    </domainuser>
  </xsl:template>

	<xsl:template name="create-server">
	<server>
		<hostname><xsl:value-of select="server/params/ip/@host"/></hostname>

		<xsl:if test="user[@type = 'admin']">
			<xsl:for-each select="user[@type='admin']">
				<xsl:call-template name="create-card"/>
			</xsl:for-each>
		</xsl:if>

		<xsl:for-each select="server/psa_key">
			<key additional="false"><xsl:value-of select="text()"/></key>
		</xsl:for-each>

		<xsl:if test="server/dns">
			<xsl:call-template name="create-server-dns"/>
		</xsl:if>

		<xsl:for-each select="/psa/server/certificates_list/certificate">
			<xsl:variable name="id" select="@id"/>
			<xsl:if test="not(/psa/user/sites/domain/certificates_list/certificate/@id = $id) and not(@bu_cert='true')">
				<xsl:call-template name="create-cert">
					<xsl:with-param name='id' select="@id"/>
				</xsl:call-template>
			</xsl:if>
		</xsl:for-each>
	</server>
	</xsl:template>

  <xsl:template name="create-password">
    <xsl:param name="passwd"/>
    <xsl:param name="type"/>
    <xsl:choose>
    	<xsl:when test="$type = 'crypt'">
   			<password type="encrypted"><xsl:value-of select="$passwd"/></password>
	    </xsl:when>
	    <xsl:otherwise>
   			<password type="{$type}"><xsl:value-of select="$passwd"/></password>
	    </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template name="create-card">
    <xsl:param name="no_name"/>
    <xsl:for-each select='card/*'>
      <xsl:variable name='node' select='name()'/>
      <xsl:variable name='content' select='.'/>
      <xsl:choose>
        <xsl:when test="$node = 'pname' and $no_name = 1">
        </xsl:when>
        <xsl:when test="$node = 'pname'">
          <pinfo name='name'><xsl:value-of select='$content'/></pinfo>
        </xsl:when>
        <xsl:when test="$node = 'cname'">
          <pinfo name='company'><xsl:value-of select='$content'/></pinfo>
        </xsl:when>
        <xsl:when test="$node = 'pcode'">
          <pinfo name='zip'><xsl:value-of select='$content'/></pinfo>
        </xsl:when>
        <xsl:otherwise>
          <pinfo name='{$node}'><xsl:value-of select='$content'/></pinfo>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:for-each>  
  </xsl:template>

  <xsl:template name="create-limits">
    <xsl:for-each select='limits/@*'>
      <xsl:variable name='name' select='name()'/>
      <xsl:variable name='limit' select='.'/>
      <limit name="{$name}"><xsl:value-of select='$limit'/></limit>
    </xsl:for-each>
  </xsl:template>

  <xsl:template name="create-permissions">
    <xsl:for-each select='permissions/@*'>
      <xsl:variable name='permission' select='name()'/>
      <xsl:variable name='allowed' select='.'/>
      <xsl:if test="$permission != 'site_builder'">
        <permission name="{$permission}" allowed="{$allowed}"/>
      </xsl:if>
    </xsl:for-each>
  </xsl:template>

  <xsl:template name="create-ip">
    <xsl:param name="id"/>
    <xsl:for-each select="/psa/server/ip_addresses_list/ip_address[@id = $id]">
        <ip>
          <ip-type><xsl:value-of select="@type"/></ip-type>
          <ip-address><xsl:value-of select="@ip_address"/></ip-address>
        </ip>
    </xsl:for-each>
  </xsl:template>

  <xsl:template name="create-mailuser">
    <mailuser>
      <xsl:attribute name="name">
        <xsl:value-of select="login/@name"/>
      </xsl:attribute>

      <xsl:attribute name="mailgroup-enabled">
        <xsl:value-of select="services/@mail_group"/>
      </xsl:attribute>

      <xsl:call-template name='create-password'>
        <xsl:with-param name='passwd' select="login/@password"/>
        <xsl:with-param name='type' select="login/@PW_TYPE"/>
      </xsl:call-template>

      <xsl:if test="services/@postbox">
        <xsl:variable name='state' select='services/@postbox'/>
        <mailbox enabled="{$state}" type="mdir">
          <xsl:attribute name="cid_whole">
            <xsl:value-of select="substring-after(@src,':')"/>
          </xsl:attribute>
        </mailbox>
      </xsl:if>

      <xsl:for-each select="aliases/alias">
        <alias><xsl:value-of select="@name"/></alias>
      </xsl:for-each>

      <xsl:if test="services/forwarding/@redirect">
        <redirect enabled="{services/forwarding/@redirect}"><xsl:value-of select="services/forwarding[@redirect]/email"/></redirect>
      </xsl:if>

      <xsl:if test="services/forwarding/@mailgroup">
        <xsl:for-each select="services/forwarding[@mailgroup]/email">
          <mailgroup-member><xsl:value-of select="."/></mailgroup-member>
        </xsl:for-each>
      </xsl:if>

      <autoresponders enabled="{services/@autoresponder}">
        <xsl:for-each select="repository/file">
          <attach file="{@name}"/>
        </xsl:for-each>
        <xsl:for-each select="autoresponder">
          <xsl:call-template name="create-autoresponder"/>
        </xsl:for-each>
      </autoresponders>

      <xsl:if test="spamassassin">
        <xsl:call-template name="create-spamassassin"/>
      </xsl:if>

      <xsl:apply-templates select="params/parameter[@name='drweb_direction']"/>

    </mailuser>
  </xsl:template>

  <xsl:template name="create-db">
    <database>
      <xsl:attribute name="name">
        <xsl:value-of select="@name"/>
      </xsl:attribute>
      <xsl:attribute name="type">
        <xsl:value-of select="@type"/>
      </xsl:attribute>
      <xsl:attribute name="cid">
        <xsl:value-of select="substring-after(@src,':')"/>
      </xsl:attribute>
      <xsl:for-each select="user">
        <xsl:call-template name="create-dbuser"/>
      </xsl:for-each>      
    </database>
  </xsl:template>

  <xsl:template name="create-dbuser">
    <dbuser name="{login/@name}">
      <xsl:call-template name="create-password">
        <xsl:with-param name="passwd" select="login/@password"/>
        <xsl:with-param name="type" select="login/@PW_TYPE"/>
      </xsl:call-template>
    </dbuser>
  </xsl:template>

  <xsl:template match="domain_services/domain_service[@type='tomcat']">
    <tomcat>
      <xsl:call-template name="create-status">
        <xsl:with-param name="status" select="@status"/>
      </xsl:call-template>
      <xsl:for-each select="web_app[@src != '']">
        <webapp name="{@name}">
          <xsl:attribute name="cid">
            <xsl:value-of select="substring-after(@src,':')"/>
          </xsl:attribute>

          <xsl:call-template name="create-status">
            <xsl:with-param name="status" select="@status"/>
          </xsl:call-template>
        </webapp>
      </xsl:for-each>
    </tomcat>
  </xsl:template>

  <xsl:template match="user[@type='hosting']">
      <phosting>
        
        <xsl:attribute name="cid_whole">
          <xsl:value-of select="substring-after(@src,':')"/>
        </xsl:attribute>

        <xsl:if test="services/@ssl='true'">
          <xsl:attribute name="https">true</xsl:attribute>
        </xsl:if>

        <xsl:if test="@at_domains!=''">
          <xsl:attribute name="at-domains">
            <xsl:value-of select='@at_domains'/>
          </xsl:attribute>
        </xsl:if>

        <xsl:call-template name='create-sysuser'>
          <xsl:with-param name='name' select='login/@name'/>
          <xsl:with-param name='password' select='login/@password'/>
          <xsl:with-param name='quota' select='login/@quota'/>
          <xsl:with-param name='pw-type' select='login/@PW_TYPE'/>
          <xsl:with-param name='shell' select='login/@shell'/>
        </xsl:call-template>

        <xsl:call-template name="create-scripting"/>

        <!--sapp-installed-->
        <!--fpuser-->
        <!--logrotation -->
        <!--anonftp-->
        <xsl:if test="anon_ftp">
          <anonftp pub="{anon_ftp/@status}" incoming="{anon_ftp/@incoming}" display-login="{anon_ftp/@display_login}">
            <xsl:if test="anon_ftp/size">
              <anonftp-limit name="incoming-disk-quota"><xsl:value-of select="anon_ftp/size/@limit"/></anonftp-limit>
            </xsl:if>
            <xsl:if test="anon_ftp/@max_conn">
              <anonftp-limit name="max-connections"><xsl:value-of select="anon_ftp/@max_conn"/></anonftp-limit>
            </xsl:if>
            <xsl:if test="anon_ftp/@bandwidth">
              <anonftp-limit name="bandwidth"><xsl:value-of select="anon_ftp/@bandwidth"/></anonftp-limit>
            </xsl:if>
            <xsl:if test="anon_ftp/@incoming_subdirs = 'true'">
              <anonftp-permission name="incoming-mkdir"/>
            </xsl:if>
            <xsl:if test="anon_ftp/@incoming_readable = 'true'">
              <anonftp-permission name="incoming-download"/>
            </xsl:if>
            <xsl:if test="anon_ftp/@login_text">
              <login-message><xsl:value-of select="anon_ftp/@login_text"/></login-message>
            </xsl:if>  
            
          </anonftp>
        </xsl:if>
        <!--pdir-->
        <xsl:for-each select="protected_dir">
          <xsl:call-template name="create-pdir"/>
        </xsl:for-each>
        <!--webuser-->
        <xsl:for-each select="user[@type='web_users']">
          <xsl:call-template name="create-webuser"/>
        </xsl:for-each>

        <!--subdomain-->
        <xsl:if test="subdomains">
          <xsl:for-each select="subdomains/subdomain">
            <xsl:call-template name="create-subdomain"/>
          </xsl:for-each>
        </xsl:if>
        <!--webalizer-->

      </phosting>
  </xsl:template>
  
  <xsl:template name="create-sysuser">
    <xsl:param name="name"/>
    <xsl:param name="password"/>
    <xsl:param name="quota"/>
    <xsl:param name="pw-type"/>
    <xsl:param name="shell"/>

	<sysuser name="{$name}">
		<xsl:if test="$quota > 0">
			<xsl:attribute name="quota">
				<xsl:value-of select="$quota"/>
			</xsl:attribute>
        </xsl:if>
          
        <xsl:if test="string-length($shell) > 0">
          <xsl:attribute name="shell">
			<xsl:value-of select="$shell"/>
		  </xsl:attribute>
		</xsl:if>

		<xsl:call-template name="create-password">
          <xsl:with-param name="passwd" select="$password"/>
          <xsl:with-param name="type" select="$pw-type"/>
		</xsl:call-template>
	</sysuser>
  </xsl:template>

  <xsl:template name="create-scripting">
    <scripting>
      <xsl:for-each select="services/@*">
        <xsl:variable name='feature' select="name()"/>
        <xsl:variable name='state' select="."/>
        <xsl:choose>
          <xsl:when test="$feature!='ssl' and $feature!='same_ssl' and $feature!='webstat' and $state!=''">
            <xsl:attribute name="{$feature}">
              <xsl:value-of select="$state"/>
            </xsl:attribute>
          </xsl:when>
        </xsl:choose>
      </xsl:for-each>
    </scripting>
  </xsl:template>

  <xsl:template name="create-autoresponder">
    <autoresponder name="{@name}" status="{@turn}">
      <xsl:choose>
        <xsl:when test="@turn='true'">
          <xsl:attribute name="status">on</xsl:attribute>  
        </xsl:when>
        <xsl:otherwise>
          <xsl:attribute name="status">off</xsl:attribute>  
        </xsl:otherwise>
      </xsl:choose>
      <xsl:if test="params/@reply_to">
        <xsl:attribute name="replyto">
          <xsl:value-of select="params/@reply_to"/>
        </xsl:attribute>
      </xsl:if>

      <xsl:if test="forwarding/@autoresponder='true'">
        <xsl:attribute name="redirect">
          <xsl:value-of select="forwarding[@autoresponder='true']/email"/>
        </xsl:attribute>
      </xsl:if>

      <xsl:if test="keystr/@where = 'body' or keystr/@where = 'subj'">
        <xsl:attribute name="require">
          <xsl:value-of select="concat(keystr/@where, ':', keystr/text())"/>
        </xsl:attribute>
      </xsl:if>

      <xsl:attribute name="subject">
        <xsl:value-of select="message/@subject"/>
      </xsl:attribute>

      <text><xsl:value-of select="message"/></text>

      <xsl:if test="params/@ans_freq">
        <autoresponder-limit name="ans-freq"><xsl:value-of select="params/@ans_freq"/></autoresponder-limit>
      </xsl:if>

      <xsl:if test="params/@mem_limit">
        <autoresponder-limit name="ans-count"><xsl:value-of select="params/@mem_limit"/></autoresponder-limit>
      </xsl:if>

      <xsl:copy-of select="attach"/>

    </autoresponder>
  </xsl:template>

  <xsl:template match="params/parameter[@name='drweb_direction']">
    <drweb>
      <xsl:choose>
        <xsl:when test="@value='any'">
          <xsl:attribute name="state">
            <xsl:text>inout</xsl:text>
          </xsl:attribute>
        </xsl:when>
        <xsl:when test="@value='from'">
          <xsl:attribute name="state">
            <xsl:text>in</xsl:text>
          </xsl:attribute>
        </xsl:when>
        <xsl:otherwise>
          <xsl:attribute name="state">
            <xsl:text>out</xsl:text>
          </xsl:attribute>
        </xsl:otherwise>
        
      </xsl:choose>
    </drweb>
  </xsl:template>

  <xsl:template name="create-spamassassin">
    <spamassassin>
      <xsl:for-each select="spamassassin/@*">
        <xsl:variable name='feature' select="name()"/>
        <xsl:variable name='state' select="."/>
        <xsl:choose>
          
          <xsl:when test="$feature = 'spam_action'">
            <xsl:attribute name="action">
              <xsl:choose>
                <xsl:when test="$state='save'">mark</xsl:when>
                <xsl:otherwise>del</xsl:otherwise>
              </xsl:choose>
            </xsl:attribute>
          </xsl:when>

          <xsl:when test="$feature='rw_subject_tag' and string-length($state)>0">
            <xsl:attribute name='modify-subj'><xsl:text>true</xsl:text></xsl:attribute>
            <xsl:attribute name='subj-text'><xsl:value-of select="$state"/></xsl:attribute>
          </xsl:when>

        </xsl:choose>
      </xsl:for-each>
    </spamassassin>  
  </xsl:template>
  
  <xsl:template name="create-subdomain">
    <subdomain name="{@name}">
      <xsl:attribute name="cid_whole">
        <xsl:value-of select="substring-after(@src,':')"/>
      </xsl:attribute>

      <xsl:if test="@sys_user_type='native'">
        <xsl:call-template name='create-sysuser'>
          <xsl:with-param name='name' select="user[@type='subdomain']/login/@name"/>
          <xsl:with-param name='password' select="user[@type='subdomain']/login/@password"/>
          <xsl:with-param name='quota' select="user[@type='subdomain']/login/@quota"/>
          <xsl:with-param name='pw-type' select="user[@type='subdomain']/login/@PW_TYPE"/>
          <xsl:with-param name='shell' select="user[@type='subdomain']/login/@shell"/>
        </xsl:call-template>
      </xsl:if>

     <scripting>
     <xsl:for-each select="@*">
        <xsl:variable name='feature' select="name()"/>
        <xsl:variable name='state' select="."/>

        <xsl:choose>
          <xsl:when test="$feature = 'python' or $feature = 'cgi' or $feature = 'php' or $feature = 'perl' or $feature = 'asp' or $feature = 'coldfusion'  or $feature = 'ssi'">
            <xsl:attribute name="{$feature}"><xsl:value-of select="$state"/></xsl:attribute>
          </xsl:when>
        </xsl:choose>          
      </xsl:for-each>
      </scripting>
    </subdomain>
  </xsl:template>

  <xsl:template name="create-webuser">
    <webuser name="{login/@name}">
      <xsl:attribute name="cid">
        <xsl:value-of select="substring-after(@src,':')"/>
      </xsl:attribute>
                     
      <xsl:call-template name="create-sysuser">
        <xsl:with-param name='name' select="login/@name"/>
        <xsl:with-param name='password' select="login/@password"/>
        <xsl:with-param name='quota' select="login/@quota"/>
        <xsl:with-param name='pw-type' select="login/@PW_TYPE"/>
        <xsl:with-param name='shell' select="login/@shell"/>
      </xsl:call-template>

      <xsl:call-template name="create-scripting"/>
    </webuser>
  </xsl:template>

  <xsl:template name="create-pdir">
    <xsl:variable name="title" select="realm"/>
    <pdir name="{@path}" title="{$title}">
      <xsl:attribute name="ssl"><xsl:value-of select="services/@ssl"/></xsl:attribute>
      <xsl:attribute name="nonssl"><xsl:value-of select="services/@non_ssl"/></xsl:attribute>
      <xsl:for-each select="user[@type='pd_users']">
        <xsl:variable name="name" select="login/@name"/>
        <pduser name="{$name}">
          <xsl:call-template name="create-password">
            <xsl:with-param name='passwd' select="login/@password"/>
            <xsl:with-param name='type' select="login/@PW_TYPE"/>
          </xsl:call-template>
        </pduser>
      </xsl:for-each>
    </pdir>
  </xsl:template>

  <xsl:template name="create-status">
    <xsl:param name="status"/>
    <status>
      <xsl:choose>
        <xsl:when test="$status != 4 and $status != 0">
          <xsl:if test="$status div 2 mod 2 >= 1">
            <disabled-by name="parent"/>
          </xsl:if>
          <xsl:if test="$status = 1 or $status = 32 or $status div 16 mod 2 >= 1">
            <disabled-by name="admin"/>
          </xsl:if>
          <xsl:if test="$status div 64 mod 2 >= 1">
            <disabled-by name="client"/>
          </xsl:if>
          <xsl:if test="$status div 256 mod 2 >= 1">
            <disabled-by name="expired"/>
          </xsl:if>
        </xsl:when>
        <xsl:otherwise>
          <enabled/>
       </xsl:otherwise>
      </xsl:choose>
    </status>
  </xsl:template>
  
  <xsl:template name="create-server-dns">
    <dns-settings recursion="localnets"> <!-- recursion is REQUIRED. Set default value -->
      <dns-zone email="root@localhost.localdomain" type="master">
        <xsl:if test="server/dns[@dns_zone_status='on']">
          <xsl:call-template name="create-status">
            <xsl:with-param name="status">0</xsl:with-param>
          </xsl:call-template>
        </xsl:if>
        <xsl:if test="server/dns[@dns_zone_status='off']">
          <xsl:call-template name="create-status">
            <xsl:with-param name="status">16</xsl:with-param>
          </xsl:call-template>
        </xsl:if>

        <xsl:for-each select="server/dns/host">
          <xsl:call-template name="create-dnsrec"/>
        </xsl:for-each>
      </dns-zone>
      <common-acl/>
    </dns-settings>
  </xsl:template>

  <xsl:template name="create-dnsrec">
    <xsl:variable name="type" select="@type"/>
    <dnsrec>
      <xsl:attribute name="type"><xsl:value-of select="$type"/></xsl:attribute>
      <xsl:choose>
        <xsl:when test="$type = 'NS'">
          <xsl:attribute name="src"><xsl:value-of select="@name"/></xsl:attribute>
          <xsl:attribute name="dst"><xsl:value-of select="@ns"/></xsl:attribute>
        </xsl:when>
        <xsl:when test="$type = 'A'">
          <xsl:attribute name="src"><xsl:value-of select="@name"/></xsl:attribute>
          <xsl:attribute name="dst"><xsl:value-of select="@ip"/></xsl:attribute>
        </xsl:when>
        <xsl:when test="$type = 'MX'">
          <xsl:attribute name="src"><xsl:value-of select="@name"/></xsl:attribute>
          <xsl:attribute name="dst"><xsl:value-of select="@mx"/></xsl:attribute>
          <xsl:attribute name="opt"><xsl:value-of select="@priority"/></xsl:attribute>
        </xsl:when>
        <xsl:when test="$type = 'CNAME'">
          <xsl:attribute name="src"><xsl:value-of select="@name"/></xsl:attribute>
          <xsl:attribute name="dst"><xsl:value-of select="@cname"/></xsl:attribute>
        </xsl:when>
        <xsl:when test="$type = 'PTR'">
          <xsl:attribute name="src"><xsl:value-of select="@ip"/></xsl:attribute>
          <xsl:attribute name="dst"><xsl:value-of select="@name"/></xsl:attribute>
          <xsl:attribute name="opt"><xsl:value-of select="@netmask"/></xsl:attribute>
        </xsl:when>

      </xsl:choose>
    </dnsrec>
  </xsl:template>

  <xsl:template name="create-cert">
    <xsl:param name="id"/>
    <xsl:for-each select="/psa/server/certificates_list/certificate">
      <xsl:if test="@id = $id">
        <certificate>
          <xsl:attribute name="name">
            <xsl:value-of select="@name"/>
          </xsl:attribute>
          <xsl:if test="@default">
            <xsl:attribute name="default">true</xsl:attribute>
          </xsl:if>
          <xsl:attribute name="cid">
            <xsl:value-of select="substring-after(pub_key/@src,':')"/>
          </xsl:attribute>
          <xsl:attribute name="cid-private">
            <xsl:value-of select="substring-after(pvt_key/@src,':')"/>
          </xsl:attribute>
          <xsl:if test="ca_cert">
            <xsl:attribute name="cid-ca">
              <xsl:value-of select="substring-after(ca_cert/@src,':')"/>
            </xsl:attribute>
          </xsl:if>
        </certificate>
      </xsl:if>
    </xsl:for-each>
  </xsl:template>

</xsl:stylesheet>  
