# Copyright 1999-2014. Parallels IP Holdings GmbH. All Rights Reserved.
import commands
import getopt
import osutil
import backup
import gzip_stream
from ReadLineAdapter import FileAdapter
import split_files
import MimeWriter
import StringIO
import sys
import libxml2
import os
import string
import binascii, re, time

from plesk_config import get
from shutil import copyfileobj
from tempfile import mktemp


def getXmlAttribute(xmlNode_, xmlAttrName_):
    for props in xmlNode_.get_properties():
        if not props.isText():
            if props.name == xmlAttrName_:
                return props.content
    return None


class InfoCommand(commands.HelpProvider):
    USAGE_STRING = """info <source-file>"""
    SHORT_HELP_STRING = "Gets info on old (pre-8.0) dump"
    HELP_STRING = "Gets info on old (pre-8.0) dump"
    def __init__(self, verbosity):
        self.verbosity = verbosity
        self.__temp_xmlfile = "/tmp/pre8dump~.xml"

    def get_parts(self, file):
        finder = backup.BackupPartsFinder('file', file)
        fh = FileAdapter(split_files.SplitFile(finder))
        if gzip_stream.isGzipped(fh):
            fh = FileAdapter(gzip_stream.GzipReadStream(fh))
        parts = backup.decodeParts(backup.enumerateParts(fh))
        return parts
    
    def parse(self, args):
        if len(args) == 0:
            raise RuntimeError("No backup file specified")
        self.backupFile = args.pop()

    def act(self):
        #self.print_backup_object()
        self.print_server_object()

    def __check_root(self):
        mime = MimeWriter.MimeWriter(StringIO.StringIO())
        mime.addheader("Mime-Version", "1.0")
        mime.startmultipartbody("related")
        for (partInfo, partContent) in self.get_parts(self.backupFile):
             if partInfo.getheader("Content-Type", "").find("text/xml") >= 0:
                 b = backup.Bundle(fh = partContent, session_dir = None)
                 b.process(osutil.createToFileSaver(self.__temp_xmlfile))
                 b.invalidate()
                 break
        doc = libxml2.parseFile(self.__temp_xmlfile)
        root = doc.children
        for child in root:
            if child.type == "element":
                if child.name == 'psa':
                    break
                else:
                    raise Exception("Root element 'psa' is not found in dump")
        return doc, root

    def print_server_object(self):
        doc = self.__check_root()[0]
        print "object_type: server"
        print "object_id: admin"
        doc.freeDoc()

    def print_backup_object(self):
        admin = None
        clients = []
        domains = []
        doc, root = self.__check_root()
        for child in root:
            if child.type == "element":
                if child.name == 'user':
                    user_type = getXmlAttribute(child,'type')
                    if user_type == 'admin':
                        admin = child
                    elif user_type == 'client':
                        clients.append(child)
                elif child.name == 'domain':
                    # test for existance of 'id' attribute, otherwise it is domain template
                    if getXmlAttribute(child,'id') is not None:
                        domains.append(child)
        if admin is not None or len(clients) > 1:
            print "object_type: server"
            print "object_id: admin"
        elif len(clients) == 1:
            print "object_type: client"
            print "object_id: " + getXmlAttribute(domains[0],'name')
        else:
            if len(domains) > 1:
                print "object_type: server"
                print "object_id: admin"
            elif len(domains) == 1:
                print "object_type: domain"
                print "object_id: " + getXmlAttribute(domains[0],'name')
            else:
                raise Exception("Could not get top-level object from backup file: '%s'. Neither client nor domain element found" % self.backupFile)
        doc.freeDoc()
        osutil.unlink_nothrow(self.__temp_xmlfile)

commands.register("info", InfoCommand, 2)
