# Copyright 1999-2014. Parallels IP Holdings GmbH. All Rights Reserved.
import string
import commands

class GenericHelp(object):
    def help(self):
        s = """Usage: backup-unpack <global-options> <command> <command-arguments>.

Global options:

  -v|--verbose  Enables verbose processing.

Commands:\n"""

        for (name, class_) in commands.items():
            s += "  " + string.ljust(name, 14)  + class_.SHORT_HELP_STRING + "\n"

        s += "\n'backup-unpack help <command>' describes <command> in details\n"
        return s

    help = classmethod(help)

class SelectObjectsHelp(object):
    def help(self):
        return """
backup-unpack supports selecting only part of backup for
processing. It is achieved providing either single object selection
or multiple objects selection in command line. Note that processing
is also controlled by specifying recursive or non-recursive mode in
commands.

== Single object.

There are two forms of selecting single entity for processing: full
and short. Short form provides convenient way to select most used
entities, while full form gives more control.

* Full single object selection form.

Object is specified as <object type> <object name>:

 * client <client> (clients are selected by their login name)
 * domain <domain>
 * mailname <mail>@<domain>
 * maillist <mail>@<domain>
 * database <name>[@<hostname>[:<port>]][/(mysql|postgresql)]
 * webuser <user>@<domain>

If hostname and port for database is omitted, it is treated as
database on local database server. Type of database is deduced
automatically if not provided.

* Short single entity selection form

This form allows selecting only clients, domains and mailnames. Only
<object name> is specified in this form, and object type is deduced
from the object name.

== Multiple objects.

Selecting all objects of specified type is specified by name of object
type:

 * clients
 * domains
 * mailnames
 * maillists
 * databases
 * webusers
"""
    help = classmethod(help)

class HelpCommand(commands.HelpProvider):
    def __init__(self, globalOptions):
        pass

    USAGE_STRING = "help [<command>|select-objects]"
    SHORT_HELP_STRING = "Shows help message"
    HELP_STRING = """Shows the help message about command line structure or specific command.

'help' without arguments shows command line structure.
'help <command>' describes specific command in details.
'help select-objects' describes syntax of selecting parts of backup for processing.
"""

    def parse(self, args):
        self.command = self._parseForCommand(args)

    def _parseForCommand(self, args):
        if len(args) == 0:
            return GenericHelp
        if len(args) == 1 and args[0] == "select-objects":
            return SelectObjectsHelp
        if len(args) == 1:
            return commands.get(args[0])
        raise RuntimeError("Too many arguments for 'help' command")

    def act(self):
        print self.command.help()

commands.register("help", HelpCommand, 100)

# Local Variables:
# mode: python
# indent-tabs-mode: nil
# tab-width: 4
# End:
