# Copyright 1999-2014. Parallels IP Holdings GmbH. All Rights Reserved.
import os
import sys
import datetime 
import random

import commands
import backup
from converter_exceptions import DumpTooOldException
import config

class InfoCommand(commands.HelpProvider):
    USAGE_STRING = """info <backup-file>"""
    SHORT_HELP_STRING = "Gets info on backup of Plesk version < 9.0"
    HELP_STRING = """Gets info on backup of Plesk version < 9.0."""

    def __init__(self, verbosity):
        self.backupFile = None
    
    def parse(self, args):
        if len(args) == 0:
            raise RuntimeError("No backup file specified")
        self.backupFile = args.pop().encode("utf-8")

    def act(self):
        backupFile = backup.backupFileFactory('file', self.backupFile)
        if not backupFile.exists():
            raise RuntimeError("Backup file '" + self.backupFile + "' does not exist")
        b = backup.Backup('file', self.backupFile)
        metadata = b.getMetadata()
        if metadata is None:
            raise Exception("Could not get Metadata from backup file: '%s'" % self.backupFile)
        info = metadata.getInfo()
        if info is None:
            raise Exception("Could not get Info from Metadata of backup file: '%s'" % self.backupFile)
        if not info.dumpVersion.startswith("8.6"):
            raise DumpTooOldException("Restore backup files from Plesk older than 8.6.0 is not supported")

        print "timestamp: " + datetime.datetime.now().strftime("%y%m%d%H%M")
        if info.server or len(info.clients) > 1 or len(info.domains) > 1:
            print "object_type: server"
            print "object_id: admin"
            print "object_guid: 00000000-0000-0000-0000-000000000000"
        else:
            if len(info.clients) == 1:
                if len(info.domains) == 1:
                    raise Exception("Could not get top-level object from backup file: '%s'. There are both top-level client and domain elements" % self.backupFile )
                print "object_type: client"
                print "object_id: " + info.clients[0].name
                print "object_guid: " + info.clients[0].guid
            elif len(info.domains) == 1:
                print "object_type: domain"
                print "object_id: " + info.domains[0].name
                print "object_guid: " + info.domains[0].guid
            else:
                raise Exception("Could not get top-level object from backup file: '%s'. Neither client nor domain element found" % self.backupFile)

commands.register("info", InfoCommand, 20)

# Local Variables:
# mode: python
# indent-tabs-mode: nil
# tab-width: 4
# End:
