# Copyright 1999-2014. Parallels IP Holdings GmbH. All Rights Reserved.
from __future__ import generators
import xml.dom.minidom
import os.path
import config
import random
import os

def normalizeStringCommon(str, strOrig, id, maxLen):
    dangerous_symbols = [' ', ':', '>', '<', '|', '&', '!']
    replacements = ['_', '-', '-gt-', '-lt-', '_', '-and-', '_']

    str_id = id
    if str_id == None:
        str_id = ""
    
    res = str
    for i in range(0, len(dangerous_symbols) - 1):
        res = res.replace(dangerous_symbols[i], replacements[i])

    oldRes = res
    res = ''
    for i in range(0, len(oldRes)):
        if oldRes[i].isalnum() or oldRes[i] == '-' or oldRes[i] == '.':
            res += oldRes[i]
        else:
            res += '_'
    
    if maxLen != None:
        if len(res) > maxLen:
            res = res[0:(maxLen - len(str_id) - 1)] + "_" + str_id

    return res

def normalizeString(str, id, maxLen):
    return normalizeStringCommon(str, str, id, maxLen)

def normalizeDomainName(str, id, maxLen):
    return normalizeStringCommon(str.encode('idna'), str, id, maxLen)
    

def childElements(x):
    for n in x.childNodes:
        if n.nodeType == xml.dom.minidom.Node.ELEMENT_NODE:
            yield n

# Python 2.2 has no x.attributes.has_key() and x.attributes[key]
def getAttr(x, name):
    for i in range(x.attributes.length):
        if x.attributes.item(i).name == name:
            return x.attributes.item(i).value
    return None

def getAttrStr(x, name):
    for i in range(x.attributes.length):
        if x.attributes.item(i).name == name:
            return x.attributes.item(i).value
    return ''

guidChars = {0:'0', 1:'1', 2:'2', 3:'3', 4:'4', 5:'5', 6:'6', 7:'7', 8:'8', 9:'9', 10:'a', 11:'b', 12:'c', 13:'d', 14:'e', 15:'f'}

def getGuidChar():
    number = random.randint(0, 15)
    return guidChars[number]

def generateGuid():
    ret = ''
    
    for i in range(0, 8):
        ret += getGuidChar()
    ret += '-'
    
    for i in range(0, 4):
        ret += getGuidChar()
    ret += '-'
    
    for i in range(0, 4):
        ret += getGuidChar()
    ret += '-'
    
    for i in range(0, 4):
        ret += getGuidChar()
    ret += '-'
    
    for i in range(0, 12):
        ret += getGuidChar()
    
    return ret

def getGuidFromParent(x, parent, suffix):
    idAttr = getAttrStr(x, 'id')
    if (idAttr == None) or (len(idAttr) == 0):
        return generateGuid()
    else:
        return parent.getGuid() + suffix + idAttr

def getAttrGuid(x, func):
    for i in range(x.attributes.length):
        if x.attributes.item(i).name == 'guid':
            if x.attributes.item(i).value != '':
                return x.attributes.item(i).value
            x.removeAttributeNode(x.attributes.item(i))
            break;
    guid = func()
    x.setAttribute('guid', guid)
    return guid

def getConstGuid(guid):
    return guid

def setAttr(x, name, value):
    for i in range(x.attributes.length):
        if x.attributes.item(i).name == name:
            x.attributes.item(i).value = value

def childElementsName(x, name):
    for n in childElements(x):
        if n.nodeName == name:
            yield n

def hasElement(x, name):
    for n in childElements(x):
        if n.nodeName == name:
            return 1
    return 0

def addChildElement(x, name):
    newNode = x.ownerDocument.createElement(name)
    x.appendChild(newNode)
    return newNode

def childTextNodes(x):
    for n in x.childNodes:
        if n.nodeType == xml.dom.minidom.Node.TEXT_NODE:
            yield n.data

def textContent(x):
    return "".join(childTextNodes(x))

def setTextContent(x, str):
    x.data = str
    for n in x.childNodes:
        if n.nodeType == xml.dom.minidom.Node.TEXT_NODE:
            n.data = str

def _ext(cid):
    if cid.endswith(":gzipped"):
        return ".gz"
    else:
        return ""

def _parseCustomButton(object, c, prefix=None):
    _processCid(object, c)
        
def _processCid(object, x):
    for c in childElementsName(x, "content"):
        for cid in childElementsName(c, "cid"):
            realName = "";
            for cFile in childElementsName(cid, "content-file"):
                realName = "".join((textContent(cFile), _ext(textContent(cFile))))
            object._addCid(cid, realName)
            object.pathFactory.registerElementObject(cid, object)

class PleskObject(object):
    def __init__(self, type_, id, name, guid, pathFactory):
        self.type_ = type_
        self.id = id
        self.name = name
        self.guid = guid
        self.cids = {}
        self.cidIds = {}
        self.pathFactory = pathFactory;
    
    def __repr__(self):
        return self.type_ + " " + self.name
    def objectDir(self):
        return self.type_ + "-" + self.name
    
    def objectFullDir(self):
        return self.objectDir()

    def _addCid(self, cid, name):
        self.cids[cid] = name
        self.cidIds[self.getCidId(cid)] = cid

    def getCidType(self, cidId):
        return self.cidIds[cidId].getAttribute("type")

    def getCid(self, cidId):
        return self.cidIds[cidId]

    def getCidId(self, cid):
        for contentFile in childElementsName(cid, 'content-file'):
            return textContent(contentFile)
        return ''
    
    def setCidFileName(self, cidId, fileName):
        for contentFile in childElementsName(self.cidIds[cidId], 'content-file'):
            setTextContent(contentFile, fileName)

    def getCids(self):
        return self.cids
    
    def getType(self):
        return self.type_
    
    def getName(self):
        return normalizeString(self.name, self.id, 25)
    
    def getPathDiff(self, obj):
        selfDir = self.objectFullDir()
        objDir = obj.objectFullDir()
        if not objDir.startswith(selfDir):
            return ""
        return objDir[len(selfDir):].lstrip("/")
    
    def needSave(self):
        return True
    
    def getGuidName(self):
        return self.getName()
    
    def getGuid(self):
        return self.guid


class PleskObjectWithParent(PleskObject):
    def __init__(self, type_, id, name, guid, parent, pathFactory):
        PleskObject.__init__(self, type_, id, name, guid, pathFactory)
        self.parent = parent
    
    def __repr__(self):
        return PleskObject.__repr__(self) + "@" + self.parent.name
    
    def objectDir(self):
        return PleskObject.objectDir(self) + "@" + self.parent.name
    
    def objectFullDir(self):
        dir = self.objectDir()
        if self.parent == None:
           return dir 
        if dir == "":
            return self.parent.objectFullDir()
        return os.path.join(self.parent.objectFullDir(), dir)
    
    def getParent(self):
        return self.parent

class Server(PleskObject):
    def __init__(self, x, pathFactory):
        PleskObject.__init__(self, "server", "", "", "00000000-0000-0000-0000-000000000000", pathFactory)

    def objectDir(self):
        return ""
    
    def getGuidName(self):
        return "admin"

class Admin(PleskObject):
    def __init__(self, x, pathFactory):
        PleskObject.__init__(self, 
                             "server", 
                             "", 
                             "", 
                             getAttrGuid(x, lambda: "00000000-0000-0000-0000-000000000000"), 
                             pathFactory)

    def objectDir(self):
        return ""
    
    def getGuidName(self):
        return "admin"

class Client(PleskObjectWithParent):
    def __init__(self, x, parent, pathFactory):
        PleskObjectWithParent.__init__(self,
                                       "client",
                                       getAttr(x, "id"),
                                       x.attributes["name"].value,
                                       getAttrGuid(x, lambda: generateGuid()),
                                       parent,
                                       pathFactory)
        _processCid(self, x)
        for c in childElementsName(x, "custom-button"):
            _parseCustomButton(self, c)

    def objectDir(self):
        return os.path.join("clients", self.getName());

class Domain(PleskObjectWithParent):
    def __init__(self, x, parent, pathFactory):
        PleskObjectWithParent.__init__(self,
                                       "domain",
                                       getAttr(x, "id"),
                                       x.attributes["name"].value,
                                       getAttrGuid(x, lambda: generateGuid()),
                                       parent,
                                       pathFactory)
        if parent != None:
            x.setAttribute('owner-guid', parent.getGuid())
			
        for n in childElements(x):
            if n.nodeName == "tomcat":
                self._parseTomcat(n)
            elif n.nodeName == "custom-button":
                _parseCustomButton(self, n)

    def _parseTomcat(self, t):
        for w in childElementsName(t, "webapp"):
            _processCid(self, w)

    def getName(self):
        return normalizeDomainName(self.name, self.id, 47)

    def objectDir(self):
        return os.path.join("domains", self.getName());

class PHosting(PleskObjectWithParent):
    def __init__(self, x, parent, pathFactory):
        PleskObjectWithParent.__init__(self,
                                       "phosting",
                                       "",
                                       "",
                                       getAttrGuid(x, lambda: parent.getGuid() + '_ph'),
                                       parent,
                                       pathFactory)
        x.setAttribute('owner-guid', parent.getGuid())
        _processCid(self, x)

        for c in childElementsName(x, "preferences"):
            for cc in childElementsName(c, "anonftp"):
                _processCid(self, cc)

        for s in childElements(x):
            if s.nodeName == "applications":
                self._parseSappInstalled(s)

    def _parseAnonymousFtp(self, a):
        _processCid(self, a)

    def _parseSappInstalled(self, node):
        for c in childElements(node): 
            if c.nodeName == "sapp-installed":
                _processCid(self, c)

    def objectDir(self):
        return "phosting"
    
    def getGuidName(self):
        return 'phosting'

class SubDomain(PleskObjectWithParent):
    def __init__(self, x, parent, pathFactory):
        PleskObjectWithParent.__init__(self,
                                       "subdomain",
                                       getAttr(x, "id"),
                                       x.attributes["name"].value,
                                       getAttrGuid(x, lambda: getGuidFromParent(x, parent, '_sb_')),
                                       parent,
                                       pathFactory)
        x.setAttribute('owner-guid', parent.getGuid())
        _processCid(self, x)

        for s in childElements(x):
            if s.nodeName == "applications":
                self._parseSappInstalled(s)

    def objectDir(self):
        return "subdomain"

    def getName(self):
        return normalizeDomainName(self.name, self.id, 25)

    def _parseSappInstalled(self, node):
        for c in childElements(node):
            if c.nodeName == "sapp-installed":
                _processCid(self, c)

class Mailname(PleskObjectWithParent):
    def __init__(self, x, parent, pathFactory):
        PleskObjectWithParent.__init__(self,
                                       "mailname",
                                       getAttr(x, "id"),
                                       x.attributes["name"].value,
                                       getAttrGuid(x, lambda: getGuidFromParent(x, parent, '_mn_')),
                                       parent,
                                       pathFactory)
        x.setAttribute('owner-guid', parent.getGuid())
        for n in childElementsName(x, "preferences"):
            for nn in childElements(n):
                if nn.nodeName == "mailbox":
                    self._parseMailbox(nn)
                elif nn.nodeName == "autoresponders":
                    self._parseAutoresponders(nn)
                elif nn.nodeName == "spamassassin":
                    self._parseSpamassassin(nn)
                elif nn.nodeName == "custom-button":
                    _parseCustomButton(self, nn)

    def _parseMailbox(self, n):
        _processCid(self, n)

    def _parseAutoresponders(self, n):
        _processCid(self, n)

    def _parseSpamassassin(self, n):
        _processCid(self, n)
                
    def objectDir(self):
        return os.path.join("mailnames", self.getName());
    
    def getName(self):
        return self.name + "@" + self.parent.getName()

    def needSave(self):
        return False


def _findDatabaseServer(x):
    host = None
    port = None
    for c in childElementsName(x, "db-server"):
        for cc in childElements(c):
            if cc.nodeName == "host":
                host = textContent(cc)
            if cc.nodeName == "port":
                port = textContent(cc)

    return (host, port)


class Database(PleskObjectWithParent):
    def __init__(self, x, parent, pathFactory):
        PleskObjectWithParent.__init__(self,
                                       "database",
                                       getAttr(x, "id"),
                                       x.attributes["name"].value,
                                       getAttrGuid(x, lambda: getGuidFromParent(x, parent, '_db_')),
                                       parent,
                                       pathFactory)
        x.setAttribute('owner-guid', parent.getGuid())
        (self.host, self.port) = _findDatabaseServer(x)
        self.dbType = x.attributes["type"].value
        _processCid(self, x)

    def __repr__(self):
        r = "database " + self.name
        if self.host:
            r += "@" + self.host
            if self.port:
                r += ":" + self.port
        if self.dbType:
            r += "/" + self.dbType
        return r
    
    def objectDir(self):
        return os.path.join("databases", self.getName());

class WebUser(PleskObjectWithParent):
    def __init__(self, x, parent, pathFactory):
        PleskObjectWithParent.__init__(self, "webuser", getAttr(x, "id"),
                                       x.attributes["name"].value, "", parent, pathFactory)
        _processCid(self, x)

    def objectDir(self):
        return "";
    
    def needSave(self):
        return False

def walkWebuser(visitor, x, parent, pathFactory):
    webuser = WebUser(x, parent, pathFactory)
    visitor.visit(webuser, x)

def walkSubDomain(visitor, x, parent, pathFactory):
    subDom = SubDomain(x, parent, pathFactory)
    visitor.visit(subDom, x)

    for c in childElements(x):
        walkDatabase(visitor, c, parent, pathFactory)

def walkPhosting(visitor, x, parent, pathFactory):
    phosting = PHosting(x, parent, pathFactory)

    for c in childElementsName(x, "subdomains"):
        for cc in childElementsName(c, "subdomain"):
            walkSubDomain(visitor, cc, phosting, pathFactory)

    for c in childElementsName(x, "webusers"):
        for cc in childElementsName(c, "webuser"):
            _processCid(phosting, cc)

    visitor.visit(phosting, x)

def walkMaillist(visitor, x, parent, pathFactory):
    _processCid(parent, x)

def walkMaillists(visitor, x, parent, pathFactory):
    for c in childElementsName(x, "maillist"):
            walkMaillist(visitor, c, parent, pathFactory)

def walkDatabase(visitor, x, parent, pathFactory):
    if x.nodeName == "database":
        database = Database(x, parent, pathFactory)
        visitor.visit(database, x)
    elif x.nodeName == "subdomain":
        return
    else:
        for sub in childElements(x):
            walkDatabase(visitor, sub, parent, pathFactory)

def walkMailname(visitor, x, parent, pathFactory):
    mailname = Mailname(x, parent, pathFactory)
    visitor.visit(mailname, x)

def walkMailsystem(visitor, x, parent, pathFactory):
    for c in childElementsName(x, "mailusers"):
        for cc in childElementsName(c, "mailuser"):
            walkMailname(visitor, cc, parent, pathFactory)

def walkDomain(visitor, x, parent, pathFactory):
    domain = Domain(x, parent, pathFactory)

    for c in childElements(x):
        if c.nodeName == "mailsystem":
            walkMailsystem(visitor, c, domain, pathFactory)
        elif c.nodeName == "maillists":
            walkMaillists(visitor, c, domain, pathFactory)
        elif c.nodeName == "phosting":
            walkPhosting(visitor, c, domain, pathFactory)

    for c in childElements(x):
        walkDatabase(visitor, c, domain, pathFactory)


    visitor.visit(domain, x)

def walkClient(visitor, x, parent, pathFactory):
    client = Client(x, parent, pathFactory)

    for c in childElementsName(x, "domains"):
        for cc in childElementsName(c, "domain"):
            walkDomain(visitor, cc, client, pathFactory)

    visitor.visit(client, x)

def walkServer(visitor, x, pathFactory):
    server = Server(x, pathFactory)

    for c in childElementsName(x, "server"):
        for cc in childElementsName(c, "keys"):
            for ccc in childElementsName(cc, "key"):
                _processCid(server, ccc)
        for cc in childElementsName(c, "application-vault"):
            for ccc in childElementsName(cc, "sapp-packages-pool"):
                for cccc in childElementsName(ccc, "sapp-package"):
                    _processCid(server, cccc)

    visitor.visit(server, x)

def walkAdmin(visitor, x, pathFactory):
    admin = Admin(x, pathFactory)

    for c in childElements(x):
        if c.nodeName == "clients":
            for cc in childElementsName(c, "client"):
                walkClient(visitor, cc, None, pathFactory)
        elif c.nodeName == "domains":
            for cc in childElementsName(c, "domain"):
                walkDomain(visitor, cc, None, pathFactory)

    visitor.visit(admin, x)

def walkDump(visitor, fh, pathFactory):
    convertedXmlPath = convertXml(fh)

    doc = xml.dom.minidom.parse(convertedXmlPath)
    root = doc.documentElement

    for c in childElements(root):
        if c.nodeName == "server":
            walkServer(visitor, root, pathFactory)
        elif c.nodeName == "admin":
            walkAdmin(visitor, c, pathFactory)
        elif c.nodeName == "client":
            walkClient(visitor, c, None, pathFactory)
        elif c.nodeName == "domain":
            walkDomain(visitor, c, None, pathFactory)
        elif c.nodeName == "dump-info":
            visitor.dumpInfo = c

    os.remove(convertedXmlPath)

def convertXml(fh):
    convertedXmlPath = "/tmp/dump-converted.xml"
    xsl = config.getWorkingDir() + "convert.xsl"
    
    oldXml = "/tmp/dump.xml"
    f = open(oldXml, 'w')
    try:
        while 1:
            str = fh.readline()
            if  str == None or str == "":
                break
            f.write(str)
    finally:
        f.close()
    
    command = 'xsltproc --output %s %s %s' % (convertedXmlPath, xsl, oldXml)
    os.system(command)
    os.remove(oldXml)
    return convertedXmlPath

# Local Variables:
# mode: python
# indent-tabs-mode: nil
# tab-width: 4
# End:
