# Copyright 1999-2014. Parallels IP Holdings GmbH. All Rights Reserved.
import sys

import plesk_log
from optparse import OptionParser
from optparse import SUPPRESS_HELP
from configstorage import PleskFileStorage
from configstorage import Formatter


def main():
    log = plesk_log.getLogger(__name__)
    parser = OptionParser()
    parser.description = 'Utility for monitoring configuration files changes'

    parser.add_option('--update', dest='filepath_update',
                        help=SUPPRESS_HELP)

    parser.add_option('--save-all', action='store_true',
                        help='Rotate and commit changes for protected files')

    parser.add_option('--list-all', action='store_true',
                        help='List all predefined configs with states')

    parser.add_option('-s', '--file-history', dest='filepath_history',
                        metavar='filepath',
                        help='''Get all revisions for config file path
                                (revision numbers may not be sequential)''')

    parser.add_option('-f', '--file', dest='filepath',
                        metavar='filepath',
                        help='''Get last file revision by filepath and
                                dump content to stdout''')

    parser.add_option('-r', '--revision', dest='revision', 
                        metavar='revision',
                        help='Get file content for specified revision',
                        type='string')

    (options, args) = parser.parse_args()
    log.debug(options)

    try:
        if len( filter(lambda x: x is not None, options.__dict__.values()) ) > 1:
            raise Exception('All options are mutually exclusive. Please specify only one.')

        pfs = PleskFileStorage()

        if options.save_all:
            pfs.storeAllFiles()
        elif options.filepath_update:
            pfs.setActor('plesk')
            pfs.storeFile(options.filepath_update)
        elif options.filepath_history:
            print Formatter.formatHistory(pfs.getFileHistory(options.filepath_history))
        elif options.filepath:
            sys.stdout.write(pfs.getLastFileRevisionByFilePath(options.filepath))
        elif options.revision:
            sys.stdout.write(pfs.getFileByRevision(options.revision))
        elif options.list_all:
            print Formatter.formatList(pfs.getAllFilesStats())
        else:
            parser.print_help()
    except Exception, e:
        log.error('%s', e)
        sys.exit(1)


if __name__ == '__main__':
    main()

# vim: ts=4 sts=4 sw=4 et :
