// Copyright 1999-2014. Parallels IP Holdings GmbH. All Rights Reserved.
/* JavaScript Widgets */

Jsw = {

    version: '1.0',
    baseUrl: '',
    ctxIdName: '',
    ctxId: '',

    _registredComponents: null,
    _initOnReady: false,
    _firedOnReady: false,
    _onReadyEvents: null,

    priority: {
        high: 'high',
        normal: 'normal',
        low: 'low'
    },

    registerComponent: function(identity, component) {
        if (!Jsw._registredComponents) {
            Jsw._registredComponents = new Hash();
        }

        Jsw._registredComponents.set(identity, component);
    },

    getComponent: function(identity) {
        return Jsw._registredComponents.get(identity);
    },

    clearStatusMessages: function() {
        var contentElement = $('content');

        ['.msg-error', '.msg-info', '.msg-warning'].each(function(msgClass) {
            contentElement.select(msgClass).each(function(messageElement) {
                messageElement.remove();
            });
        });
    },

    addStatusMessage: function(type, message, params) {
        var params = ('undefined' != typeof params) ? params : {};

        var elementParams = { 'class': 'msg-box msg-'  + type };
        if (params['id']) {
            elementParams['id'] = params['id'];
        }

        if (params['title']) {
            message = '<b>' + params['title'] + ':</b> ' + message;
        }

        var statusElement = new Element('div', elementParams);
        statusElement.update(
            '<div><div><div><div><div>' +
            '<div class="msg-content">' + message + '</div>' +
            '</div></div></div></div></div>'
        );

        if (params['closable']) {
            var closeElement = new Element('span', { 'class': 'close' });
            closeElement.observe('click', params['onClose']);
            statusElement.down('.msg-content').insert({top: closeElement});
        }

        try {
        	var target;
        	if (params['renderTo']) {
        		target = params['renderTo'];
        	} else {
        		target = 'main';
        	}
        	var mode = 'top';
        	if (params['renderMode']) {
        		mode = params['renderMode'];
        	}
            var insertion = {};
            insertion[mode] = statusElement;

        	$(target).insert(insertion);
        } catch (e) {
            alert(type + ' | ' + message);
        }
    },

    onReady: function(priority, callback) {
        if ('function' == typeof priority) {
            callback = priority;
            priority = Jsw.priority.normal;
        }

        if (Jsw._firedOnReady) {
            callback();
            return;
        }

        if (!Jsw._initOnReady) {
            Event.observe(window, 'load', Jsw._fireOnReady);
            Jsw._initOnReady = true;
            Jsw._onReadyEvents = [];
        }

        if ('undefined' == typeof Jsw._onReadyEvents[priority]) {
            Jsw._onReadyEvents[priority] = [];
        }

        Jsw._onReadyEvents[priority].push(callback);
    },

    _fireOnReady: function() {
        var eventsToFire = [];
        var predefinedPriorities = [Jsw.priority.high, Jsw.priority.normal, Jsw.priority.low];

        predefinedPriorities.each(function(priority) {
            if ('undefined' != typeof Jsw._onReadyEvents[priority]) {
                eventsToFire = eventsToFire.concat(Jsw._onReadyEvents[priority]);
            }
        });

        Jsw._onReadyEvents.each(function(events, priority) {
            if (-1 == predefinedPriorities.indexOf(priority)) {
                eventsToFire = eventsToFire.concat(events);
            }
        });

        eventsToFire.each(function(callback) {
            callback();
        });

        Jsw._firedOnReady = true;
    },

    namespace: function(ns) {
        var nsParts = ns.split('.');
        var root = window;

        for (var partIndex = 0; partIndex < nsParts.length; partIndex++) {
            if ('undefined' == typeof root[nsParts[partIndex]]) {
                root[nsParts[partIndex]] = {};
            }

            root = root[nsParts[partIndex]];
        }
    },

    prepareUrl: function(url) {
        if ('undefined' == typeof url || null === url) {
            return null;
        }

        url = Jsw.prepareCtxUrl(url);

        if (
                url.startsWith('/services/') ||
                url.startsWith('/server/spamassassin') ||
                url.startsWith('/server/mail_') ||
                url.startsWith('/plesk/') ||
                (Jsw.baseUrl != '' && url.startsWith(Jsw.baseUrl + '/')) ||
                (!url.startsWith('/') && (url.substr(0, 2) != '^/')) ||
                url.startsWith('/modules/')
        ) {
            return url;
        }
        if (url.substr(0, 2) == '^/') {
            return url.toString().substr(1);
        }

        return Jsw.baseUrl + url;
    },

    prepareCtxUrl: function(url, id) {
        if (('undefined' == typeof id || null === id) && Jsw.ctxId) {
            id = Jsw.ctxId;
        }

        if (! id
            || url.startsWith('#')
            || url.include(Jsw.ctxIdName+'=')
            || url.startsWith('javascript:')
            || (url.startsWith('http') && !url.include(window.location.hostname))) {
            return url;
        }

        var fragment = '';
        if (url.include('#')) {
            var splitUrl = url.split('#', 2);
            url = splitUrl[0];
            fragment = '#' + splitUrl[1];
        }
        url = Jsw.addUrlParams(url, Jsw.ctxIdName+'=' + encodeURIComponent(id) + fragment);
        return url;
    },

    addUrlParams: function(url, params) {
        return url + (url.include('?') ? '&' : '?') + params;
    },

    clonePosition: function(element, source, options) {
        // workaround for: https://prototype.lighthouseapp.com/projects/8886/tickets/90-cloneposition-problem-in-ie7

        try {
            element.clonePosition(source, options);
        } catch (e) {
            element.clonePosition(source, options);
        }
    },

    showInternalError: function(message) {
        Jsw.clearStatusMessages();
        message = 'Internal error.<br/>' + message;
        Jsw.addStatusMessage('error', message);
    }
};

Jsw.keyCode = {
    ESC: 27,
    ENTER: 13,
    LEFT_ARROW: 37,
    UP_ARROW: 38,
    RIGHT_ARROW: 39,
    DOWN_ARROW: 40
};

// Monkey patching Prototype for proper handling of session expiration
Ajax.PatchedRequest = Class.create(Ajax.Request, {
    initialize: function($super, url, options) {
        options = Object.extend({
            on400: function() {
                top.window.location.reload();
            },
            on500: function(transport) {
                Jsw.showInternalError(transport.responseText);
            },
            on0: function() {
                // do nothing in case of internal/network error
                return false;
            }
        }, options || {});

        $super(url, options);
    },
    request: function($super, url) {
        $super(Jsw.prepareCtxUrl(url));
    }
});

Ajax.Request.prototype = Ajax.PatchedRequest.prototype;

// cross-site request forgery protection
(function() {
    Ajax.Base.prototype.initialize = Ajax.Base.prototype.initialize.wrap(function() {
        var args = $A(arguments), proceed = args.shift();
        args[0] = args[0] || {};
        var options = args[0];
        if (!(options.method && options.method.toUpperCase() == "GET")) {
            var addParamToOptions = function(param, value) {
                if (Object.isString(options.parameters)) {
                    if (options.parameters.indexOf(param) == -1) {
                        options.parameters += '&' + param + '=' + value;
                    }
                } else if (Object.isHash(options.parameters)) {
                    options.parameters = options.parameters.toObject();
                    options.parameters[param] = value;
                } else {
                    options.parameters = options.parameters || {};
                    options.parameters[param] = value;
                }
            };

            if ($('forgery_protection_token')) {
                addParamToOptions('forgery_protection_token', encodeURIComponent($('forgery_protection_token').content));
            }
            if (Jsw.ctxId) {
                addParamToOptions(Jsw.ctxIdName, Jsw.ctxId);
            }
        }
        proceed.apply(null, args);
    });
})();

Jsw.Component = Class.create({

    _tag: 'div',

    initialize: function(config) {
        this._initConfiguration(config);
        this._registerComponent();

        this._initComponentElement();

        if (this._autoRender && ((null != $(this._renderTargetId)) || (null != $(this._applyTargetId)))) {
            this.render();
        }
    },

    _initConfiguration: function(config) {
        this.initialConfig = config;
        this._config = this.initialConfig;

        this._cls = this._getConfigParam('cls', '');
        this._attrs = this._getConfigParam('attrs', {});
        this._wrapperClass = this._getConfigParam('wrapperClass', '');
        this._autoRender = this._getConfigParam('autoRender', true);
        this._renderTargetId = this._getConfigParam('renderTo', null);
        this._applyTargetId = this._getConfigParam('applyTo', null);
        this._locale = this._getConfigParam('locale', {});

        this._id = (this._applyTargetId)
            ? this._applyTargetId
            : this._getConfigParam('id', null);

        if (null === this._id) {
            this._id = this._getRandomId();
            this._idIsRandom = true;
        }

        this._tag = (this._applyTargetId)
            ? $(this._applyTargetId).tagName
            : this._getConfigParam('tag', this._tag);
    },

    _registerComponent: function() {
        Jsw.registerComponent(this._id, this);
    },

    _initComponentElement: function() {
        if (this._applyTargetId) {
            this._componentElement = $(this._applyTargetId);
            return ;
        }

        var attrs = Object.extend(this._attrs, {
            'id': this._id,
            'class': this._cls
        });

        this._componentElement = new Element(this._tag, attrs);
    },

    _initDisablerOverlay: function() {
        this._disablerOverlay = new Element('div');

        this._disablerOverlay.setStyle({
            display: 'none',
            zIndex: this._componentElement.style.zIndex + 1000
        });

        this._disablerOverlay.addClassName('content-blocker');
        this._componentElement.appendChild(this._disablerOverlay);
    },

    addEventObserver: function(eventName, handler) {
        if ('undefined' != typeof handler) {
            this._componentElement.observe(eventName, handler);
        }
    },

    removeEventObserver: function(eventName, handler) {
        if ('undefined' != typeof handler) {
            this._componentElement.stopObserving(eventName, handler);
        }
    },

    _hasConfigParam: function(name)
    {
        return 'undefined' != typeof this._config[name];
    },

    _getConfigParam: function(name, defaultValue) {
        return this._hasConfigParam(name)
            ? this._config[name]
            : defaultValue;
    },

    _getRandomId: function() {
        var id = 'gen-id-' + this._getRandomNumber();

        while(null != $(id)) {
            id = 'gen-id-' + this._getRandomNumber();
        }

        return id;
    },

    _getRandomNumber: function() {
        return Math.floor(1000000*Math.random());
    },

    _addEvents: function() {
    },

    _addTooltips: function() {
        var description = this._getConfigParam('description');
        if  (description) {
            this._tooltip = Jsw.Tooltip.init(this._componentElement, {text: description});
        }
    },

    _updateComponentElement: function(content) {
        if (this._componentElement.empty()) {
            this._componentElement.update(content);
            return ;
        }

        var oldElements = this._componentElement.childElements();

        Element.insert(this._componentElement.firstChild, {before: content});

        oldElements.each(function(oldElement) {
            if (
                'undefined' == typeof oldElement.id ||
                '' == oldElement.id
            ) {
                return ;
            }

            var newElement = this._componentElement.select('#' + oldElement.id).first();

            if (newElement) {
                newElement.replace(oldElement);
            }
        }, this);
    },

    getWrapperClass: function() {
        return this._wrapperClass;
    },

    getComponentHtml: function() {
        var wrapper = new Element('span');
        wrapper.update(this._componentElement);

        return wrapper.innerHTML;
    },

    getId: function() {
        return this._id;
    },

    setRenderTarget: function(targetId) {
        this._renderTargetId = targetId;
    },

    getRenderTarget: function() {
        return this._renderTargetId;
    },

    show: function() {
        this._componentElement.show();
    },

    hide: function() {
        this._componentElement.hide();
    },

    render: function() {
        if ($(this._renderTargetId)) {
            var insertion = {};
            insertion[this._getConfigParam('renderMode', 'bottom')] = this._componentElement;
            $(this._renderTargetId).insert(insertion);
        }

        this._addEvents();
        this._addTooltips();
    }

});

Jsw.Hint = Class.create(Jsw.Component, {
    _tag: 'p',

    _initConfiguration: function($super, config) {
        $super(config);
        this._cls = this._getConfigParam('cls', '');
    },

    _initComponentElement: function($super) {
        $super();

        this.setHint(
            this._getConfigParam('hint', '')
        );

        if (this._getConfigParam('expandable', false)) {
            var contentElement = this._componentElement;
            var componentElement = new Element('div', {'class': 'screen-description'});
            var wrapElement = new Element('div', {'class': 'screen-description-wrap'});
            var btnShowMore = new Element('span', {'class': 'screen-description-control'});
            btnShowMore.observe('click', function() { this.up('.screen-description').addClassName('expanded'); });
            wrapElement.insert(contentElement);
            wrapElement.insert(btnShowMore);
            this._componentElement = componentElement.update(wrapElement);
            this._screenControl = function() {
                if (componentElement.getWidth() > wrapElement.getWidth()) {
                    btnShowMore.addClassName('off');
                } else {
                    btnShowMore.removeClassName('off');
                }
            };
        }
    },

    _addEvents: function($super) {
        $super();

        if (this._screenControl) {
            Event.observe(window, 'resize', this._screenControl);
            this._screenControl();
        }
    },

    setHint: function(hint) {
        this._componentElement.update(hint);
    }
});

Jsw.Button = Class.create(Jsw.Component, {

    _tag: 'a',

    _initConfiguration: function($super, config) {
        $super(config);

        this._handler = this._getConfigParam('handler', false);
        if (Object.isString(this._handler)) {
            eval('this._handler = ' + this._handler);
        }
        this._cls = this._getConfigParam('cls', 'b-btn');
        this._disabledClass = this._getConfigParam('disabledClass', 'btn-disabled');
        this._isNew = this._getConfigParam('isNew', false);
        this._newTitle = this._getConfigParam('newTitle', '');
    },

    _initComponentElement: function($super) {
        $super();

        this._componentElement.update(
            '<i><i><i><span>'
            + this._config['title']
            + (this._isNew ? ' <span class="badge-new">' + this._newTitle + '</span>' : '')
            + '</span></i></i></i>'
        );

        this._initVisibility();
    },

    _initVisibility: function() {
        this._disabled = this._getConfigParam('disabled', false);

        if (this._disabled) {
            this._disabled = false;
            this.disable();
        } else {
            this._disabled = true;
            this.enable();
        }
    },

    _addAttribute: function(name, value) {
        if ('undefined' != typeof value) {
            this._componentElement.writeAttribute(name, value);
        }
    },

    setTooltip : function(tooltipText) {
        if (this._tooltip) {
            this._tooltip.setText(tooltipText);
        } else {
            this._tooltip = Jsw.Tooltip.init(this._componentElement, {text: tooltipText});
        }
    },

    enable: function() {
        if (!this._disabled) {
            return ;
        }
        this._disabled = false;

        var classNames = this._config['addCls'] || '';
        if (!Object.isArray(classNames)) {
            classNames = [classNames];
        }
        classNames.each(function(item) {
            this._componentElement.addClassName(item);
        }, this);
        classNames.each(function(item) {
            this._componentElement.removeClassName(item + '-disabled');
        }, this);
        this._componentElement.removeClassName(this._disabledClass);

        this._addAttribute('href', Jsw.prepareUrl(this._getConfigParam('href', 'javascript:;')));
        this._addAttribute('target', this._getConfigParam('target', null));
        if (this._handler) {
            this.addEventObserver('click', this._handler);
        }
    },

    disable: function() {
        if (this._disabled) {
            return ;
        }
        this._disabled = true;

        this._componentElement.addClassName(this._disabledClass);

        var classNames = this._config['addCls'];
        if (!Object.isArray(classNames)) {
            classNames = [classNames];
        }
        classNames.each(function(item) {
            this._componentElement.addClassName(item + '-disabled');
        }, this);
        classNames.each(function(item) {
            this._componentElement.removeClassName(item);
        }, this);
        this._componentElement.writeAttribute('href', null);
        this._componentElement.writeAttribute('target', null);
        if (this._handler) {
            this.removeEventObserver('click', this._handler);
        }
    }
});

Jsw.SmallButton = Class.create(Jsw.Button, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._cls = this._getConfigParam('cls', 's-btn');
    }

});

Jsw.BigButton = Class.create(Jsw.Button, {

    _initConfiguration: function($super, config) {
        $super(config);
        this._icon = this._getConfigParam('icon', null);
    },

    _initComponentElement: function($super) {
        $super();

        this._componentElement.update(
            '<span class="db-content">' +
                '<span class="db-icon"><img src="' + this._icon + '"></span>' +
                '<span class="db-name">' + this._config['title'] + '</span>' +
                '<span class="db-info">' + this._config['description'] + '</span>' +
            '</span>'
        );

        this._initVisibility();
    }
});

Jsw.submit = function(element) {
    element = $(element);
    if (element.tagName.toLowerCase() == 'form') {
        element.submit();
    } else {
        element.up("form").submit();
    }
};

Jsw.redirect = function(action, data, target) {
    if (data || target) {
        Jsw.FormRedirect.redirect(Jsw.prepareUrl(action), 'get', data, target);
    } else {
        $(document).location.href = Jsw.prepareUrl(action);
  }
};

Jsw.redirectPost = function(action, data, target) {
    Jsw.FormRedirect.redirect(Jsw.prepareUrl(action), 'post', data, target);
};

Jsw.FormRedirect = {
    redirect: function(action, method, data, target) {

    // workaround for http://code.google.com/p/chromium/issues/detail?id=10671
    var randomId = Math.floor(1000000*Math.random());
    action = Jsw.addUrlParams(action, '_randomId=' + randomId);

        var form = new Element(
            'form',
            {
                method: method,
                action: action
            }
        );
        if (target) {
            form.target = target;
        }
        if (data) {
            var elements = Jsw.FormRedirect.subToElements(data);
            $A(elements).each(function (element) {
                form.insert(element);
            });
        }

        if ($('forgery_protection_token')) {
            form.insert(new Element('input', {
                type: 'hidden',
                name: 'forgery_protection_token',
                value: $('forgery_protection_token').content
            }));
        }
        if (Jsw.ctxId) {
            form.insert(new Element('input', {
                type: 'hidden',
                name: Jsw.ctxIdName,
                value: Jsw.ctxId
            }));
        }

        $(document.body).insert(form);
        form.submit();
    },

    stringToElements: function(value, prefix)
    {
        return [new Element('input', {
            type: 'hidden',
            name: prefix,
            value: value
        })];
    },

    booleanToElements: function(value, prefix)
    {
        return [new Element('input', {
            type: 'hidden',
            name: prefix,
            value: value ? 1 : 0
        })];
    },

    arrayToElements: function(data, prefix)
    {
        var elements = [];
        $A(data).each(function(item) {
            elements = elements.concat(Jsw.FormRedirect.subToElements(item, '', prefix));
        });
        return elements;
    },

    hashToElements: function(data, prefix)
    {
        var elements = [];
        $H(data).each(function(pair) {
            elements = elements.concat(Jsw.FormRedirect.subToElements(pair.value, pair.key, prefix));
        });
        return elements;
    },

    subToElements: function(value, key, prefix)
    {
        var elements = [];
        var name = prefix
            ? prefix + '[' + key + ']'
            : key;
        if (Object.isString(value) || Object.isNumber(value)) {
            elements = elements.concat(Jsw.FormRedirect.stringToElements(value, name));
        } else
        if ('boolean' == typeof value) {
            elements = elements.concat(Jsw.FormRedirect.booleanToElements(value, name));
        } else
        if(Object.isArray(value)) {
            elements = elements.concat(Jsw.FormRedirect.arrayToElements(value, name));
        } else {
            elements = elements.concat(Jsw.FormRedirect.hashToElements(value, name));
        }
        return elements;
    }
}

Jsw.CommandButton = Class.create(Jsw.Component, {
    _initConfiguration: function($super, config) {
        $super(config);

        this._handler = this._getConfigParam('handler', false);
        if (Object.isString(this._handler)) {
            eval('this._handler = ' + this._handler);
        }

        this._value = this._config['value'] || ''
        this._tabIndex = this._getConfigParam('tabIndex', null);
    },

    _initComponentElement: function($super) {
        $super();

        var buttonElement = new Element('button', {
            type: 'button',
            value: this._value,
            name: this._config['name']
        }).update(this._config['title']);

        if (null != this._tabIndex) {
            buttonElement.writeAttribute('tabindex', this._tabIndex);
        }

        this._updateComponentElement(buttonElement);

        this._initVisibility();
    },

    _initVisibility: function() {
        this._disabled = this._getConfigParam('disabled', false);

        if (this._disabled) {
            this._disabled = false;
            this.disable();
        } else {
            this._disabled = true;
            this.enable();
        }
    },

    enable: function() {
        if (!this._disabled) {
            return ;
        }

        this._disabled = false;
        if ('send' == this._config['name']) {
            this._componentElement.addClassName('action');
        }
        this._componentElement.removeClassName('disabled');
        this._componentElement.disabled = false;
        if (this._handler) {
            this.addEventObserver('click', this._handler);
        }
    },

    disable: function() {
        if (this._disabled) {
            return ;
        }

        this._disabled = true;
        this._componentElement.disabled = true;
        if ('send' == this._config['name']) {
            this._componentElement.removeClassName('action');
        }
        this._componentElement.addClassName('disabled');
        if (this._handler) {
            this.removeEventObserver('click', this._handler);
        }
    }
});

Jsw.namespace('Jsw.form');

Jsw.form.DisplayField = Class.create(Jsw.Component, {
    _tag: 'div',

    _initConfiguration: function($super, config) {
        $super(config);
        this._cls = this._getConfigParam('cls', 'form-row');
        this._valueRenderer = this._getConfigParam('valueRenderer', null);
    },

    _initComponentElement: function($super) {
        $super();

        var values = ('function' == typeof this._valueRenderer)
        	? this._valueRenderer()
        	: this._getConfigParam('value', '');

        values = (typeof values == 'string') ? [values] : values;

        var valuesElements = '';
        values.each(function(value) {
            valuesElements += '<div class="text-value">' + value + '</div>';
        });

        if (this._getConfigParam('singleRow', false)) {
        	if (1 == values.length) {
	        	valuesElements = values.valueOf();
	        }
    		this._componentElement.update('<div class="single-row">' + valuesElements + '</div>');
    	} else {
    		this._componentElement.update(
    			'<div class="field-name">' + this._getConfigParam('fieldLabel', '') + '</div>' +
    	        '<div class="field-value">' + valuesElements + '</div>'
    	    );
    	}
    }
});

Jsw.FormAjax = Class.create(Jsw.Component, {
    _sendButtonTitle: '',

    _initConfiguration: function($super, config) {
        $super(config);
        this._sendButtonId = this._getConfigParam('sendButtonId', 'btn-send');
        this._cancelButtonId = this._getConfigParam('cancelButtonId', 'btn-cancel');
        this._submitHandler = this._getConfigParam('submitHandler', function() { return true; });
    },

    disable: function() {
        var sendButtonWrapper = $(this._sendButtonId);
        if (sendButtonWrapper) {
            var sendButton = sendButtonWrapper.down('button');
            sendButton.disabled = true;
            sendButtonWrapper.addClassName('disabled');
            if (!this._sendButtonTitle) {
                this._sendButtonTitle = sendButton.innerHTML;
            }
            sendButton.update('<span class="wait">' + this._config['waitButtonTitle'] + '</span>');
        }

        var cancelButtonWrapper = $(this._cancelButtonId);
        if (cancelButtonWrapper) {
            cancelButtonWrapper.down('button').disabled = true;
            cancelButtonWrapper.addClassName('disabled');
        }
    },

    enable: function() {
        var sendButtonWrapper = $(this._sendButtonId);
        if (sendButtonWrapper) {
            sendButton = sendButtonWrapper.down('button');
            sendButton.disabled = false;
            sendButton.update(this._sendButtonTitle);
            sendButtonWrapper.removeClassName('disabled');
        }

        var cancelButtonWrapper = $(this._cancelButtonId);
        if (cancelButtonWrapper) {
            cancelButtonWrapper.down('button').disabled = false;
            cancelButtonWrapper.removeClassName('disabled');
        }
    },

    _initComponentElement: function($super) {
        this._componentElement = $(this._id);
        this._initDisablerOverlay();

        var firstElement = this._componentElement.findFirstElement();

        if ('undefined' != typeof firstElement) {
            try {
                firstElement.focus();
            } catch (e) {}
        }

        this._submitInProgress = false;
    },

    _addEvents: function() {
        this._componentElement._formSubmit = this._componentElement.submit;
        this._componentElement.submit = this._onSubmit.bind(this);
        this._componentElement.observe('submit', this._onSubmitEvent.bind(this));
        this._addChoiceRadioButtonsOnClickEvent();
    },

    _addChoiceRadioButtonsOnClickEvent: function() {
        $$('input[type="radio"]').each(function(element) {
            if (!$(element.id).up('div.choice-block')) {
                return;
            }
            element.observe('click', function() {
                $(this.id).up('div.choice-block').select('span').each(function(element) {
                    element.removeClassName('selected');

                    if ('0' == $(this.id).value) {
                        element.removeClassName('no');
                    }
                }, this);

                $(this.id).up('span').addClassName('selected');

                if ('0' == $(this.id).value) {
                    $(this.id).up('span').addClassName('no');
                }
            });
        });
    },

    _onSubmitEvent: function(event) {
        this._onSubmit();
        Event.stop(event);
        return false;
    },

    _onSubmit: function() {
        if (false === this._submitHandler()) {
            return false;
        }

        if (this._submitInProgress) {
            return false;
        } else {
            this._submitInProgress = true;
        }

        this.disable();

        if (this._componentElement.enctype == "multipart/form-data") {
            this._componentElement._formSubmit();
            return true;
        }

        var actionUrl = this._componentElement.getAttribute('action');

        if (!actionUrl) {
            actionUrl = $(document).location.href;
        }

        // remove hash symbol (and the following) cause this will encoded and lead to problems
        actionUrl = actionUrl.replace(/#.*$/, "");

        var params = this._componentElement.serialize();
        var context = this;
        var request = new Ajax.Request(
            actionUrl,
            {
                method:'post',
                parameters: params,
                onSuccess: this._onSuccess.bind(this),
                onFailure: this._onFailure.bind(this)
            }
        );
        return true;
    },

    _onFailure: function(req) {
        this._submitInProgress = false;
        this._clearMessages();

        if (504 == req.status) {
            Jsw.addStatusMessage('error', this._config['timeoutMessage']);
        } else {
            Jsw.showInternalError(req.responseText);
        }

        this.enable();
    },

    _onSuccess: function(req) {
        this._submitInProgress = false;
        Jsw.clearStatusMessages();

        try {
            var response = req.responseText.evalJSON();
            this._processForm(response);
        } catch (e) {
            this._clearMessages();
            this.enable();

            // show error, if request wasn't aborted by user
            if (0 != req.status) {
                Jsw.showInternalError(req.responseText);
            }
        }
    },

    _processForm: function(response) {
        this._processResponseRedirect(response);

        if (!response.redirect) {
            this._clearMessages();
            this._processResponseStatus(response.status);
            this._processResponseStatusMessages(response.statusMessages);
            this._processResponseFormMessages(response.formMessages);

            if (this._hasFieldErrors) {
                this._showFieldErrorArea();
            }
            this.enable();
        }
    },

    _showFieldErrorArea: function() {
        var errorsElements = this._componentElement.select('.field-errors');
        var firstError = null;

        errorsElements.each(function(errorElement) {
            if (errorElement.visible()) {
                firstError = errorElement;
            }
        });

        var hiddenContainerId = null;

        firstError.ancestors().each(function(element) {
            if (!element.visible()) {
                hiddenContainerId = element.id;
            }
        });

        if (!hiddenContainerId) {
            return ;
        }

        // in case of tabbed form we must switch to corresponding tab
        var tabsBar = Jsw.getComponent('form-tab-buttons');

        if (tabsBar) {
            tabsBar.switchTab(hiddenContainerId);
        }
    },

    _processResponseStatus: function (status) {
        if ('success' != status) {
            this._hasErrors = true;
        }
    },

    _clearMessages: function() {
        //clear field messages
        try {
            this._componentElement.select('.field-errors').each(function(errors){
                errors.hide();
                errors.up('.form-row').removeClassName('error');
                errors.select('.error-hint').each(function(error) {
                    error.remove();
                });
            });
        } catch (e) {
        }

        this._hasErrors = false;
        this._hasFieldErrors = false;
    },

    _processResponseStatusMessages: function (messages) {
        $A(messages).each(function(value) {
            this._addFormMessage(value.status, value.content, value.title);
        }, this);
    },

    _processResponseFormMessages: function (messages) {
        this._processFieldMessages(messages, []);
    },

    _addFieldMessage: function(errors, message) {
        errors.up('.form-row').addClassName('error');
        errors.insert({bottom: '<span class="error-hint">' + message.escapeHTML() + '</span>' });
        errors.show();
    },

    _processFieldMessage: function(prefix, name, message) {
        var fieldErrors;
        var formElement = this._componentElement.down('#' + prefix.join('-'));
        fieldErrors = formElement ? formElement.next('.field-errors') : null;
        if (!fieldErrors) {
            fieldErrors = formElement ? formElement.up('.form-row').down('.field-errors') : null;
        }
        if (!fieldErrors) {
            fieldErrors = this._componentElement.down('#' + prefix.join('-') + '-form-row').select('.field-errors').last();
        }
        this._addFieldMessage(fieldErrors, message);
        this._hasFieldErrors = true;
    },

    _processFieldMessages: function(messages, prefix) {
        if (Object.isArray(messages)) {
            $A(messages).each(function(message) {
                if (Object.isString(message)) {
                    this._processFieldMessage(prefix, 'error', message);
                } else {
                    prefix.push(name);
                    this._processFieldMessages(message, prefix);
                    prefix.pop();
                }
            }, this);
        } else {
            $H(messages).each(function(pair) {
                if (Object.isString(pair.value)) {
                    this._processFieldMessage(prefix, pair.key, pair.value);
                } else {
                    prefix.push(pair.key);
                    this._processFieldMessages(pair.value, prefix);
                    prefix.pop();
                }
            }, this);
        }
    },

    _addFormMessage: function (type, message, title) {
        Jsw.addStatusMessage(type, message, { 'title': title });
    },

    _processResponseRedirect: function (response) {
        if (response.redirect) {
            if (response.postData) {
                Jsw.redirectPost(response.redirect, response.postData, response.target);
            } else {
                Jsw.redirect(response.redirect, null, response.target);
            }
        }
    }

});

Jsw.Container = Class.create(Jsw.Component, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._contentAreaId = this._id;

        this._initItems();
    },

    _initItems: function() {
        this._items = this._getConfigParam('items', null);

        if (null != this._items) {
            this._items.each(function(item, index) {
                if (item && ('undefined' != typeof item.componentType)) {
                    // recreate proper object defined by componentType
                    eval('var resultItem = new ' + item.componentType + '(item);');
                    this._items[index] = resultItem;
                }
            }, this);
        }
    },

    _renderItems: function() {
        if (null != this._items) {
            this._items.each(function(item) {
                if (item) {
                    this._renderItem(item);
                }
            }, this);
        }
    },

    _renderItem: function(item) {
        item.setRenderTarget(this._contentAreaId);
        item.render();
    },

    render: function($super) {
        $super();

        this._renderItems();
    },

    getItems: function() {
        return this._items;
    }

});

Jsw.ListContainer = Class.create(Jsw.Container, {

    _tag: 'ul',

    _renderItem: function(item) {
        var renderTargetId = null;
        var isEmptyItem = Object.isArray(item) && !item.size();

        if (!isEmptyItem) {
            renderTargetId = this._id + '-item-' + item.getId();
        }

        var cls = this._getConfigParam('itemClass', '');

        if (this._getConfigParam('renderLastItemClass', '') && this._items.last() == item) {
            cls = 'last';
        }

        if (isEmptyItem) {
            cls += ' ' + this._getConfigParam('emptyItemClass', '');
        }

        if (cls) {
            cls = ' class="' + cls + '"';
        }

        $(this._contentAreaId).insert('<li ' + (renderTargetId ? 'id="' + renderTargetId + '"' : '') + cls + '></li>');

        if (!isEmptyItem) {
            item.setRenderTarget(renderTargetId);
            item.render();
        }
    }

});

Jsw.namespace('Jsw.layout');

Jsw.layout.TwoColumns = Class.create(Jsw.Container, {

    _initConfiguration: function($super, config) {
        $super(config);
        this._cls = this._getConfigParam('cls', 'two-cols-block clearfix');
        this._columnCls = 'first-col';
    },

    _initComponentElement: function() {
        this._componentElement = new Element('div', {
                'class': this._cls
            }).update(
                '<div id="' + this._contentAreaId + '" class="columns-wrapper">' +
                '</div>'
            );
    },

    _renderItem: function(item) {
        var renderTargetId = this._id + '-item-' + item.getId();
        $(this._contentAreaId).insert(
            '<div class="' + this._columnCls + '"><div class="column-box" id="' + renderTargetId +'"></div></div>'
        );
        item.setRenderTarget(renderTargetId);
        item.render();

        this._columnCls = 'second-col';
    }

});

Jsw.Dashboard = Class.create(Jsw.Container, {

    _initConfiguration: function($super, config) {
        $super(config);
        this._buttonsAreaId =this._id + '-buttons-area';
    },

    _initComponentElement: function() {
        this._componentElement = new Element('div', {
                'class': this._cls
            }).update(
                '<ul id="' + this._buttonsAreaId + '" class="d-buttons">' +
                    this._getConfigParam('html', '') +
                '</ul>'
            );
    },

    _renderItem: function(item) {
        var renderTargetId = this._id + '-button-' + item.getId();
        $(this._buttonsAreaId).insert('<li id="' + renderTargetId +'"></li>');
        item.setRenderTarget(renderTargetId);
        item.render();
    }

});

Jsw.Panel = Class.create(Jsw.Container, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._title = this._getConfigParam('title', '');
        this._titleClass = this._getConfigParam('titleClass', '');
        this._hideContentTitle = this._getConfigParam('hideContentTitle', this._title);
        this._titleAreaId = this._id + '-title-area';
        this._contentAreaId = this._id + '-content-area';
        this._boxItemsAreaId = this._id + '-box-area';
        this._initBoxItems();
    },

    _initBoxItems: function() {
        this._boxItems = this._getConfigParam('boxItems', null);

        if (null != this._boxItems) {
            this._boxItems.each(function(item, index) {
                if ('undefined' != typeof item.componentType) {
                    // recreate proper object defined by componentType
                    eval('var resultItem = new ' + item.componentType + '(item);');
                    this._boxItems[index] = resultItem;
                }
            }, this);
        }
    },

    _initComponentElement: function($super) {
        $super();

        this._updateComponentElement(
            '<div class="box-area">' +
            this._getTitleHtml() +
            this._getSubTitleHtml() +
                '<div id="' + this._boxItemsAreaId + '"></div>' +
                '<div class="content">' +
                    '<div class="content-area">' +
                        '<div class="content-wrapper" id="' + this._contentAreaId + '">' +
                            this._getConfigParam('html', '') +
                        '</div>' +
                    '</div>' +
                '</div>' +
            '</div>'
        );
    },

    _getTitleHtml: function() {
        return this._title
            ?   '<div class="title ' + (this._titleClass ? this._titleClass : '') + '">' +
                    '<div class="title-area" id="' + this._titleAreaId + '">' +
                        (('-' == this._title) ? '' : '<h3><span>' + this._title + '</span></h3>') +
                    '</div>' +
                '</div>'
            : '';
    },

    _getSubTitleHtml: function() {
    	return '';
    },

    render: function($super) {
        $super();

        this._renderBoxItems();
    },

    _renderBoxItems: function() {
        if (null != this._boxItems) {
            this._boxItems.each(function(item) {
                this._renderBoxItem(item);
            }, this);
        }
    },

    _renderBoxItem: function(item) {
        item.setRenderTarget(this._boxItemsAreaId);
        item.render();
    },

    _isHidden: function()
    {
        var rememberedStatus = Jsw.Cookie.get(this._id + '-hidden');
        if (this._idIsRandom || null === rememberedStatus) {
            return this._getConfigParam('hidden', false);
        }
        return 1 == rememberedStatus;
    },

    _setHidden: function(hidden)
    {
        if (this._getConfigParam('hidden', false) == hidden) {
            Jsw.Cookie.remove(this._id + '-hidden');
        } else {
            Jsw.Cookie.set(this._id + '-hidden', hidden ? 1 : 0);
        }
    },

    _addEvents: function($super) {
        $super();
        if (this._isHidden()) {
            this.hideContent();
        } else {
            this.showContent();
        }
        this._componentElement.select('#' + this._titleAreaId).each(function(titleAreaElement) {
            Event.observe(titleAreaElement, 'click', this.toggleContent.bindAsEventListener(this));
        }, this);
    },

    _updateTitle: function(title) {
        if (title && ('-' != title)) {
            this._componentElement.down('div.title-area > h3 > span').update(title);
        }
    },

    showContent: function() {
        this._componentElement.removeClassName('hide');
        this._updateTitle(this._hideContentTitle);
        if (!this._idIsRandom) {
            this._setHidden(false);
        }
    },

    hideContent: function() {
        this._componentElement.addClassName('hide');
        this._updateTitle(this._title);
        if (!this._idIsRandom) {
            this._setHidden(true);
        }
    },

    toggleContent: function() {
        this._componentElement.hasClassName('hide')
            ? this.showContent()
            : this.hideContent();
    }

});

Jsw.SmallTools = Class.create(Jsw.Container, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._operations = this._getConfigParam('operations', null);
        this._operationsClass = this._getConfigParam('operationsClass', 'objects-toolbar clearfix');
        this._operationsTag = this._getConfigParam('operationsTag', 'div');

        if (Object.isArray(this._operations) && (0 == this._operations.size())) {
            this._operations = null;
        }
    },

    _initComponentElement: function($super) {
        $super();

        this._updateComponentElement('<div id="' + this._id + '-container"></div>');
    },

    _getActionsBoxHtml: function() {
        if (null == this._operations) {
            return '';
        }

        var html = '<div class="actions-box">';

        if (this._operations) {
            html +=
                '<' + this._operationsTag + ' id="' + this._id +
                    '-operations" class="' + this._operationsClass + '">' +
                '</' + this._operationsTag + '>';
        }

        html +=
            '</div>';

        return html;
    },

    _addOperations: function() {
        this._operations.each(function(config) {
            eval('var operation = new ' + config.componentType + '(config);');
            operation.setRenderTarget(this._id + '-operations');
            operation.render();
        }, this);
    },

    render: function($super) {
        $super();

        this._componentElement.down().replace(
            '<div id="' + this._id + '-container">' +
                this._getActionsBoxHtml() +
            '</div>'
        );

        if (this._operations) {
            this._addOperations();
        }

        Jsw.Tooltip.initData(this._componentElement);
    }
});


Jsw.CollapsibleBlock = Class.create(Jsw.Container, {
    _initConfiguration: function($super, config) {
        $super(config);

        this._blockAreaId = this._id + '-block';
        this._titleAreaId = this._id + '-title-area';
        this._contentAreaId = this._id + '-content-area';
    },

    _initComponentElement: function($super) {
        $super();

        var containerContentChilds = this._componentElement.childElements();

        this._updateComponentElement(
            '<div class="' + this._cls +  '-block' + (this._getConfigParam('collapsed', true) ? ' hide' : '') + '" id="' + this._blockAreaId + '">' +
                '<div class="' + this._cls +  '-title" id="' + this._titleAreaId + '">' +
                    '<span>' + this._getConfigParam('title', '') +  '</span>' +
                '</div>' +
                '<div class="' + this._cls +  '-content" id="' + this._contentAreaId + '"></div>' +
            '</div>'
        );
    },

    _addEvents: function($super) {
        $super();
        $(this._titleAreaId).down('span').observe('click', this._onTitleClickEvent.bind(this));
    },

    _onTitleClickEvent: function(event) {
        Event.stop(event);
        $(this._blockAreaId).toggleClassName('hide');
    }
});

Jsw.Collapsible = Class.create(Jsw.Container, {

    _initConfiguration: function($super, config) {
        $super(config);
        this._contentAreaId = this._id + '-content-area';
        this._anchorAreaId = this._id + '-anchor-area';
    },

    _initComponentElement: function($super) {
        $super();

        var containerContentChilds = this._componentElement.childElements();

        this._updateComponentElement(
            '<div class="collapsible-box collapsed">' +
                '<div class="collapsible-anchor" id="' + this._anchorAreaId + '"></div>' +
                '<div class="collapsible-area">' +
                    '<div class="tl"><div class="br"><div class="bl"><div class="tr">' +
                        '<div class="collapsible-content" id="' + this._contentAreaId + '">' +
                            this._getConfigParam('html', '') +
                            //...collapsible elements
                        '</div>' +
                    '</div></div></div></div>' +
                '</div>' +
             '</div>'
        );
    },

    _addEvents: function($super) {
        $super();

        //init collapsed status
        if (this._getConfigParam('collapsed', true)) {
            this.collapse();
        } else {
            this.expand();
        }
        //handle click event on anchor
        this._componentElement.select('.collapsible-anchor').each(function(anchorElement) {
            Event.observe(anchorElement, 'click', this._onAnchorClickEvent.bind(this));
        }, this);
    },

    _onAnchorClickEvent: function(event) {
        if (this._collapsed) {
            this.expand();
        } else {
            this.collapse();
        }
        Event.stop(event);
    },

    _getCollapseBinding: function() {
        if (!this._collapseBinding) {
            this._collapseBinding = this.collapse.bind(this);
        }
        return this._collapseBinding;
    },

    collapse: function() {
        if (this._collapsed) {
            return;
        }
        this._collapsed = true;
        Event.stopObserving(document, 'click', this._getCollapseBinding());
        var content = $(this._contentAreaId).up('.collapsible-box');
        content.removeClassName('expanded');
        content.addClassName('collapsed');
    },

    expand: function() {
        if (!this._collapsed) {
            return;
        }
        this._collapsed = false;
        var content = $(this._contentAreaId).up('.collapsible-box');
        content.removeClassName('collapsed');
        content.addClassName('expanded');
        Event.observe(document, 'click', this._getCollapseBinding());
    }

});

Jsw.DropdownArea = Class.create(Jsw.Collapsible, {
    _initComponentElement: function($super) {
	    $super();
	    this._componentElement.down('#' + this._anchorAreaId).update(
	        '<h4><span>' + this._getConfigParam('title', '') + '</span></h4>'
	    );
	}
});

Jsw.DropdownSplit = Class.create(Jsw.Component, {
    _initConfiguration: function($super, config) {
    	$super(config);
    	this._cls = this._getConfigParam('cls', 'split');
	}
});

Jsw.Dropdown = Class.create(Jsw.Collapsible, {

    _initConfiguration: function($super, config) {
        $super(config);
        this._valuesAreaId = this._id + '-values-area';
        this.onChange = this._getConfigParam('onChange', this.onChange);
    },

    _initComponentElement: function($super) {
        $super();
        this._componentElement.down('#' + this._contentAreaId).update(
            '<h4><span>' + this._getConfigParam('title', '') + '</span></h4>' +
            '<ul id="' + this._valuesAreaId + '"></ul>'
        );
    },

    _renderItem: function(item) {
        var dropdownItem = new Jsw.DropdownItem(item);

        var valueItem = new Element('a').update(dropdownItem.title.escapeHTML());

        $(this._valuesAreaId).insert(
            new Element('li').update(valueItem)
        );

        valueItem.observe('click', this._onClick.bindAsEventListener(this, dropdownItem));
    },

    _onClick: function(event, item)
    {
        this.onChange(item);
    },

    onChange: function(item) {
        this.setValue(item.value);
    },

    setValue: function(value) {
        this._value = value;
    },

    getValue: function() {
        return this._value;
    }

});

Jsw.DropdownItem = Class.create({

    initialize: function(item) {
        if (Object.isString(item)) {
            this.value = item;
            this.title = item;
        } else
        if (item.title || item.value) {
            this.value = item.value;
            this.title = item.title;
        }
    }
});

Jsw.Pathbar = Class.create(Jsw.Container, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._contentAreaId = this._id + '-content-area';
    },

    _initComponentElement: function($super) {
        if ($(this._id)) {
            this._applyTargetId = this._id;
            $super();

            return;
        }

        $super();

        this._updateComponentElement(
            '<ul id="' + this._contentAreaId + '">' +
            '</ul>'
        );
    },

    _renderItem: function(item) {
        var renderTargetId = this._id + '-item-' + item.getId();
        $(this._contentAreaId).insert('<li id="' + renderTargetId +'"></li>');
        item.setRenderTarget(renderTargetId);
        item.render();
        this._renderItemSuffix(item);
    },

    _renderItemSuffix: function(item) {
        var renderTargetId = this._id + '-item-' + item.getId();
        $(renderTargetId).insert("<b>&gt;</b>");
    }

});

Jsw.Pathbar.Item = Class.create(Jsw.Component, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._title = this._getConfigParam('title', '');
        this._href = this._getConfigParam('href', '');
    },

    _initComponentElement: function($super) {
        $super();

        this._componentElement = new Element('a', {
            'href': this._href
        }).update('<span>' + this._title + '</span>');
    },

    getHref: function() {
        return this._href;
    }
});

Jsw.Pathbar.SimpleItem = Class.create(Jsw.Component, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._title = this._getConfigParam('title', '');
    },

    _initComponentElement: function($super) {
        $super();

        this._componentElement = new Element('span');
        this._componentElement.update(this._title);
    }
});


Jsw.Bar = Class.create(Jsw.Container, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._contentAreaId = this._id + '-content-area';

        // allowed types: static, dynamic
        this._type = this._getConfigParam('type', 'static');
    },

    _initComponentElement: function($super) {
        $super();

        var content = '<ul id="' + this._contentAreaId + '"></ul>';
        if ('navbar' != this._id) {
            content = '<div class="tabs-area">' + content + '</div>';
        }

        this._updateComponentElement(content);
    },

    _renderItems: function() {
        if (null != this._items) {
            var size = this._items.size();
            this._items.each(function(item, i) {
                this._renderItem(item, i, size);
            }, this);
        }
    },

    _renderItem: function(item, i, size) {
        if (0 == i && 'navbar' != this._id && 'classic-mode-navigation' != this._id) {
            var renderTargetId = 'first-tab';
        } else if (size - 1 == i && 'navbar' != this._id && 'classic-mode-navigation' != this._id) {
            var renderTargetId = 'last-tab';
        } else {
            var renderTargetId = this._id + '-item-' + item.getId();
        }
        var containerCls = item.getContainerClass();

        $(this._contentAreaId).insert('<li class="' + containerCls + '" id="' + renderTargetId +"\"></li>");
        item.setRenderTarget(renderTargetId);
        item.render();

        if ('dynamic' == this._type) {
            item.addEventObserver('click', item.switchTab.bindAsEventListener(item));
        }

        if ($('main')) {
            if (!$('main').hasClassName('tabed') && 'navbar' != this._id) {
                $('main').addClassName('tabed');
            }
        }
    }
});

Jsw.bar = {};

Jsw.bar.Tabs = Class.create(Jsw.Bar, {

    _initConfiguration: function($super, config) {
        $super(config);
        this._isResponsive = this._getConfigParam('responsive', true);
    },

    _renderItems: function($super) {
        $super();

        if (this._isResponsive) {
            this._addResponsiveTabs();
        }
    },

    _addResponsiveTabs: function() {
        var element = $(this._id);
        element.select('li a').each(function(el) {
            el.observe('click', function(event) {
                var listItem = el.up('li');
                if (listItem.hasClassName('active') || listItem.id == 'current') {
                    Event.stop(event);
                }
                element.toggleClassName('responsive-tabs-visible');
            });
        });
    },

    switchTab: function(tabId) {
        var activeTab = null;

        this.getItems().each(function(item) {
            if (tabId == item.getTabId()) {
                activeTab = item;
            }

            $(item.getTabId()).hide();
            $(item.getRenderTarget()).removeClassName('active');
        });

        $(tabId).show();
        $(activeTab.getRenderTarget()).addClassName('active');
    },

    _renderItem: function($super, item, i, size) {
        $super(item, i, size);

        var content = item.getTabContent();

        if (content) {
            var tabContentElement = new Element('div', {
                'id': item.getTabId()
            });

            if (!item.isActive()) {
                tabContentElement.setStyle({ 'display': 'none' });
            }

            this._componentElement.up().insert({ bottom: tabContentElement });

            if (Object.isString(content)) {
                tabContentElement.update(content);
            } else {
                content.setRenderTarget(item.getTabId());
                content.render();
            }
        }
    }
});

Jsw.bar.Button = Class.create(Jsw.Button, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._isActive = this._getConfigParam('active', false);
        this._cls = this._getConfigParam('cls', '');
        this._containerCls = this._getConfigParam('containerCls', this._isActive ? 'active' : '');
        this._tabId = this._getConfigParam('tabId', null);
        this.navigationTab = this._getConfigParam('navigationTab', false);
        this._content = this._getConfigParam('content', null);
    },

    _initComponentElement: function() {
        this._componentElement = new Element('a', {
            'class': this._cls,
            'id': this._tabId ? this._tabId + '-button' : null
        }).update('<span>' + this._config['title'] + '</span>');

        this._initVisibility();
    },

    getContainerClass: function() {
        return this._containerCls;
    },

    getTabId: function() {
        return this._tabId;
    },

    switchTab: function(event) {
        var bar = Jsw.getComponent(Event.element(event).up('ul').up('div').up('div').id);
        bar.switchTab(this._tabId);
        Event.stop(event);
    },

    getTabContent: function() {
        return this._content;
    },

    isActive: function() {
        return this._isActive;
    }

});

Jsw.bar.Separator = Class.create(Jsw.Component, {
    _tag: 'span',

    _initConfiguration: function($super, config) {
        $super(config);
        this._cls = this._getConfigParam('cls', 'separator');
    },

    _initComponentElement: function($super) {
        $super();
        this._componentElement.update('<span></span>');
    }
});

Jsw.bar.HorizontalSeparator = Class.create(Jsw.Component, {
    _tag: 'span',

    _initConfiguration: function($super, config) {
        $super(config);
        this._wrapperClass = this._getConfigParam('wrapperClass', 'separator');
    }
});

Jsw.list = {

    COLUMN_SELECTION: {
        cls: 'select',
        headerCls: 'select',
        hideDisabled: false,
        headerRenderer: function(column) {
            return '<input type="checkbox" class="checkbox" name="listGlobalCheckbox"/>';
        },
        renderer: function(item, isDisabled) {
            if (!isDisabled) {
                return '<input type="checkbox" class="checkbox" name="listCheckbox[]" value="' + item.id.toString().escapeHTML().replace(/"/g,'&quot;') + '"/>';
            }

            var html =
                '<div>' +
                    '<span class="checkbox-disabled-wrapper"></span>' +
                    '<input style="z-index: 1;" disabled="disabled" type="checkbox" class="checkbox"/>' +
                '</div>';

            return html;
        }
    },

    ITEMS_UNLIMITED: 100000

};

Jsw.list.AdditionalActions = Class.create(Jsw.Component, {

    _tag: 'div',

    _initConfiguration: function($super, config) {
        $super(config);

        this._operations = this._getConfigParam('operations', null);
        this._cls = this._getConfigParam('cls', 'actions-menu');
        this._titleAddCls = this._getConfigParam('titleAddCls', 'sb-more-actions');
    },

    _initComponentElement: function($super) {
        $super();

        this._componentElement.insert({
            bottom:
                '<div class="popup-box collapsed">' +
                    '<table class="popup-wrapper" cellspacing="0"><tbody><tr>' +
                        '<td class="popup-container">' +
                            '<div class="c1"><div class="c2"><div class="c3"><div class="c4"><div class="c5">' +
                                '<div class="popup-heading">' +
                                    '<div class="heading-area">' +
                                        '<span class="close"></span>' +
                                        '<h4>' +
                                            '<span>' +
                                                this._getConfigParam('title', '') +
                                            '</span>' +
                                        '</h4>' +
                                    '</div>' +
                                '</div>' +
                                '<div class="popup-content">'+
                                    '<div class="popup-content-area">' +
                                        '<ul id="' + this._id + '-operations">' +
                                        '</ul>' +
                                    '</div>' +
                                '</div>' +
                            '</div></div></div></div></div>' +
                        '</td>' +
                    '</tr></tbody></table>' +
                '</div>'
        });
    },

    _addEvents: function($super) {
        $super();

        var context = this;
        $(document.body).observe('click', function() {
            context.collapse();
        });
    },

    enable: function() {
        this._titleButton.enable();
    },

    disable: function() {
        this._titleButton.disable();
    },

    toggle: function() {
        var popup = this._componentElement.down('.popup-box');

        if (popup) {
            popup.toggleClassName('collapsed');
        }

        var button = this._componentElement.down('.' + this._titleAddCls);

        if (button) {
            button.toggleClassName('active');
        }
    },

    collapse: function() {
        var popup = this._componentElement.down('.popup-box');

        if (popup) {
            popup.addClassName('collapsed');
        }

        var button = this._componentElement.down('.' + this._titleAddCls);

        if (button) {
            button.removeClassName('active');
        }
    },

    render: function($super) {
        $super();

        this._titleButton = new Jsw.SmallButton({
            title: this._getConfigParam('title', ''),
            cls: 's-btn',
            addCls: this._titleAddCls,
            renderTo: this._id,
            handler: (function(event) {
                Event.stop(event);
                this.toggle();
            }).bind(this)
        });

        if (!this._operations) {
            return;
        }

        var operationsArea = $(this._id + '-operations');
        this._operations.each(function(config) {
            var liElement = new Element('li');
            operationsArea.insert(liElement);

            eval('var operation = new ' + config.componentType + '(config);');

            var wrapperClass = operation.getWrapperClass();
            if (wrapperClass) {
                liElement.addClassName(wrapperClass);
            }

            operation.setRenderTarget(liElement);
            operation.render();
        }, this);

        if (this._getConfigParam('disabled', false)) {
            this.disable();
        } else {
            this.enable();
        }
    }

});

Jsw.List = Class.create(Jsw.Component, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._listCls = this._getConfigParam('listCls', 'list');
        this._tableId = this._id + '-table';
        this._dataUrl = this._getConfigParam('dataUrl', '');
        this._operations = this._getConfigParam('operations', null);
        this._operationsClass = this._getConfigParam('operationsClass', 'objects-toolbar clearfix');
        this._operationsTag = this._getConfigParam('operationsTag', 'div');
        this._columns = this._getConfigParam('columns', null);
        this._limitValues = this._getConfigParam('limitValues', [10, 25, 100, Jsw.list.ITEMS_UNLIMITED]);
        this._splitListData(this._getConfigParam('data', { data: null, pathbar: null, state: null, pager: null, locale: null, additional: null }));
        this._rowSelectable = this._isColumnSelectionPresent();
        this._searchFilters = $H(this._getConfigParam('searchFilters', {}));
        this._searchable = this._getConfigParam('searchable', this._searchFilters.size() > 0);
        this._searchOveral = this._getConfigParam('searchOveral', null);
        this._searchExtended = this._getConfigParam('searchExtended', this._searchFilters.size() > 1);
        this._pageable = this._getConfigParam('pageable', true);
        this._disabledSelectHint = this._getConfigParam('disabledSelectHint', '');
        this._clearStatusOnReload = this._getConfigParam('clearStatusOnReload', true);
        this.isDisabledItem = this._getConfigParam('isDisabledItem', function() {
            return false;
        });
        this._onRedraw = this._getConfigParam('onRedraw', null);
        this._getTotalInfo = this._getConfigParam('getTotalInfo', function() {
            return this._locale.recordsTotal.replace('%%total%%', this._pager.totalItemCount);
        });

        if (Object.isArray(this._operations)) {
            this._operations = this._operations.compact();
            if (0 == this._operations.size()) {
                this._operations = null;
            }
        }
    },

    _getSearchOveralField: function()
	{
    	if (null === this._searchOveral) {
    		this._searchFilters.each(function (pair){
        		if(pair.value.overal) {
        			this._searchOveral = pair.key;
        		}
    		}, this);
    		if (null === this._searchOveral && 1 == this._searchFilters.size()) {
    			this._searchOveral = this._searchFilters.keys().first();
    		}
    		if (null === this._searchOveral) {
    			this._searchOveral = false;
    		}
    	}

    	return this._searchOveral;
    },

    _initComponentElement: function($super) {
        $super();

        this._updateComponentElement('<div id="' + this._id + '-container"></div>');
        this._initDisablerOverlay();
    },

    _splitListData: function(listData) {
        if (!listData) {
            return false;
        }

        if (!listData.data || !listData.state || !listData.pager || !listData.locale) {
            return false;
        }
        this._data = listData.data;
        this._state = listData.state;
        this._pager = listData.pager;
        this._locale = listData.locale;

        if (listData.pathbar) {
            this._pathbar = listData.pathbar;
        }
        if (listData.additional) {
            this._additionalData = listData.additional;
        }
        return true;
    },

    _isColumnSelectionPresent: function() {
        for (var index = 0; index < this._columns.length; index++) {
            if (Jsw.list.COLUMN_SELECTION == this._columns[index]) {
                return true;
            }
        }

        return false;
    },

    onHeaderClick: function(dataIndex) {
        this._state.sortField = dataIndex;
        this._state.sortDirection = ('down' == this._state.sortDirection)
            ? 'up'
            : 'down';

        this.reload('/sort-field/' + dataIndex + '/sort-dir/' + this._state.sortDirection);
    },

    onLimitClick: function(value) {
        this.reload('/items-per-page/' + value);
    },

    onPagerClick: function(page) {
        this.reload('/page/' + page);
    },

    onEnumSearchFilterChange: function(element) {
        this._onSearchClick(Element.up(element, 'form'));
    },

    onSelectButtonClick: function(event) {
        var globalCheckbox = this._componentElement.select('input[name="listGlobalCheckbox"]').first();
        globalCheckbox.checked = !globalCheckbox.checked;
        this.onListGlobalCheckboxClick(event);
        event.stopPropagation();
    },

    onResponsiveButtonClick: function(event, name) {
        var actionsBox = this._componentElement.select('.actions-box').first();
        if (actionsBox) {
            var button = actionsBox.select('.r-actions-' + name).first();
            var show = !button.hasClassName('r-actions-item-active');
            this._hideResponsiveButtonBlocks();
            if (show) {
                this._showResponsiveButtonBlocks(name);
            }
        }
        event.stopPropagation();
    },

    onListGlobalCheckboxClick: function(event) {
        var actionsBox = this._componentElement.select('.actions-box').first();
        var globalCheckbox = this._componentElement.select('input[name="listGlobalCheckbox"]').first();

        if (actionsBox) {
            var selectButton = actionsBox.select('.r-actions-select').first();
            if (globalCheckbox.checked) {
                selectButton.addClassName('r-actions-item-active');
            } else {
                selectButton.removeClassName('r-actions-item-active');
            }
        }

        this._componentElement.select('input[name="listCheckbox[]"]').each(function(checkbox) {
            if (globalCheckbox.checked) {
                checkbox.up('tr').addClassName('selected');
            } else {
                checkbox.up('tr').removeClassName('selected');
            }
            checkbox.checked = globalCheckbox.checked;
        });
    },

    onPagerButtonClick: function(event) {
        var paging = this._componentElement.select('.paging').first();
        var show = !paging.hasClassName('r-paging-visible');
        this._hideResponsiveButtonBlocks();
        if (show) {
            paging.addClassName('r-paging-visible');
        }
        event.stopPropagation();
    },

    _hideResponsiveButtonBlocks: function() {
        var actionsBox = this._componentElement.select('.actions-box').first();
        if (actionsBox) {
            actionsBox.removeClassName('r-actions-active');
            actionsBox.removeClassName('r-actions-active-search');
            actionsBox.removeClassName('r-actions-active-toolbar');
            actionsBox.select('.r-actions-item-active').each(function(item) {
                if (item.hasClassName('r-actions-select')) {
                    return;
                }
                item.removeClassName('r-actions-item-active');
            });
        }
        var paging = this._componentElement.select('.paging').first();
        if (paging) {
            paging.removeClassName('r-paging-visible');
        }
    },

    _showResponsiveButtonBlocks: function(name) {
        var actionsBox = this._componentElement.select('.actions-box').first();
        if (actionsBox) {
            var button = actionsBox.select('.r-actions-' + name).first();
            actionsBox.addClassName('r-actions-active');
            actionsBox.addClassName('r-actions-active-' + name);
            button.addClassName('r-actions-item-active');
        }
    },

    _getListHeadersHtml: function() {
        var html = '<thead><tr>';
        var colspan = 0;

        this._columns.each(function(column) {
            if (colspan) {
                colspan --;
                return;
            }

            if ('function' === typeof column.isVisible) {
                if (!column.isVisible()) {
                    return;
                }
            }

            if (column.headerColspan) {
                colspan = column.headerColspan;
            }

            var data = '';

            var headerCls = '';

            if ('undefined' != typeof column.headerCls) {
                headerCls = column.headerCls;
            }

            var headerParams = {};
            var headerParamsString = '';

            if ('undefined' != typeof column.headerParams) {
                headerParams = $H(column.headerParams);
                headerParams.each(function(param) {
                    headerParamsString = param.key + "='" + param.value + "' ";
                });
            }

            if ('undefined' != typeof column.headerRenderer) {
                data = column.headerRenderer(column);
            } else {
                data = column.header;

                var sortCssClass = '';

                if (this._state.sortField == column.dataIndex) {
                    sortCssClass = ('down' == this._state.sortDirection)
                        ? ''
                        : 'sort-up';

                    headerCls = (headerCls)
                        ? (headerCls + ' ' + 'sort')
                        : 'sort';
                }

                if (column.sortable) {
                    data = '<a class="' + sortCssClass + '" href="#" onclick="Jsw.getComponent(\'' + this._id
                        + '\').onHeaderClick(\'' + column.dataIndex + '\')">' + data + '</a>';
                }
            }

            html += '<th class="' + headerCls + '"' + (colspan ? 'colspan="' + colspan + '"' : '') + ' ' + headerParamsString + '>' + data + '</th>';
        }, this);

        html += '</tr></thead>';

        return html;
    },

    _getListDataHtml: function() {
        if (!this._data) {
            return ;
        }

        var html = '';
        var currentRowClass = 'odd';

        this._data.each(function(item) {
            html += '<tr class="' + currentRowClass + '">';

            this._columns.each(function(column) {
                if ('function' === typeof column.isVisible) {
                    if (!column.isVisible()) {
                        return;
                    }
                }

                var data = '';

                if ('undefined' != typeof column.renderer) {
                    var isDisabled = this.isDisabledItem(item, this._data);

                    if (isDisabled && ('undefined' != typeof column.hideDisabled) && column.hideDisabled) {
                        data = '';
                    } else {
                        data = column.renderer(item, isDisabled);
                    }
                } else if ('undefined' != typeof column.dataIndex) {
                    if (column.noEscape) {
                        data = item[column.dataIndex];
                    } else {
                        data = item[column.dataIndex].escapeHTML();
                    }
                }

                var itemCls = '';

                if ('undefined' != typeof column.cls) {
                    itemCls = column.cls;
                }

                html += '<td class="' + itemCls + '">' + data + '</td>';
            }, this);

            html += '</tr>';

            currentRowClass = ('odd' == currentRowClass) ? 'even' : 'odd';
        }, this);

        return html;
    },

    _getListHtml: function() {
        if (0 == this._pager.totalItemCount) {
            return '<span class="empty-list">' + this._locale.noEntriesFound + '</span>';
        }

        var html =
            this._getPagerHtml() +
            '<div class="' + this._listCls + '">' +
                '<table width="100%" cellspacing="0" id="' + this._tableId + '">' +
                    this._getListHeadersHtml() +
                    this._getListDataHtml() +
                '</table>' +
            '</div>' +
            this._getPagerHtml();

        return html;
    },

    _getPathbarHtml: function() {
        if (!this._pathbar) {
            return '';
        }

        return '<div id="' + this._id + '-pathbar"></div>';
    },

    _getPagerHtml: function() {
        if (!this._pageable) {
            return '';
        }
        var totalRecords = this._getTotalInfo();

        var html =
            '<div class="paging">' +
                '<div class="paging-area">' +
                    '<span class="paging-info"  onclick="Jsw.getComponent(\'' + this._id +
                        '\').onPagerButtonClick(event);">' + totalRecords + '</span>' +
                    '<span class="paging-view">' + this._locale.numberOfItemPerPage + ': ' +
                        this._getLimitValuesHtml() +
                    '</span>';

        if (this._pager.pageCount > 1) {
            html += '<span class="paging-nav">' + this._locale.pages + ': ' + this._getPagesItemsHtml() + '</span>';
        }

        html += '</div></div>';

        return html;
    },

    _getLimitValuesHtml: function() {
        var html = '';

        this._limitValues.each(function(limitValue) {
            var limitValueTitle = (Jsw.list.ITEMS_UNLIMITED == limitValue)
                ? this._locale.allItems
                : limitValue;

            if (this._pager.itemCountPerPage != limitValue) {
                html += '<a href="#" onclick="Jsw.getComponent(\'' + this._id +
                    '\').onLimitClick(' + limitValue + ');">' + limitValueTitle + '</a> ';
            } else {
                html += '<span>' + limitValueTitle + '</span> ';
            }
        }, this);

        return html;
    },

    _getPagesItemsHtml: function() {
        var html = '';

        if (this._pager.first != this._pager.current) {
            html += '<a href="#" onclick="' + this._getPageItemLink(this._pager.first) + '">'
                + this._locale.firstPage + '</a> ';
        } else {
            html += '<span>' + this._locale.firstPage + '</span> ';
        }

        if (this._pager.previous) {
            html += '<a href="#" onclick="' + this._getPageItemLink(this._pager.previous) + '">&lt;&lt;</a> ';
        } else {
            html += '<span>&lt;&lt;</span> ';
        }

        $H(this._pager.pagesInRange).values().each(function(page) {
            if (page != this._pager.current) {
                html += '<a href="#" onclick="' + this._getPageItemLink(page) + '">' + page + '</a> ';
            } else {
                html += page + ' ';
            }
        }, this);

        if (this._pager.next) {
            html += '<a href="#" onclick="' + this._getPageItemLink(this._pager.next) + '">&gt;&gt;</a> ';
        } else {
            html += '<span>&gt;&gt;</span> ';
        }

        if (this._pager.last != this._pager.current) {
            html += '<a href="#" onclick="' + this._getPageItemLink(this._pager.last) + '">'
                + this._locale.lastPage + '</a> ';
        } else {
            html += '<span>' + this._locale.lastPage + '</span> ';
        }

        return html;
    },

    _getPageItemLink: function(page) {
        return "Jsw.getComponent('" + this._id + "').onPagerClick(" + page + ");";
    },

    _getActionsBoxHtml: function() {
        if (!this._searchable && null == this._operations) {
            return '';
        }

        var searchStateClass = '';

        if (this._state.forceShowSearch) {
            Jsw.Cookie.set(this._id + '-search-show', 'true');
        }

        if (
            this._searchable &&
            this._searchExtended &&
            'true' == Jsw.Cookie.get(this._id + '-search-show')
        ) {
            searchStateClass = ' actions-box-search-show';
        }

        var html = '<div class="actions-box' + searchStateClass + '">';

        html += this._getResponsiveButtonsHtml();

        if (this._operations || this._searchable) {
            html +=
                '<' + this._operationsTag + ' id="' + this._id +
                    '-operations" class="' + this._operationsClass + '">' +
                '</' + this._operationsTag + '>';
        }

        if (this._searchable && this._searchExtended) {
            html +=
                '<div class="search-box">' +
                    '<form>' +
                    	'<ul>' +
                        	this._getSearchFiltersHtml() +
                    	'</ul>' +
                        '<a class="s-btn sb-search" href="#">' +
	                        '<i><span>' + this._locale.buttonSearch + '</span></i>' +
	                    '</a>' +
	                    '<a class="s-btn sb-show-all" href="#">' +
	                        '<i><span>' + this._locale.buttonResetSearch + '</span></i>' +
	                    '</a>' +
	                    '<input type="image" style="border: 0pt none ; height: 0pt; width: 0pt; position: absolute; left: -9999px;" src="' + Jsw.skinUrl + '/images/blank.gif">' +
                	'</form>' +
                '</div>';
        }

        html +=
            '</div>';

        return html;
    },

    _getSearchFiltersHtml: function() {
    	if (!this._searchable) {
    		return '';
    	}

    	var searchFilterHtml = '';

    	this._searchFilters.each(function(pair) {
    		var filterName = pair.key;
    		var filterConfig = pair.value;

    		var filterTitle = filterConfig.title;
    		var filterTitleHtml = '';
    		if (filterTitle) {
    			filterTitleHtml = '<span>' + filterTitle + '</span>';
            }

    		var filterBodyHtml = this._getSearchFilterBodyHtml(filterName, filterConfig);

    		searchFilterHtml +=
    			'<li>' +
    				'<div>' +
    					filterTitleHtml +
    					filterBodyHtml +
    				'</div>' +
    			'</li>';
    	}, this);

    	return searchFilterHtml;
    },

    _getSearchFilterBodyHtml: function(filterName, filterConfig) {
		var filterBodyHtml = '';

		if (filterConfig.render) {
			filterBodyHtml = filterConfig.render(filterConfig);
		} else if(filterConfig.options) {
			filterBodyHtml = this._getSearchFilterHtmlEnum(filterName, filterConfig);
		} else {
			filterBodyHtml = this._getSearchFilterHtmlText(filterName, filterConfig);
		}

		return filterBodyHtml;
    },

    _getSearchFilterHtmlText: function(filterName, filterConfig) {
    	var filterHtml = '<input' +
    			' id="' + this._id + '-search-text-' + filterName + '"' +
    			' name="searchFilter[' + filterName + '][searchText]"' +
    			' type="text"' +
    			' value="' + $H(this._state.searchFilters).get(filterName).searchText.escapeHTML().replace(/"/g, '&quot;') + '"' +
			'/>';
    	return filterHtml;
    },

    _getSearchFilterHtmlEnum: function(filterName, filterConfig) {
    	var currentValue = $H(this._state.searchFilters).get(filterName).searchText;
    	var optionsHtml = '';

    	filterOptions = $A(filterConfig.options);

    	filterOptions.each(function(item) {
    		if ('value' == item.type) {
    			optionsHtml += this._getSearchFilterHtmlEnumValue(currentValue, item.value, item.label);
    		} else {
    			optionsHtml += this._getSearchFilterHtmlEnumGroup(currentValue, item.options, item.label);
    		}
    	}, this);

    	var filterHtml = '<select' +
			' id="' + this._id + '-type-filter"' +
			' name="searchFilter[' + filterName + '][searchText]"' +
			' onchange="Jsw.getComponent(\'' + this._id + '\').onEnumSearchFilterChange(this)"' +
		'>' + optionsHtml + '</select>';

    	return filterHtml;
    },

    _getSearchFilterHtmlEnumValue: function(currentValue, value, label)
    {
        var selected = currentValue == value ? 'selected="selected"' : '';
    	var ret = '<option value="' + value.escapeHTML().replace(/"/g, '&quot;') + '"' + selected + '>' + label.escapeHTML() + '</option>';
    	return ret;
    },

    _getSearchFilterHtmlEnumGroup: function(currentValue, options, label)
    {
    	var ret = '';
		ret += '<optgroup label="' + label.escapeHTML().replace(/"/g, '&quot;') + '">';
		$A(options).each(function(item) {
    		if ('value' == item.type) {
    			ret += this._getSearchFilterHtmlEnumValue(currentValue, item.value, item.label);
    		} else {
    			ret += this._getSearchFilterHtmlEnumGroup(currentValue, item.options, item.label);
    		}
		}, this);
		ret += '</optgroup>';
    	return ret;
    },

    _getResponsiveButtonsHtml :function() {
        var html = '';
        html += '<div class="r-actions">';
        html += '<ul class="r-actions-list">';
        if (this._isColumnSelectionPresent && this._pager.totalItemCount != 0) {
            html += '<li class="r-actions-item r-actions-select" onclick="Jsw.getComponent(\'' + this._id +
                '\').onSelectButtonClick(event);"><span></span></li>';
        }
        if (this._operations) {
            html += '<li class="r-actions-item r-actions-toolbar" onclick="Jsw.getComponent(\'' + this._id +
                '\').onResponsiveButtonClick(event, \'toolbar\');"><span></span></li>';
        }
        if (this._searchable) {
            html += '<li class="r-actions-item r-actions-search" onclick="Jsw.getComponent(\'' + this._id +
                '\').onResponsiveButtonClick(event, \'search\');"><span></span></li>';
        }
        html += '</ul>';
        html += '</div>';
        return html;
    },

    _addSearchHandlers: function() {
        if (!this._searchable) {
            return;
        }

        var context = this;

        if (this._searchExtended) {
	        this._componentElement.select('.search-control .s-btn').each(function(element) {
	            element.observe('click', function() {
                    this.up('.actions-box').toggleClassName('actions-box-search-show');
	                Jsw.Cookie.set(context._id + '-search-show', this.up('.actions-box').hasClassName('actions-box-search-show'));
	                context._hideItemsNotSelectedWarning();
	                return false;
	            });
	        });
        }

        if (this._searchExtended) {
	        var searchLink = this._componentElement.select('.sb-search').first();
	        var form = searchLink.up('form');
	        searchLink.observe('click', function(event) {
	        	context._onSearchClick(form);
	        	Event.stop(event);
	        });
	        form.observe('submit', function(event) {
	        	context._onSearchClick(form);
	        	Event.stop(event);
	        });


	        var resetSearchLink = this._componentElement.select('.sb-show-all').first();
	        resetSearchLink.observe('click', function(event) {
	        	context._onResetSearchClick();
	        	Event.stop(event);
	        });
        }

        if (this._getSearchOveralField()) {
            this._componentElement.select('.quick-search-box form em').each(function(element) {
        		element.observe('click', function(event) {
        			element.up('form').submit();
        		});
        	});
        	this._componentElement.select('.quick-search-box form').each(function(element) {
        		element.observe('submit', function(event) {
        			context._onSearchClick(element);
        			Event.stop(event);
        		});
        	});
        }
    },

    _onSearchClick: function(form) {
        var searchQuery = '?';
        if (this._dataUrl.include('?')) {
            searchQuery = '&';
        }
        searchQuery += form.serialize();
        this.reload(searchQuery);
    },

    _onResetSearchClick: function() {
    	this.reload('/reset-search/true/');
    },

    _addSelectionHandlers: function() {
        var context = this;

        var globalCheckbox = this._componentElement.select('input[name="listGlobalCheckbox"]').first();
        var actionsBox = this._componentElement.select('.actions-box').first();
        var selectButton = actionsBox ? actionsBox.select('.r-actions-select').first() : null;

        globalCheckbox.observe('click', this.onListGlobalCheckboxClick.bind(this));

        this._componentElement.select('input[name="listCheckbox[]"]').each(function(checkbox) {
            checkbox.observe('click', function() {
                if (checkbox.checked) {
                    checkbox.up('tr').addClassName('selected');
                } else {
                    checkbox.up('tr').removeClassName('selected');
                    globalCheckbox.checked = false;
                    if (selectButton) {
                        selectButton.removeClassName('r-actions-item-active');
                    }
                }
            });
        });

        $(document).observe('click', function(event) {
            if ((context._operations || context._searchable) &&  (element = event.findElement('#' + context._id + '-operations'))) {
                return;
            }

            context._hideItemsNotSelectedWarning();
        });
    },

    _addResponsiveHandlers: function() {
        var list = this;
        $(document).observe('click', function(event) {
            if ((list._operations || list._searchable) &&  (element = event.findElement('#' + list._id + '-operations'))) {
                return;
            }

            if ((list._searchable || list._searchExtended) &&  (element = event.findElement('.search-box'))) {
                return;
            }

            list._hideResponsiveButtonBlocks();
        });

        $(document).observe('touchstart', function() {
            list._hideItemsNotSelectedWarning();
        });
    },

    _addOperations: function() {
        this._operations.each(function(config) {
            eval('var operation = new ' + config.componentType + '(config);');
            operation.setRenderTarget(this._id + '-operations');
            operation.render();
        }, this);
    },

    _addPathbar: function() {
        if (!this._pathbar) {
            return;
        }

        eval('this._pathbarComponent = new ' + this._pathbar.componentType + '(this._pathbar);');
        this._pathbarComponent.setRenderTarget(this._id + '-pathbar');
        this._pathbarComponent.render();
    },

    disable: function() {
        this._disablerOverlay.show();
        Jsw.clonePosition(this._disablerOverlay, this._componentElement);
    },

    enable: function() {
        this._disablerOverlay.hide();
    },

    getSelectedItemsIds: function() {
        var itemsIds = new Array();

        this._componentElement.select('input[name="listCheckbox[]"]').each(function(checkbox) {
            if (checkbox.checked) {
                itemsIds.push(checkbox.value);
            }
        });

        return itemsIds;
    },

    getSelectedItems: function() {
        var ids = [];
        var selectedItems = [];

        this.getSelectedItemsIds().each(function(id) {
            ids.push(id);
        });

        this._data.each(function(dataItem) {
            var itemIndex = ids.indexOf(dataItem.id);
            if (itemIndex < 0) {
                return;
            }

            selectedItems.push(dataItem);
        });

        return selectedItems;
    },

    checkNonEmptySelection: function() {
    	if (0 == this.getSelectedItemsIds().length) {
    		this._showItemsNotSelectedWarning();
    		return false;
    	}
    	this._hideItemsNotSelectedWarning();
    	return true;
    },

    execGroupOperation: function(params) {
        var context = this;

        var ids = $H();
        var count = 0;

        var idsParamName = params.submitVarName
            ? params.submitVarName
            : 'ids';

        var selectedItems = params.selectedItems ? params.selectedItems : this.getSelectedItemsIds();
        selectedItems.each(function(id) {
            ids.set(idsParamName + '[' + count + ']', id);
            count++;
        });

        if (params.checkSelection) {
            if (!params.checkSelection()) {
                return;
            }
        } else if (0 == count) {
            this._showItemsNotSelectedWarning();
            return ;
        } else {
            this._hideItemsNotSelectedWarning();
        }

        var sendRequestHandler = params.submitHandler
            ? params.submitHandler
            : function(url, ids) {
                new Ajax.Request(Jsw.prepareUrl(url), {
                    method: 'post',
                    parameters: ids,
                    reloading: false,

                    onSuccess: function (transport) {
                        try {
                            var status = transport.responseText.evalJSON();
                        } catch (e) {
                            Jsw.showInternalError(transport.responseText);
                            return;
                        }
                        if (status.redirect) {
                            Jsw.redirect(status.redirect);
                        } else if (!status.noReload) {
                            this.reloading = true;
                            context.reload();
                        }

                        if (context._clearStatusOnReload) {
                            Jsw.clearStatusMessages();
                        }
                        $A(status.statusMessages).each(function(message) {
                            Jsw.addStatusMessage(message.status, message.content);
                        });

                        if (params.onSuccess) {
                            params.onSuccess();
                        }
                    },

                    onCreate: function() {
                        context.disable();
                    },

                    onComplete: function() {
                        if (!this.reloading) {
                            context.enable();
                        }
                    },

                    onFailure: function(req) {
                        if (504 == req.status) {
                            Jsw.addStatusMessage('error', context._locale.timeoutMessage);
                        } else {
                            Jsw.showInternalError(req.responseText);
                        }

                        context.enable();
                    }
                });
            };

        var sendRequest = function() {
            sendRequestHandler(params.url, ids);
        };

        var skipConfirmation = params.skipConfirmation;
        var mouseEvent = params.mouseEvent;

        if (mouseEvent && !skipConfirmation) {
            mouseEvent.preventDefault();
            skipConfirmation = mouseEvent.shiftKey;
        }

        if (skipConfirmation) {
            sendRequest();
        } else {
            Jsw.messageBox.show({
                type: Jsw.messageBox.TYPE_YESNO,
                buttonTitles: {
                    yes: this._locale.messageBoxButtonYes,
                    no: this._locale.messageBoxButtonNo,
                    wait: this._locale.messageBoxButtonWait
                },
                text: params.getConfirmOnGroupOperation
                    ? params.getConfirmOnGroupOperation()
                    : params.locale.confirmOnGroupOperation,
                subtype: params.subtype,
                needAttention: params.needAttention ? params.needAttention : false,
                needAttentionText: params.needAttentionText ? params.needAttentionText : '',
                needAttentionBlockSubmit: params.needAttentionBlockSubmit ? params.needAttentionBlockSubmit : false,
                onYesClick: sendRequest,

                isAjax: params.isAjax,
                requestUrl: params.requestUrl,
                requestParams: ids,
                loadingTitle: params.loadingTitle
            });
        }
    },

    _showItemsNotSelectedWarning: function() {
        this._hideItemsNotSelectedWarning();
        var element = this._componentElement.down('items-not-selected-warning');
        if (element) {
            element.show();
        } else {
            this._componentElement.down('.actions-box').insert({ top:
                '<div class="actions-msg-container" id="items-not-selected-warning"><span class="list-actions-msg"><span>' +
                this._locale.itemsNotSelected + '</span></span></div>'
            });
        }
    },

    _hideItemsNotSelectedWarning: function() {
        var element = this._componentElement.down('.actions-msg-container');
        if (element) {
            element.hide();
        }
    },

    render: function($super) {
        $super();

        if (!this._data) {
            this.reload();
            return ;
        }

        this.redraw();
    },

    reload: function(params) {
        params = params || '';
        var context = this;

        if (!this._dataUrl) {
        	return;
        }

        new Ajax.Request(Jsw.prepareUrl(this._dataUrl + params), {
            method: 'get',
            onSuccess: function(transport) {
                if ('' == transport.responseText) {
                    //:INFO: sometimes happens in FF if you are go to other page during request
                    return;
                }

                try {
                    var data = transport.responseText.evalJSON();
                } catch (e) {
                    Jsw.showInternalError(transport.responseText);
                    return;
                }

                if (context._splitListData(data)) {
                    context.redraw();
                } else {
                    context.processReloadError(data);
                }
            },

            onFailure: this._onReloadFailure.bind(this),

            onException: this._onReloadException.bind(this),

            onCreate: this._onReloadCreate.bind(this),

            onComplete: this._onReloadComplete.bind(this)
        });
    },

    _onReloadFailure: function(transport) {
        Jsw.showInternalError(transport.responseText);
    },

    _onReloadException: function(transport, exception) {
        Jsw.showInternalError(exception + "\n" + transport.responseText);
    },

    _onReloadCreate: function(transport) {
        if (this._clearStatusOnReload) {
            Jsw.clearStatusMessages();
        }
        this.disable();
    },

    _onReloadComplete: function() {
        this.enable();
    },

    _addDisabledCheckboxesHints: function() {
        var checkboxes = this._componentElement.select('input[type="checkbox"][disabled]');

        checkboxes.each(function(checkbox) {
            new Jsw.Tooltip.Instance(checkbox.previous(), {text: this._disabledSelectHint });
        }, this);
    },

    redraw: function() {
        this._componentElement.down().replace(
            '<div id="' + this._id + '-container">' +
                this._getActionsBoxHtml() +
                this._getPathbarHtml() +
                this._getListHtml() +
            '</div>'
        );

        if (this._operations) {
            this._addOperations();
        }

        if (this._searchable) {
        	this._addSearchTools();
        }

        if (this._pager.totalItemCount > 0) {
            if (this._rowSelectable) {
                this._addSelectionHandlers();
            }
        }

        this._addPathbar();
        this._addSearchHandlers();
        this._addResponsiveHandlers();

        if (this._disabledSelectHint) {
            this._addDisabledCheckboxesHints();
        }

        this._addResponsiveHtml();

        Jsw.Tooltip.initData(this._componentElement);
        if (this._onRedraw) {
        	this._onRedraw();
        }
    },

    _addSearchTools: function() {
        var searchBoxHtml =
            '<div class="quick-search-box">';

    	if (this._getSearchOveralField()) {
    		searchBoxHtml +=
                '<div class="search-field">' +
                    '<form>' +
						this._getSearchFilterBodyHtml(this._getSearchOveralField(), this._searchFilters.get(this._getSearchOveralField())) +
						'<em><span></span></em>' +
                    '</form>' +
                '</div>';
    	}

    	if (this._searchExtended) {
            searchBoxHtml +=
                    '<span class="search-control">' +
                        '<a class="s-btn sb-search-show" href="#">' +
                            '<i><i><i><span>' + this._locale.showSearch + '</span></i></i></i>' +
                        '</a>' +
                        '<a class="s-btn sb-search-hide" href="#">' +
                            '<i><i><i><span>' + this._locale.hideSearch + '</span></i></i></i>' +
                        '</a>' +
                    '</span>' +
                '</div>';
    	}

    	searchBoxHtml += '</div>';

        $(this._id + '-operations').insert(searchBoxHtml);
    },

    processReloadError: function(data) {
        Jsw.showInternalError('Unable to load list data.');
    },

    hasSelectedItems: function(callback) {
        var selected = this.getSelectedItemsIds();

        if (0 == selected.length) {
            this._showItemsNotSelectedWarning();
            return false;
        } else {
            this._hideItemsNotSelectedWarning();
        }

        if (callback) {
            callback.bind(this)();
        }

        return true;
    },

    getItemById: function(itemId) {
        for (var i = 0; i < this._data.length; i++) {
            if (this._data[i].id == itemId) {
                return this._data[i];
            }
        }
    },

    _addResponsiveHtml: function() {
        var list = this;
        var columns = this._componentElement.select('thead th');
        this._componentElement.select('#' + this._tableId + ' tbody tr').each(function(row){
            var cells = row.childElements();
            for (var i = 0; i < cells.length; i++) {
                var element = cells[i];
                var columnName = columns[i].innerHTML.strip().stripTags();
                var columnNameHtml = '<span class="r-visible">' + columnName + '</span>';

                list._addResponsiveHtmlToButtons(element, columnNameHtml);

                list._addResponsiveHtmlToImages(element, columnNameHtml);

                list._addResponsiveHtmlToIcons(element, columnNameHtml);

                list._addResponsiveHtmlToNumbers(element, columnName)
            };
        });
    },

    _addResponsiveHtmlToButtons: function(element, columnNameHtml) {
        var buttons = element.select('a.s-btn');
        buttons.each(function(button) {
            var tooltip = button.previous('.tooltipData');
            if (!button.down('span').innerHTML && tooltip) {
                button.down('span').innerHTML = '<span class="r-visible">' + tooltip.innerHTML + '</span>';
            }
        });
        if (buttons.length == 1 && !buttons[0].down('span').innerHTML) {
            buttons[0].down('span').innerHTML = columnNameHtml;
        }
    },

    _addResponsiveHtmlToImages: function(element, columnNameHtml) {
        var images = element.select('a img');
        images.each(function(image) {
            var text = '';
            var tooltip = image.up().previous('.tooltipData');
            if (!tooltip) {
                if (images.length != 1) {
                    return;
                }
                text = columnNameHtml;
            } else {
                text = tooltip.innerHTML;
            }
            if (!image.up('.b-indent-icon') || !image.up('a').innerHTML.replace(image.up('.b-indent-icon').outerHTML, '').strip()) {
                image.wrap('i', {'class' : 'icon'});
                image.up().insert({after: ' <span class="r-visible">' + text + '</span>'});
                image.up('a').addClassName('i-link');
            }
        });
    },

    _addResponsiveHtmlToIcons: function(element, columnNameHtml) {
        var icons = element.select('span.b-indent-icon');
        icons.each(function(icon) {
            var tooltip = icon.select('.tooltipData');
            if (tooltip.length > 0 && !icon.up().innerHTML.replace(icon.outerHTML, '').strip()) {
                icon.down('img').insert({after: ' <span class="r-visible">' + tooltip[0].innerHTML + '</span>'});
            }
        });
        if (icons.length == 1 && !icons[0].down('span.r-visible') && !icons[0].up().innerHTML.replace(icons[0].outerHTML, '').strip()) {
            icons[0].down('img').insert({after: columnNameHtml});
        }
    },

    _addResponsiveHtmlToNumbers: function(element, columnName) {
        var number = element.innerHTML;
        var columnNameHtml = '<span class="r-visible">' + columnName + ': </span>';
        if (!isNaN(parseFloat(number)) && isFinite(number)) {
            element.insert({top: columnNameHtml});
        }
        var links = element.select('a');
        if (links.length == 1 && !isNaN(parseFloat(links[0].innerHTML)) && isFinite(links[0].innerHTML)) {
            element.insert({top: columnNameHtml});
        }
    }


});

Jsw.DynamicSubForm = Class.create(Jsw.Component, {

    _initConfiguration: function($super, config) {
        $super(config);
        var getDynamicContent = this._getConfigParam('getDynamicContent', null);
        if (getDynamicContent) {
            this._getDynamicContent = getDynamicContent.bind(this);
        }
        var prepareDynamicContent = this._getConfigParam('prepareDynamicContent', null);
        if (prepareDynamicContent) {
            this._prepareDynamicContent = prepareDynamicContent.bind(this);
        }
        var bindDynamicContent = this._getConfigParam('bindDynamicContent', null);
        if (bindDynamicContent) {
            this._bindDynamicContent = bindDynamicContent.bind(this);
        }
        var insertDynamicContent = this._getConfigParam('insertDynamicContent', null);
        if (insertDynamicContent) {
            this._insertDynamicContent = insertDynamicContent.bind(this);
        }
        var onUpdateDynamic = this._getConfigParam('onUpdateDynamic', null);
        if (onUpdateDynamic) {
            this._onUpdateDynamic = onUpdateDynamic.bind(this);
        }
        var onInitDynamic = this._getConfigParam('onInitDynamic', null);
        if (onInitDynamic) {
            this._onInitDynamic = onInitDynamic.bind(this);
        }
    },

    _addEvents: function($super) {
        $super();
        var dynamicElements = this._componentElement.select('.dynamicSubFormContent');
        if (0 == dynamicElements.size()) {
            this._addEmptyContent();
        } else {
            dynamicElements.each(function(element, index) {
                this._bindDynamicContent(element, 0 == index);
            }, this);
        }
        this._onInitDynamic();
        this._onUpdateDynamic();
    },

    _addEmptyContent: function() {
        var element = this._prepareDynamicContent(this._getDynamicContent());
        this._insertDynamicContent(element);
        this._bindDynamicContent(element, true);
    },

    _getDynamicContent: function() {
        var dynamicElement = this._componentElement.down('.dynamicSubFormTemplate').cloneNode(true);
        dynamicElement.removeClassName('dynamicSubFormTemplate').addClassName('dynamicSubFormContent').show();
        return dynamicElement;
    },

    _prepareDynamicContent: function(content, value) {
        return content;
    },

    _insertDynamicContent: function(content) {
        var dynamicElements = this._componentElement.select('.dynamicSubFormContent');
        if (0 == dynamicElements.size()) {
            this._componentElement.insert({top : content});
        } else {
            dynamicElements.last().insert({after : content});
        }
    },

    _bindDynamicContent: function(content, isFirstElement) {
    },

    _onInitDynamic: function() {
    },

    _onUpdateDynamic: function() {
    },

    deleteContent: function(content) {
        content.remove();
        this._onUpdateDynamic();
    },

    addContent: function(value) {
        var content = this._prepareDynamicContent(this._getDynamicContent(), value);
        this._insertDynamicContent(content);
        this._bindDynamicContent(content, false);
        this._onUpdateDynamic();
    }

});

Jsw.AddRemoveDynamicSubForm = Class.create(Jsw.DynamicSubForm, {
    _initConfiguration: function($super, config) {
        $super(config);
        this._addButtonId = this._getConfigParam('addButtonId', '');
        this._addButtonTitle = this._getConfigParam('addButtonTitle', '');
        this._removeButtonId = this._getConfigParam('removeButtonId', '');
        this._removeButtonTitle = this._getConfigParam('removeButtonTitle', '');
    },

    _onInitDynamic: function() {
        $(this._addButtonId).replace('<div id="' + this._addButtonId + '" class="text-value"></div>');
        new Jsw.SmallButton({
            id: 'dynamicSubFormContentAddButton-' + this._id,
            title: this._addButtonTitle,
            addCls: 'sb-item-add dynamicSubFormContentAddButton',
            handler: this.addContent.bind(this),
            disabled: this._isAddButtonDisabled(),
            renderTo: this._addButtonId
        });
    },

    _isAddButtonDisabled: function() {
        return false;
    },

    _bindDynamicContent: function(content, isFirstElement) {
        if (!isFirstElement) {
            var wrapperId = this._initRemoveButtonWrapper(content);
            new Jsw.SmallButton({
                id: wrapperId,
                title: this._removeButtonTitle,
                addCls: 'sb-item-remove dynamicSubFormContentRemoveButton',
                handler: this.deleteContent.bind(this, content),
                renderTo: wrapperId
            });
        }
        return content;
    },

    _initRemoveButtonWrapper: function(content) {
        var wrapperId = this._removeButtonId + '-' + this._getRandomNumber();
        content.select('.field-value input, .field-value select').last().insert({
            after: '<span id="' + wrapperId + '" class="' + this._removeButtonId + '"></span>'
        });
        return wrapperId;
    },

    _prepareDynamicContent : function(content) {
        var newId = Math.floor(1000000*Math.random());

        var nameRegExp = new RegExp("(name=.*?\\[" + this._id + "\\]\\[)dynamicSubFormTemplate(\\])", "g");
        var idRegExp = new RegExp("(id=.*?\\-" + this._id + "-)dynamicSubFormTemplate(-)", "g");

        content = content.update(content.innerHTML
            .replace(nameRegExp, '\$1c' + newId +'\$2')
            .replace(idRegExp, '\$1c' + newId +'\$2')
        );

        return content;
    },

    _onUpdateDynamic: function() {
        var childs = this._componentElement.select('.dynamicSubFormContent');
        if (0 < childs.size()) {
            childs.each(function(child, i) {
                if (0 == i) {
                    child.select('label').last().show();
                } else {
                    child.select('label').last().hide();
                }
            });
            $$('label[for="' + this._addButtonId + '"]').first().hide();
        } else {
            $$('label[for="' + this._addButtonId + '"]').first().show();
        }
    },

    addContent: function($super, event) {
        $super();
        Event.stop(event);
    },

    deleteContent: function($super, content, event) {
        $super(content);
        Event.stop(event);
    }
});

Jsw.Box = Class.create(Jsw.Component, {
    _tag: 'div',

    _initConfiguration: function($super, config) {
        $super(config);
    },

    _initComponentElement: function($super) {
        $super();

        this._componentElement.update(
            this._getConfigParam('html', '')
        );
    }
});

Jsw.Gauge = Class.create(Jsw.Component, {
    _tag: 'span',

    _initConfiguration: function($super, config) {
        $super(config);
        this._cls = this._getConfigParam('cls', 'gauge-wrapper f9 not_empty');
    },

    _initComponentElement: function($super) {
        $super();

        this.setProgress(this._getConfigParam('value', ''));
    },

    setProgress: function(value)
    {
        this._componentElement.update(
            '<span class="gauge-container">' +
                '<span style="width: ' + value + '%" class="gauge-filled"></span>' +
            '</span>'
        );
    }
});

Jsw.Cookie = {
    get: function(name) {
        var cookies = document.cookie.split("; ");
        for (var i = 0; i < cookies.length; i++) {
            var crumbs = cookies[i].split("=");
            if (name == crumbs[0]) {
               return unescape(crumbs[1]);
            }
        }
        return null;
    },

    set: function(name, value, expires, path) {
        var cookie = name + "=" + escape(value) + "; ";

        if (('undefined' != typeof expires) && expires) {
            cookie += 'expires=' + expires + '; '
        }

        if (('undefined' != typeof path) && path) {
            cookie += "path=" + path + '; ';
        }

        document.cookie = cookie;
    },

    setPermanent: function(name, value, path) {
        var date = new Date();
        date.setTime(date.getTime()+(365*24*60*60*1000));
        var expires = date.toGMTString();
        this.set(name, value, expires, path);
    },

    remove: function(name, path) {
        var cookie = name + "=; expires=Fri, 31 Dec 1999 23:59:59 GMT;";

        if (('undefined' != typeof path) && path) {
            cookie += "path=" + path + '; ';
        }

        document.cookie = cookie;
    }
};

Jsw.Tooltip = {
    showTimeoutStatus: true,    //show timeout
    showTimeout: 0.55,            //show timeout
    minDelta: 5,                //minimum viewport delta
    xOffsetLeft: -5,            //horisonatl offset from cursor
    xOffsetRight: 5,            //horisonatl offset from cursor
    yOffsetTop: 1,                //vertical offset from cursor
    yOffsetBottom: 13,            //vertical offset from cursor

    current: null,

    initData: function(contentElement) {
        var elements;

        if ('undefined' != typeof contentElement) {
            elements = contentElement.select('.tooltipData');
        } else {
            elements = $$('.tooltipData');
        }

        elements.each(function(tooltipData) {
            Jsw.Tooltip.init(tooltipData.parentNode, {element: tooltipData});
        });
    },

    init: function(element, config) {
        return new Jsw.Tooltip.Instance(element, config);
    },

    set: function(tooltip) {
        if (Jsw.Tooltip.current && Jsw.Tooltip.current !== tooltip) {
            Jsw.Tooltip.hide();
        }
        if (Jsw.Tooltip.current === tooltip) {
            return;
        }
        Jsw.Tooltip.current = tooltip;
        Element.observe(tooltip._element, 'mousemove', Jsw.Tooltip._onMouseMove);
        Element.observe(tooltip._element, 'mouseout', Jsw.Tooltip._onMouseOut);

        Jsw.Tooltip.show();
    },

    show: function() {
        Jsw.Tooltip.showCancel();
        Jsw.Tooltip.hideCancel();
        if (Jsw.Tooltip.showTimeoutStatus) {
            Jsw.Tooltip.showTask = new PeriodicalExecuter(Jsw.Tooltip.showFinish, Jsw.Tooltip.showTimeout);
        } else {
            Jsw.Tooltip.showFinish();
        }
    },

    showCancel: function() {
        if (Jsw.Tooltip.showTask) {
            Jsw.Tooltip.showTask.stop();
            Jsw.Tooltip.showTask = null;
        }
    },

    showFinish: function() {
        Jsw.Tooltip.showCancel();
        var tooltipElement = Jsw.Tooltip._fetchTooltipElement();
        var tooltip = Jsw.Tooltip.current;
        if (tooltip._config.text) {
            tooltipElement.down('span').update(tooltip._config.text);
        } else
        if(tooltip._config.element) {
            tooltipElement.down('span').update(tooltip._config.element.innerHTML);
        }
        tooltipElement.show();
        Jsw.Tooltip.update();
    },

    hide: function() {
        Jsw.Tooltip.showCancel();
        if (!Jsw.Tooltip.current)
            return;
        var tooltip = Jsw.Tooltip.current;
        Jsw.Tooltip.current = null;
        var tooltipElement = Jsw.Tooltip._fetchTooltipElement();
        tooltipElement.hide();
        Element.stopObserving(tooltip._element, 'mousemove', Jsw.Tooltip._onMouseMove);
        Element.stopObserving(tooltip._element, 'mouseout', Jsw.Tooltip._onMouseOut);
        Jsw.Tooltip.hideTask = new PeriodicalExecuter(Jsw.Tooltip.hideFinish, Jsw.Tooltip.showTimeout);
    },

    hideCancel: function() {
        if (Jsw.Tooltip.hideTask) {
            Jsw.Tooltip.hideTask.stop();
            Jsw.Tooltip.hideTask = null;
        }
    },

    hideFinish: function() {
        Jsw.Tooltip.showTimeoutStatus = true;
    },

    update: function() {
        if (!Jsw.Tooltip.current)
            return;
        var tooltipElement = Jsw.Tooltip._fetchTooltipElement();
        var position = Jsw.Tooltip._fetchTooltipPosition(tooltipElement);
        Element.setStyle(tooltipElement, {
            top: position.y + 'px',
            left: position.x + 'px'
        });
    },

    _onMouseOut: function(event) {
        Jsw.Tooltip.hide();
    },

    _onMouseMove: function(event) {
        Jsw.Tooltip.tooltipX = Event.pointerX(event);
        Jsw.Tooltip.tooltipY = Event.pointerY(event);
        Jsw.Tooltip.update();
    },

    _fetchTooltipElement: function() {
        if (!Jsw.Tooltip._tooltipElement) {
            Jsw.Tooltip._tooltipElement = $$('.tooltip').first();
        }

        return Jsw.Tooltip._tooltipElement;
    },

    _fetchTooltipPosition: function(tooltipElement) {
        var x = Jsw.Tooltip.tooltipX;
        var y = Jsw.Tooltip.tooltipY;

        var tDim = Element.getDimensions(tooltipElement);
        var vDim = document.viewport.getDimensions();
        var vScr = document.viewport.getScrollOffsets();

        var freeRight = vDim.width + vScr.left - Jsw.Tooltip.tooltipX;
        var freeLeft = Jsw.Tooltip.tooltipX - vScr.left;
        var freeTop = Jsw.Tooltip.tooltipY - vScr.top;
        var freeBottom = vDim.height + vScr.top - Jsw.Tooltip.tooltipY;

        //apply better x
        if( freeRight > tDim.width + Jsw.Tooltip.minDelta + Jsw.Tooltip.xOffsetRight) {
            x = Jsw.Tooltip.tooltipX + Jsw.Tooltip.xOffsetRight;
        } else
        if( freeLeft > tDim.width + Jsw.Tooltip.minDelta + Jsw.Tooltip.xOffsetLeft) {
            x = Jsw.Tooltip.tooltipX - tDim.width - Jsw.Tooltip.xOffsetLeft;
        } else {
            x = Jsw.Tooltip.tooltipX - (tDim.width + Jsw.Tooltip.xOffsetLeft) / 2;
        }

        //apply better y
        if (freeBottom > tDim.height + Jsw.Tooltip.minDelta + Jsw.Tooltip.yOffsetBottom) {
            y = Jsw.Tooltip.tooltipY + Jsw.Tooltip.yOffsetBottom;
        } else
        if( freeTop > tDim.height + Jsw.Tooltip.minDelta + Jsw.Tooltip.yOffsetTop) {
            y = Jsw.Tooltip.tooltipY - tDim.height - Jsw.Tooltip.yOffsetTop;
        } else {
            y = Jsw.Tooltip.tooltipY - (tDim.height + Jsw.Tooltip.yOffsetTop) / 2;
        }
        if (x < 0) {
        	x = Jsw.Tooltip.minDelta;
    	}

        return { x: x, y: y };
    }

}

Event.observe(window, 'load', function() {
    Jsw.Tooltip.initData();
});

Jsw.Tooltip.Instance = Class.create({

    initialize: function(element, config) {
        this._element = element;
        this._config = config;
        //observe events
        Event.observe(this._element, 'mouseover', this.set.bind(this));
    },

    set: function() {
        Jsw.Tooltip.set(this);
    },

    setText: function(text) {
        this._config.text = text;
    }
});

Jsw.AjaxTooltip = {
    windowId: 'ajaxTooltipWindow',
    delay: 0.55,
    _isInitialized: false,
    _currentTooltip: null,

    _init: function(cls) {
        if (this._isInitialized) {
            return ;
        }

        this._cls = cls;
        this._initWindowElement();
        this._initWindowEvents();
        this._isInitialized = true;
    },

    _initWindowElement: function($super) {
        $(document.body).insert({
            top: '<div id="' + this.windowId + '" class="' + this._cls + '" style="display: none; z-index: 10000"></div>'
        });

        $(this.windowId).update(
            '<table cellspacing="0" class="popup-wrapper"><tbody><tr><td class="popup-container">' +
                '<div class="c1"><div class="c2"><div class="c3"><div class="c4"><div class="c5">' +
                    '<div class="popup-heading">' +
                        '<div class="heading-area">' +
                            '<span class="close" id="' + this.windowId + '-close' + '"></span>' +
                            '<h4><span id="' + this.windowId + '-title' + '"></span></h4>' +
                        '</div>' +
                    '</div>' +
                    '<div class="popup-content">' +
                        '<div id="' + this.windowId + '-content' + '" class="popup-content-area"></div>' +
                    '</div>' +
                '</div></div></div></div></div>' +
            '</td></tr></tbody></table>'
        );
    },

    _initWindowEvents: function() {
        Event.observe($(this.windowId), 'mouseover', this._onWindowMouseOver.bindAsEventListener(this));
        Event.observe($(this.windowId), 'mouseout', this._onWindowMouseOut.bindAsEventListener(this));
        Event.observe($(this.windowId + '-close'), 'click', this._onCloseClick.bindAsEventListener(this));
    },

    _onWindowMouseOver: function(event, title) {
        this._showWindow();
    },

    _onWindowMouseOut: function() {
        $(this.windowId).hide();
    },

    _onCloseClick: function() {
        $(this.windowId).hide();
    },

    set: function(tooltip) {
        this._init(tooltip.getCls());
        var title = tooltip.getTitle();
        var titleElement = $(this.windowId + '-title');
        if (false === title) {
        	$(this.windowId + '-title').update('');
        	titleElement.up('.popup-heading').hide();
        } else {
        	$(this.windowId + '-title').update(title);
        	titleElement.up('.popup-heading').show();
        }
        $(this.windowId + '-content').update(tooltip.getContent());

        Jsw.clonePosition($(this.windowId), tooltip.getTargetElement(), {offsetTop: document.viewport.getScrollOffsets().top, setWidth: false, setHeight: false });
        this._showWindow();
        this._currentTooltip = tooltip;
    },

    updateContent: function(tooltip) {
        if (this._currentTooltip == tooltip) {
            $(this.windowId + '-content').update(tooltip.getContent());
        }
        this._updatePosition();
    },

    _showWindow: function() {
        $(this.windowId).show();
        this._updatePosition();
    },

    _updatePosition: function()
    {
    	var element = $(this.windowId);

        var vWidth = document.viewport.getWidth();
        var vLeft = document.viewport.getScrollOffsets().left;
        var eWidth = Element.getWidth(element);
        var eLeft = Element.cumulativeOffset(element).left;
        if (eLeft + eWidth > vWidth + vLeft) {
        	Jsw.clonePosition(element, element, {offsetLeft: (vWidth + vLeft) - (eLeft + eWidth),  offsetTop: document.viewport.getScrollOffsets().top, setWidth: false, setHeight: false });
        }
    }
}

Jsw.AjaxTooltip.Instance = Class.create(Jsw.Component, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._targetElement = $(this._getConfigParam('target', ''));
        this._title = this._getConfigParam('title', '');
        this._waitMsg = this._getConfigParam('waitMsg', '');
        this._url = this._getConfigParam('url', '');
        this._cls = this._getConfigParam('cls', 'popup-box');

        this._content = '<div class="ajax-loading">' + this._waitMsg + '</div>';
        this._contentLoaded = false;
        this._delayedExecuter = null;
    },

    _initComponentElement: function($super) {
        this._initTargetEvents();
    },

    _initTargetEvents: function() {
        Event.observe(this._targetElement, 'mouseover', this.set.bind(this));
        Event.observe(this._targetElement, 'mouseout', this.unset.bind(this));
    },

    _loadContent: function() {
        var context = this;
        new Ajax.Request(Jsw.prepareUrl(this._url), {
            method: 'get',
            onSuccess: function(transport) {
                if ('' == transport.responseText) {
                    return;
                }

                context._content = transport.responseText;
                Jsw.AjaxTooltip.updateContent(context);
            }
        });

        this._contentLoaded = true;
    },

    setText: function(text)
    {
    	this._contentLoaded = true;
    	this._content = text;
    },

    set: function() {
        var context = this;
        this._delayedExecuter = new PeriodicalExecuter(function(pe) {
            Jsw.AjaxTooltip.set(context);
            pe.stop();
            this._delayedExecuter = null;
        }, Jsw.AjaxTooltip.delay);
    },

    unset: function() {
        if (this._delayedExecuter) {
            this._delayedExecuter.stop();
            this._delayedExecuter = null;
        }
    },

    getTitle: function() {
        return this._title;
    },

    getContent: function() {
        if (!this._contentLoaded) {
            this._loadContent();
        }

        return this._content;
    },

    getTargetElement: function() {
        return this._targetElement;
    },

    getCls: function() {
        return this._cls;
    }

});

Jsw.messageBox = {

    TYPE_OK: 'TYPE_OK',
    TYPE_YESNO: 'TYPE_YESNO',

    _messageBox: null,

    show: function(config) {
        if (!Jsw.messageBox._messageBox) {
            Jsw.messageBox._messageBox = config.isAjax ? new Jsw.AjaxMessageBox(config) : new Jsw.MessageBox(config);
        } else {
            Jsw.messageBox._messageBox.initialize(config);
        }

        Jsw.messageBox._messageBox.show();
    }
};

Jsw.ModalDialogBox = Class.create(Jsw.Component, {
    _initConfiguration: function($super, config) {
        $super(config);

        this._screenDisablerId = this._getConfigParam('screenDisablerId', 'screenDisabler');

        if (!$(this._screenDisablerId)) {
            $(document.body).insert({
                top: '<div id="' + this._screenDisablerId + '" class="main-disabled-block" style="display: none;"></div>'
            });
        }

        this._screenDisabler = $(this._screenDisablerId);

        this._renderTargetId = this._getConfigParam('renderTo', 'modalDialogBox');

        if ($(this._renderTargetId)) {
            $(this._renderTargetId).remove();
        }

        this._screenDisabler.insert({ after: '<div id="' + this._renderTargetId + '"></div>' });
    },

    _makeCentered: function() {
        var windowElement = this._componentElement.down();
        var viewportTopOffset = document.viewport.getScrollOffsets().top;
        var windowTopOffset = document.viewport.getHeight() / 2 - windowElement.getHeight() / 2;

        if (windowTopOffset > 0) {
            windowElement.setStyle({ top: (viewportTopOffset + windowTopOffset) + 'px' });
        }
    },

    show: function($super) {
        this._screenDisabler.show();
        $super();
        this._makeCentered();
    },

    hide: function($super) {
        $super();

        this._screenDisabler.hide();
    }

});

Jsw.PopupForm = Class.create(Jsw.ModalDialogBox, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._popupContentAreaId = this._id + '-popup-content-area';
        this._titleAreaId = this._id + '-title-area';
        this._hintAreaId = this._id + '-hint-area';
        this._hint1AreaId = this._id + '-hint1-area';
        this._boxAreaId = this._id + '-box-area';
        this._contentAreaId = this._id + '-content-area';

        this._actionButtonsId = this._id + '-action-buttons';
        this._leftActionButtonsAreaId = this._id + '-left-action-buttons-area';
        this._rightActionButtonsAreaId = this._id + '-right-action-buttons-area';

        this._titleCls = this._getConfigParam('titleCls', '');
        this._isRemoved = false;
    },

    _initComponentElement: function($super) {
        $super();
        this._updateComponentElement(
            '<table class="popup-wrapper" cellspacing="0"><tbody><tr><td class="popup-container">' +
                '<div class="c1"><div class="c2"><div class="c3"><div class="c4"><div class="c5">' +
                    '<div class="popup-heading">' +
                        '<div class="heading-area">' +
                            '<h2 class="' + this._titleCls + '"><span id="' + this._titleAreaId + '">' +
                                this._getConfigParam('title', '') +
                            '</span></h2>' +
                        '</div>' +
                    '</div>'+
                    '<div class="popup-content">' +
                        '<div class="popup-content-area" id="' + this._popupContentAreaId + '">' +
                            this._getContentArea() +
                        '</div>' +
                    '</div>' +
                '</div></div></div></div></div>' +
            '</td></tr><tr/></tbody></table>'
        );
        this._screenDisabler.show();
        this._makeCentered();
    },

    _getContentArea: function() {
        return '' +
            '<p id="' + this._hint1AreaId + '"></p>' +
            '<span id="' + this._hintAreaId + '"></span>' +
            this._getBoxArea() +
            this._getButtonsArea();
    },

    _getBoxArea: function() {
        return '' +
            '<div class="form-box" id="' + this._boxAreaId + '">' +
                '<div class="box-area">' +
                    '<div class="content">' +
                        '<div class="content-area" id="' + this._contentAreaId + '">' +
                        '</div>' +
                    '</div>' +
                '</div>' +
            '</div>';
    },

    _getButtonsArea: function() {
        return '' +
            '<div class="btns-box" id="' + this._actionButtonsId + '">' +
                '<div class="box-area">' +
                    '<div class="form-row">' +
                        '<div class="field-name" id="' + this._leftActionButtonsAreaId + '"> </div>' +
                        '<div class="field-value" id="' + this._rightActionButtonsAreaId + '"> </div>' +
                    '</div>' +
                '</div>' +
            '</div>';
    },

    _makeCentered: function() {
        var viewportTopOffset = document.viewport.getScrollOffsets().top;
        this._componentElement.setStyle({ top: (viewportTopOffset + 110) + 'px' });
    },

    setBoxType: function(type) {
        $(this._boxAreaId).className = type;
        if ('list-box' == type) {
            $(this._actionButtonsId).addClassName('no-separator');
            $(this._actionButtonsId).addClassName('simple-box');
        }
        if ('form-box' == type) {
            $(this._actionButtonsId).removeClassName('no-separator');
            $(this._actionButtonsId).removeClassName('simple-box');
        }
        if ('fm-box' == type) {
            $(this._boxAreaId).className = 'list-box';
            $(this._actionButtonsId).addClassName('no-separator');
        }
    },

    setTitleType: function(cls) {
        $(this._titleAreaId).parentNode.className = cls;
    },

    setTitle: function(text) {
        $(this._titleAreaId).update(text);
    },

    setHint: function(text) {
        if ('' == text) {
            $(this._hintAreaId).hide();
        } else {
            $(this._hintAreaId).update(text);
            $(this._hintAreaId).show();
        }
    },

    setHint1: function(text) {
        if ('' == text) {
            $(this._hint1AreaId).hide();
        } else {
            $(this._hint1AreaId).update(text);
            $(this._hint1AreaId).show();
        }
    },

    remove: function() {
        this._isRemoved = true;
        $(this._renderTargetId).remove();
    },

    addRightButton: function(title, handler, isDefault, isAction, params) {
        var button = this._createButton(title, handler, isDefault, isAction, params);
        $(this._rightActionButtonsAreaId).insert(button);
        return button;
    },

    addLeftButton: function(title, handler, isDefault, isAction, params) {
        var button = this._createButton(title, handler, isDefault, isAction, params);
        $(this._leftActionButtonsAreaId).insert(button);
        return button;
    },

    removeRightButtons: function() {
        $(this._rightActionButtonsAreaId).update(' ');
    },

    removeLeftButtons: function() {
        $(this._leftActionButtonsAreaId).update(' ');
    },

    _createButton: function(title, handler, isDefault, isAction, params) {
        params = (typeof params !== 'undefined') ? params : {};

        var buttonContainer = new Element('span', params);
        buttonContainer.addClassName('btn');
        if (isAction) {
            buttonContainer.addClassName('action');
        }

        buttonContainer.update(
            '<button value="" type="' + (isDefault ? 'submit' : 'button') + '">' +
                title +
            '</button>'
        );
        buttonContainer.down('button').observe(
            'click',
            this._onClick.bindAsEventListener(this, handler.bind(this))
        );

        return buttonContainer;
    },

    _updateButton: function(element, config) {
        $H(config).each(function(pair) {
            switch (pair.key) {
            case 'title':
                element.down('button').update(pair.value);
                break;
            case 'disabled':
                element.down('button').disabled = pair.value ? true : false;
                pair.value ? Element.addClassName(element, 'disabled')
                           : Element.removeClassName(element, 'disabled');
                break;
            }
        });
    },

    _onClick: function(event, handler) {
        Event.stop(event);
        handler(event);
    },

    _addEvents: function($super) {
        $super();

        if ($(this._hintAreaId)) {
            $(this._hintAreaId).hide();
        }
        if ($(this._hint1AreaId)) {
            $(this._hint1AreaId).hide();
        }
    },

    _clearMessages: function() {
        var contentElement = $(this._popupContentAreaId);

        ['.msg-error', '.msg-info', '.msg-warning'].each(function(msgClass) {
            contentElement.select(msgClass).each(function(messageElement) {
                messageElement.remove();
            });
        });
    },

    _addErrorMessage: function(message) {
        $(this._popupContentAreaId).insert(
            {
                top:
                    '<div class="msg-box msg-error">' +
                        '<div>' +
                            '<div>' +
                                '<div>' +
                                    '<div>' +
                                        '<div>' +
                                            '<div class="msg-content">' +
                                                message +
                                            '</div>' +
                                        '</div>' +
                                    '</div>' +
                                '</div>' +
                            '</div>' +
                        '</div>' +
                    '</div>'
            }
        );
    },

    _userError: function(error) {
        Jsw.clearStatusMessages();
        Jsw.addStatusMessage('error', error);
        this._close();
    },

    _internalError: function(error) {
        Jsw.clearStatusMessages();
        Jsw.addStatusMessage('error', 'Internal error: ' + error);
        this._close();
    }

});

Jsw.MessageBox = Class.create(Jsw.ModalDialogBox, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._type = this._getConfigParam('type', Jsw.messageBox.TYPE_YESNO);
        this._text = this._getConfigParam('text', '');
        this._subtype = this._getConfigParam('subtype', null);
        this._description = this._getConfigParam('description', '');
        this._descriptionWrapperTag = this._getConfigParam('descriptionWrapperTag', 'p');
        this._buttonTitles = this._getConfigParam('buttonTitles', null);

        this._onYesClick = this._getConfigParam('onYesClick', null);
        this._onNoClick = this._getConfigParam('onNoClick', null);
        this._onOkClick = this._getConfigParam('onOkClick', null);
        this._needAttention = this._getConfigParam('needAttention', false);
        this._needAttentionText = this._getConfigParam('needAttentionText', '');
        this._needAttentionBlockSubmit = this._getConfigParam('needAttentionBlockSubmit', false);

        if (Jsw.messageBox.TYPE_YESNO == this._type) {
           if (this._onYesClick) {
               this._onYesClickHandler = this._onYesClick.bindAsEventListener(this);
           } else {
               this._onYesClickHandler = this._defaultOnButtonClick.bindAsEventListener(this);
           }
        }
    },

    _getDescriptionCode: function() {
        var descriptionCode = '';
        if (Object.isArray(this._description)) {
            this._description.each(function(item) {
                descriptionCode += '<' + this._descriptionWrapperTag + '>' + item + '</' + this._descriptionWrapperTag +'>';
            })
        } else {
            descriptionCode += '<' + this._descriptionWrapperTag + '>' + this._description + '</' + this._descriptionWrapperTag + '>';
        }
        return descriptionCode;
    },

    _getNeedAttentionCode: function() {
        var needAttentionCode = '';
        if (this._needAttention) {
           needAttentionCode = '<hr>';
           if (this._needAttentionBlockSubmit) {
               needAttentionCode += '<p><span class="b-indent"><span class="b-indent-icon"><input type="checkbox" class="checkbox" id="needAttentionConfirmationCheckbox"></span><label for="needAttentionConfirmationCheckbox">' + this._needAttentionText + '</label></span></p>';
           } else {
               needAttentionCode += '<p class="hint-warning">' + this._needAttentionText + '</p>';
           }
           needAttentionCode += '<hr>';
        }
        return needAttentionCode;
    },

    _initComponentElement: function($super) {
        $super();

        var descriptionCode = this._getDescriptionCode();
        var needAttentionCode = this._getNeedAttentionCode();

        var contentHtml =
            '<div class="confirmation-box">' +
                '<table cellspacing="0"><tr><td>' +
                    '<div class="c1"><div class="c2"><div class="c3"><div class="c4"><div class="c5">' +
                        '<div class="confirmation-msg' + (this._subtype ? ' mw-' + this._subtype : '') + '">' +
                            '<h3>' + this._text + '</h3>' +
                            descriptionCode + needAttentionCode +
                            '<div class="btns-container">';

        if (Jsw.messageBox.TYPE_YESNO == this._type) {
            var buttonDefault = this._getConfigParam('buttonDefault', 'no');
            contentHtml += '<span class="btn' + ('yes' == buttonDefault ? ' action' : '') + '"><button type="submit" value="">' + this._buttonTitles.yes + '</button></span>';
            contentHtml += '<span class="btn' + ('no' == buttonDefault ? ' action' : '') + '"><button type="submit" value="">' + this._buttonTitles.no + '</button></span>';
        }

        if (Jsw.messageBox.TYPE_OK == this._type) {
            contentHtml += '<span class="btn action"><button type="submit" value="">' + this._buttonTitles.ok + '</button></span>';
        }

        contentHtml +=      '</div>' +
                        '</div>' +
                    '</div></div></div></div></div>' +
                '</td></tr></table>' +
            '</div>';

        this._updateComponentElement(contentHtml);

        this._componentElement.hide();
    },

    _disableYesButton: function() {
        if (Jsw.messageBox.TYPE_YESNO == this._type) {
            this._componentElement.select('.btn').first().addClassName('disabled');
            var buttonYes = this._componentElement.select('button').first();
            buttonYes.disable();
            Event.stopObserving(buttonYes, 'click', this._onYesClickHandler);
        }
    },

    _enableYesButton: function(button) {
        if (Jsw.messageBox.TYPE_YESNO == this._type) {
           this._componentElement.select('.btn').first().removeClassName('disabled');
           var buttonYes = this._componentElement.select('button').first();
           buttonYes.enable();
           Event.observe(buttonYes, 'click', this._onYesClickHandler);
        }
    },

    _addEvents: function($super) {
        $super();

        // add default behavior - hide dialog
        this._componentElement.select('button').each(function(button) {
            Event.observe(button, 'click', this._defaultOnButtonClick.bindAsEventListener(this));
        }, this);

        if (Jsw.messageBox.TYPE_YESNO == this._type) {
            if (this._onYesClick) {
                var buttonYes = this._componentElement.select('button').first();
                Event.observe(buttonYes, 'click', this._onYesClickHandler);
            }

            if (this._onNoClick) {
                var buttonNo = this._componentElement.select('button').last();
                Event.observe(buttonNo, 'click', this._onNoClick.bindAsEventListener(this));
            }

           if (this._needAttention && this._needAttentionBlockSubmit) {
                var confirmationCheckbox = this._componentElement.select('input').last();
                var top = this;
                Event.observe(confirmationCheckbox, 'click',
                              function() { if ($F(confirmationCheckbox)) {top._enableYesButton()} else {top._disableYesButton()}} );
           }
        }

        if (Jsw.messageBox.TYPE_OK == this._type) {
            if (this._onOkClick) {
                var buttonOk = this._componentElement.select('button').first();
                Event.observe(buttonOk, 'click', this._onOkClick.bindAsEventListener(this));
            }
        }
    },

    _addTooltips: function() {
    },

    _defaultOnButtonClick: function() {
        this.hide();
    },

    show: function($super) {
        $super();
        this._componentElement.select('.action').first().select('button').first().focus();
        if (this._needAttention && this._needAttentionBlockSubmit) {
           this._disableYesButton();
        }
    }

});

Jsw.AjaxMessageBox = Class.create(Jsw.MessageBox, {
    _initConfiguration: function($super, config) {
        $super(config);
        this._requestUrl = this._getConfigParam('requestUrl', '');
        this._requestMethod = this._getConfigParam('requestMethod', 'post');
        this._requestParams = this._getConfigParam('requestParams', '');
        this._contentContainerId = this._getConfigParam('contentContainerId', this._id + 'contentContainer');
        this._confirmationCheckboxId = this._getConfigParam('confirmationCheckboxId', this._id + 'confirmationCheckbox');
        this._loadingTitle = this._getConfigParam('loadingTitle', '');
    },

    _getDescriptionCode: function() {
        return '';
    },

    _getNeedAttentionCode: function($super) {
        var needAttentionCode = $super();
        return '<div id="' + this._contentContainerId + '"></div>' + needAttentionCode;
    },

    _initComponentElement: function($super) {
        $super();
    },

    _showSpinner: function() {
        if (Jsw.messageBox.TYPE_YESNO == this._type) {
            this._componentElement.select('button').first().update('<span class="wait">' + this._buttonTitles.wait + '</span>');
        }
    },

    _hideSpinner: function(button) {
        if (Jsw.messageBox.TYPE_YESNO == this._type) {
            this._componentElement.select('button').first().update(this._buttonTitles.yes);
        }
    },

    show: function($super) {
        $super();
        if (!this._requestUrl) {
            return;
        }

        new Ajax.Request(Jsw.prepareUrl(this._requestUrl), {
            method: this._requestMethod,
            parameters: this._requestParams,
            onCreate: this._onCreate.bind(this),
            onSuccess: this._onSuccess.bind(this),
            onFailure: this._onFailure.bind(this)
        });
    },

    _onCreate: function() {
        this._disableYesButton();
        this._showSpinner();
    },

    _onSuccess: function(transport) {
        this._hideSpinner();

        var result = transport.responseText.evalJSON();

        var content = result.content;
        var contentCode = '';
        if (Object.isArray(content)) {
            if (content.length > 0) {
                contentCode = '<ul class="ul">';
                content.each(function(item) {
                    contentCode += '<li>' + item + '</li>';
                })
                contentCode += '</ul>';
            }
        } else {
            contentCode = content;
        }

        var descriptionCode = '';
        if (result.description) {
            descriptionCode = '<p><span class="hint-warning">' + result.description + '</span></p>';
        }

        var confirmationCode = '';
        if (result.confirmation) {
            confirmationCode = '<p><span class="b-indent">' +
                '<span class="b-indent-icon">' +
                    '<input type="checkbox" class="checkbox" id="' + this._confirmationCheckboxId + '"/>' +
                '</span>' +
                '<label for="' + this._confirmationCheckboxId + '">' + result.confirmation + '</label>' +
            '</span></p>';
        }

        $(this._contentContainerId).update('' + contentCode + descriptionCode + confirmationCode + '');

        var checkboxes = this._componentElement.select('input[type="checkbox"]');
        var top = this;
        checkboxes.each(function(checkbox) {
            Event.stopObserving(checkbox, 'click');
            Event.observe(checkbox, 'click',
                function() {
                    var enableYesButton = true;
                    top._componentElement.select('input[type="checkbox"]').each(function(confirmationCheckbox) {
                        if (!$F(confirmationCheckbox)) {
                            enableYesButton = false;
                        }
                    });
                    if (enableYesButton) {
                        top._enableYesButton()
                    } else {
                        top._disableYesButton()}
                }
            );
        });

        if (0 == checkboxes.length) {
            this._enableYesButton();
        }
    },

    _onFailure: function() {
        this._hideSpinner();
        this._enableYesButton();
    }

});


Jsw.Popup = Class.create(Jsw.ModalDialogBox, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._popupContentAreaId = this._id + '-popup-content-area';
        this._titleAreaId = this._id + '-title-area';
        this._actionButtonsId = this._id + '-action-buttons';
        this._leftActionButtonsAreaId = this._id + '-left-action-buttons-area';
        this._rightActionButtonsAreaId = this._id + '-right-action-buttons-area';

	    this._contentHeight = this._getConfigParam('contentHeight', null);
	    this._closeButtonEnabled = this._getConfigParam('closeButtonEnabled', false);
	    this._popupCls = this._getConfigParam('popupCls', 'popup-panel');
        this._titleCls = this._getConfigParam('titleCls', '');
        this._buttons = this._getConfigParam('buttons', []);
        this._onShow = this._getConfigParam('onShow', function(){});
    },

    _initComponentElement: function($super) {
        $super();
	    var closeButton = '';
	    if (this._closeButtonEnabled) {
	        closeButton = '<span class="close" id="' + this._id + '-close"></span>';
	    }
	    var contentExtraStyle = '';
	    if (this._contentHeight) {
		    contentExtraStyle = ' style="height:' + this._contentHeight + 'px"';
	    }
        this._updateComponentElement(
            '<div class="' + this._popupCls + '"><table class="popup-wrapper" cellspacing="0"><tbody><tr><td class="popup-container">' +
                '<div class="c1"><div class="c2"><div class="c3"><div class="c4"><div class="c5">' +
                    '<div class="popup-heading">' +
                        '<div class="heading-area">' +
		                    closeButton +
                            '<h2 class="' + this._titleCls + '"><span id="' + this._titleAreaId + '">' +
                                this._getConfigParam('title', '') +
                            '</span></h2>' +
                        '</div>' +
                    '</div>'+
                    '<div class="popup-content">' +
                        '<div class="popup-content-area" id="' + this._popupContentAreaId + '"' + contentExtraStyle + '>' +
                            this._getContentArea() +
                        '</div>' +
                    '</div>' +
                '</div></div></div></div></div>' +
            '</td></tr><tr/></tbody></table></div>'
        );
    },

    _getContentArea: function() {
	    var buttonsArea = '';
	    if (this._buttons.size() > 0) {
			buttonsArea = this._getButtonsArea();
	    }
        return '' +
            '<form>' +
                this._getConfigParam('content', '') +
                buttonsArea +
            '</form>';
    },

    _getButtonsArea: function() {
        return '' +
            '<div class="btns-box" id="' + this._actionButtonsId + '">' +
                '<div class="box-area">' +
                    '<div class="form-row">' +
                        '<div class="field-name" id="' + this._leftActionButtonsAreaId + '"> </div>' +
                        '<div class="field-value" id="' + this._rightActionButtonsAreaId + '"> </div>' +
                    '</div>' +
                '</div>' +
            '</div>';
    },

    _addEvents: function($super) {
        $super();

        var context = this;

        this._buttons.each(function(buttonDesc) {
            var buttonId = '';
            if ('undefined' != typeof buttonDesc['id']) {
                buttonId = ' id="' + buttonDesc['id'] + '" ';
            }
            if ('undefined' == typeof buttonDesc['class']) {
                buttonDesc['class'] = '';
            }
            var buttonDisabled = '';
            if ('undefined' != typeof buttonDesc['disabled'] && buttonDesc['disabled']) {
                buttonDisabled = 'disabled="disabled"';
            }
            var buttonContainer = new Element('span', { 'class': 'btn ' + buttonDesc['class'] });
            buttonContainer.update('<button ' + buttonId + ' value="" type="button" ' + buttonDisabled + '>' + buttonDesc['title'] + '</button>');
            buttonContainer.down('button').observe(
                'click',
                buttonDesc['handler'].bindAsEventListener(this, context)
            );
            $(context._rightActionButtonsAreaId).insert(buttonContainer);
        });

	    if (this._closeButtonEnabled) {
			Event.observe($(this._id + '-close'), 'click', this._onCloseClick.bindAsEventListener(this));
	    }

        this.show();
    },

	_onCloseClick: function() {
	    this.hide();
	},

    show: function($super) {
        $super();
        this._onShow();
    }

});

Jsw.DoubleListSelector = Class.create(Jsw.Component, {
    _initConfiguration: function($super, config) {
        $super(config);
        this._cls = 'doubleListSelect';
        this._name = this._getConfigParam('name', '');
        this._list = this._getConfigParam('list', $A());
        this._dataUrl = this._getConfigParam('dataUrl', null);
        this._selectedMarked = 0;
        this._unselectedMarked = 0;
        this._prevSelectedItem = -1;
        this._isSearchable = this._getConfigParam('isSearchable', false);
        this._isLarge = this._getConfigParam('isLarge', false);
    },

    _initList: function() {
        this._list.each(function(element, index) {
            element.marked = false;
            if ('undefined' == typeof element.selected) {
                element.selected = false;
            }

            this._initListItem(element, index, true);
            this._initListItem(element, index, false);
            this._initInputElement(element, index);
        }, this);
    },

    _initComponentElement: function() {
        this._componentElement = $(this._applyTargetId);
        this._componentElement.addClassName('double-list-select');
        this._componentElement.addClassName('dls-icons-1');
        if (this._isLarge) {
            this._componentElement.addClassName('dls-large');
        }

        var searchControlUnselected = this._isSearchable ? this._getSearchControl('unselected') : '';
        var searchControlSelected = this._isSearchable ? this._getSearchControl('selected') : '';
        var ajaxLoading = null == this._dataUrl ? ''
            : '<div class="ajax-loading" style="display: none;">' + this._locale.loadingTitle + '</div>';

        var listHtml =
            '<div class="dls-first-box">' +
                searchControlUnselected +
                '<h4><span>' + this._locale.unselectedTitle + '</span></h4>' +
                '<div  id="' + this._id + '-container-unselected" class="dls-box-area">' + ajaxLoading + '<ul id="' + this._id + '-unselected"></ul></div>' +
            '</div>' +
            '<div class="dls-second-box">' +
                 searchControlSelected +
                '<h4><span>' + this._locale.selectedTitle +'</span></h4>' +
                '<div id="' + this._id + '-container-selected" class="dls-box-area">' + ajaxLoading + '<ul id="' + this._id + '-selected"></ul></div>' +
            '</div>' +
            '<div id="' + this._id + '-controls" class="dls-controls">' +
                '<span id="' + this._id + '-controls-select" class="btn disabled">' +
                    '<button id="'+ this._id +'-submitSelect" onclick="return false;"><span class="dls-add">&gt;&gt;</span></button>' +
                '</span>' +
                '<span id="' + this._id + '-controls-unselect" class="btn disabled">' +
                    '<button id="'+ this._id +'-submitUnselect" onclick="return false;"><span class="dls-remove">&lt;&lt;</span></button>' +
                '</span>' +
            '</div>';

        this._updateComponentElement(listHtml);
        this._initList();
    },

    _initListItem: function(element, index, isSelected) {
        var container = null;
        var isHidden = false;
        if (isSelected) {
            container = $(this._id + '-selected');
            isHidden = !element.selected;
        } else {
            container = $(this._id + '-unselected');
            isHidden = element.selected;
        }

        var item = new Element('div', {
            'class': 'dls-item-block'
        })
        .insert('<b>' + element.title.escapeHTML() + '</b>');

        if (element.description) {
            item.insert('<span>' + element.description + '</span>');
        }

        if ('undefined' != typeof element.icons && element.icons.length > 0) {
            var itemIcons = new Element('span', {
                'class': 'dls-icons'
            });

            element.icons.each(function(icon) {
                itemIcons
                .insert('<i class="icon"><img src="' + Jsw.skinUrl + icon.src + '" title="' + icon.title + '"></i>');
            }, this);

            item.insert(itemIcons);
        }

        var listItem = new Element('li', {
            'id': this._id + '-' + index + '-' + (isSelected ? 'selected' : 'unselected') + '-item'
        })
        .insert(item);


        container.insert(listItem);
        var context = this;
        listItem.observe('click', function(event) {
            context._toggleItem(index, isSelected, event);
        });
        if (isHidden) {
            listItem.hide();
        }
    },

    _initInputElement: function(element) {
        this._componentElement.insert(new Element('input', {
            'id': this._id + '-' + element.id,
            'name': this._name + '[' + element.id + ']',
            'type': 'hidden',
            'value': (element.selected ? 'true' : 'false')
        }));
    },

    _getSearchControl: function(columnId) {
        var html =
            '<div class="dls-search">' +
                '<input id="' + this._id + '-search-' + columnId + '" class="search-input search-empty" type="text" autocomplete="off"' +
                    ' value="' + this._locale.searchTitle + '"' +
                    ' onfocus="Jsw.getComponent(\'' + this._id + '\').onFocus(this)"' +
                    ' onblur="Jsw.getComponent(\'' + this._id + '\').onBlur(this)"' +
                    ' onkeyup="Jsw.getComponent(\'' + this._id + '\').onToggleSearch(\'' + columnId + '\', this.value);">' +
                '<input id="' + this._id + '-search-button-' + columnId + '" class="search-button" type="image" src="' + Jsw.skinUrl + '/icons/16/plesk/search.png"' +
                    ' onmousedown="Jsw.getComponent(\'' + this._id + '\').onResetSearch(\'' + columnId + '\');"' +
                    ' onclick="return false;">' +
            '</div>';
        return html;
    },

    _addEvents: function() {
        $(this._id + '-controls-select').observe('click', this._onSubmitSelect.bind(this));
        $(this._id + '-controls-unselect').observe('click', this._onSubmitUnselect.bind(this));
    },

    _toggleItem: function(index, isSelected, event) {
        if (this._list[index].marked) {
            this._unmarkItem(index, isSelected, event);
        } else {
            this._markItem(index, isSelected, event);
        }
    },

    _markItem: function(index, isSelected, event) {
        if (('undefined' != typeof event) && event.shiftKey && (-1 != this._prevSelectedItem)) {
            var startIndex = Math.min(index, this._prevSelectedItem);
            var finishIndex= Math.max(index, this._prevSelectedItem);
            for (i = startIndex; i < finishIndex; i++) {
                this._markItem(i, isSelected);
            }
        }

        this._prevSelectedItem = index;

        this._list[index].marked = true;
        $(this._id + '-' + index + '-' + (isSelected ? 'selected' : 'unselected') + '-item').addClassName('marked');
        this._updateSelectControls(isSelected, true);
    },

    _unmarkItem: function(index, isSelected, event) {
        this._list[index].marked = false;
        $(this._id + '-' + index + '-' + (isSelected ? 'selected' : 'unselected') + '-item').removeClassName('marked');
        this._updateSelectControls(isSelected, false);
    },

    _onSubmitSelect: function() {
        this._list.each(function(element, index) {
            if (element.marked && !element.selected && $(this._id + '-' + index + '-unselected-item').visible()) {
                this._unmarkItem(index, false);
                element.selected = true;
                $(this._id + '-' + element.id).value = 'true';
                $(this._id + '-' + index + '-unselected-item').hide();
                $(this._id + '-' + index + '-selected-item').show();
            }
        }, this);

        this.onChange();
    },

    _onSubmitUnselect: function() {
        this._list.each(function(element, index) {
            if (element.marked && element.selected && $(this._id + '-' + index + '-selected-item').visible()) {
                this._unmarkItem(index, true);
                element.selected = false;
                $(this._id + '-' + element.id).value = 'false';
                $(this._id + '-' + index + '-selected-item').hide();
                $(this._id + '-' + index + '-unselected-item').show();
            }
        }, this);

        this.onChange();
    },

    onChange: function() {},

    onToggleSearch: function(columnId, searchString) {
        var isSelected = (columnId == 'selected');
        searchString = searchString.strip().toLowerCase();

        if (0 != searchString.length) {
            this._toggleSearchButtonIcon(columnId, true);
            this._list.each(function(element, index) {
                if (isSelected == element.selected) {
                    var item = $(this._id + '-' + index + '-' + columnId + '-item');
                    if (this._isSearchMatched(element, searchString)) {
                        if (!item.visible() && element.marked) {
                            this._updateSelectControls(isSelected, true);
                        }
                        item.show();
                    } else {
                        if (item.visible() && element.marked) {
                            this._updateSelectControls(isSelected, false);
                        }
                        item.hide();
                    }
                }
            }, this);
        } else {
            this._toggleSearchButtonIcon(columnId, false);
            this._showAllItems(columnId);
        }
    },

    _isSearchMatched: function(element, searchString) {
        var searchIndex = 'undefined' == typeof element.searchIndex
            ? element.title
            : element.searchIndex;

        return -1 != searchIndex.toLowerCase().indexOf(searchString);
    },

    onFocus: function(element) {
        if (this._locale.searchTitle == element.value) {
            element.value = '';
            element.removeClassName('search-empty');
        }
    },

    onBlur: function(element) {
        if ('' == element.value) {
            this._resetSearchControl(element);
        }
    },

    onResetSearch: function(columnId) {
        this._showAllItems(columnId);
        var element = $(this._id + '-search-' + columnId);
        this._resetSearchControl(element);
        this._toggleSearchButtonIcon(columnId, false);
    },

    _showAllItems: function(columnId) {
        var isSelected = (columnId == 'selected');
        this._list.each(function(element, index) {
            if (isSelected == element.selected) {
                var item = $(this._id + '-' + index + '-' + columnId + '-item');
                if (!item.visible() && element.marked) {
                    this._updateSelectControls(isSelected, true);
                }
                item.show();
            }
        }, this);
    },

    _resetSearchControl: function(element) {
        element.value = this._locale.searchTitle;
        element.addClassName('search-empty');
    },

    _toggleSearchButtonIcon: function(columnId, isActive) {
        var inputIcon = $(this._id + '-search-button-' + columnId);
        inputIcon.toggleClassName('search-button-clear', isActive);
        inputIcon.src = (isActive)
            ? Jsw.skinUrl + '/images/clear-search.png'
            : Jsw.skinUrl + '/icons/16/plesk/search.png';
    },

    _updateSelectControls: function(isSelected, setMarked) {
        var counter = isSelected ? this._selectedMarked : this._unselectedMarked;
        var controlId = isSelected ? 'unselect' : 'select';

        setMarked ? counter++ : counter--;
        if (0 == counter) {
            $(this._id + '-controls-' + controlId).addClassName('disabled');
        } else {
            $(this._id + '-controls-' + controlId).removeClassName('disabled');
        }

        isSelected ? this._selectedMarked = counter : this._unselectedMarked = counter;
    },

    isEmpty: function() {
        return !this._list.any(function(element) { return element.selected; });
    },

    reload: function(params) {
        params = params || '';
        if (!this._dataUrl) {
        	return;
        }

        new Ajax.Request(Jsw.prepareUrl(this._dataUrl + params), {
            method: 'get',
            onSuccess: this._onReloadSuccess.bind(this),
            onFailure: this._onReloadFailure.bind(this),
            onException: this._onReloadException.bind(this),
            onCreate: this._onReloadCreate.bind(this),
            onComplete: this._onReloadComplete.bind(this)
        });
    },

    _onReloadSuccess: function(transport) {
        if ('' == transport.responseText) {
            //:INFO: sometimes happens in FF if you are go to other page during request
            return;
        }

        try {
            var data = transport.responseText.evalJSON();
        } catch (e) {
            Jsw.showInternalError(transport.responseText);
            return;
        }

        if (data.list) {
            this._list = $A(data.list);
        } else if (data.status) {
            Jsw.clearStatusMessages();
            $A(data.statusMessages).each(function(message) {
                Jsw.addStatusMessage(message.status, message.content);
            }.bind(this));
        } else {
            Jsw.showInternalError('Unable to load list data.');
        }

        this._initList();
    },

    _onReloadFailure: function(transport) {
        Jsw.showInternalError(transport.responseText);
    },

    _onReloadException: function(transport, exception) {
        Jsw.showInternalError(exception + "\n" + transport.responseText);
    },

    _onReloadCreate: function(transport) {
        this._list = $A();
        this._componentElement.select('.dls-box-area').each(function(listArea) {
            listArea.select('ul').first().update();
            listArea.select('.ajax-loading').first().show();
        });
    },

    _onReloadComplete: function() {
        this._componentElement.select('.dls-box-area').each(function(listArea) {
            listArea.select('.ajax-loading').first().hide();
        });
    }
});

Jsw.EnhancedDoubleListSelector = Class.create(Jsw.DoubleListSelector, {
    _initComponentElement: function() {
        this._componentElement = $(this._applyTargetId);
        this._componentElement.addClassName('edls');

        var searchControlUnselected = this._isSearchable ? this._getSearchControl('unselected') : '';
        var searchControlSelected = this._isSearchable ? this._getSearchControl('selected') : '';

        var listHtml = '' +
            '<div class="edls-wrap">' +
                '<div class="edls-box edls-box-first">' +
                    '<div class="edls-header">' +
                        '<div class="edls-header-title">' + this._locale.unselectedTitle + '</div>' +
                    '</div>' +
                    '<div class="edls-box-wrap">' +
                        searchControlUnselected +
                        '<div class="edls-box-body">' +
                            '<ul class="edls-list" id="' + this._id + '-unselected"></ul>' +
                            '<div class="edls-list-empty" style="display: none;"><span>' + this._locale.unselectedEmpty + '</span></div>' +
                            '<div class="edls-ajax-loading" style="display: none;"><span class="ajax-loading">' + this._locale.loadingTitle + '</span></div>' +
                        '</div>' +
                    '</div>' +
                '</div>' +
                '<div class="edls-control">' +
                    '<div class="edls-control-wrap">' +
                        '<div class="edls-control-body">' +
                            '<div class="edls-control-text">' +
                                '<div class="edls-control-text-none">' + this._locale.controlTextNone + '</div>' +
                                '<div class="edls-control-text-first">' +
                                    this._locale.controlTextFirst.replace('%checked%', '<b>' + this._locale.controlChecked.replace('%count%', '<span class="edls-count">0</span>') + '</b>') +
                                '</div>' +
                                '<div class="edls-control-text-second">' +
                                    this._locale.controlTextSecond.replace('%checked%', '<b>' + this._locale.controlChecked.replace('%count%', '<span class="edls-count">0</span>') + '</b>') +
                                '</div>' +
                            '</div>' +
                            '<div class="edls-control-indicator"><i></i></div>' +
                        '</div>' +
                    '</div>' +
                '</div>' +
                '<div class="edls-box edls-box-second">' +
                    '<div class="edls-header">' +
                        '<div class="edls-header-title">' + this._locale.selectedTitle + '</div>' +
                    '</div>' +
                    '<div class="edls-box-wrap">' +
                        searchControlSelected +
                        '<div class="edls-box-body">' +
                            '<ul class="edls-list" id="' + this._id + '-selected"></ul>' +
                            '<div class="edls-list-empty" style="display: none;"><span>' + this._locale.selectedEmpty + '</span></div>' +
                            '<div class="edls-ajax-loading" style="display: none;"><span class="ajax-loading">' + this._locale.loadingTitle + '</span></div>' +
                        '</div>' +
                    '</div>' +
                '</div>' +
            '</div>';

        this._updateComponentElement(listHtml);
        this._initList();
    },

    _initList: function($super) {
        $super();

        this._fixListZebra();
    },

    _fixListZebra: function() {
        [$(this._id + '-unselected'), $(this._id + '-selected')].each(function(list) {
            var count = 0;
            list.select('.edls-item').each(function(el) {
                if (el.visible()) {
                    el.addClassName(count % 2 ? 'odd' : 'even');
                    el.removeClassName(count % 2 ? 'even' : 'odd');
                    count++;
                }
            });
            list.next('.edls-list-empty')[count ? 'hide' : 'show']();
        });
    },

    _getSearchControl: function(columnId) {
        var html =
            '<div class="edls-search">' +
                '<span class="edls-check"><input type="checkbox" value="" class="checkbox"></span>' +
                '<span class="edls-search-control">' +
                    '<input type="text" value="" autocomplete="off" class="search-input search-empty"' +
                        ' id="' + this._id + '-search-' + columnId + '"' +
                        ' onfocus="Jsw.getComponent(\'' + this._id + '\').onFocus(this)"' +
                        ' onblur="Jsw.getComponent(\'' + this._id + '\').onBlur(this)"' +
                        ' onkeyup="Jsw.getComponent(\'' + this._id + '\').onToggleSearch(\'' + columnId + '\', this.value);"' +
                        '>' +
                    '<input type="image" src="' + Jsw.skinUrl + '/images/search-input.png" class="search-button"' +
                        ' id="' + this._id + '-search-button-' + columnId + '"' +
                        ' onmousedown="Jsw.getComponent(\'' + this._id + '\').onResetSearch(\'' + columnId + '\');"' +
                        ' onclick="return false;"' +
                        '>' +
                '</span>' +
            '</div>';

        return html;
    },

    _toggleSearchButtonIcon: function(columnId, isActive) {
        var inputIcon = $(this._id + '-search-button-' + columnId);
        inputIcon.toggleClassName('search-button-clear', isActive);
        inputIcon.src = (isActive)
            ? Jsw.skinUrl + '/images/clear-search.png'
            : Jsw.skinUrl + '/images/search-input.png';
    },

    onFocus: function(element) {
        element.removeClassName('search-empty');
    },

    _resetSearchControl: function(element) {
        element.value = '';
        element.addClassName('search-empty');
    },

    _initListItem: function(element, index, isSelected) {
        var container = null;
        var isHidden = false;
        if (isSelected) {
            container = $(this._id + '-selected');
            isHidden = !element.selected;
        } else {
            container = $(this._id + '-unselected');
            isHidden = element.selected;
        }

        var itemCheckbox = new Element('input', {
            'type': 'checkbox',
            'class': 'checkbox',
            'value': ''
        });

        var item = new Element('div', {
            'class': 'edls-item-wrap'
        })
        .insert(new Element('span', {'class': 'edls-check'}).insert(itemCheckbox))
        .insert('<span class="edls-text">' + this._getItemText(element) + '</span>');

        var listItem = new Element('li', {
            'id': this._id + '-' + index + '-' + (isSelected ? 'selected' : 'unselected') + '-item',
            'class': 'edls-item'
        })
        .insert(item);

        container.insert(listItem);
        var context = this;
        listItem.observe('click', function(event) {
            context._toggleItemSelection(index, isSelected, event);
        });
        itemCheckbox.observe('click', function(event) {
            event.stopPropagation();
            context._toggleItemCheckbox(itemCheckbox, itemCheckbox.checked);
        });
        if (isHidden) {
            listItem.hide();
        }
    },

    _getItemText: function(element, highlight) {
        var itemIcons = '';
        if ('undefined' != typeof element.icons && element.icons.length > 0) {
            element.icons.each(function(icon) {
                itemIcons += '<i class="icon"><img src="' + Jsw.skinUrl + icon.src + '" title="' + icon.title + '"></i>';
            });
            itemIcons += ' ';
        }
        var itemText = element.title.escapeHTML();
        if (highlight) {
            var resultPos = itemText.indexOf(highlight);
            if (resultPos > 0) {
                itemText = itemText.substr(0, resultPos) +
                    '<b class="search-result-label">' +
                        itemText.substr(resultPos, highlight.length) +
                    '</b>' +
                    itemText.substr(resultPos + highlight.length);
            }
        }

        return itemIcons + itemText;
    },

    onToggleSearch: function(columnId, searchString) {
        var isSelected = (columnId == 'selected');
        searchString = searchString.strip().toLowerCase();

        if (0 != searchString.length) {
            this._toggleSearchButtonIcon(columnId, true);
            this._list.each(function(element, index) {
                var item = $(this._id + '-' + index + '-' + columnId + '-item');
                if (this._isSearchMatched(element, searchString)) {
                    item.down('.edls-text').update(this._getItemText(element, searchString));
                    if (isSelected == element.selected) {
                        item.show();
                    }
                } else {
                    item.down('.edls-text').update(this._getItemText(element));
                    var checkbox = item.down('.edls-check input');
                    if (item.visible() && checkbox.checked) {
                        this._toggleItemCheckbox(checkbox, false);
                    }
                    item.hide();
                }
            }, this);
        } else {
            this._toggleSearchButtonIcon(columnId, false);
            this._showAllItems(columnId);
        }
    },

    _showAllItems: function(columnId) {
        var isSelected = (columnId == 'selected');
        this._list.each(function(element, index) {
            var item = $(this._id + '-' + index + '-' + columnId + '-item');
            item.down('.edls-text').update(this._getItemText(element));
            if (isSelected == element.selected) {
                item.show();
            }
        }, this);
    },

    _toggleItemSelection: function(index, isSelected, event, batchMode) {
        event.preventDefault();
        this._list[index].selected = !isSelected;
        $(this._id + '-' + this._list[index].id).value = !isSelected ? 'true' : 'false';
        $(this._id + '-' + index + '-' + (isSelected ? 'selected' : 'unselected') + '-item').hide();
        $(this._id + '-' + index + '-' + (isSelected ? 'unselected' : 'selected') + '-item').show();
        this._toggleItemCheckbox($(this._id + '-' + index + '-' + (isSelected ? 'selected' : 'unselected') + '-item').down('.edls-check input'), false);
        if (!batchMode) {
            this._fixListZebra();
            this.onChange();
        }
    },

    _toggleItemCheckbox: function(checkbox, value) {
        checkbox.checked = value;

        var side = checkbox.up('.edls-box-first') ? 'first' : 'second';
        var count = checkbox.up('.edls-box-' + side).select('.edls-list .edls-check input:checked').length;
        if (value) {
            checkbox.up('.edls-item').addClassName('selected');
        } else {
            checkbox.up('.edls-item').removeClassName('selected');
            checkbox.up('.edls-box').down('.edls-search .checkbox').checked = false;
        }
        checkbox.up('.edls')[count ? 'addClassName' : 'removeClassName']('edls-selected-' + side);
        checkbox.up('.edls').down('.edls-control-text-' + side + ' .edls-count').update(count);
    },

    _updateSelectControls: function(isSelected, setMarked) {},

    _addEvents: function() {
        var context = this;
        var componentElement = this._componentElement;

        this._componentElement.select('.edls-box-wrap').each(function(el) {
            el.observe('mouseover', function() {
                componentElement.addClassName('edls-over');
                componentElement.addClassName(this.up('.edls-box-first') ? 'edls-over-first' : 'edls-over-second');
                componentElement.removeClassName(this.up('.edls-box-first') ? 'edls-over-second' : 'edls-over-first');
            });
            el.observe('mouseleave', function() {
                if (!componentElement.hasClassName('edls-selected-first') && !componentElement.hasClassName('edls-selected-second')) {
                    componentElement.removeClassName('edls-over');
                    componentElement.removeClassName('edls-over-first');
                    componentElement.removeClassName('edls-over-second');
                }
            });
        });

        this._componentElement.observe('mouseleave', function() {
            this.removeClassName('edls-over');
            this.removeClassName('edls-over-first');
            this.removeClassName('edls-over-second');
        });

        this._componentElement.select('.edls-box .edls-search .edls-check input').each(function(element) {
            element.observe('click', function(event) {
                var value = this.checked;
                this.up('.edls-box').select('.edls-list .edls-check input').each(function(checkbox) {
                    if (value && checkbox.up('.edls-item').visible()) {
                        context._toggleItemCheckbox(checkbox, true);
                    } else {
                        context._toggleItemCheckbox(checkbox, false);
                    }
                });
            });
        });

        this._componentElement.select('.edls-control-wrap').first().observe('click', function(event) {
            var isSelected;
            if (componentElement.hasClassName('edls-over-first')) {
                isSelected = false;
            } else if (componentElement.hasClassName('edls-over-second')) {
                isSelected = true;
            } else {
                return;
            }

            context._list.each(function(element, index) {
                if (element.selected == isSelected) {
                    var item = $(context._id + '-' + index + '-' + (isSelected ? 'selected' : 'unselected') + '-item');
                    if (item.hasClassName("selected")) {
                        context._toggleItemSelection(index, isSelected, event, true);
                    }
                }
            });
            context._fixListZebra();
            context.onChange();
        });
    },

    _onReloadCreate: function(transport) {
        this._list = $A();
        this._componentElement.select('.edls-box').each(function(listArea) {
            listArea.select('ul').first().update();
            listArea.select('.edls-ajax-loading').first().show();
            listArea.select('.edls-list-empty').first().hide();
        });
    },

    _onReloadComplete: function() {
        this._componentElement.select('.edls-box').each(function(listArea) {
            listArea.select('.edls-ajax-loading').first().hide();
        });
    }
});

Jsw.ImageSelector = Class.create(Jsw.Component, {
    _initConfiguration: function($super, config) {
        $super(config);
        var list = this._getConfigParam('list', {});
        this._list = Object.isArray(list) ? $H() : $H(list);
        var captions = this._getConfigParam('captions', {});
        this._captions = Object.isArray(captions) ? $H() : $H(captions);
        this._name = this._getConfigParam('name', '');
        this._value = this._getConfigParam('value', null);

        this._currentId = this._id + '-current';
        this._listId = this._id + '-list';
    },

    _initComponentElement: function($super) {
        $super();

        this._updateComponentElement(
            '<div class="' + this._cls + '-area">' +
                '<div id="' + this._currentId + '" class="' + this._cls + '-current">' +
                    '<img/>' +
                    '<p>' + this._captions.get('selectedImage').escapeHTML() + '</p>' +
                '</div>' +
                '<div id="' + this._listId + '" class="' + this._cls + '-select clearfix"></div>' +
            '</div>'
        );

        this._valueElement = new Element('input', {
            'id': this._id + '-value',
            'name': this._name,
            'type': 'hidden',
            'value': null
        });

        this._componentElement.insert(this._valueElement);
    },

    _addEvents: function() {
        this._list.each(function(pair) {
            this._addImage(pair.key, pair.value);
        }, this);

        this.set(this._value);
    },

    _addImage: function(value, image) {
        this._componentElement.select('#' + this._listId).each(function(target) {
            var element = new Element('img', {
                'src': image.url,
                'alt': image.title
            });

            target.insert(element);

            element.observe('click', this._onImageClick.bindAsEventListener(this, value));
        }, this);
    },

    _onImageClick: function(event, value)
    {
        this.set(value);
    },

    set: function(value)
    {
        this._valueElement.value = value;
        var image = this._list.get(value);
        this._componentElement.select('#' + this._currentId + ' img').each(function(target) {
            if (image) {
                target.src = image.url;
                target.alt = image.title;
                target.show();
            } else {
                target.hide();
            }
        }, this);

    }

});

Jsw.ToolList = Class.create(Jsw.Container, {

    _initConfiguration: function($super, config) {
		$super(config);
        this._tools = this._getConfigParam('tools', null);
        this._cls = this._getConfigParam('cls', 'tools-list-box');
        this._contentAreaId = this._id + '-tools-list';

        if (Object.isArray(this._tools) && (0 == this._operations.size())) {
            this._operations = null;
        }
    },

    _initComponentElement: function($super) {
    	$super();
        this._updateComponentElement(
            '<div id="' + this._id + '-box-area" class="box-area">' +
	            '<div id="' + this._id + '-content" class="content">' +
		            	'<div id="' + this._id + '-content-area" class="content-area">' +
		            		'<ul id="' + this._id + '-tools-list" class="tools-list"></ul>' +
		            	'</div>' +
	            '</div>' +
            '</div>'
        );
    },

    render: function($super) {
    	$super();
        Jsw.Tooltip.initData(this._componentElement);
    }
});

Jsw.toollist = {};

Jsw.toollist.Panel = Class.create(Jsw.Container, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._contentAreaId = this._id + '-tools-list';
        this._image = this._getConfigParam('image', '');
        this._title = this._getConfigParam('title', '');
    },

    _initComponentElement: function($super) {
        var attributes = { 'class': 'tools-item' };
        var forcedId = this._getConfigParam('id', null);
        if (forcedId) {
            attributes.id = forcedId;
        }

        this._componentElement = new Element('li', attributes);
        this._componentElement.update(new Element('div', {
	        	'class': 'tool-block'
	        }).insert(
                '<span class="tool-icon">'+
                    '<img src="' + this.getImagePath(this._image) + '"/>'+
                '</span>'+
                '<span class="tool-name">' + this._title + '</span>'
            ).insert(new Element('ul', {
                'id': this._id + '-tools-list',
                'class': 'tool-info' })
            )
        );
    },

    getImagePath: function(path) {
    	 return Jsw.skinUrl + "/" + path;
    }
});

Jsw.toollist.Item = Class.create(Jsw.Component, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._handler = this._getConfigParam('handler', false);
        if (Object.isString(this._handler)) {
            eval('this._handler = ' + this._handler);
        }
        this._href = this._getConfigParam('href', '#');
        this._target = this._getConfigParam('target', null);
        this._title = this._getConfigParam('title', '');
        this._description = this._getConfigParam('description', '');
        this._disabled = this._getConfigParam('disabled', false);
        this._attention = this._getConfigParam('attention', false);
    },

    _initComponentElement: function($super) {
        if (this._disabled) {
            href = '#';
        } else {
            href = this._href;
        }

        if (this._attention) {
            this._title = new Element('span', {
                'class': 'hint-attention'
            }).update(this._title);
        }

        this._componentElement = new Element('li', {
            'id': this._getConfigParam('id', null)
        }).update(new Element('a', {
            'href': href,
            'target': this._target
        }).update(this._title));

        if (this._handler && !this._disabled) {
            this._componentElement.observe('click', this._handler);
        }
    }
});

Jsw.toollist.Button = Class.create(Jsw.Component, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._handler = this._getConfigParam('handler', false);
        if (Object.isString(this._handler)) {
            eval('this._handler = ' + this._handler);
        }
        this._href = this._getConfigParam('href', '#');
        this._target = this._getConfigParam('target', null);
        this._image = this._getConfigParam('image', '');
        this._disabledImage = this._getConfigParam('disabledImage', this._image);
        this._title = this._getConfigParam('title', '');
        this._description = this._getConfigParam('description', '');
        this._disabled = this._getConfigParam('disabled', false);
    },

    _initComponentElement: function($super) {
        var attributes = { 'class': 'tools-item' };
        var forcedId = this._getConfigParam('id', null);
        if (forcedId) {
            attributes.id = forcedId;
        }

        this._componentElement = new Element('li', attributes);

    	cls = 'tool-block';
    	if (this._disabled) {
    		cls += ' tool-block-disabled';
    		href = '#';
    		image = this.getImagePath(this._disabledImage);
    	} else {
    		href = this._href;
    		image = this.getImagePath(this._image);
    	}
        this._componentElement.update(new Element('a', {
	        	'class': cls,
	            'href': href,
	            'target': this._target
	        }).insert(
	            '<span class="tool-icon">'+
	            	'<img src="' + image + '"/>'+
	            '</span>'+
	            '<span class="tool-name">' + this._title + '</span>'+
	            '<span class="tool-info">' + this._description + '</span>'
	        )
        );

        if (this._handler && !this._disabled) {
        	this._componentElement.observe('click', this._handler);
        }
    },

    _addTooltips: function() {
        // don't show tooltips, 'cause we've inline descriptions
    },

    getImagePath: function(path) {
    	 return Jsw.skinUrl + "/" + path;
    }
});

Jsw.CustomButton =  Class.create(Jsw.toollist.Button, {
	getImagePath: function(path) {
		return path;
	}
});

Object.extend(Form.Element.Methods, {
    disable: function(element) {
        element = $(element);
        element.disabled = 'disabled';
        element.addClassName('disabled');
        return element;
    },

    enable: function(element) {
        element = $(element);
        element.disabled = false;
        element.removeClassName('disabled');
        return element;
    }
});

Element.addMethods();

Jsw.PasswordMeter = Class.create(Jsw.Component, {

	_initConfiguration: function($super, config) {
    	$super(config);
    	this._passwordElement = $(this._getConfigParam('observe', null));
    	this._passwordElement.observe('keyup', this._onChange.bind(this));
        this._passwordElement.observe('plesk:passwordGenerated', this._onChange.bind(this));
        this._passwordElement.observe('blur', this._onBlur.bind(this));
        this._requiredStrength = this._getConfigParam('strength', -1);
    	this._tag = 'span';
    	this._cls = 'password-strength';
	},

	_initComponentElement: function($super) {
    	$super();

		this._hintContainer = new Element('span', {cls: 'hint-inline hint-info'});
		this._hint = new Element('span');
		this._hint.insert('(?)');
		this._hintContainer.insert(this._hint);

		this._progress = new Element('i');
		this._componentElement.insert(this._progress);

		this._verdict = new Element('b');
		this._componentElement.insert(this._verdict);
	},

    render: function($super) {
        $super();

        if (!this._tooltip) {
	        this._componentElement.insert({'after':this._hintContainer});
	        this._tooltip = new Jsw.AjaxTooltip.Instance({
	                title: this._locale['title'],
	                waitMsg: '',
	                url: '',
	                target: this._hint
	            });
        }
        this._onChange();
    },

    setProgress: function(value)
    {
    	this._progress.writeAttribute('style', 'width: ' + value + '%');
    },

    _onBlur: function() {
        var password = this._passwordElement.value;
        if (!password.empty()) {
            var result = this._applyRules(password);
            this._updateError(result.score);
        }
    },

    _onChange: function() {
		var result = this._applyRules(this._passwordElement.value);
		this._updateProgress(result.score);
		this._updateHint(result.unusedRules);
		this._updateVerdict(result.score);
		this._updateColor(result.score);
	},

	_applyRules: function(passwd)
	{
		var unusedRules = new Array();
		var sum = 0;
		$H(this.rules).each(function(pair) {
			var mark = pair.value(passwd);
			if (mark < 0) {
				unusedRules.push({rule: pair.key, value: -mark});
			} else {
				sum += mark;
			}
	    });
		return {score: sum, unusedRules: unusedRules};

	},

    _updateError: function(score)
    {
        var rowElement = this._passwordElement.up('.form-row');
        if (!rowElement) {
            return;
        }
        if (score <= this._requiredStrength) {
            rowElement.addClassName('error');
            var errorHint = '<span class="error-hint">' + this._locale['errorHint'] + '</span>';
            rowElement.down('.field-errors').update(errorHint);
            rowElement.down('.field-errors').show();
        } else {
            rowElement.removeClassName('error');
            rowElement.down('.field-errors').update();
            rowElement.down('.field-errors').hide();
        }
    },

	_verdictClasses: {
		verdictNone: 'verdictNone',
		verdictVeryWeak: 'password-strength-very-weak',
		verdictWeak: 'password-strength-weak',
		verdictMediocre: 'password-strength-medium',
		verdictStrong: 'password-strength-strong',
		verdictStronger: 'password-strength-very-strong'
	},

	_updateColor: function(score)
	{
		var verdict = this._getVerdict(score);
		var newClass = this._verdictClasses[verdict];
		if (this._oldClass == newClass ) {
			return;
		}
		if (this._oldClass) {
			this._componentElement.removeClassName(this._oldClass);
		}
		this._componentElement.addClassName(newClass);
		this._oldClass = newClass;
	},

	_updateHint: function(unusedRules)
	{
		var description = '';
		if (unusedRules.length) {
			description =  this._locale['description'] + '<br/><ul>'
			for(var i=0; i < unusedRules.length && i < 3; ++i) {
				description += "<li>" +  this._locale[unusedRules[i].rule] + "</li>";
			}
			description += "</ul>";
		} else {
			description += this._locale['yourPasswordIsStrong'];
		}

		this._tooltip.setText(description);
	},

	_updateVerdict: function(score)
	{
		this._verdict.update(this._locale[this._getVerdict(score)])
	},

	_updateProgress: function(score)
	{
		if (score > 45) {
			score = 45;
		}
		var value = ((score - 3) / 42) * 100;
		this.setProgress(value);
	},

	_getVerdict: function(score) {
		if(score < 1) {
			return "verdictNone"
		}
		else if(score < 16)
		{
		   return "verdictVeryWeak"
		}
		else if (score > 15 && score < 25)
		{
			return "verdictWeak"
		}
		else if (score > 24 && score < 35)
		{
			return "verdictMediocre"
		}
		else if (score > 34 && score < 45)
		{
			return "verdictStrong"
		}
		return "verdictStronger"
	},

	rules: {
		passwordTooShort: function(passwd) {
			if (passwd.length<5) {
				return -1;
			}
			return 0;
		},
		passwordLength: function(passwd) {
			// PASSWORD LENGTH
			if (passwd.length<5)                         // length 4 or less
			{
				return 3;
			}
			else if (passwd.length>4 && passwd.length<8) // length between 5 and 7
			{
				return 6;
			}
			else if (passwd.length>7 && passwd.length<16)// length between 8 and 15
			{
				return 12;
			}
			else           // length 16 or more
			{
				return 18;
			}
			return -1;
		},
		lettersLowerCase: function(passwd) {
			if (passwd.match(/[a-z]/))                              // [verified] at least one lower case letter
			{
				return 1;
			}
			return -1;
		},
		lettersUpperCase: function(passwd) {
			if (passwd.match(/[A-Z]/))                              // [verified] at least one upper case letter
			{
				return 5;
			}
			return -1;
		},
		numbers1: function(passwd) {
			if (passwd.match(/\d+/))                                 // [verified] at least one number
			{
				return 5;
			}
			return -1;
		},
		numbers3: function (passwd) {
			if (passwd.match(/(.*[0-9].*[0-9].*[0-9])/))             // [verified] at least three numbers
			{
				return 5;
			}
			return -1;
		},
		specialChar1: function(passwd) {
			if (passwd.match(/[!@#$%^&*?_~]/))            // [verified] at least one special character
			{
				return 5;
			}
			return -1;
		},
		specialChar2: function(passwd) {
			// [verified] at least two special characters
			if (passwd.match(/(.*[!@#$%^&*?_~].*[!@#$%^&*?_~])/))
			{
				return 5;
			}
			return -1;
		},
		comboUpperAndLower: function(passwd) {
			if (passwd.match(/([a-z].*[A-Z])|([A-Z].*[a-z])/))        // [verified] both upper and lower case
			{
				return 2;
			}
			return -1;
		},
		comboLettersAndNumbers: function(passwd) {
			if (passwd.match(/([a-zA-Z])/) && passwd.match(/([0-9])/)) // [verified] both letters and numbers
			{
				return 2;
			}
			return -1;
		},
		comboLettersNumbersSpecial: function(passwd) {
			// [verified] letters, numbers, and special characters
			if (passwd.match(/([a-zA-Z0-9].*[!@#$%^&*?_~])|([!@#$%^&*?_~].*[a-zA-Z0-9])/))
			{
				return 2;
			}
			return -1;
		}
	}
});

Jsw.PasswordGenerator = Class.create(Jsw.Component, {

    _initConfiguration: function($super, config) {
        $super(config);
        this._passwordElement = $(this._getConfigParam('password', null));
        this._passwordConfirmationElement = $(this._getConfigParam('passwordConfirmation', null));
        this._generateButtonElement = $(this._getConfigParam('generateButton', null));
        this._generateButtonElement.observe('click', this._onGeneratePasswordClick.bind(this));
        this._showButtonElement = $(this._getConfigParam('showButton', null));
        this._showButtonElement.observe('click', this._onShowPasswordClick.bind(this));
        this._showPasswordVisible = true;
        this._showPasswordTitle = this._getConfigParam('showPasswordTitle', '');
        this._hidePasswordTitle = this._getConfigParam('hidePasswordTitle', '');
        this._passwordStrength = this._getConfigParam('passwordStrength', '');
        this._tag = 'span';
    },

    _onGeneratePasswordClick: function() {
        var password = this._generatePassword();
        this._passwordElement.value = password;
        if(this._passwordConfirmationElement) {
            this._passwordConfirmationElement.value = password;
        }
        this._passwordElement.fire('plesk:passwordGenerated');
    },

    _onShowPasswordClick: function() {
        if (this._showPasswordVisible) {
            this._passwordElement.writeAttribute('type', 'text');
            if ('text' == this._passwordElement.type) {
                this._showButtonElement.down().update(this._hidePasswordTitle);
                this._showPasswordVisible = !this._showPasswordVisible;
            } else if (Prototype.Browser.IE && 'undefined' != typeof(prompt)) {
                var newPassword;
                if (newPassword = prompt('', this._passwordElement.value)) {
                    this._passwordElement.value = newPassword;
                    if(this._passwordConfirmationElement) {
                        this._passwordConfirmationElement.value = newPassword;
                    }
                }
            }
        } else {
            this._passwordElement.writeAttribute('type', 'password');
            this._showButtonElement.down().update(this._showPasswordTitle);
            this._showPasswordVisible = !this._showPasswordVisible;
        }
    },

    _symbolClasses: {
        upper: new ObjectRange('A', 'Z').entries(),
        lower: new ObjectRange('a', 'z').entries(),
        number: new ObjectRange('0', '9').entries(),
        special: "!@#$%^&*?_~"
    },

    _generatePassword: function() {
        var password = "";
        var symbolClasses = ['upper', 'lower', 'lower', 'lower', 'number', 'number', 'special', 'number'];

        if ('Stronger' == this._passwordStrength) {
            symbolClasses.push('upper', 'upper', 'lower', 'lower', 'lower', 'lower', 'lower', 'lower');
        }

        symbolClasses.sort(function(a, b) {
            return Math.floor(b.length * Math.random()) - Math.floor(a.length * Math.random());
        });
        symbolClasses.each(function(i) {
            password += this._generateSymbol(i);
        }, this);
        return password;
    },

    _generateSymbol: function(symbolClass) {
        var index = Math.floor(this._symbolClasses[symbolClass].length*Math.random());
        return this._symbolClasses[symbolClass][index];
    }
});

Jsw.Promo = Class.create(Jsw.Container, {
	_tag: 'div',

    _initConfiguration: function($super, config) {
		$super(config);
        this._title = this._getConfigParam('title', '');
        this._rows = this._getConfigParam('rows', 3);
        this._cls = this._getConfigParam('cls', 'p-box home-promo-block');
        this._contentAreaId = this._id + '-content-area';
        this._itemCount = 0;
        this._current = 0;
    },

    _initComponentElement: function($super) {
        $super();
        this._initLayout();
    },

    _initLayout: function() {
        this._componentElement.update(
        		'<div class="p-box-tl"><div class="p-box-tr"><div class="p-box-tc"><!----></div></div></div>' +
        		'<div class="p-box-ml"><div class="p-box-mr"><div class="p-box-mc">' +
        		'<div class="p-box-header">' +
        		  '<div class="p-box-header-wrap">' +
        		    '<div class="p-box-header-wrap-inner">' +
        		      '<div class="p-box-header-nav">' +
        		        '<a class="link-01" id="' + this._id + '-prev" href="#"><span>' + this._locale.prev + '</span></a> ' +
        		        '<a class="link-01" id="' + this._id + '-next" href="#"><span>' + this._locale.next + '</span></a>' +
        		      '</div>' +
        		      '<div class="p-box-header-text">' +
        		        this._title +
        		      '</div>' +
        		    '</div>' +
        		  '</div>' +
        		'</div>' +
        		'<div class="p-box-content">' +
        		  '<ul class="tools-list" id="' + this._contentAreaId + '">' +
        		  '</ul>' +
        		'</div>' +
        		'</div></div></div>' +
        		'<div class="p-box-bl"><div class="p-box-br"><div class="p-box-bc"><!----></div></div></div>'
        );
    },

    _renderItems: function() {
    	this._showCurrentItems();
    },

    _onHide: function(item) {
    	for (var i=0; i < this._items.length; ++i) {
    		if (this._items[i] == item) {
    			this._items.splice(i, 1);
    			break;
    		}
    	}

    	this._showCurrentItems();
    },

    _addEvents: function() {
    	$(this._id + '-next').observe('click', this._onNext.bind(this));
    	$(this._id + '-prev').observe('click', this._onPrev.bind(this));
    },

    _onNext: function(event) {
        Event.stop(event);
		this._current += this._rows;
		this._showCurrentItems();
    },

    _onPrev: function(event) {
        Event.stop(event);
		this._current -= this._rows;
		this._showCurrentItems();
    },

    _clampCurrentItem: function () {
    	if (this._items.length > this._rows) {
    		$(this._id + '-prev').up('div').show();
    		$(this._id + '-next').up('div').show();
    	} else {
    		$(this._id + '-next').up('div').hide();
    		$(this._id + '-prev').up('div').hide();
    	}

    	if (this._current >= this._items.length) {
    		this._current = 0;
    	} else if (this._current < 0) {
            if (this._rows == 1) {
                this._current = this._items.length - 1;
            } else {
                this._current = this._items.length - this._items.length % this._rows;
            }
    	}
    },

    _showCurrentItems: function() {
    	if (this._items.length == 0) {
    		this.hide();
    		return;
    	}

    	this._clampCurrentItem();

    	for (var i=0; i < this._items.length; ++i) {
    		var item = this._items[i];
            item.setRenderTarget(this._contentAreaId);
            item.setHideHandler(this._onHide.bind(this));
    		item.hide();
    		if (i >= this._current && i < this._current + this._rows) {
    			item.render();
    			item.show();
    		}
    	}
    }
});


Jsw.Promo.Item = Class.create(Jsw.Component, {

	_tag: 'li',

    _initConfiguration: function($super, config) {
        $super(config);

        this._text = this._getConfigParam('text', '');
        this._title = this._getConfigParam('title', '');
        this._buttonUrl = this._getConfigParam('buttonUrl', '');
        this._buttonText = this._getConfigParam('buttonText', '');
        this._buttonTarget = this._getConfigParam('buttonTarget', '');
        this._hideUrl = this._getConfigParam('hideUrl', '');
        this._hideText = this._getConfigParam('hideText', '');
        this._iconUrl = this._getConfigParam('iconUrl', '');
        this._cls = "tools-item tools-item-active";
    },

    render: function($super) {
    	this._componentElement.update(
			'<a class="tool-block" href="' + this._buttonUrl + '" target="' + this._buttonTarget + '">' +
				'<span class="tool-icon"><img src="' + this._iconUrl + '"></span>' +
				'<span class="tool-name">' + this._title.escapeHTML() + '</span>' +
				'<span class="tool-info">' + this._text.escapeHTML() + '</span>' +
			'</a>' +
			'<div class="tool-actions">' +
				'<a href="' + this._buttonUrl + '" class="btn" target="' + this._buttonTarget + '">' + this._buttonText.escapeHTML() + '</a>' +
				'<a href="#" class="link-02" id="' + this._id + '-hide"><span>' + this._hideText.escapeHTML() + '</span></a>' +
			'</div>'
        );
    	$super();
    },

    setHideHandler: function (handler) {
    	this._hideHandler = handler;
    },

    _onClose: function(event)
    {
    	Event.stop(event);
        this._hideHandler(this);
        this._componentElement.remove();
        if (this._hideUrl) {
        	new Ajax.Request(this._hideUrl, { method: 'post' });
        }
        return false;
    },

    _addEvents: function() {
    	var el = $(this._id + '-hide');
    	if (el) {
    		el.observe('click', this._onClose.bind(this));
    	}
    }
});

Jsw.namespace('Jsw.FileSharing');

Jsw.FileSharing = {
    trimLastSlash: function(str) {
        while (str[str.length-1] == '/') {
            str = str.substring(0, str.length-1);
        }
        return str;
    }
}

/* testing mode support */
Jsw.onReady(function() {
    if ('true' != Jsw.Cookie.get('debug-testing-mode')) {
        return;
    }

    if ($('bottomAnchor')) {
        var date = new Date();
        $('bottomAnchor').innerHTML = date.getTime();
    }
});

Jsw.UpgradeFailureNotification = Class.create(Jsw.Component, {

    updateInterval: 5,

    _initConfiguration: function($super, config) {

        $super(config);
        this._isBootstrapRunning = this._getConfigParam('isBootstrapRunning');
        this._bootstrapExecutionUrl = this._getConfigParam('bootstrapExecutionUrl');
        this._bootstrapStatusUrl = this._getConfigParam('bootstrapStatusUrl');
        this._upgradeFailedMessage = this._getConfigParam('upgradeFailedMessage');
        this._bootstrapInProgressMessage = this._getConfigParam('bootstrapInProgressMessage');
        this._bootstrapLinkTitle = this._getConfigParam('bootstrapLinkTitle');
    },

    _initComponentElement: function($super) {
        $super();

        this._componentElement.update(
            '<div class="msg-box msg-warning">' +
            '<div><div><div><div><div>' +
            '<div class="msg-content" id="execute-bootstrap-message">' +
                (this._isBootstrapRunning ?
                    '<span class="ajax-loading">' + this._bootstrapInProgressMessage + '</span>' :
                    this._upgradeFailedMessage + ' <a id="execute-bootstrap-link" href="#">' + this._bootstrapLinkTitle +'</a>') +
            '</div>' +
            '</div></div></div></div></div>' +
            '</div>'
        );
    },

    _addEvents: function($super) {
        $super();
        if (this._isBootstrapRunning) {
            this._scheduleUpdateStatus();
        } else {
            $('execute-bootstrap-link').observe('click', this._executeBootstrapper.bind(this));
        }
    },

    _executeBootstrapper: function() {
        new Ajax.Request(Jsw.prepareUrl(this._bootstrapExecutionUrl), {
            method: 'post',
            onCreate: this._onCreateExecute.bind(this),
            onSuccess: this._scheduleUpdateStatus.bind(this),
            onFailure: this._scheduleUpdateStatus.bind(this)
        });
    },

    _onCreateExecute: function() {
        $('execute-bootstrap-message').update('<span class="ajax-loading">' + this._bootstrapInProgressMessage + '</span>');
    },

    _scheduleUpdateStatus: function() {
        this._statusUpdate.bind(this).delay(this.updateInterval);
    },

    _statusUpdate: function() {
        new Ajax.Request(Jsw.prepareUrl(this._bootstrapStatusUrl), {
            method: 'get',
            onSuccess: this._scheduleUpdateStatus.bind(this),
            onFailure: this._scheduleUpdateStatus.bind(this),
            on0: this._scheduleUpdateStatus.bind(this)
        });
    }
});

Jsw.namespace('Jsw.CustomDescription');

Jsw.CustomDescription.PopupForm = Class.create(Jsw.PopupForm, {
    _initConfiguration: function($super, config) {
        $super(config);
        this._hint = this._getConfigParam('hint', '');
        this._value = this._getConfigParam('value', '');
        this._handler = this._getConfigParam('handler', null);
        this._sendButtonId = this._getConfigParam('sendButtonId', 'btn-send');
        this._cancelButtonId = this._getConfigParam('cancelButtonId', 'btn-cancel');
    },

    render: function($super) {
        $super();

        this.setBoxType('form-box');
        this.setTitle(this._locale.popupTitle);
        this.setTitleType(this._getConfigParam('titleCls', 'pp-edit-description'));

        $(this._contentAreaId).update(
            '<p>' + this._hint + '</p>' +
            '<textarea id="' + this._id + '-description" class="js-auto-resize" rows="4" maxlength="255" cols="80">' +
                this._value + '</textarea>'
        );

        this.addOkButton();
        this.addCancelButton();
    },

    addOkButton: function() {
        this.addRightButton(this._locale.button.ok, this._onOkClick, true, true, {id: this._sendButtonId} );
    },

    addCancelButton: function() {
        this.addRightButton(this._locale.button.cancel, this._onCancelClick, false, false, {id: this._cancelButtonId} );
    },

    _onOkClick: function() {
        this._handler($(this._id + '-description').value);
        this.hide();
    },

    _onCancelClick: function() {
        this.hide();
    }
});

Jsw.namespace('Jsw.ConfirmationPopupManager');

Jsw.ConfirmationPopupManager.PopupForm = Class.create(Jsw.PopupForm, {
    _sendButtonTitle: '',

    _initConfiguration: function($super, config) {
        $super(config);

        this._prepareUrl = this._getConfigParam('prepareUrl', '');
        this._handlerUrl = this._getConfigParam('handlerUrl', '');
        this._ids = this._getConfigParam('ids', '');
        this._formListItemsAreaId = this._id + '-form-list-items';
        this._formListAreaId = this._id + '-form-list';
        this._formBoxAreaId = this._id + '-form-box';
        this._sendButtonId = this._getConfigParam('sendButtonId', 'btn-send');
        this._cancelButtonId = this._getConfigParam('cancelButtonId', 'btn-cancel');
        this._formDescriptionId =  this._id + '-form-desc';
    },

    render: function($super) {
        $super();

        this.setBoxType('form-box');
        this._setTitle();

        $(this._contentAreaId).update(
            this._getHeadDescription() +
            this._getActionsBoxHtml() +
            '<form method="post" action="" enctype="application/x-www-form-urlencoded" id="' + this._id + '-form">' +
            '<div class="box" id="' + this._formBoxAreaId + '"></div>' +
            '<div class="list" id="' + this._formListAreaId + '"></div>' +
            '</form>' +
            this._getBottomDescription()
        );

        var form = $(this._id + '-form');
        form._formSubmit = form.submit;
        form.submit = this._onSubmit.bind(this);
        form.observe('submit', this._onSubmitEvent.bind(this));

        $(this._formBoxAreaId).insert('<div class="ajax-loading">' + this._locale.loading + '</div>');
        this._renderPreparePopup();

        this._addButtons();
        this._addActionBoxHandlers();
    },

    reload: function() {
        $(this._formListAreaId).update('');
        $(this._formBoxAreaId).update('<div class="ajax-loading">' + this._locale.loading + '</div>');
        this._preparePopup();
    },

    _getActionsBoxHtml: function() {
        return '';
    },

    _addActionBoxHandlers: function() {
        return;
    },

    _addButtons: function() {
        this.addRightButton(this._locale.buttonOk, this._onOkClick, true, true, {id: this._sendButtonId} );
        this.addRightButton(this._locale.buttonCancel, this._onCancelClick, false, false, {id: this._cancelButtonId} );
    },

    _setTitle: function() {
        this.setTitle(this._locale.title);
    },

    _getHeadDescription: function() {
        return '';
    },

    _getBottomDescription: function() {
        return '<p id="' + this._formDescriptionId + '">' + this._locale.description + '</p>';
    },

    _getButtonsArea: function() {
        return '' +
            '<div class="btns-box" id="' + this._actionButtonsId + '">' +
            '<div class="box-area">' +
            '<div class="form-row">' +
            '<div class="single-row">' +
            '<div class="field-value" id="' + this._rightActionButtonsAreaId + '"> </div>' +
            '</div>' +
            '</div>' +
            '</div>' +
            '</div>';
    },

    _onException: function(transport, exception) {
        this._addErrorMessage('Internal error: ' + exception);
        this.enable();
    },

    _onOkClick: function() {
        this._onSubmit();
    },

    _onCancelClick: function() {
        this.hide();
    },

    _preparePopup: function() {
        var ids = $H();
        var count = 0;

        this._ids.each(function(id) {
            ids.set('ids[' + count + ']', id);
            count++;
        });

        new Ajax.Request(
            Jsw.prepareUrl(this._prepareUrl), {
                method: 'post',
                parameters: ids,
                onSuccess: this._onSuccessPreparePopup.bind(this),
                onException: this._onException.bind(this)
            }
        );
    },

    _renderPreparePopup: function() {
        this._preparePopup();
    },

    _onSubmitEvent: function(event) {
        this._onSubmit();
        Event.stop(event);
        return false;
    },

    _onSuccessResponse: function(response) {
        this.hide();
        Jsw.redirect(response.redirect);
    },

    _onSuccess: function(transport) {
        this._clearMessages();
        try {
            var response = transport.responseText.evalJSON();
            if ('success' == response.status) {
                this._onSuccessResponse(response);
            } else {
                if (response.redirect) {
                    Jsw.redirect(response.redirect);
                    return;
                }
                this._addErrorMessage(response.message);
                this.enable();
            }
        } catch (e) {
            this._addErrorMessage(e.message);
            this._addErrorMessage(
                'Internal error: ' + transport.responseText
            );
            this.enable();
        }
    },

    disable: function() {
        var sendButtonWrapper = $(this._sendButtonId);
        if (sendButtonWrapper) {
            var sendButton = sendButtonWrapper.down('button');
            sendButton.disabled = true;
            sendButtonWrapper.addClassName('disabled');
            if (!this._sendButtonTitle) {
                this._sendButtonTitle = sendButton.innerHTML;
            }
            sendButton.update('<span class="wait">' + this._locale.loading + '</span>');
        }

        var cancelButtonWrapper = $(this._cancelButtonId);
        if (cancelButtonWrapper) {
            cancelButtonWrapper.down('button').disabled = true;
            cancelButtonWrapper.addClassName('disabled');
        }
    },

    enable: function() {
        var sendButtonWrapper = $(this._sendButtonId);
        if (sendButtonWrapper) {
            sendButton = sendButtonWrapper.down('button');
            sendButton.disabled = false;
            sendButton.update(this._sendButtonTitle);
            sendButtonWrapper.removeClassName('disabled');
        }

        var cancelButtonWrapper = $(this._cancelButtonId);
        if (cancelButtonWrapper) {
            cancelButtonWrapper.down('button').disabled = false;
            cancelButtonWrapper.removeClassName('disabled');
        }
    },

    _onSuccessPreparePopup: function() {
        //should be implemented in child classes
        return true;
    },

    _getLoadingIndicatorItems: function() {
        return [];
    },

    _addLoadingIndicator: function(items) {
        //should be implemented in child classes
    },

    _getAdditionalParams: function(params) {
        //should be implemented in child classes if it necessary
        return params;
    },

    _isValid: function() {
        return true;
    },

    _onSubmit: function() {

        this._clearMessages();

        if (!this._isValid()) {
            return;
        }

        var params = $H();
        var count = 0;
        this._ids.each(function(id) {
            params.set('ids[' + count + ']', id);
            count++;
        });
        params = this._getAdditionalParams(params);

        this.disable();
        this._addLoadingIndicator(this._getLoadingIndicatorItems());
        new Ajax.Request(
            this._handlerUrl,
            {
                method: 'post',
                parameters: params,
                onSuccess: this._onSuccess.bind(this),
                onException: this._onException.bind(this)
            }
        );
    }
});

Jsw.LookUp = Class.create(Jsw.Component, {
    _initConfiguration: function($super, config) {
        $super(config);

        this._name = this._getConfigParam('name', null);
        this._cls = this._getConfigParam('cls', 'lookup');
        this._data = this._getConfigParam('data', []);
        this._currentValue = this._getConfigParam('value', null);
    },

    _initComponentElement: function($super) {
        $super();

        this._valueField = new Element('input', {
            'type': 'hidden',
            'name': this._name,
            'value': this._currentValue ? this._currentValue.id : ''
        });

        this._inputField = new Element('input', {
            'type': 'text',
            'class': 'lookup-input input-text f-large-size',
            'value': this._currentValue ? this._currentValue.title : ''
        });

        this._lookUpButton =  new Element('span', {'class': 'btn lookup-button'})
            .insert(new Element('button', {'type': 'button'}));

        this._dropdownList = new Element('ul', {'class': 'dropdown-menu-list'});

        this._componentElement
            .insert(this._valueField)
            .insert(this._inputField)
            .insert(this._lookUpButton)
            .insert(new Element('div', {'class': 'dropdown-menu'})
                .insert(new Element('div', {'class': 'lookup-empty'}).update(this._locale.nothingFound))
                .insert(this._dropdownList)
            );

        this._updateData(this._data);
    },

    onChange: function() {},

    isEmpty: function() {
        return null === this._currentValue;
    },

    getValue: function() {
        return this._currentValue ? this._currentValue.id : null;
    },

    _openList: function(showAll) {
        this._componentElement.addClassName('lookup-open');
        this._lookUpButton.addClassName('lookup-button-clear');
        this._selectResults(showAll);
    },

    _closeList: function() {
        this._inputField.setValue(this._currentValue ? this._currentValue.title : '');
        this._valueField.setValue(this._currentValue ? this._currentValue.id : '');
        this._componentElement.removeClassName('lookup-open');
        this._lookUpButton.removeClassName('lookup-button-clear');
    },

    _selectResults: function(showAll) {
        var value = showAll ? '' : this._inputField.getValue().toLocaleLowerCase();
        var itemsFound = false;
        this._dropdownList.select('li').each(function(item) {
            item.removeClassName('active');
            var itemLink = item.select('a').first();
            var itemData = itemLink.innerHTML.stripTags();
            if (itemData.toLowerCase().startsWith(value)) {
                itemsFound = true;
                item.show();
                itemData = '' +
                    '<b class="search-result-label">' +
                        itemData.substr(0, value.length) +
                    '</b>' +
                    itemData.substr(value.length);
            } else {
                item.hide();
            }
            itemLink.update(itemData);
        });

        this._componentElement.select('.lookup-empty').first().toggle(!itemsFound);
    },

    _addEvents: function($super) {
        $super();

        this._inputField.observe('focus', this._inputOnFocus.bindAsEventListener(this));
        this._inputField.observe('blur', this._inputOnBlur.bindAsEventListener(this));
        this._inputField.observe('paste', this._inputOnPaste.bindAsEventListener(this));
        this._inputField.observe('keyup', this._inputOnKeyUp.bindAsEventListener(this));
        this._inputField.observe('keydown', this._inputOnKeyDown.bindAsEventListener(this));
        this._lookUpButton.observe('click', this._lookUpButtonOnClick.bindAsEventListener(this));
    },

    _inputOnFocus: function() {
        this._openList(true);
    },

    _inputOnBlur: function(event) {
        // workaround for Google Chrome
        setTimeout(function() {
            if ('' == this._inputField.getValue()) {
                this._currentValue = null;
                this.onChange();
            }
            this._closeList();
        }.bind(this), 300);
    },

    _inputOnPaste: function(event) {
        setTimeout(function() {
            this._openList();
        }.bind(this), 300);
    },

    _inputOnKeyUp: function(event) {
        if ([Jsw.keyCode.UP_ARROW,  Jsw.keyCode.DOWN_ARROW, Jsw.keyCode.LEFT_ARROW, Jsw.keyCode.RIGHT_ARROW, Jsw.keyCode.ENTER, Jsw.keyCode.ESC].indexOf(event.keyCode) != -1) {
            return;
        }

        this._openList();
    },

    _inputOnKeyDown: function(event) {
        if ([Jsw.keyCode.UP_ARROW,  Jsw.keyCode.DOWN_ARROW].indexOf(event.keyCode) != -1) {
            this._onArrowKeyPressed(event.keyCode);
        }

        if (Jsw.keyCode.ESC == event.keyCode) {
            this._closeList();
        }

        if (Jsw.keyCode.ENTER == event.keyCode) {
            event.preventDefault();
            var activeItem = this._dropdownList.select('li.active').first();
            if (activeItem) {
                var itemLink = activeItem.select('a').first();
                this._currentValue = {
                    id: itemLink.readAttribute('id'),
                    title: itemLink.innerHTML.stripTags()
                };
                this._closeList();
                this.onChange();
            } else {
                this._openList();
            }
        }
    },

    _onArrowKeyPressed: function(keyCode) {
        var nextItem = null;
        var activeItem = this._dropdownList.select('li.active').first();
        if (!activeItem) {
            nextItem = this._dropdownList.select('li').first();
            while (nextItem && !nextItem.visible()) {
                nextItem = nextItem.next();
            }
        } else if (Jsw.keyCode.DOWN_ARROW == keyCode) {
            nextItem = activeItem.next();
            while (nextItem && !nextItem.visible()) {
                nextItem = nextItem.next();
            }
        } else if (Jsw.keyCode.UP_ARROW == keyCode) {
            nextItem = activeItem.previous();
            while (nextItem && !nextItem.visible()) {
                nextItem = nextItem.previous();
            }
        }

        if (nextItem && nextItem.visible()) {
            nextItem.addClassName('active');
            if (activeItem) {
                activeItem.removeClassName('active');
            }
        }
    },

    _lookUpButtonOnClick: function(event) {
        event.preventDefault();

        if (this._componentElement.hasClassName('lookup-open')) {
            this._closeList();
        } else {
            this._openList(true);
        }
    },

    _itemOnClick: function(event, item) {
        event.preventDefault();
        this._currentValue = item;
        this._closeList();
        this.onChange();
    },

    _itemOnOver: function() {
        this._dropdownList.select('li.active').each(function(element) {
            element.removeClassName('active');
        });
    },

    _updateData: function(data) {
        this._dropdownList.update();

        data.each(function(item) {
            var listItem = new Element('li').insert(
                new Element('a', {'href': '#', 'id': item.id}).update(item.title)
            );
            listItem.observe('click', this._itemOnClick.bindAsEventListener(this, item));
            listItem.observe('mouseover', this._itemOnOver.bindAsEventListener(this));

            this._dropdownList.insert(listItem);
        }.bind(this));
    }
});

Jsw.Observer = (function(){
    var observers = {};
    var getObservers = function(ns) {
        if ('undefined' == typeof observers[ns]) {
            observers[ns] = [];
        }
        return observers[ns];
    };
    return {
        append: function(callback, ns) {
            getObservers(ns).push(callback);
        },
        appendAndCall: function(callback, ns) {
            getObservers(ns).push(callback);
            callback();
        },
        notify: function(event, ns) {
            getObservers(ns).each(function(callback) {
                callback(event);
            });
        }
    };
})();